package gov.va.med.mhv.api.patient.service.impl;

import gov.va.med.mhv.api.common.service.SessionAPIService;
import gov.va.med.mhv.api.enumeration.ErrorEnum;
import gov.va.med.mhv.api.patient.service.BBAPIService;
import gov.va.med.mhv.api.transfer.DataClassTO;
import gov.va.med.mhv.api.transfer.FacilityExtractStatusTO;
import gov.va.med.mhv.api.transfer.FacilityExtractStatusesTO;
import gov.va.med.mhv.api.transfer.GenerateReportTO;
import gov.va.med.mhv.api.transfer.Session;
import gov.va.med.mhv.api.util.MHVApiUtility;
import gov.va.med.mhv.api.util.PropertiesHelper;
import gov.va.med.mhv.usermgmt.enumeration.AccountStatusEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActionTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.service.FacilityExtractStatusCollectionServiceResponse;
import gov.va.med.mhv.usermgmt.service.MhvDownloadReportCollectionServiceResponse;
import gov.va.med.mhv.usermgmt.service.MhvDownloadReportIterator;
import gov.va.med.mhv.usermgmt.service.PatientServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.DownloadYourReportServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.PatientServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.transfer.FacilityExtractStatus;
import gov.va.med.mhv.usermgmt.transfer.MhvDownloadReport;
import gov.va.med.mhv.usermgmt.transfer.Patient;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;
import gov.va.med.mhv.usermgmt.util.Auditor;
import gov.va.med.mhv.usermgmt.util.PatientCorrelationStatusUtils;
import gov.va.med.mhv.usermgmt.util.bbdownload.DownloadHelper;
import gov.va.med.mhv.usermgmt.util.bbdownload.PdfXHTMLWrapper;

import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import javax.annotation.Resource;
import javax.naming.NamingException;
import javax.ws.rs.core.Response;

import oracle.net.aso.e;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.cxf.jaxrs.ext.MessageContext;
import org.apache.cxf.jaxrs.impl.ResponseBuilderImpl;
import org.tigris.atlas.service.VoidServiceResponse;

public class BBAPIServiceImpl implements BBAPIService{
	private static final Log log = LogFactory.getLog(BBAPIServiceImpl.class);
	
	private SessionAPIService sessionApiService;

	@Resource
	MessageContext mc;
	
	@Override
	public String getBlueButtonReport() {
		Session session = sessionApiService.checkSession(mc);
//		UserProfileServiceDelegate profDelegate = ServiceDelegateFactory.createUserProfileServiceDelegate();
//		UserProfileServiceResponse userProfRes = profDelegate.getUserProfileById(id);
//		UserProfile uf = userProfRes.getUserProfile();
		UserProfile uf = session.getPatient().getUserProfile();
		if(uf != null)
			System.out.println("USER NAME: "+ uf.getUserName());
		return null;
	}

	@Override
	public Response doRefresh() {
		Session session = sessionApiService.checkSession(mc);
		UserProfile uf = session.getPatient().getUserProfile();
		if(uf != null) {
			PatientServiceResponse patRes = getPatientServiceDelegate().getPatientForUser(uf);
			Patient pat = patRes.getPatient();
			if(pat != null) {
				try {
				getDownloadYourReportServiceDelegate().doRefresh(pat);
				}catch ( Exception e) {
					e.printStackTrace();
					MHVApiUtility.throwException(ErrorEnum.UNKNOWN_99);
				}
			}
			
		}
		ResponseBuilderImpl builder = new ResponseBuilderImpl();
		String r = "{\"status\" : \"OK\",\"description\" : null,\"nextDate\" : {} }";
		return builder.entity(r).status(200).build();
	}

	@Override
	public Response generateBBReport(GenerateReportTO genRepTO) {
		Session session = sessionApiService.checkSession(mc);
		UserProfile uf = session.getPatient().getUserProfile();
		
		String RFC1123_PATTERN  =	"EEE, dd MMM yyyy HH:mm:ss z";
		SimpleDateFormat formatter = new SimpleDateFormat(RFC1123_PATTERN);
		Date fromDate = null;
		Date toDate = null;
		try {
			fromDate = formatter.parse(genRepTO.getFromDate());
			toDate = formatter.parse(genRepTO.getToDate());
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			MHVApiUtility.throwException(ErrorEnum.DATA_INTEGRITY_ERROR_117);
		}
		VoidServiceResponse voidRes = null;
		PatientServiceResponse patRes = getPatientServiceDelegate().getPatientForUser(uf);
		Patient pat = patRes.getPatient();
		//Collection dataList = constructEligibleDataClassList(pat, Arrays.asList(genRepTO.getDataClasses()));
		Collection dataList = constructEligibleDataClassList(pat, genRepTO.getDataClasses());
		if(uf != null && dataList != null) {
			voidRes = getDownloadYourReportServiceDelegate().generateSelectedDownload(uf.getId(),dataList, fromDate, toDate, 4l, 4l);
		}
		else {
			Auditor.auditBBEventFromMHVAPI(uf.getId(), false,session.getClientApplication().getName(),ActivityActionTypeEnumeration.CUSTOM_DOWNLOAD_REQUESTED );
			MHVApiUtility.throwException(ErrorEnum.DATA_INTEGRITY_ERROR_117);
		}
		if (voidRes.getMessages().hasErrorMessages()) {
			Auditor.auditBBEventFromMHVAPI(uf.getId(), false,session.getClientApplication().getName(),ActivityActionTypeEnumeration.CUSTOM_DOWNLOAD_REQUESTED );
		}
		MHVApiUtility.throwExceptionOnErrors(ErrorEnum.UNKNOWN_99, voidRes);
		Auditor.auditBBEventFromMHVAPI(uf.getId(), true,session.getClientApplication().getName(),ActivityActionTypeEnumeration.CUSTOM_DOWNLOAD_REQUESTED );
		ResponseBuilderImpl builder = new ResponseBuilderImpl();
		String r = "{\"status\" : \"OK\",\"description\" : null,\"nextDate\" : {} }";
		return builder.entity(r).status(200).build();
	}
	
	@Override
	public FacilityExtractStatusesTO getExtractStatus() {
		
		Session session = sessionApiService.checkSession(mc);
		UserProfile uf = session.getPatient().getUserProfile();
		
		List<FacilityExtractStatusTO> facilityExtractStatusList = new ArrayList<FacilityExtractStatusTO>();
		if(uf != null) {
			PatientServiceResponse patRes = getPatientServiceDelegate().getPatientForUser(uf);
			Patient pat = patRes.getPatient();
			if(pat != null) {
				try {
					FacilityExtractStatusCollectionServiceResponse facilityExtractStatusRes = 
						getDownloadYourReportServiceDelegate().getExtractStatusList(pat);
					Collection<FacilityExtractStatus> statusses = facilityExtractStatusRes.getFacilityExtractStatuss();
					MHVApiUtility.throwExceptionOnErrors(ErrorEnum.UNKNOWN_99, facilityExtractStatusRes);
					for(FacilityExtractStatus status : statusses) {
						FacilityExtractStatusTO fac = new FacilityExtractStatusTO();
						fac.setCreatedOn(status.getCreatedOn());
						fac.setExtractType(status.getExtractType().getName());
						//fac.setFacility(status.getFacility());
						fac.setInProgress(status.getInProgress());
						fac.setLastUpdated(status.getLastUpdated());
						if(status.getFacility() != null) {
							fac.setStationNumber(status.getFacility().getFacilityInfo().getStationNumber());
						}
						else {
							fac.setStationNumber("");
						}
						facilityExtractStatusList.add(fac);
					}		
				}catch ( Exception e) {
					e.printStackTrace();
					MHVApiUtility.throwException(ErrorEnum.UNKNOWN_99);
				}
			}
			
		}

		return new FacilityExtractStatusesTO(facilityExtractStatusList);
	}	
	
	
	@Override
	public Response getBBReport(String filetype) {
		
		Session session = sessionApiService.checkSession(mc);
		UserProfile uf = session.getPatient().getUserProfile();
		MhvDownloadReport downloadReport = null;
		String expectedFileType = null;
		FileTypes fileTEnum = null;
		String contentType;
		String fileName = null;
		try {
			fileTEnum = FileTypes.valueOf(filetype);
		} catch (Exception e) {
			e.printStackTrace();
			Auditor.auditBBEventFromMHVAPI(uf.getId(),false, session.getClientApplication().getName(),ActivityActionTypeEnumeration.DOWNLOAD_MY_CUSTOM_DATA);
			MHVApiUtility.throwException(ErrorEnum.DOC_TYPE_NOT_SUPPORTED_133);
		}
		if(uf != null) {
			MhvDownloadReportCollectionServiceResponse response =
				getDownloadYourReportServiceDelegate().getDownloadedReport(uf.getId());
				
				if(!response.getMhvDownloadReports().isEmpty()) {
					MhvDownloadReportIterator it = response.iterator();
					while(it.hasNext()) {
						downloadReport = it.next();
					}
				}
		}
		if(downloadReport == null) {
			MHVApiUtility.throwException(ErrorEnum.UNKNOWN_99);
		}
		
		switch (fileTEnum) {
		case pdf:
			expectedFileType = "pdfFormat"; 
			contentType = "application/pdf";
			StringBuffer pdfFileName = new StringBuffer();  
			pdfFileName.append(downloadReport.getFileName().substring(0, downloadReport.getFileName().indexOf(".")));
			pdfFileName.append(".pdf");
			fileName = pdfFileName.toString();
			break;
		case txt:
			expectedFileType = "textFormat"; 
			contentType = "text/plain";
			StringBuffer txtFileName = new StringBuffer();  
			txtFileName.append(downloadReport.getFileName().substring(0, downloadReport.getFileName().indexOf(".")));
			txtFileName.append(".txt");
			fileName = txtFileName.toString();
			break;
		case bb:
			expectedFileType = "bbFormat"; 
			contentType = "application/octet-stream";
			StringBuffer bbFileName = new StringBuffer();  
			bbFileName.append(downloadReport.getFileName().substring(0, downloadReport.getFileName().indexOf(".")));
			bbFileName.append(".txt");
			fileName = bbFileName.toString();
			break;
		default:
			expectedFileType = "pdfFormat";	
			contentType = "application/octet-stream";
			break;
		}
		byte[] allPdfBytes = null;
		byte[] fileBytes =null;
		if(uf != null) {
			try {
				if(expectedFileType.equals("pdfFormat")) {
					PdfXHTMLWrapper p = new PdfXHTMLWrapper();
					allPdfBytes = p.makePdfFromXhtmlAPI(getUserFNameLName(uf),
							DownloadHelper.retrieveReport(DownloadHelper.getMHVDatasource(), expectedFileType, downloadReport.getDownloadReportId()));
				}
				else {
					String fileContents = DownloadHelper.retrieveReport(DownloadHelper.getMHVDatasource(), expectedFileType, 
							downloadReport.getDownloadReportId());
					fileBytes = fileContents.getBytes();
				}
			} catch (SQLException e) {
				Auditor.auditBBEventFromMHVAPI(uf.getId(),false, session.getClientApplication().getName(),ActivityActionTypeEnumeration.DOWNLOAD_MY_CUSTOM_DATA);
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (NamingException e) {
				Auditor.auditBBEventFromMHVAPI(uf.getId(),false, session.getClientApplication().getName(),ActivityActionTypeEnumeration.DOWNLOAD_MY_CUSTOM_DATA);
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			catch (Exception e) {
				Auditor.auditBBEventFromMHVAPI(uf.getId(),false, session.getClientApplication().getName(),ActivityActionTypeEnumeration.DOWNLOAD_MY_CUSTOM_DATA);
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		System.out.println("Client Name:" +session.getClientApplication().getName());
		if(expectedFileType.equals("pdfFormat"))
			Auditor.auditBBEventFromMHVAPI(uf.getId(), true,session.getClientApplication().getName(),ActivityActionTypeEnumeration.DOWNLOAD_MY_CUSTOM_DATA_PDF );
		else if (expectedFileType.equals("bbFormat"))
			Auditor.auditBBEventFromMHVAPI(uf.getId(), true,session.getClientApplication().getName(),ActivityActionTypeEnumeration.DOWNLOAD_MY_CUSTOM_DATA_BLUEBUTTON );
		else 
			Auditor.auditBBEventFromMHVAPI(uf.getId(), true,session.getClientApplication().getName(),ActivityActionTypeEnumeration.DOWNLOAD_MY_CUSTOM_DATA);
		if (expectedFileType.equals("pdfFormat")) {
			return Response.ok().entity(allPdfBytes).header("Content-Type",contentType).header
				("Content-Length", allPdfBytes.length).header("Content-Disposition","attachment; filename="+fileName).build();
		}
		else {
			return Response.ok().entity(fileBytes).header("Content-Type",contentType).header
			("Content-Length", fileBytes.length).header("Content-Disposition","attachment; filename="+fileName).build();

		}
	
	}
	
	@Override
	public DataClassTO getEligibleDataClasses() {
		
		String account_type = null;
		Session session = sessionApiService.checkSession(mc);
		UserProfile uf = session.getPatient().getUserProfile();
		Patient patient = null;
		if(uf != null) {
			PatientServiceResponse patRes = getPatientServiceDelegate().getPatientForUser(uf);
			patient = patRes.getPatient();
		}	
		Properties p = PropertiesHelper.getProperties();
		account_type = MHVApiUtility.getPatientAcctType(patient);
	   	String acctTypepropname = "bb."+account_type+".eligible.dataclasses";
		String dataClassesList = p.getProperty(acctTypepropname);
		String[] dataClasses = dataClassesList.split(",");
		return new DataClassTO(dataClasses);
	}
	
	public String getUserFNameLName(UserProfile userProfile) {
		StringBuffer userFirstLastName = new StringBuffer();
		userFirstLastName.append(userProfile.getLastName() != null ? userProfile.getLastName(): "");

		if(userProfile.getFirstName() != null) {
			userFirstLastName.append(", ");
			userFirstLastName.append(userProfile.getFirstName());
		}
		else {
			userFirstLastName.append("");
		}
		if(userProfile.getMiddleName() != null) {
			userFirstLastName.append(" " +userProfile.getMiddleName());
		}

		return userFirstLastName.toString();
	}

	//private Collection constructEligibleDataClassList(Patient patient, List<String> dataClassesList) {
	private Collection constructEligibleDataClassList(Patient patient, String[] array) {
		
		if(isValidList(patient, array)) {
			Set<String> set = new HashSet<String>(Arrays.asList(array));
			Collection dataClasses = Arrays.asList(set);
			return dataClasses;
		}
	
		return null;
	}
	private boolean isValidList(Patient patient, String[] selectedDataListArray) {
		
		Properties p = PropertiesHelper.getProperties();
		String account_type = MHVApiUtility.getPatientAcctType(patient);
	   	String acctTypepropname = "bb."+account_type+".eligible.dataclasses";
		String eligibleDataClassesList = p.getProperty(acctTypepropname);
		String[] eligibleDataClassesArray = eligibleDataClassesList.split(",");
		List eligibleDataClasses = Arrays.asList(eligibleDataClassesArray);
		
		for (String selectedDataList: selectedDataListArray)	{
			if(!eligibleDataClasses.contains(selectedDataList.toLowerCase())) {
				return false; 
			}
		}
		return true;
	}
	
	public PatientServiceDelegate getPatientServiceDelegate() {
		return ServiceDelegateFactory.createPatientServiceDelegate();
	}
	public DownloadYourReportServiceDelegate getDownloadYourReportServiceDelegate() {
		return ServiceDelegateFactory.createDownloadYourReportServiceDelegate();
	}
	
	public SessionAPIService getSessionApiService() {
		return sessionApiService;
	}

	public void setSessionApiService(SessionAPIService sessionApiService) {
		this.sessionApiService = sessionApiService;
	}

}

enum FileTypes {
  pdf,
  txt,
  bb,
}
