package gov.va.med.mhv.api.patient.service.impl;

import gov.va.med.mhv.api.cache.CacheHandler;
import gov.va.med.mhv.api.common.service.SessionAPIService;
import gov.va.med.mhv.api.enumeration.ErrorEnum;
import gov.va.med.mhv.api.patient.service.RxAPIService;
import gov.va.med.mhv.api.transfer.ErrorTO;
import gov.va.med.mhv.api.transfer.OtherPresListIncluded;
import gov.va.med.mhv.api.transfer.PrescriptionDetailTO;
import gov.va.med.mhv.api.transfer.PrescriptionTO;
import gov.va.med.mhv.api.transfer.PrescriptionsTO;
import gov.va.med.mhv.api.transfer.Session;
import gov.va.med.mhv.api.transfer.TrackingInfoDetailsTO;
import gov.va.med.mhv.api.transfer.TrackingInfoTO;
import gov.va.med.mhv.api.util.MHVApiUtility;
import gov.va.med.mhv.api.util.RxAPIServiceHelper;
import gov.va.med.mhv.rxrefill.form.PrescriptionForm;
import gov.va.med.mhv.rxrefill.service.PrescriptionServiceResponse;
import gov.va.med.mhv.rxrefill.service.RequestServiceResponse;
import gov.va.med.mhv.rxrefill.service.delegate.PrescriptionServiceDelegate;
import gov.va.med.mhv.rxrefill.service.delegate.RxTrackingServiceDelegate;
import gov.va.med.mhv.rxrefill.transfer.PharmacyPatient;
import gov.va.med.mhv.rxrefill.transfer.Prescription;
import gov.va.med.mhv.rxrefill.transfer.Request;
import gov.va.med.mhv.rxrefill.transfer.RxTracking;
import gov.va.med.mhv.rxrefill.util.ServiceResponseUtil;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActionTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityTypeEnumeration;
import gov.va.med.mhv.usermgmt.service.PatientServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.ActivityAuditServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.PatientServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.transfer.Activity;
import gov.va.med.mhv.usermgmt.transfer.Patient;
import gov.va.med.mhv.usermgmt.transfer.TransferObjectFactory;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.annotation.Resource;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.cxf.jaxrs.ext.MessageContext;
import org.apache.cxf.jaxrs.impl.ResponseBuilderImpl;
import org.tigris.atlas.messages.Message;
import org.tigris.atlas.messages.MessageUtils;
import org.tigris.atlas.messages.Messages;
import org.tigris.atlas.service.VoidServiceResponse;
/**
 * @author muazzam
 *
 */
public class RxAPIServiceImpl implements RxAPIService {
	private static final Log log = LogFactory.getLog(RxAPIServiceImpl.class);

	private SessionAPIService sessionApiService;

	@Resource
	MessageContext mc;

	/* (non-Javadoc)
	 * @see gov.va.med.mhv.api.patient.service.RxAPIService#getActivePrescriptions()
	 */
	@Override
	public PrescriptionsTO getActivePrescriptions() {
		Session session = sessionApiService.checkSession(mc);
		UserProfile uf = session.getPatient().getUserProfile();
		PrescriptionsTO presTOs = new PrescriptionsTO();
		List<PrescriptionTO> presList = new ArrayList<PrescriptionTO>();;
		RxAPIServiceHelper rxServceHelper = new RxAPIServiceHelper();
		if(uf != null) {
			boolean isSigned = rxServceHelper.isRxAgreementSigned(uf);
			if( !isSigned ) {
				MHVApiUtility.throwException(ErrorEnum.RX_AGREEMENT_NOT_ACCEPTED_135);
			}
			PatientServiceResponse patRes = getPatientServiceDelegate().getPatientForUser(uf);
			Patient pat = patRes.getPatient();

			if(pat != null) {
				try {
					PharmacyPatient pharmPat = getPharmacyPatient(uf);
					if (pharmPat != null) {
						PrescriptionServiceDelegate presServiceDelegate = getPrescriptionServiceDelegate();
						org.tigris.atlas.service.EntityCollectionServiceResponse response =
							presServiceDelegate.getActivePrescriptions(pharmPat);
						// Adding for Testing
						if (response.getMessages().hasErrorMessages()) {
							Collection<Message> messages = (Collection<Message>)response.getMessages().getErrorMessages();
							for(Message message: messages) {
								System.out.println((message.getSeverity()+":"+message.getKey()+";"));
								System.out.println("ERROR MSG: " +message.getInserts());
								
							}
							
						}
						if (response.getMessages().hasInformationalMessages()) {
							Collection<Message> messages = (Collection<Message>)response.getMessages().getInformationalMessages();
							for(Message message: messages) {
								System.out.println((message.getSeverity()+":"+message.getKey()+";"));
								System.out.println("INFO MSG: " +message.getInserts());
							}
							
						}
						//Ending Here
						//Starting to clean up message...
						StringBuffer failedStationList = new StringBuffer();
						Messages filteredMessages = ServiceResponseUtil.convertMessages(null, response, "rxrefill.");
				        System.out.println("INFO MSG COUNT: " +filteredMessages.getInformationalMessageCount() );
				        RequestServiceResponse reqServRes =  presServiceDelegate.getLastSuccessfulExtractRequestForUser(pharmPat);
				        String lastUpdatedtime = null;
				        if (reqServRes.getRequest() != null && reqServRes.getRequest().getModifiedDate() != null) {
				        	lastUpdatedtime = MHVApiUtility.convertDateToRFC1123Pattern(reqServRes.getRequest().getModifiedDate());
				        }	
						if ( !filteredMessages.getInformationalMessages().isEmpty() ){
							Collection<Message> messages = filteredMessages.getInformationalMessages();
							for(Message message: messages) {
								System.out.println((message.getSeverity()+":"+message.getKey()+";"));
								System.out.println("INFO 2 MSG: " +message.getInserts());
								Iterator it = message.getInserts().iterator();
								while (it.hasNext()) {
									failedStationList.append(it.next().toString());
									if (it.hasNext())
											failedStationList.append(",");
								}	
							//	String formattedMsg = MessageUtils.getSummaryMessage(message.getKey(), message.getInserts(), PrescriptionForm.class, null);
							//	System.out.println("FORMATTED MESSAGE: " +formattedMsg);
							}
						}	
						auditPrecriptionTracking(pharmPat.getPatient(), !response.getMessages().hasErrorMessages(), session, ActivityActionTypeEnumeration.VIEW_ACTIVE_PRESCRIPTION);
						MHVApiUtility.throwExceptionOnErrors(ErrorEnum.UNKNOWN_99, response);
						Collection entities = response.getEntities();
						presTOs.setFailedStationList(failedStationList.toString());
						presTOs.setLastUpdatedTime(lastUpdatedtime);
						if (entities != null) {
							for (Object e: entities) {
								Prescription rx = (Prescription) e;
								presList.add(rxServceHelper.convertToPrescriptionTO(rx));
							}
							presTOs.setPrescriptionList(presList);	
						}

					}
					else {
						MHVApiUtility.throwException(ErrorEnum.DATA_INTEGRITY_ERROR_117);
					}
				}catch ( Exception e) {
					e.printStackTrace();
					MHVApiUtility.throwException(ErrorEnum.UNKNOWN_99);
				}
			}

		}

		return presTOs;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.mhv.api.patient.service.RxAPIService#getHistoricalPrescriptions()
	 */
	@Override
	public PrescriptionsTO getHistoricalPrescriptions() {
		Session session = sessionApiService.checkSession(mc);
		UserProfile uf = session.getPatient().getUserProfile();
		PrescriptionsTO presTOs = new PrescriptionsTO();
		List<PrescriptionTO> presList = new ArrayList<PrescriptionTO>();
		RxAPIServiceHelper rxServceHelper = new RxAPIServiceHelper();
		if(uf != null) {
			boolean isSigned = rxServceHelper.isRxAgreementSigned(uf);
			if( !isSigned ) {
				MHVApiUtility.throwException(ErrorEnum.RX_AGREEMENT_NOT_ACCEPTED_135);
			}
			PatientServiceResponse patRes = getPatientServiceDelegate().getPatientForUser(uf);
			Patient pat = patRes.getPatient();

			if(pat != null) {
				try {
					PharmacyPatient pharmPat = getPharmacyPatient(uf);
					if (pharmPat != null) {
						PrescriptionServiceDelegate presServiceDelegate = getPrescriptionServiceDelegate();
						org.tigris.atlas.service.EntityCollectionServiceResponse response =
							presServiceDelegate.getHistoricalPrescriptions(pharmPat);
						
						// Adding for Testing
						if (response.getMessages().hasErrorMessages()) {
							Collection<Message> messages = (Collection<Message>)response.getMessages().getErrorMessages();
							for(Message message: messages) {
								System.out.println((message.getSeverity()+":"+message.getKey()+";"));
								System.out.println("ERROR MSG: " +message.getInserts());
								
							}
							
						}
						if (response.getMessages().hasInformationalMessages()) {
							Collection<Message> messages = (Collection<Message>)response.getMessages().getInformationalMessages();
							for(Message message: messages) {
								System.out.println((message.getSeverity()+":"+message.getKey()+";"));
								System.out.println("INFO MSG: " +message.getInserts());
							}
							
						}
						//Ending Here
						//Starting to clean up message...
						Messages filteredMessages = ServiceResponseUtil.convertMessages(null, response, "rxhistory");
				        System.out.println("INFO MSG COUNT: " +filteredMessages.getInformationalMessageCount() );
				        StringBuffer failedStationList = new StringBuffer();
				        RequestServiceResponse reqServRes =  presServiceDelegate.getLastSuccessfulExtractRequestForUser(pharmPat);
				        String lastUpdatedtime = null;
				        if (reqServRes.getRequest() != null && reqServRes.getRequest().getModifiedDate() != null) {
				        	lastUpdatedtime = MHVApiUtility.convertDateToRFC1123Pattern(reqServRes.getRequest().getModifiedDate());
				        }
						if ( !filteredMessages.getInformationalMessages().isEmpty() ){
							Collection<Message> messages = filteredMessages.getInformationalMessages();
							for(Message message: messages) {
								System.out.println((message.getSeverity()+":"+message.getKey()+";"));
								System.out.println("INFO 2 MSG: " +message.getInserts());
								Iterator it = message.getInserts().iterator();
								while (it.hasNext()) {
									failedStationList.append(it.next().toString());
									if (it.hasNext())
										failedStationList.append(",");
								}
							//	String formattedMsg = MessageUtils.getSummaryMessage(message.getKey(), message.getInserts(), PrescriptionForm.class, null);
							//	System.out.println("FORMATTED MESSAGE: " +formattedMsg);
							}
						}
						auditPrecriptionTracking(pharmPat.getPatient(), !response.getMessages().hasErrorMessages(), session, ActivityActionTypeEnumeration.VIEW_HISTORICAL_PRESCRIPTION);
						MHVApiUtility.throwExceptionOnErrors(ErrorEnum.UNKNOWN_99, response);
						Collection entities = response.getEntities();
						presTOs.setFailedStationList(failedStationList.toString());
						presTOs.setLastUpdatedTime(lastUpdatedtime);
						if (entities != null) {
							for (Object e: entities) {
								Prescription rx = (Prescription) e;
								presList.add(rxServceHelper.convertToPrescriptionTO(rx));
							}
							presTOs.setPrescriptionList(presList);	
						}

					}
					else {
						MHVApiUtility.throwException(ErrorEnum.DATA_INTEGRITY_ERROR_117);
					}
				}catch ( Exception e) {
					e.printStackTrace();
					MHVApiUtility.throwException(ErrorEnum.UNKNOWN_99);
				}
			}

		}

		return presTOs;

	}

/* 
	@Override
	public PrescriptionDetailTO getPrescriptionDetail(String rxid) {
		if(rxid == null) {
			if (log.isErrorEnabled())
				log.error("MHV API received NULL string from client");
			MHVApiUtility.throwException(ErrorEnum.PRESCRIPTIONID_NOT_FOUND_138);
		}
		RxAPIServiceHelper rxServceHelper = new RxAPIServiceHelper();
		Session session = sessionApiService.checkSession(mc);
		UserProfile uf = session.getPatient().getUserProfile();
		PrescriptionServiceDelegate presServiceDelegate = getPrescriptionServiceDelegate();
		PrescriptionServiceResponse preRes = presServiceDelegate.getPrescriptionDetailsForPrescriptionID(Long.parseLong(rxid));
		MHVApiUtility.throwExceptionOnErrors(ErrorEnum.UNKNOWN_99, preRes);
		Prescription pres = preRes.getPrescription();
		if(pres == null) {
			if (log.isErrorEnabled())
				log.error("MHV API could NOT find the prescription ID: " + rxid +" user has requested");
			MHVApiUtility.throwException(ErrorEnum.PRESCRIPTIONID_NOT_FOUND_138);
		}
		if (pres.getUserId().longValue() != uf.getId().longValue()) {
			if(log.isInfoEnabled())
				log.info("The prescription ID: " + rxid +" does not belong to USER: " + uf.getId());
			MHVApiUtility.throwException(ErrorEnum.RX_USER_NOT_OWNER_136);
		}	

		return rxServceHelper.convertToPrescriptionDetailTO(pres);
	}
/*	

	/* (non-Javadoc)
	 * @see gov.va.med.mhv.api.patient.service.RxAPIService#refillPrescriptions(java.lang.String)
	 */
	@Override
	public Response refillPrescriptions(String rxid) {
		Session session = sessionApiService.checkSession(mc);
		UserProfile uf = session.getPatient().getUserProfile();
		PrescriptionServiceDelegate presServiceDelegate = getPrescriptionServiceDelegate();
		List<Prescription> rxidList = new ArrayList<Prescription>();
		//		for (Long rxid: rxids) {
		PrescriptionServiceResponse preRes = presServiceDelegate.getPrescriptionDetailsForPrescriptionID(Long.parseLong(rxid));
		MHVApiUtility.throwExceptionOnErrors(ErrorEnum.UNKNOWN_99, preRes);
		Prescription pres = preRes.getPrescription();
		if(pres == null) {
			MHVApiUtility.throwException(ErrorEnum.PRESCRIPTIONID_NOT_FOUND_138);
		}
		rxidList.add(pres);
		if (!pres.getUserId().equals(uf.getId()))
			MHVApiUtility.throwException(ErrorEnum.RX_USER_NOT_OWNER_136);	
		//}
		
		//ALL prescriptionId belongs to the same user. proceed
		
		if(pres.getIsRefillable() != null && pres.getIsRefillable() ) {
			PharmacyPatient pharmPat = getPharmacyPatient(uf);
			VoidServiceResponse response = presServiceDelegate.refillPrescriptionsAPI(pharmPat, rxidList);
			MHVApiUtility.throwExceptionOnErrors(ErrorEnum.UNKNOWN_99, response);

			// Beauty lies in the eyes of beholder 
			if (response.getMessages().hasErrorMessages()) {
				Collection<Message> messages = (Collection<Message>)response.getMessages().getErrorMessages();
				for(Message message: messages) {
					System.out.println((message.getSeverity()+":"+message.getKey()+";"));
					System.out.println("ERROR MSG in REFILL PRESCRIPTION: " +message.getInserts());
				}
			}
			Collection<Message> infoMessages = null;;
			List<Request> requests = new ArrayList<Request>();
			if (response.getMessages().hasInformationalMessages()) {
				infoMessages = (Collection<Message>)response.getMessages().getInformationalMessages();
		        List messagesToRemove = new ArrayList();
				for (Iterator i = infoMessages.iterator(); i.hasNext();) {
		            Message message = (Message) i.next();
		            if ("prescriptionservice.request".equals(message.getKey())) {
		            	Iterator inserts = message.getInserts().iterator();
		            	if (inserts.hasNext()) {
		            		Object object = inserts.next();
		            		if (object instanceof Request) {
		            			requests.add((Request) object);
		            			messagesToRemove.add(message); 
		            		}
		            	}
		            }
		        }
				infoMessages.removeAll(messagesToRemove); 
		        
				System.out.println("Modified InfoMesgs Size is " +infoMessages.size());
				
				
				
			}
			// 1st Beauty ends and 2nd starts
			ResponseBuilderImpl builder = new ResponseBuilderImpl();
			if( infoMessages != null && !infoMessages.isEmpty() ) {
				builder.status(400);
				ErrorTO error = new ErrorTO(ErrorEnum.PRESCRIPTIONID__REFILL_ERROR_140);
				StringBuffer sb = new StringBuffer();
				for(Message message: infoMessages) {
					Iterator it = message.getInserts().iterator();
					while (it.hasNext()) {
						sb.append(it.next().toString());
					}
				}
				error.setDeveloperMessage(sb.toString());
				CacheHandler.getInstance().setError(error);
				Response responseException = builder.build();
				auditRefillrequest(pharmPat.getPatient(), pres,	false, session);
				throw new WebApplicationException(responseException);
			}
			else {
				org.tigris.atlas.service.EntityCollectionServiceResponse actResponse =	presServiceDelegate.getActivePrescriptions(pharmPat);
				Messages filteredMessages = ServiceResponseUtil.convertMessages(requests, actResponse, "rxrefill.");
				System.out.println("INFO MSG COUNT in RXAPI: " +filteredMessages.getInformationalMessageCount() );
				StringBuffer failedStationList = new StringBuffer();
				List<String> infoList = new ArrayList();
				if ( !filteredMessages.getInformationalMessages().isEmpty() ){
					Collection<Message> messages = filteredMessages.getInformationalMessages();
					for(Message message: messages) {
						infoList.add(message.getKey());
						System.out.println((message.getSeverity()+":"+message.getKey()+";"));
						System.out.println("INFO 2 MSG in RXAPI: " +message.getInserts());
						Iterator it = message.getInserts().iterator();
						while (it.hasNext()) {
							failedStationList.append(it.next().toString());
							if (it.hasNext())
								failedStationList.append(",");
						}	
					}
				}
				if(infoList.contains("rxrefill.info.vista.refillsuccess")) {
					System.out.println("Refill is SUCCESSFUL FINALLY");
					auditRefillrequest(pharmPat.getPatient(), pres,	!response.getMessages().hasErrorMessages(), session);
					return Response.ok("Successfully submitted to: "+failedStationList.toString()).build();
				}
				else if(infoList.contains("rxrefill.error.vista.systemfailure")) {
					ErrorTO error = new ErrorTO(ErrorEnum.PRESCRIPTIONID__REFILL_ERROR_140);
					error.setDeveloperMessage("MHV is unable to process your request: System Failed to Connect : "+failedStationList.toString());
					CacheHandler.getInstance().setError(error);
					Response responseException = builder.build();
					auditRefillrequest(pharmPat.getPatient(), pres,	false, session);
					throw new WebApplicationException(responseException);
				}
				else if(infoList.contains("rxrefill.info.vista.inProgress")) {
					ErrorTO error = new ErrorTO(ErrorEnum.PRESCRIPTIONID__REFILL_ERROR_140);
					error.setDeveloperMessage("MHV is unable to process your request: Refill is in Process ");
					CacheHandler.getInstance().setError(error);
					Response responseException = builder.build();
					auditRefillrequest(pharmPat.getPatient(), pres,	false, session);
					throw new WebApplicationException(responseException);
				}
				else if(infoList.contains("rxrefill.error.vista.prescriptionrejected")) {
					ErrorTO error = new ErrorTO(ErrorEnum.PRESCRIPTIONID__REFILL_ERROR_140);
					error.setDeveloperMessage("MHV is unable to process your request: Prescription Rejected: " +failedStationList.toString());
					CacheHandler.getInstance().setError(error);
					Response responseException = builder.build();
					auditRefillrequest(pharmPat.getPatient(), pres,	false, session);
					throw new WebApplicationException(responseException);
				}
				else if(infoList.contains("rxrefill.error.vista.maxfailureexceeded")) {
					ErrorTO error = new ErrorTO(ErrorEnum.PRESCRIPTIONID__REFILL_ERROR_140);
					error.setDeveloperMessage("MHV is unable to process your request: Max Failure Exceeded");
					CacheHandler.getInstance().setError(error);
					Response responseException = builder.build();
					auditRefillrequest(pharmPat.getPatient(), pres,	false, session);
					throw new WebApplicationException(responseException);
				}
				
				
				
			}	
		}
		else {
			MHVApiUtility.throwException(ErrorEnum.PRESCRIPTIONID_NOT_REFILLABLE_139);
		}
		// if the flow comes this far. then its a serious problem. return nothing. 
		return null;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.mhv.api.patient.service.RxAPIService#getTrackingDetail(java.lang.String)
	 */
	@Override
	public TrackingInfoDetailsTO getTrackingDetail(String rxid) {
		if(rxid == null) {
			if (log.isErrorEnabled())
				log.error("MHV API received NULL string from client");
			MHVApiUtility.throwException(ErrorEnum.PRESCRIPTIONID_NOT_FOUND_138);
		}
		Session session = sessionApiService.checkSession(mc);
		UserProfile uf = session.getPatient().getUserProfile();
		PrescriptionServiceDelegate presServiceDelegate = getPrescriptionServiceDelegate();
		PrescriptionServiceResponse preRes = presServiceDelegate.getPrescriptionDetailsForPrescriptionID(Long.parseLong(rxid));
		MHVApiUtility.throwExceptionOnErrors(ErrorEnum.UNKNOWN_99, preRes);
		Prescription pres = preRes.getPrescription();
		if(pres == null) {
			if (log.isErrorEnabled())
				log.error("MHV API could NOT find the prescription ID: " + rxid +" user: " + uf.getId() + " has requested");
			MHVApiUtility.throwException(ErrorEnum.PRESCRIPTIONID_NOT_FOUND_138);
		}
		if (pres.getUserId().longValue() != uf.getId().longValue()) {
			if(log.isInfoEnabled())
				log.info("The prescription ID: " + rxid +" does not belong to USER: " + uf.getId());
			MHVApiUtility.throwException(ErrorEnum.RX_USER_NOT_OWNER_136);
		}
		// Good to go Start Tracking

		String mainStationNumber = pres.getInstitution().getStationNumber();
		String rxnumber = pres.getPrescriptionNumber();
		Date releaseDateTime = pres.getReleaseDateTime();

		RxTrackingServiceDelegate trackingService = getRxTrackingServiceDelegate();
		Collection<RxTracking> trackingNumbers = trackingService.getPrescriptionTrackingNumbers(rxnumber, mainStationNumber).getRxTrackings();

		TrackingInfoDetailsTO trackingInfoDetailsTO = new TrackingInfoDetailsTO();
		trackingInfoDetailsTO.setPrescriptionName(pres.getDrugName());
		trackingInfoDetailsTO.setPrescriptionNumber(pres.getPrescriptionNumber());
		trackingInfoDetailsTO.setFacilityName(pres.getInstitution().getName());
		String divisionPhone = "";
		String ndcNumber ="";
		String lastUpdatedtime ="";
		for (RxTracking rx: trackingNumbers) {
			divisionPhone = rx.getDivisionPhone();
			if (rx.getDateLoaded() != null) {
				lastUpdatedtime = MHVApiUtility.convertDateToRFC1123Pattern (new Date(rx.getDateLoaded().getTime()));
			}	
			ndcNumber = rx.getNdc();
			if (divisionPhone.isEmpty() || ndcNumber.isEmpty() || rx.getDateLoaded() == null) 
				continue;
			else
				break;
		}
		
		trackingInfoDetailsTO.setRxInfoPhoneNumber(divisionPhone);
		trackingInfoDetailsTO.setLastUpdatedtime(lastUpdatedtime);
		trackingInfoDetailsTO.setNdcNumber(ndcNumber);
		List <TrackingInfoTO> trackingInfo= new ArrayList<TrackingInfoTO>();
		for (RxTracking rx: trackingNumbers) {
			TrackingInfoTO trackInf = new TrackingInfoTO();
			trackInf.setTrackingNumber(rx.getTrackingNumber());
			trackInf.setDeliveryService(rx.getCarrier());
			trackInf.setShippedDate(MHVApiUtility.convertDateToRFC1123Pattern(rx.getCompleteDateTime()));

			// Now pull other Prescription infos for the Tracking Number
			List<OtherPresListIncluded> otherPresListIncludedList = new ArrayList<OtherPresListIncluded>();
			// Get the RxIds associated with the Tracking number
			if(log.isInfoEnabled())
				log.info("Dealing with Tracking Number: " + rx.getTrackingNumber());
			Collection<RxTracking> rxTrackingNumbers = trackingService.getPrescriptionDataByTrackingNumber(rx.getTrackingNumber()).getRxTrackings();
			if(log.isInfoEnabled())
				log.info("Tracking Number: " + rx.getTrackingNumber() +" has total Rx: " +rxTrackingNumbers.size()); 
			for (RxTracking rx1: rxTrackingNumbers) {
				OtherPresListIncluded otherPresListIncluded = new OtherPresListIncluded();
				//Get the Prescription of the RxId
				if(log.isInfoEnabled()) {
					log.info("Dealing with Rx1: " + rx1.getRxNumber() +" of STATION: " + rx1.getStationNumber() 
						+ " OF Tracking: " + rx.getTrackingNumber() );
				}	
				Prescription pres1 = presServiceDelegate.getPrescriptionDetailsForPrescriptionNumber(rx1.getRxNumber(),rx1.getStationNumber()).getPrescription();
				//NOTE: pres1 is NULL does NOT mean the prescription Does NOT exist. System might NOT be able find coz 
				//	of combination of pres number and station is missing in PRESCRIPTIONS. IN Reality HOWZ that possible I don't know.
				// Following what GUI does in those scenario.. ADD N/A for PRESSCIPTION NAME. and ADD PRESCRIPTION NUMBER from 
				// RX_TRACKING table.
				otherPresListIncluded.setNdcNumber(rx1.getNdc());
				if(pres1 == null) {
					if(log.isInfoEnabled()) {
						log.info("NO prescritionS found of Rx1: " + rx1.getRxNumber() +" of STATION: " + rx1.getStationNumber() 
							+ " OF Tracking: " + rx.getTrackingNumber() );
					}
					otherPresListIncluded.setPrescriptionName("N/A");
					otherPresListIncluded.setPrescriptionNumber(rx1.getRxNumber());
					otherPresListIncluded.setStationNumber(rx1.getStationNumber());
				}
				//NOTE: to EXCLUDE the parent prescription in the OTHER LIST
				//pres1.getPrescriptionNumber().equals(pres.getPrescriptionNumber()

				if(pres1 != null && !pres1.getPrescriptionNumber().equals(pres.getPrescriptionNumber())) {
					if(log.isInfoEnabled())
						log.info("PRES1: " +pres1.getPrescriptionNumber() + " PRES: " + pres.getPrescriptionNumber());
					if (pres1.getDrugName() != null)
						otherPresListIncluded.setPrescriptionName(pres1.getDrugName());
					otherPresListIncluded.setPrescriptionNumber(pres1.getPrescriptionNumber());
					otherPresListIncluded.setStationNumber(pres1.getInstitution().getStationNumber());

				}
				if(otherPresListIncluded.getPrescriptionNumber() != null)
					otherPresListIncludedList.add(otherPresListIncluded);	

			}
			trackInf.setOtherPrescriptionListIncluded(otherPresListIncludedList);
			trackingInfo.add(trackInf);
		}
		trackingInfoDetailsTO.setTrackingInfo(trackingInfo);
		return trackingInfoDetailsTO;
	}	

	
	private void auditRefillrequest(Patient patient, Prescription pres,	boolean success, Session session) {
		Activity activity = TransferObjectFactory.createActivity();
		activity.setAction(ActivityActionTypeEnumeration.getEnum(ActivityActionTypeEnumeration.REQUEST_REFILL));
		activity.setActionType(ActivityTypeEnumeration.RXREFILL);
		activity.setCompletionTime(new Timestamp(new Date().getTime()));
		activity.setPerformerType(ActivityActorTypeEnumeration.getEnum(ActivityActorTypeEnumeration.SYSTEM));
		activity.setSubjectId(patient.getUserProfile().getId());
		activity.setDetailValue("Remote System: " +session.getClientApplication().getName() +" :: " +pres.getDrugName());
		activity.setStatus(success);
		getAuditServiceDelegate().audit(activity);


	}

	private void auditPrecriptionTracking(Patient patient, boolean success, Session session, String activityActionType ) {
		Activity activity = TransferObjectFactory.createActivity();
		activity.setAction(ActivityActionTypeEnumeration.getEnum(activityActionType));
		activity.setActionType(ActivityTypeEnumeration.RXREFILL);
		activity.setCompletionTime(new Timestamp(new Date().getTime()));
		activity.setPerformerType(ActivityActorTypeEnumeration.getEnum(ActivityActorTypeEnumeration.SYSTEM));
		activity.setSubjectId(patient.getUserProfile().getId());
		activity.setDetailValue("Remote System: " +session.getClientApplication().getName());
		activity.setStatus(success);
		getAuditServiceDelegate().audit(activity);


	}
	
	public ActivityAuditServiceDelegate getAuditServiceDelegate() { 
		return ServiceDelegateFactory.createActivityAuditServiceDelegate();
	}	

	public PatientServiceDelegate getPatientServiceDelegate() {
		return ServiceDelegateFactory.createPatientServiceDelegate();
	}
	public PrescriptionServiceDelegate getPrescriptionServiceDelegate() {
		return gov.va.med.mhv.rxrefill.service.delegate.ServiceDelegateFactory.
		createPrescriptionServiceDelegate();
	}
	
	public RxTrackingServiceDelegate getRxTrackingServiceDelegate() {
		return gov.va.med.mhv.rxrefill.service.delegate.ServiceDelegateFactory.
		createRxTrackingServiceDelegate();
	}

	private PharmacyPatient getPharmacyPatient(UserProfile up) {
		Patient patient = gov.va.med.mhv.usermgmt.service.delegate.
		ServiceDelegateFactory. createPatientServiceDelegate().
		getPatientForUser(up).getPatient();

		if (patient == null) return null;
		return getPrescriptionServiceDelegate().getPharmacyPatient(patient).getPharmacyPatient();
	}
	

	public SessionAPIService getSessionApiService() {
		return sessionApiService;
	}

	public void setSessionApiService(SessionAPIService sessionApiService) {
		this.sessionApiService = sessionApiService;
	}



}
