package gov.va.med.mhv.api.transfer;


import gov.va.med.mhv.api.transfer.ClientApplication;
import gov.va.med.mhv.api.util.ClientApplicationHelper;
import gov.va.med.mhv.usermgmt.transfer.Patient;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;


import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.concurrent.TimeUnit;

import org.apache.commons.httpclient.util.DateUtil;

import com.sun.corba.se.impl.io.TypeMismatchException;

public class Session implements Serializable {
	private static final long serialVersionUID = -5454873976482094676L;
	
	private long userId;
	private Date timestamp;
	private Patient patient;
	private ClientApplication clientApplication;
	private int expirationSeconds = 300; //Default to 5 minutes

	public Session(long userId, ClientApplication clientApplication) {
		this.userId = userId;
		this.clientApplication = clientApplication;
		this.expirationSeconds = clientApplication.getTimeoutSeconds().intValue();
		touch();
	}

	private Session(long userId, ClientApplication clientApplication, Date timestamp) {
		this.userId = userId;
		this.clientApplication = clientApplication;
		this.expirationSeconds = clientApplication.getTimeoutSeconds().intValue();
		this.timestamp = timestamp;
	}

	public long getUserId() {
		return userId;
	}
	public void setUserId(long userId) {
		this.userId = userId;
	}
	public Date getTimestamp() {
		return timestamp;
	}
	public void setTimestamp(Date timestamp) {
		this.timestamp = timestamp;
	}
	public long getExpirationSeconds() {
		return expirationSeconds;
	}
	public void setExpirationSeconds(int expirationSeconds) {
		this.expirationSeconds = expirationSeconds;
	}
	public Patient getPatient() {
		return patient;
	}
	public void setPatient(Patient patient) {
		this.patient = patient;
	}

	public ClientApplication getClientApplication() {
		return clientApplication;
	}
	public void setClientApplication(ClientApplication clientApplication) {
		this.clientApplication = clientApplication;
	}
	public String getFormattedTimestamp() {
		Date expiresDate = timestamp;
		return DateUtil.formatDate(expiresDate);
	}


	public void touch() {
		this.timestamp = new Date(System.currentTimeMillis() + (getExpirationSeconds() * 1000));
	}

	public boolean isExpired() {
		long currentTimeinSec = TimeUnit.MILLISECONDS.toSeconds(System.currentTimeMillis());
		long timeSeconds = TimeUnit.MILLISECONDS.toSeconds(timestamp.getTime());
		if ((currentTimeinSec - timeSeconds) <= (getExpirationSeconds()) ) {
			return false;
		} else {
			return true;
		}
	}
	
	public byte[] marshall() throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		baos.write(String.valueOf(getUserId()).getBytes());
		baos.write("|".getBytes());
		baos.write(String.valueOf(getClientApplication().getId()).getBytes());
		baos.write("|".getBytes());
		baos.write(String.valueOf(getTimestamp().getTime()).getBytes());
		
		return baos.toByteArray();
	}
	public static Session unmarshall(byte[] bytes) throws IOException,
			ClassNotFoundException {
		String sessionStr = new String(bytes);
		StringTokenizer st = new StringTokenizer(sessionStr,"|");
		if( st.countTokens() == 3 ) {
			try {
				return new Session(Long.parseLong(st.nextToken()),ClientApplicationHelper.findClientApplication(Long.parseLong(st.nextToken())), new Date(Long.parseLong(st.nextToken())) );
			} catch (Exception e) {
				throw new IOException("Unable to unmarshall object");
			}
		} else {
			throw new IOException("Unable to unmarshall object");
		}
	}
	
	public String toString() { 
		return "Session {userId: " + userId + ";expirationTimestamp: " + timestamp + "; User: " + patient.getUserProfile().getUserName() + ";ClientApplication: " + clientApplication  +"}";
	}

}
