package gov.va.med.mhv.imaging;

import gov.va.med.mhv.api.util.PropertiesHelper;
import gov.va.med.mhv.imaging.transfer.FileTO;
import gov.va.med.mhv.imaging.transfer.FilesTO;
import gov.va.med.mhv.imaging.util.CVIXDateUtil;
import gov.va.med.mhv.integration.phr.client.cvix.CVIXService;
import gov.va.med.mhv.integration.phr.client.cvix.Study;
import gov.va.med.mhv.integration.phr.client.cvix.Study.Serieses.Series;
import gov.va.med.mhv.integration.phr.client.cvix.Study.Serieses.Series.Images.Image;
import gov.va.med.mhv.integration.phr.client.cvix.model.ImageTO;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActionTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.service.PatientServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.transfer.Patient;
import gov.va.med.mhv.usermgmt.util.Auditor;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MultivaluedHashMap;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.StreamingOutput;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.cxf.jaxrs.client.WebClient;
import org.apache.cxf.jaxrs.ext.MessageContext;

public class ImagingServiceImpl implements ImagingService {
	private static final String CONTENT_TYPE = "application/json";

	private static final Log log = LogFactory.getLog(ImagingServiceImpl.class);

	// public static ZipFile ZIP;
	//
	// static {
	// try {
	// URL zipUrl = PhrServiceImpl.class.getResource("/html.zip");
	// File zipFile = new File(zipUrl.toURI());
	// ZIP = new ZipFile(zipFile);
	// } catch (IOException e) {
	// e.printStackTrace();
	// } catch (URISyntaxException e) {
	// e.printStackTrace();
	// }
	// }

	@Resource
	MessageContext mc;
	@Context
	private HttpServletResponse response;
	@Context
	private HttpServletRequest request;

	private static String ACCESSCODE = "XXXXXX";
	private static String VERIFYCODE = "XXXXXX";
	private static final String ICN_FIELD = "icn";
	private static String MHV_URL = null;
	private static String CVIX_URL = null;

	static {
		Properties p = PropertiesHelper.getProperties();
		MHV_URL = p.getProperty("mhv-host");
		if (log.isInfoEnabled()) {
			log.info("ImagingService is using host: '" + MHV_URL
					+ "' for icn retrieval");
		}

		CVIX_URL = p.getProperty("cvix-host");
		if (log.isInfoEnabled()) {
			log.info("ImagingService is using host: '" + CVIX_URL
					+ "' for cvix integration");
		}

		ACCESSCODE = p.getProperty("accesscode");
		if (log.isDebugEnabled()) {
			log.debug("ImagingService is using accesscode: '" + ACCESSCODE
					+ "'");
		}

		VERIFYCODE = p.getProperty("verifycode");
		if (log.isDebugEnabled()) {
			log.debug("ImagingService is using verifycode: '" + VERIFYCODE
					+ "'");
		}

	}

	@Override
	public FilesTO getFileList(String studyId) {
		FilesTO resp = new FilesTO();
		String icn = "";
		Patient pat = null;
		String activityDetailValue = studyId;
		try {
			// TRANSFORM URI (Getting ICN from MHV)
			icn = getIcnFromMHV();
			PatientServiceResponse patResponse = ServiceDelegateFactory
					.createPatientServiceDelegate().getPatientByIcn(icn);
			pat = patResponse.getPatient();

			List<FileTO> fileList = new ArrayList<FileTO>();
			SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
			// ADD DIAGNOSTIC IMAGES
			Study study = new CVIXService().getCVIXClient().getStudyResult(
					CVIX_URL, ACCESSCODE, VERIFYCODE, icn, studyId);
			String procedureDateStr = "UNKNOWN";
			try {
				procedureDateStr = sdf.format(CVIXDateUtil.parseDate(study
						.getProcedureDate().toString()));
			} catch (Exception e) {
				log.warn("Found an invalid date for procedureDate: '"
						+ study.getProcedureDate() + "' for ICN: '" + icn
						+ "', for StudyId: '" + studyId + "'");
			}
			// SET THE ZIP FILE NAME FOR CONTAINER [mhv_{patient
			// name}_{procedureDate}_{study description}.zip]
			String fileName = "mhv_"
					+ (study.getPatientName()).substring(0, study
							.getPatientName().indexOf(','))
					+ "_"
					+ procedureDateStr
					+ "_"
					+ (study.getDescription().replaceAll(" ", "_")).replaceAll(
							",", "_") + ".zip";
			resp.setZipFileName(fileName);

			// ADD TEXT REPORT
			fileList.add(new FileTO(study.getStudyId().replaceAll(
					"-[0-9]{10}V[0-9]{6}", "-{ICN}"), "report", "/Report/"));

			// ADD TEMPLATE FILES (FUTURE)
			// fileList.add(new
			// FileTO("/mhv-portal-web/ShowBinary/BEA%20Repository/pdf/BlueButtonUserGuide.pdf",
			// "content", "/User Guide/"));

			// ADD DIAGNOSTIC IMAGE FILES
			List<Series> list = study.getSerieses().getSeries();
			for (Series s : list) {
				String path = "/Images/Series " + s.getSeriesNumber() + "/";
				for (Image img : s.getImages().getImage()) {
					// If the Reference quality image is not available respond
					// with the diagnostic quality
					String imageUri = (img.getReferenceImageUri() != null ? img
							.getReferenceImageUri() : img
							.getDiagnosticImageUri());
					String xformName = imageUri.replaceAll(
							"-[0-9]{10}V[0-9]{6}", "-{ICN}");
					fileList.add(new FileTO(xformName, "image", path));
					resp.setFile(fileList);
				}
			}
			if (!study.getEvent().isEmpty()) {
				activityDetailValue = study.getEvent() + " "
						+ study.getDescription() + "(" + studyId + ")";
			} else {
				activityDetailValue = study.getDescription() + "(" + studyId
						+ ")";
			}
			Auditor.auditDownloadBBMIDataRequestEvent(pat.getUserProfile()
					.getId(), ActivityActorTypeEnumeration.SELF, true,
					ActivityActionTypeEnumeration.DOWNLOAD_MY_VA_BBMI,
					activityDetailValue);
			// OR ADD INTEGRATION TO ZIP FILE IN CLASSPATH? CMS IS BETTER THOUGH
		} catch (Exception e) {
			Auditor.auditDownloadBBMIDataRequestEvent(pat.getUserProfile()
					.getId(), ActivityActorTypeEnumeration.SELF, false,
					ActivityActionTypeEnumeration.DOWNLOAD_MY_VA_BBMI,
					activityDetailValue);
			throw new WebApplicationException(e, 400);
		}

		return resp;
	}

	@Override
	public StreamingOutput getFile(String fileType, String fileId) {
		try {
			if (log.isInfoEnabled())
				log.info("Streaming " + fileId);

			// TRANSFORM URI (Getting ICN from MHV)
			String icn = getIcnFromMHV();
			fileId = fileId.replace("{ICN}", icn);
		} catch (Exception e) {
			throw new WebApplicationException(e, 400);
		}

		// test for each type of file
		// if image grab from getImage interface; if report get from getReport
		// interface; if other than get from CMS/Classpath
		if (fileType.equalsIgnoreCase("image")) {
			return new SIO(new CVIXService(), fileId, response) {
				@Override
				public void write(final OutputStream os) {
					try {
						ImageTO image = cvixService.getCVIXClient().getImage(
								CVIX_URL, ACCESSCODE, VERIFYCODE, fileId);
						response.setHeader("Content-Disposition",
								"attachment; filename=\"" + image.getName()
										+ "\"");
						response.setHeader("Content-Type",
								"application/octet-stream");
						response.setHeader("Pragma", "public");
						response.setHeader("Expires", "0");
						response
								.setHeader("Cache-Control",
										"private, must-revalidate, post-check=0, pre-check=0");
						// response.setHeader("Transfer-Encoding", "chunked");
						// //?
						response.setHeader("Content-Transfer-Encoding",
								"binary"); // ?
						response.setHeader("Content-Length", String
								.valueOf(image.getContent().length));
						response.flushBuffer();
						os.write(image.getContent());
					} catch (IOException e) {
						throw new WebApplicationException(e, 500);
					} finally {
						try {
							os.flush();
							os.close();
						} catch (Exception e1) {
							e1.printStackTrace();
						}
					}
				}
			};
		} else if (fileType.equalsIgnoreCase("report")) {
			return new SIO(new CVIXService(), fileId, response) {
				@Override
				public void write(final OutputStream os) {
					try {
						String report = cvixService.getCVIXClient()
								.getReportResultByUri(CVIX_URL, ACCESSCODE,
										VERIFYCODE, fileId);
						response.setHeader("Content-Disposition",
								"attachment; filename=\"Report.txt\"");
						response.setHeader("Content-Type",
								"application/octet-stream");
						response.setHeader("Pragma", "public");
						response.setHeader("Expires", "0");
						response
								.setHeader("Cache-Control",
										"private, must-revalidate, post-check=0, pre-check=0");
						// response.setHeader("Transfer-Encoding", "chunked");
						response.setHeader("Content-Transfer-Encoding",
								"binary");
						response.flushBuffer();
						os.write(report.getBytes());
					} catch (IOException e) {
						throw new WebApplicationException(e, 500);
					} finally {
						try {
							os.flush();
							os.close();
						} catch (Exception e1) {
							e1.printStackTrace();
						}
					}
				}
			};
		} else {
			throw new WebApplicationException(new Exception(
					"Not Implemented Yet"));
		}
	}

	/**
	 * 
	 * Call the MHV Portal mhv-portal-web servlet with session id and get the
	 * ICN -->Sample output from MHV: {"icn":"XXXXXX"}
	 * 
	 * @return
	 * @throws IOException
	 */
	private String getIcnFromMHV() throws IOException {
		List<Object> PROVIDERS = new ArrayList<Object>();
		HttpServletRequest req = mc.getHttpServletRequest();
		HttpHeaders httpHeaders = mc.getHttpHeaders();
		String jSessionId = req.getRequestedSessionId();

		PROVIDERS.add(new org.codehaus.jackson.jaxrs.JacksonJsonProvider());
		WebClient client = null;
		MultivaluedMap<String, String> headers = new MultivaluedHashMap<String, String>();

		// Call the MHV Portal mhv-portal-web servlet with session id and get
		// the ICN
		MultivaluedMap<String, String> headersMap = new MultivaluedHashMap<String, String>();
		Enumeration enumHeaders = req.getHeaderNames();
		for (; enumHeaders.hasMoreElements();) {
			String name = (String) enumHeaders.nextElement();
			headersMap.putSingle(name, (String) request.getHeaders(name).nextElement());
		}
		String host = req.getContextPath();
		client = WebClient.create(MHV_URL, PROVIDERS);
		client.headers(headersMap);
		client.type(CONTENT_TYPE).accept(CONTENT_TYPE);
		String resp = client.get(String.class);
		MultivaluedMap resHeader = client.getResponse().getMetadata();
		java.util.ArrayList list = (java.util.ArrayList) resHeader.get(ICN_FIELD);
		if (log.isDebugEnabled()) {
			log.debug("MHV Response for get ICN: " + list.get(0));
		}
		return (String) list.get(0);
	}

	@Override
	public StreamingOutput getZipFile(String studyId) {
		String icn = "";
		Patient pat = null;
		String activityDetailValue = studyId;
		Study study = null;
		String report = null;
		String fileName = null;
		CVIXService cvixService = new CVIXService();
		
		try {
			// TRANSFORM URI (Getting ICN from MHV)
			icn = getIcnFromMHV();
			PatientServiceResponse patResponse = ServiceDelegateFactory.createPatientServiceDelegate().getPatientByIcn(icn);
			pat = patResponse.getPatient();

			SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
			
			//GET STUDY
			study = new CVIXService().getCVIXClient().getStudyResult(CVIX_URL, ACCESSCODE, VERIFYCODE, icn, studyId);

			//GET REPORT
			report = new CVIXService().getCVIXClient().getReportResult(CVIX_URL, ACCESSCODE, VERIFYCODE, icn, studyId);

			//CREATE FILENAME
			String procedureDateStr = "UNKNOWN";
			try {
				procedureDateStr = sdf.format(CVIXDateUtil.parseDate(study.getProcedureDate().toString()));
			} catch (Exception e) {
				log.warn("Found an invalid date for procedureDate: '" + study.getProcedureDate() + "' for ICN: '" + icn + "', for StudyId: '" + studyId + "'");
			}
			
			//SET THE ZIP FILE NAME FOR CONTAINER [mhv_{patient name}_{procedureDate}_{study description}.zip]
			fileName = "mhv_" + (study.getPatientName()).substring(0, study.getPatientName().indexOf(',')) + "_" + procedureDateStr + "_" + (study.getDescription().replaceAll(" ", "_")).replaceAll(",", "_") + ".zip";
		} catch (Exception e) {
			Auditor.auditDownloadBBMIDataRequestEvent(pat.getUserProfile().getId(), ActivityActorTypeEnumeration.SELF, false, ActivityActionTypeEnumeration.DOWNLOAD_MY_VA_BBMI, activityDetailValue);
			throw new WebApplicationException(e, 400);
		}

		return new SO(cvixService, fileName, study, report, pat, activityDetailValue) {
			@Override
			public void write(final OutputStream os) {
				List<Series> list = study.getSerieses().getSeries();

				ZipOutputStream zos = new ZipOutputStream(os);
				// zos.setLevel(LEVEL);
				
				// ADD STUDY REPORT
				try {
					zos.putNextEntry(new ZipEntry("images/Report/Report.txt"));
					zos.write(report.getBytes());
					zos.closeEntry();
					zos.flush();
				} catch (Exception e) {
					e.printStackTrace();
				}
				int imageCount = 0;
				int seriesCount = 1;
				
				StringBuffer seriesJson = new StringBuffer("{ \"patientId\": \"patient id\", \"modality\": \"\", \"studyDescription\": \"\", \"numImages\": \"0\", \"studyId\": \"study id\", \"seriesList\": [");
				for (Series s : list) {
					seriesJson.append("{ \"seriesDescription\": \"\", \"seriesNumber\": \""+ s.getSeriesNumber() +"\", \"numImages\": "+s.getImages().getImage().size()+", \"instanceList\": [");
					
					if (log.isInfoEnabled()) {
						log.info(" Writing Series " + (seriesCount++) + " of " + list.size());
					}
					
					// FETCH AND ADD EACH IMAGE
					String directory = "Series " + s.getSeriesNumber();
					ImageTO image = null;

					// GET REFERENCE QUALITY (IMAGE/JPEG with 70 quality)
					for (Image img : s.getImages().getImage()) {
						try {
							String urn = (img.getReferenceImageUri() != null ? img.getReferenceImageUri() : img.getDiagnosticImageUri());
							image = cvixService.getCVIXClient().getImage(CVIX_URL, ACCESSCODE, VERIFYCODE, urn);
							String fileName = "images/Images" + "/" + directory + "/" + image.getName();
							if (log.isInfoEnabled()) {
								log.info("Adding " + fileName + "(size: " + image.getContent().length + " bytes)");
							}
							imageCount++;
							seriesJson.append("{ \"imageId\": \"images/Images/Series "+s.getSeriesNumber()+"/"+fileName+"\" },");

							zos.putNextEntry(new ZipEntry(fileName));
							zos.write(image.getContent());
							zos.closeEntry();
							zos.flush();
						} catch (Exception e) {
							throw new WebApplicationException(e, 500);
						}
					}
					
					//DELETE THE LAST ','
					seriesJson.deleteCharAt(seriesJson.length()-1);
					seriesJson.append("},");
				}
				
				//DELETE THE LAST ','
				seriesJson.deleteCharAt(seriesJson.length()-1);
				seriesJson.append("]}");

				// BUILD AND RECORD THE STUDYLIST JSON FILE
				//
				// /studyList.json
				// Example:
				//{"studyList":[{"patientId":"patient id","modality":"modality","studyDescription":"660-7577","numImages":6,"studyId":"660-7577"}]}

				String studyList_json = "{\"studyList\":[{\"patientId\":\"patient id\",\"modality\":\"modality\",\"studyDescription\":\""+activityDetail+"\",\"numImages\":"+imageCount+",\"studyId\":\""+activityDetail+"\"}]}";
				
				// /studies/Series.json
				// Example:
				//{"patientId":"patient id","modality":"","studyDescription":"","numImages":"0","studyId":"study id","seriesList":[{"seriesDescription":"","seriesNumber":"1","instanceList":[{"imageId":"images/Images/Series 1/7579.dcm"},{"imageId":"images/Images/Series 1/7583.dcm"},{"imageId":"images/Images/Series 1/7580.dcm"},{"imageId":"images/Images/Series 1/7578.dcm"},{"imageId":"images/Images/Series 1/7581.dcm"},{"imageId":"images/Images/Series 1/7582.dcm"}],"numImages":6}]}
				
				// /studies/Report.json
				// Example:
				//{"patientId":"patient id","modality":"","studyDescription":"","numImages":"0","studyId":"study id","seriesList":[{"seriesDescription":"images","seriesNumber":"1","instanceList":[{"imageId":"images/Report/Report.txt"}]}]}
				String report_json = "{\"patientId\":\"patient id\",\"modality\":\"\",\"studyDescription\":\"\",\"numImages\":\"0\",\"studyId\":\"study id\",\"seriesList\":[{\"seriesDescription\":\"images\",\"seriesNumber\":\"1\",\"instanceList\":[{\"imageId\":\"images/Report/Report.txt\"}]}]}";
				
				try {
					zos.putNextEntry(new ZipEntry("studyList.json"));
					zos.write(studyList_json.getBytes());
					zos.closeEntry();
					
					zos.putNextEntry(new ZipEntry("studies/Series.json"));
					zos.write(seriesJson.toString().getBytes());
					zos.closeEntry();
					
					zos.putNextEntry(new ZipEntry("studies/Report.json"));
					zos.write(report_json.getBytes());
					zos.closeEntry();

					zos.flush();
				} catch (Exception e) {
					e.printStackTrace();
				}
				
				// LAST ADD THE HTML FILE
				try {
					String files[] = {
							"viewer.html",
							"support/bootstrap.min.css",
							"support/cornerstone.min.css",
							"support/css/font-awesome.min.css",
							"support/fonts/FontAwesome.otf",
							"support/fonts/fontawesome-webfont.eot",
							"support/fonts/fontawesome-webfont.svg",
							"support/fonts/fontawesome-webfont.ttf",
							"support/fonts/fontawesome-webfont.woff",
							"support/js/bbmiImageLoader.js",
							"support/js/cornerstoneMath.min.js",
							"support/js/cornerstoneTools.min.js",
							"support/js/jquery-2.1.0.min.js",
							"support/js/bootstrap.min.js",
							"support/js/cornerstone.min.js",
							"support/js/cornerstoneTools.js",
							"support/js/dicomParser.min.js",
							"support/js/jquery.hammer-full.js" };
					
					String path = "https://abc-serv.myhealth.domain/mhv-portal-web/ShowBinary/BEA%20Repository/bbmi/viewer/";
					HTTPIO io = new HTTPIO();
					for (String filename : files) {
						zos.putNextEntry(new ZipEntry(filename));
						zos.write(io.httpGET(path+filename));
						zos.closeEntry();
						zos.flush();
					}
				} catch (Exception e) {
					e.printStackTrace();
				}

				try {
					zos.finish();
					zos.flush();
					Auditor.auditDownloadBBMIDataRequestEvent(patient.getUserProfile().getId(), ActivityActorTypeEnumeration.SELF, true, ActivityActionTypeEnumeration.DOWNLOAD_MY_VA_BBMI,	activityDetail);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

		};

	}

	class SO implements StreamingOutput {
		Study study;
		String report;
		CVIXService cvixService;
		String filename;
		Patient patient;
		String activityDetail;
		
		public SO(CVIXService cvixService, String filename, Study study, String report, Patient patient, String activityDetail) {
			this.cvixService = cvixService;
			this.study = study;
			this.report = report;
			this.filename = filename;
			this.patient = patient;
			this.activityDetail = activityDetail;
		}

		@Override
		public void write(OutputStream arg0) throws IOException,
				WebApplicationException {
			throw new IOException("Base Class does not support writing");
		}
	}

	class SIO implements StreamingOutput {
		String fileId;
		CVIXService cvixService;
		HttpServletResponse response;

		public SIO(CVIXService cvixService, String fileId,
				HttpServletResponse response) {
			this.cvixService = cvixService;
			this.fileId = fileId;
			this.response = response;
		}

		@Override
		public void write(OutputStream arg0) throws IOException,
				WebApplicationException {
			throw new IOException("Base Class does not support writing");
		}
	}

	class HTTPIO {
		private HttpURLConnection httpConn;
		
		public HTTPIO() {
		}
		
		public byte[] httpGET(String url) throws IOException {
			setupGETRequest(url, null);
			return sendGETRequest(); 
		}
		
		private void setupGETRequest(String requestURL, Map<String, String> headers) throws IOException {
			//audit.add(new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS").format(new Date())+" - "+requestURL);

			URL url = new URL(requestURL);
			httpConn = (HttpURLConnection) url.openConnection();

			httpConn.setUseCaches(false);
			httpConn.setDoOutput(false); // false indicates GET method
			httpConn.setDoInput(true);
			for (String key : headers.keySet()) {
				httpConn.setRequestProperty(key, headers.get(key));
			}
			httpConn.setRequestProperty("User-Agent", "CodeJava Agent");
		}

		private byte[] sendGETRequest() throws IOException {
			ByteArrayOutputStream baos = new ByteArrayOutputStream(8*1024);
			// checks server's status code first
			int status = httpConn.getResponseCode();
			if (status == HttpURLConnection.HTTP_OK) {
				BufferedInputStream input = new BufferedInputStream( httpConn.getInputStream() );
				byte[] bucket = new byte[32*1024];
				int bytesRead = 0;
		        while(bytesRead != -1){
		          bytesRead = input.read(bucket);
		          if(bytesRead > 0){
		            baos.write(bucket, 0, bytesRead);
		          }
		        }
			} else {
				throw new IOException("Response: " + status + " was returned by mhv");
			}
			return baos.toByteArray();
		}

	}
}
