package gov.va.med.mhv.integration.phr.client.cvix.client;

import gov.va.med.mhv.integration.phr.client.cvix.RestExceptionMessage;
import gov.va.med.mhv.integration.phr.client.cvix.RestStringType;
import gov.va.med.mhv.integration.phr.client.cvix.StudiesResult;
import gov.va.med.mhv.integration.phr.client.cvix.Study;
import gov.va.med.mhv.integration.phr.client.cvix.model.ImageTO;

import java.io.BufferedInputStream;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class CVIXClient {
	private static final Log log = LogFactory.getLog(CVIXClient.class);

	private static final String SITEKEY = "{site}";
	private static final String ICNKEY = "{icn}";
	private static final String STUDYIDKEY = "{studyId}";
	private static final String IMAGEKEY = "{imageId}";
	private static Date START_DATE;
	private static final String ALL_RESULT_TYPES = "all";

	private static Map<String, String> EXTENSIONS = new HashMap<String,String>();
	private HttpURLConnection httpConn;

	private List<String> audit;

	static {
		try {
			START_DATE = new SimpleDateFormat("yyyy-MM-dd").parse("1900-01-01");
		} catch (ParseException e) {
			e.printStackTrace();
		}

		EXTENSIONS.put("application/msword","doc");
		EXTENSIONS.put("application/ogg","ogg");
		EXTENSIONS.put("application/pdf","pdf");
		EXTENSIONS.put("application/srgs","gram");
		EXTENSIONS.put("application/vnd.mif","mif");
		EXTENSIONS.put("application/vnd.mozilla.xul+xml","xul");
		EXTENSIONS.put("application/vnd.ms-excel","xls");
		EXTENSIONS.put("application/vnd.ms-powerpoint","ppt");
		EXTENSIONS.put("application/vnd.wap.wbxml","wbxml");
		EXTENSIONS.put("application/voicexml+xml","vxml");
		EXTENSIONS.put("application/x-bcpio","bcpio");
		EXTENSIONS.put("application/x-cdlink","vcd");
		EXTENSIONS.put("application/x-chess-pgn","pgn");
		EXTENSIONS.put("application/x-cpio","cpio");
		EXTENSIONS.put("application/x-csh","csh");
		EXTENSIONS.put("application/x-dvi","dvi");
		EXTENSIONS.put("application/x-futuresplash","spl");
		EXTENSIONS.put("application/x-gtar","gtar");
		EXTENSIONS.put("application/x-hdf","hdf");
		EXTENSIONS.put("application/xhtml+xml","xhtml");
		EXTENSIONS.put("application/x-javascript","js");
		EXTENSIONS.put("application/x-latex","latex");
		EXTENSIONS.put("application/xml","xml");
		EXTENSIONS.put("application/xml-dtd","dtd");
		EXTENSIONS.put("application/x-sh","sh");
		EXTENSIONS.put("application/x-shar","shar");
		EXTENSIONS.put("application/x-shockwave-flash","swf");
		EXTENSIONS.put("application/xslt+xml","xslt");
		EXTENSIONS.put("application/x-stuffit","sit");
		EXTENSIONS.put("application/x-sv4cpio","sv4cpio");
		EXTENSIONS.put("application/x-sv4crc","sv4crc");
		EXTENSIONS.put("application/x-tcl","tcl");
		EXTENSIONS.put("application/x-tex","tex");
		EXTENSIONS.put("application/x-ustar","ustar");
		EXTENSIONS.put("application/x-wais-source","src");
		EXTENSIONS.put("application/x-x509-ca-cert",".crt");
		EXTENSIONS.put("application/zip","zip");
		EXTENSIONS.put("audio/basic","au");
		EXTENSIONS.put("audio/midi","mid");
		EXTENSIONS.put("audio/mpeg","mp3");
		EXTENSIONS.put("audio/x-aiff","aif");
		EXTENSIONS.put("audio/x-mpegurl","m3u");
		EXTENSIONS.put("audio/x-pn-realaudio","rm");
		EXTENSIONS.put("audio/x-pn-realaudio-plugin","rpm");
		EXTENSIONS.put("audio/x-realaudio","ra");
		EXTENSIONS.put("audio/x-wav","wav");
		EXTENSIONS.put("chemical/x-pdb","pdb");
		EXTENSIONS.put("chemical/x-xyz","xyz");
		EXTENSIONS.put("image/bmp","bmp");
		EXTENSIONS.put("image/cgm","cgm");
		EXTENSIONS.put("image/gif","gif");
		EXTENSIONS.put("image/ief","ief");
		EXTENSIONS.put("image/jpeg","jpg");
		EXTENSIONS.put("image/png","png");
		EXTENSIONS.put("image/svg+xml","svg");
		EXTENSIONS.put("image/tiff","tif");
		EXTENSIONS.put("image/x-cmu-raster","ras");
		EXTENSIONS.put("image/x-icon","ico");
		EXTENSIONS.put("image/x-portable-anymap","pnm");
		EXTENSIONS.put("image/x-portable-bitmap","pbm");
		EXTENSIONS.put("image/x-portable-graymap","pgm");
		EXTENSIONS.put("image/x-portable-pixmap","ppm");
		EXTENSIONS.put("image/x-rgb","rgb");
		EXTENSIONS.put("image/x-xbitmap","xbm");
		EXTENSIONS.put("image/x-xpixmap","xpm");
		EXTENSIONS.put("image/x-xwindowdump","xwd");
		EXTENSIONS.put("text/calendar","ics");
		EXTENSIONS.put("text/css","css");
		EXTENSIONS.put("text/plain","txt");
		EXTENSIONS.put("text/richtext","rtx");
		EXTENSIONS.put("text/rtf","rtf");
		EXTENSIONS.put("text/tab-separated-values","tsv");
		EXTENSIONS.put("text/x-setext","etx");
		EXTENSIONS.put("video/mpeg","mpg");
		EXTENSIONS.put("video/quicktime","mov");
		EXTENSIONS.put("video/x-msvideo","avi");
		//Added
		EXTENSIONS.put("image/x-targa","tga");
		EXTENSIONS.put("image/j2k","j2k");
		EXTENSIONS.put("application/dicom","dcm");
	}

	/**
	 *
	 * @param audit
	 */
	public CVIXClient(List<String> audit) {
		this.audit=audit;
	}

	/**
	 *
	 * @param accesscode
	 * @param verifycode
	 * @param imageId
	 * @return
	 * @throws IOException
	 */
	public ImageTO getImage( String cvixHost, String accesscode, String verifycode, String imageId ) throws IOException {
		String requestURL = cvixHost+"/ImageWebApp/image?{imageId}";
		byte[] response = null;
		String imageContentType = null;
		String name = null;
		try {
			Map<String, String> headers = new HashMap<String, String>();
			setupGETRequest(requestURL.replace(IMAGEKEY, imageId), headers, accesscode, verifycode );
			response = sendGETRequest();

			imageContentType = getResponseHeaderValue("Content-Type");
			int pos1 = ("imageURN=urn:vaimage:").length();
			int pos2 = imageId.indexOf('-',pos1);
			int pos3 = imageId.indexOf('-',pos2+1);

			name = imageId.substring(pos2+1,pos3) + "." + lookupExtension(imageContentType);
		} catch (Exception e) {
			throw new IOException(e);
		}
		return new ImageTO(name, imageContentType, response);
	}

	private String lookupExtension(String imageContentType) {
		String ext = EXTENSIONS.get(imageContentType.toLowerCase());
		if(ext == null) {
			log.error("Error: Unable to find extension for '" + imageContentType);
			ext = "unk";
		}
		return ext;
	}

	/**
	 *
	 * @param accesscode
	 * @param verifycode
	 * @param icn
	 * @param studyId
	 * @return
	 * @throws IOException
	 */
	public String getReportResult( String cvixHost, String accesscode, String verifycode, String icn, String studyId ) throws IOException {
		String requestURL = cvixHost+"/Study/restservices/study/report/urn:vastudy:{studyId}-{icn}";
		String report = null;

		try {
			Map<String, String> headers = new HashMap<String, String>();
			headers.put("Accept", "application/xml");

			setupGETRequest(requestURL.replace(STUDYIDKEY, studyId).replace(ICNKEY, icn), headers, accesscode, verifycode );
			byte[] response = sendGETRequest();
			report = getReport(new String(response));
		} catch (Exception ex) {
			throw new IOException("Cvix: " + ex);
		}

		return report;
	}

	/**
	 *
	 * @param accesscode
	 * @param verifycode
	 * @param uri
	 * @return
	 * @throws IOException
	 */
	public String getReportResultByUri( String cvixHost, String accesscode, String verifycode, String uri ) throws IOException {
		String requestURL = cvixHost+"/Study/restservices/study/report/{uri}";
		String report = null;

		try {
			Map<String, String> headers = new HashMap<String, String>();
			headers.put("Accept", "application/xml");
			setupGETRequest(requestURL.replace("{uri}", uri), headers, accesscode, verifycode );
			byte[] response = sendGETRequest();
			report = getReport(new String(response));
		} catch (Exception ex) {
			throw new IOException("Cvix: " + ex);
		}

		return report;
	}

	/**
	 * <restStringType><value>
	 *
	 * @param xml
	 * @return
	 */
	public String getReport(String xml) {
		RestStringType result = null;
		String report = "[EMPTY]";
		try {
			JAXBContext jaxbContext = JAXBContext.newInstance(RestStringType.class);
			Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();
			result = (RestStringType) jaxbUnmarshaller.unmarshal(new ByteArrayInputStream(xml.getBytes()));
			if(result.getValue() != null && result.getValue().length() > 0)
				report = result.getValue();
		} catch (JAXBException e) {
			e.printStackTrace();
		}

		return report;
	}

	/**
	 *
	
	 *
	 * @param accesscode
	 * @param verifycode
	 * @param icn
	 * @param studyId
	 * @return
	 * @throws IOException
	 */
	public Study getStudyResult( String cvixHost, String accesscode, String verifycode, String icn, String studyId ) throws IOException {
		String requestURL = cvixHost+"/Study/restservices/study/urn:vastudy:{studyId}-{icn}";
		Study study = null;

		try {
			Map<String, String> headers = new HashMap<String, String>();
			headers.put("Accept", "application/xml");

			setupGETRequest(requestURL.replace(STUDYIDKEY, studyId).replace(ICNKEY, icn), headers, accesscode, verifycode );
			byte[] response = sendGETRequest();
			study = getStudy(new String(response));
		} catch (Exception ex) {
			throw new IOException("Cvix: " + ex);
		}

		return study;
	}

	/**
	 *
	 * @param xml
	 * @return
	 */
	public Study getStudy(String xml) {
		//System.out.println(xml);
		Study study = null;
		try {
			JAXBContext jaxbContext = JAXBContext.newInstance(Study.class);
			Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();
			study = (Study) jaxbUnmarshaller.unmarshal(new ByteArrayInputStream(xml.getBytes()));
		} catch (JAXBException e) {
			e.printStackTrace();
		}

		return study;
	}

	/**
	 *
	 * @param accesscode
	 * @param verifycode
	 * @param icn
	 * @param site
	 * @return
	 */
	public StudiesResult getStudiesResult( String cvixHost, String accesscode, String verifycode, String icn, String site ) {
		return getStudiesResultWithFilter(cvixHost, accesscode, verifycode, icn, site, START_DATE, new Date(), ALL_RESULT_TYPES);
	}

	public StudiesResult getStudiesResultWithFilter( String cvixHost, String accesscode, String verifycode, String icn, String site, Date startDate, Date endDate, String resultType ) {
		String requestURL = cvixHost+"/Study/restservices/studies/site/icn/{site}/{icn}";
		StudiesResult sr = null;

		try {
			Map<String, String> headers = new HashMap<String, String>();
			headers.put("Accept", "application/xml");

			setupPOSTRequest(requestURL.replace(SITEKEY, site).replace(ICNKEY, icn), headers, accesscode, verifycode );
			byte[] response = sendPOSTRequest(buildStudyFilter(startDate, endDate, resultType));

			sr = getStudies(new String(response));

			for( gov.va.med.mhv.integration.phr.client.cvix.StudiesResult.Studies.Study s: sr.getStudies().getStudy() ) {
				//Date/Time Performed, Procedure/Test Name, # images
				if(log.isDebugEnabled()) {
					log.debug("getOrigin"+ ":" + s.getOrigin());  //REQUIRED "VA"
					log.debug("getPackage"+":"+ s.getPackage());  //RAD, NOTE, NONE
					log.debug("getStudyId"+ ":"+s.getStudyId());
				}
			}

		} catch (Exception ex) {
			System.err.println(ex);
		}

		return sr;
	}

	public StudiesResult getStudies(String xml) {
		StudiesResult studies = null;
		try {
			JAXBContext jaxbContext = JAXBContext.newInstance(StudiesResult.class);
			Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();
			studies = (StudiesResult) jaxbUnmarshaller.unmarshal(new ByteArrayInputStream(xml.getBytes()));
		} catch (JAXBException e) {
			e.printStackTrace();
		}

		return studies;
	}

	/**
	 *
	 * @param xml
	 * @return
	 */
	public RestExceptionMessage getRestExceptionMessage(String xml) {
		RestExceptionMessage exception = null;
		try {
			JAXBContext jaxbContext = JAXBContext.newInstance(RestExceptionMessage.class);
			Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();
			exception = (RestExceptionMessage) jaxbUnmarshaller.unmarshal(new ByteArrayInputStream(xml.getBytes()));
		} catch (JAXBException e) {
			e.printStackTrace();
		}

		return exception;
	}

	/**
	 *
	 * @param requestURL
	 * @param headers
	 * @param username
	 * @param password
	 * @throws IOException
	 */
	private void setupGETRequest(String requestURL, Map<String, String> headers, String username, String password) throws IOException {
		//System.out.println("Calling GET " + requestURL);

		audit.add(new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS").format(new Date())+" - "+requestURL);

		URL url = new URL(requestURL);
		httpConn = (HttpURLConnection) url.openConnection();

		sun.misc.BASE64Encoder enc = new sun.misc.BASE64Encoder();
		String encoded = enc.encode((username + ":" + password).getBytes());
		httpConn.setRequestProperty("Authorization", "Basic " + encoded);

		httpConn.setUseCaches(false);
		httpConn.setDoOutput(false); // false indicates GET method
		httpConn.setDoInput(true);
		for (String key : headers.keySet()) {
			httpConn.setRequestProperty(key, headers.get(key));
		}
		httpConn.setRequestProperty("User-Agent", "CodeJava Agent");
	}

	/**
	 *
	 * @param key
	 * @return
	 */
	private String getResponseHeaderValue(String key) {
		List<String> vals = httpConn.getHeaderFields().get(key);
		return vals.get(0);
	}

	/**
	 *
	 * <studyFilter>
	 * <fromDate>2010-01-01</fromDate>
	 * <toDate>2012-12-31</toDate>
	 * <resultType>all</resultType>
	 * </studyFilter>
	 *
	 * resultType can be one of: all, radiology, or artifacts.
	 *
	 * @param start
	 * @param end
	 * @param types
	 * @return
	 */
	private String buildStudyFilter( Date start, Date end, String type ) {
		StringBuffer sb = new StringBuffer("<studyFilter>");
		sb.append("<fromDate>"+new SimpleDateFormat("yyyy-MM-dd").format(start)+"</fromDate>");
		sb.append("<toDate>"+new SimpleDateFormat("yyyy-MM-dd").format(end)+"</toDate>");
		sb.append("<resultType>"+type+"</resultType>");
		sb.append("</studyFilter>");
		return sb.toString();
	}

	/**
	 *
	 * @return
	 * @throws IOException
	 */
	private byte[] sendGETRequest() throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream(32*1024);

		// checks server's status code first
		int status = httpConn.getResponseCode();
		if (status == HttpURLConnection.HTTP_OK) {
			BufferedInputStream input = new BufferedInputStream( httpConn.getInputStream() );
			byte[] bucket = new byte[32*1024];
			int bytesRead = 0;
	        while(bytesRead != -1){
	          bytesRead = input.read(bucket);
	          if(bytesRead > 0){
	            baos.write(bucket, 0, bytesRead);
	          }
	        }
			//System.out.println("Result size: " + baos.size());
		} else {
			throw new IOException("Response: " + status + " was returned by cvix");
		}
		return baos.toByteArray();
	}

	/**
	 *
	 * @param requestURL
	 * @param headers
	 * @param username
	 * @param password
	 * @throws IOException
	 */
	private void setupPOSTRequest(String requestURL, Map<String, String> headers, String username, String password) throws IOException {
		//System.out.println("Calling GET " + requestURL);

		audit.add(new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS").format(new Date())+" - "+requestURL);

		URL url = new URL(requestURL);
		httpConn = (HttpURLConnection) url.openConnection();

		sun.misc.BASE64Encoder enc = new sun.misc.BASE64Encoder();
		String encoded = enc.encode((username + ":" + password).getBytes());
		httpConn.setRequestProperty("Authorization", "Basic " + encoded);
		httpConn.setUseCaches(false);
		httpConn.setDoOutput(true); //Indicates POST
		httpConn.setDoInput(true);
		for (String key : headers.keySet()) {
			httpConn.setRequestProperty(key, headers.get(key));
		}
		httpConn.setRequestProperty("User-Agent", "CodeJava Agent");
		httpConn.setRequestProperty("Content-Type", "application/xml");
	}

	/**
	 *
	 * @param jsonParams
	 * @return
	 * @throws IOException
	 */
	private byte[] sendPOSTRequest(String studyFilter) throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream(32*1024);
		//studyFilter = URLEncoder.encode(studyFilter, "UTF-8");

		System.out.println("POST: " + studyFilter);


		OutputStream os = httpConn.getOutputStream();
		BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(os, "UTF-8"));
		writer.write(studyFilter);
		writer.flush();
		writer.close();
		os.close();

		// checks server's status code first
		int status = httpConn.getResponseCode();
		if (status == HttpURLConnection.HTTP_OK) {
			BufferedInputStream input = new BufferedInputStream( httpConn.getInputStream() );
			byte[] bucket = new byte[32*1024];
			int bytesRead = 0;
	        while(bytesRead != -1){
	          bytesRead = input.read(bucket);
	          if(bytesRead > 0){
	            baos.write(bucket, 0, bytesRead);
	          }
	        }
			System.out.println("Result size: " + baos.size() );
			System.out.println("Result: " + new String(baos.toByteArray()) );

		} else {
			throw new IOException("Response: " + status + " was returned by cvix");
		}
		return baos.toByteArray();
	}

}
