package gov.va.med.mhv.eauth.authentication;

import java.security.spec.KeySpec;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.login.AppConfigurationEntry;
import javax.servlet.Filter;

import weblogic.management.security.ProviderMBean;
import weblogic.security.spi.AuthenticationProvider;
import weblogic.security.spi.IdentityAsserter;
import weblogic.security.spi.IdentityAssertionException;
import weblogic.security.spi.PrincipalValidator;
import weblogic.security.spi.SecurityProvider;
import weblogic.security.spi.SecurityServices;
import weblogic.security.spi.ServletAuthenticationFilter;
import weblogic.utils.encoders.BASE64Decoder;


/**

 */
public final class EauthIdentityAsserterProviderImpl implements AuthenticationProvider, IdentityAsserter, ServletAuthenticationFilter
{
  final static private String TOKEN_TYPE   = "EauthToken"; // the kind of token's we handle
  final static private String TOKEN_PREFIX = "username="; // the token contains a string in the form "username=someusername"

	private static final String ENCRYPTION_KEY = "098765432109876543210987654321";

	private static final String DES_ENCRYPTION_SCHEME = "DES";

	private KeySpec keySpec;

	private SecretKeyFactory keyFactory;

	private Cipher cipher;

	private static final String UNICODE_FORMAT = "UTF-8";


  private String description;

  /**
   * Initialize the simple sample identity asserter.
   *
   * @param mbean A ProviderMBean that holds the simple sample identity asserter's
   * configuration data.  This mbean must be an instance of the simple sample
   * identity asserter's mbean.
   *
   * @param services The SecurityServices gives access to the auditor
   * so that the provider can to post audit events.
   * The simple sample role mapper doesn't use this parameter.
   *
   * @see SecurityProvider
   */
  public void initialize(ProviderMBean mbean, SecurityServices services)
  {
 //   System.out.println("EauthIdentityAsserterProviderImpl.initialize");
    EauthIdentityAsserterMBean myMBean = (EauthIdentityAsserterMBean)mbean;
    description = myMBean.getDescription() + "\n" + myMBean.getVersion();

	try{
		byte[] keyAsBytes = ENCRYPTION_KEY.getBytes(UNICODE_FORMAT);
		keySpec = new DESKeySpec(keyAsBytes);
		keyFactory = SecretKeyFactory.getInstance(DES_ENCRYPTION_SCHEME);
		cipher = Cipher.getInstance(DES_ENCRYPTION_SCHEME);
	}catch(Exception e){
		e.printStackTrace();
	}
  }

  /**
   * Get the simple sample identity asserter's description.
   *
   * @return A String containing a brief description of the simple sample identity asserter.
   *
   * @see SecurityProvider
   */
  public String getDescription()
  {
    return description;
  }

  /**
   * Shutdown the simple sample identity asserter.
   *
   * A no-op.
   *
   * @see SecurityProvider
   */
  public void shutdown()
  {
//    System.out.println("EauthIdentityAsserterProviderImpl.shutdown");
  }

  /**
   * Gets the simple sample identity assertion provider's identity asserter object.
   *
   * @return The simple sample identity assertion provider's IdentityAsserter object.
   *
   * @see AuthenticationProvider
   */
  public IdentityAsserter getIdentityAsserter()
  {
    return this;
  }

  /**
   * Assert identity given a token that identifies the user.
   *
   * @param type A String containing the token type.  The simple sample identity
   * asserter only supports tokens of type "SamplePerimeterAtnToken".
   * Also, the simple sample identity asserter's mbean's "ActiveTypes" attribute
   * must be set to "SamplePerimeterAtnToken" (which is done by default
   * when the mbean is created).
   *
   * @param token An Object containing the token that identifies the user.
   * The simple sample identity asserter's token must be an array of bytes
   * containing a String of the form "username=someusername".
   *
   * While, for simplicity, this sample does not validate the
   * contents of the token, identity asserters typically should do
   * this (to prevent someone from forging a token).  For
   * example, when using Kerberos, the token may be generated
   * and "signed" by a Kerberos server and the identity asserter
   * hands the token back to the Kerberos server to get it
   * validated.  Another example: when asserting identity from
   * X509 certificates, then identity asserter should validate the
   * certificate - that it hasn't been tampered, that it's been
   * signed by a trusted CA, that it hasn't expired or revoked, etc.
   *
   * @return a CallbackHandler that stores the username from the token.
   * The username can only be retrieved from the callback handler by
   * passing in a NameCallback.  The sample returns an instance of
   * its CallbackHandler implementation (SimpleSampleCallbackHandlerImpl).
   *
   * @throws IdentityAssertionException if another token type is passed
   * in or the token doesn't have the correct form.
   */
  public CallbackHandler assertIdentity(String type, Object token) throws IdentityAssertionException
  {
    System.out.println("EauthIdentityAsserterProviderImpl.assertIdentity");
   System.out.println("\tType\t\t= "  + type);
    System.out.println("\tToken\t\t= " + token);

    // check the token type
    if (!(TOKEN_TYPE.equals(type))) {
      String error =
        "EauthIdentityAsserter received unknown token type \"" + type + "\"." +
        " Expected " + TOKEN_TYPE;
      System.out.println("\tError: " + error);
      throw new IdentityAssertionException(error);
    }

    // make sure the token is an array of bytes
    if (!(token instanceof byte[])) {
      String error =
        "EauthIdentityAsserter received unknown token class \"" + token.getClass() + "\"." +
        " Expected a byte[].";
      System.out.println("\tError: " + error);
      throw new IdentityAssertionException(error);
    }

    // convert the array of bytes to a string
    byte[] tokenBytes = (byte[])token;
    if (tokenBytes == null || tokenBytes.length < 1) {
      String error =
        "EauthIdentityAsserter received empty token byte array";
      System.out.println("\tError: " + error);
      throw new IdentityAssertionException(error);
    }

    String tokenStr = new String(tokenBytes);

    // make sure the string contains "username=someusername
    if (!(tokenStr.startsWith(TOKEN_PREFIX))) {
      String error =
        "EauthIdentityAsserter received unknown token string \"" + type + "\"." +
        " Expected " + TOKEN_PREFIX + "username";
      System.out.println("\tError: " + error);
      throw new IdentityAssertionException(error);
    }

    // extract the username from the token
    String userName = tokenStr.substring(TOKEN_PREFIX.length());
    try{
    	System.out.println("ia userName: " + userName);
    	BASE64Decoder bd = new BASE64Decoder();
//    	userName = bd.decodeBuffer(userName);
    	userName = decrypt(userName);
    System.out.println("\tuserName\t= " + userName);
    }catch(Exception e){
    	e.printStackTrace();
    }
    // store it in a callback handler that authenticators can use
    // to retrieve the username.
    return new EauthCallbackHandlerImpl(userName);
  }

  /**
   * Return how to call the login module to perform authentication.
   *
   * @return A null AppConfigurationEntry since the simple sample identity
   * asserter is not an authenticator (thus doesn't have a login module).
  */
  public AppConfigurationEntry getLoginModuleConfiguration()
  {
    return null;
  }

  /**
   * Return how to call the login module to complete identity
   * assertion (where the identity asserter finds the user name
   * and the authenticator puts the user and its groups into the
   * subject).
   *
   * @return A null AppConfigurationEntry since the simple sample identity
   * asserter is not an authenticator (thus doesn't have a login module).
  */
  public AppConfigurationEntry getAssertionModuleConfiguration()
  {
    return null;
  }

  /**
   * Return an object that can validate principals (eg. users
   * and groups) that this provider puts into the subject.
   *
   * @return A null PrincipalValidator since the simple sample identity asserter
   * is not an authenticator (thus doesn't put principals into the subject).
   */
  public PrincipalValidator getPrincipalValidator()
  {
    return null;
  }

  private String decrypt(String es) throws Exception{
	  byte[] ciphertext = null;
	  try{
		  SecretKey key = keyFactory.generateSecret(keySpec);
		  cipher.init(Cipher.DECRYPT_MODE,key);
		  BASE64Decoder bd = new BASE64Decoder();
		  byte[] b = bd.decodeBuffer(es);
		  ciphertext = cipher.doFinal(b);
	  }catch(Exception e){
		  e.printStackTrace();
	  }
	  return bytesToString(ciphertext);
  }

  private static String bytesToString(byte[] bytes){
	  StringBuffer sb = new StringBuffer();
	  for(int i=0;i<bytes.length;i++){
		  sb.append((char)bytes[i]);
	  }
	  return sb.toString();
  }
/*
  public Filter[] getServletAuthenticationFilters() {
		Filter[] filters = new Filter[1];
		filters[0] = (Filter)new gov.va.med.mhv.eauth.authentication.EauthServletFilter();
		return filters;
	}
  */

public Filter[] getServletAuthenticationFilters() {
	// TODO Auto-generated method stub
	return null;
}
}
