package gov.va.med.mhv.eauth.service.impl;

import gov.va.med.mhv.eauth.bizobj.BusinessObjectFactory;
import gov.va.med.mhv.eauth.bizobj.EAuthHashBO;
import gov.va.med.mhv.eauth.bizobj.EAuthHashBaseBO;
import gov.va.med.mhv.eauth.messages.EauthMessages;
import gov.va.med.mhv.eauth.service.EAuthHashService;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.service.delegate.UserProfileServiceDelegate;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;

import java.util.Collection;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.service.LongServiceResponse;
import org.tigris.atlas.service.StringServiceResponse;
import org.tigris.atlas.service.VoidServiceResponse;
/**
 * Service implementation class for the EAuthHash service
 */
public class EAuthHashServiceImpl implements EAuthHashService {

	private static Log log = LogFactory.getLog(EAuthHashServiceImpl.class);
	
	/**
	 * Execute the SaveHash service
	 *
	 * @return VoidServiceResponse
	 */
	public VoidServiceResponse saveHash(String userName, String hashValue) {
		VoidServiceResponse response = new VoidServiceResponse();
		
		EAuthHashBO eabo = (EAuthHashBO)BusinessObjectFactory.createEAuthHashBO();
		eabo.setHashValue(hashValue);
		
		UserProfileServiceDelegate delegate = ServiceDelegateFactory.createUserProfileServiceDelegate();
		UserProfile userProfile = delegate.getProfileForUser( userName ).getUserProfile();
		if( userProfile == null ) {
			eabo.addError( EauthMessages.NO_USER_PROFILE_FOUND);
			return response;
		}
		eabo.setUserProfile(userProfile);
		eabo.save();
		return response;
	}


	public StringServiceResponse findUserNameByHash(String hashValue) {
		StringServiceResponse response = new StringServiceResponse();
		Collection c = EAuthHashBaseBO.findEAuthHashByHashValue(hashValue);
		if(c != null && !c.isEmpty()){
			Iterator it = c.iterator();
			while(it.hasNext()){
				EAuthHashBO e = (EAuthHashBO)it.next();
				response.setValue(e.getUserProfile().getUserName());
				break;
			}
		}
		return response;
	}


	public LongServiceResponse findHashIdByHashValue(String hashValue) {
		LongServiceResponse response = new LongServiceResponse();
		Collection c = EAuthHashBaseBO.findEAuthHashByHashValue(hashValue);
		if(c != null && !c.isEmpty()){
			Iterator it = c.iterator();
			while(it.hasNext()){
				EAuthHashBO e = (EAuthHashBO)it.next();
				response.setValue(e.getId());
				break;
			}
		}
		return response;
	}

}