package gov.va.med.mhv.eauth;

import gov.va.med.mhv.core.util.ServiceResponseUtils;
import gov.va.med.mhv.eauth.service.delegate.EAuthHashServiceDelegate;
import gov.va.med.mhv.eauth.service.delegate.EAuthLogServiceDelegate;
import gov.va.med.mhv.eauth.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.phr.util.AccessFeatureValueUtils;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.CountryEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.GenderEnumeration;
import gov.va.med.mhv.usermgmt.service.AccessControlCollectionServiceResponse;
import gov.va.med.mhv.usermgmt.service.PatientServiceResponse;
import gov.va.med.mhv.usermgmt.service.UserProfileServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.AccessControlServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.MviIntegrationServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.UserProfileServiceDelegate;
import gov.va.med.mhv.usermgmt.transfer.Patient;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;
import gov.va.med.mhv.usermgmt.transfer.UserProfileDeactivationReason;
import gov.va.med.mhv.usermgmt.util.AccessControlUtils;
import gov.va.med.mhv.usermgmt.util.AccessDomainUtils;
import gov.va.med.mhv.usermgmt.util.AccessPermissionUtils;
import gov.va.med.mhv.usermgmt.util.Auditor;
import gov.va.med.mhv.usermgmt.util.UserProfileDeactivationUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.security.InvalidKeyException;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.KeySpec;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Properties;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.security.auth.Subject;
import javax.security.auth.login.LoginException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.messages.Message;
import org.tigris.atlas.messages.Messages;
import org.tigris.atlas.service.BooleanServiceResponse;
import org.tigris.atlas.service.StringServiceResponse;
import org.tigris.atlas.service.VoidServiceResponse;

import com.rsa.certj.cert.attributes.CountryOfResidence;

import sun.misc.BASE64Encoder;
import weblogic.security.SubjectUtils;

public class EAuthGatewayLoginServlet extends HttpServlet implements EauthConstants{
	//JAZZ EPIC # 110142
	Log log = LogFactory.getFactory().getInstance(EAuthGatewayServlet.class);

	private static final long serialVersionUID = 5589615804128963698L;

	private static final String ACTION_DETAIL_DS_LOGON_ACCESS_VA = "";
	
	private static final String USERNAME = "username";
	
	private static final String PASSWORD = "password";
	
	private static final String DISPLAY_ERROR = "displayError";	
	
	private static final String INVALID_LOGIN_INFO = "invalidLoginInfo";	
	
	private static final String USERNAME_NOT_FOUND = "usernameNotFound";	
	
	private static final String INVALID_PASSWORD = "invalidPassword";	
	
	private static final String USER_NAME = "username";

	private KeySpec keySpec;

	private SecretKeyFactory keyFactory;

	private Cipher cipher;

	private static final String UNICODE_FORMAT = "UTF-8";

	private String login_url;
	
	private String splash_url;
	
	private String eauth_portal;

	private String mhv_portal_custom = null;
	
	private String mhv_portal = null;

	private String error_url = null;
	
	private boolean useExistingDslogonProcess = false;

	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		Properties props = new Properties();
		String propertyFile = "eauth.properties";
		String path = System.getProperty("config.location");
		try{
			props.load(new FileInputStream(path + File.separator + propertyFile));
			login_url = props.getProperty("eauth.login.url", "eauth.portal?_nfpb=true&_nfto=false&_pageLabel=mhvEauthLogin");
			splash_url = props.getProperty("eauth.splash.screen.url");
			error_url = props.getProperty("eauth.error.url");
			eauth_portal = props.getProperty("eauth.portal.url");
			mhv_portal = props.getProperty("mhv.portal.url");
			mhv_portal_custom = props.getProperty("mhv.portal.url.custom");
			
			if(props.getProperty("use.existing.dslogon.process")!=null && props.getProperty("use.existing.dslogon.process").equalsIgnoreCase("true")) {
				useExistingDslogonProcess = true;
			}
		} catch (IOException io) {
			login_url = "eauth.portal?_nfpb=true&_nfto=false&_pageLabel=mhvEauthLogin";
			log.error("error getting eauth related properties", io);
		}
	}

	private String populateEauthHash(UserProfile uProfile, String ahash, String csid) {
		EAuthLogServiceDelegate elsd = ServiceDelegateFactory.createEAuthLogServiceDelegate();
		EAuthHashServiceDelegate esd = ServiceDelegateFactory.createEAuthHashServiceDelegate();
		
		if(log.isDebugEnabled()){
			log.debug("++Found that user (Level 2) needs to be auto mapped for UserProfileId: " + uProfile.getId());
		}
		esd.saveHash(uProfile.getUserName(), ahash);
		VoidServiceResponse vsr = elsd.logEauthEvent(ahash, uProfile.getUserName(), csid, "MAP");
		//Write the mapping event to the Activity table
		if(uProfile != null && uProfile.getId() != null){
			//Jazz #110471 US REQ#3.2 RemoteID Proofing -  On the re-authentication page , when user selects CONTINUE 
			//button and the user is successfully authenticated, MHV system shall create an AAL entry to record the account was connected: 
			Auditor.auditDSLogonMappingEvent(uProfile.getId(), ActivityActorTypeEnumeration.SELF, csid, ACTION_DETAIL_DS_LOGON_ACCESS_VA);
    	}

		if (vsr.getMessages().hasErrorMessages()) {
	          for (Object item:vsr.getMessages().getErrorMessages()) {
			    log.error("Error logging to EAUTHLOG: " + item);
	          }
	        }
		StringServiceResponse ssr1 = esd.findUserNameByHash(ahash);
		return ssr1.getString();
		
	}
	
	public String retrieveTraitFromHeader(HttpServletRequest req, String attribute) {
		String trait = (String) req.getSession().getAttribute(attribute);
		if(trait == null) {
			trait = req.getHeader(attribute);
			req.getSession().setAttribute(attribute, trait);
		}
		return trait;
	}
	
	private boolean isValidParam(String param) {
		if(param == null || param.length()==0) {
			return false;
		}
		return true;
	}
	
	private static Message getError(Messages messages, String key) {
		Message message = null;
		if (messages != null) {
			for (Iterator i = messages.getErrorMessages().iterator(); 
			     (message == null) && i.hasNext();)
			{
				message = (Message) i.next();
				if ((message != null) && !message.getKey().equals(key)) {
					message = null; // not found, keep looking
				}
			}
		}
		return message;
	}
	
	private boolean isNotFound(String str) {
		if(str!=null && str.equalsIgnoreCase("NOT_FOUND")) {
			return true;
		}
		return false;
	}
	
	private void processLastLogin(UserProfile uProfile) {
		if(uProfile.getCurrentLogin()!=null) {
			uProfile.setLastLogin(uProfile.getCurrentLogin());
		}
		//Jazz136316 Logout box date time change
		uProfile.setCurrentLogin(new Timestamp( new Date().getTime()));
		gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createEntityMaintenanceServiceDelegate().save(uProfile);
	}
	
	protected void doPost(HttpServletRequest req, HttpServletResponse res)
		throws ServletException, IOException {
		req.getSession().removeAttribute(DISPLAY_ERROR);
		
		String username = req.getParameter(USERNAME);
		String password = req.getParameter(PASSWORD);
		
		//Jazz #110471 US REQ#3.2 RemoteID Proofing - On the re-authentication page, both User ID and Password shall be required. 
		//If the user leaves either User ID or Password blank or both, MHV system shall display a user-friendly error message:   
		if(!isValidParam(username)||!isValidParam(password)) {
			req.getSession().setAttribute(DISPLAY_ERROR, INVALID_LOGIN_INFO);
			res.sendRedirect(login_url);
			return;
		}
		
		UserProfileServiceResponse userProfileSerRes = 
			gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserProfileServiceDelegate().getProfileForUser(username);
		
		String correctUsername = (String)req.getSession().getAttribute(USER_NAME);
		
		UserProfile uProfile = userProfileSerRes.getUserProfile();
		//Jazz #110471 US REQ#3.3 RemoteID Proofing - On the re-authentication page, if MHV user selects Forgot User ID 
		//hyperlink then system shall take the user into existing Forgot User ID page.    
		if (uProfile == null || !correctUsername.toLowerCase().equalsIgnoreCase(username.toLowerCase())) {
System.out.println("correctUsername.toLowerCase():"+correctUsername.toLowerCase());			
System.out.println("username.toLowerCase():"+username.toLowerCase());			
			req.getSession().setAttribute(DISPLAY_ERROR, USERNAME_NOT_FOUND);
			res.sendRedirect(login_url);
			return;
		}

		BooleanServiceResponse response = 
			gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserServiceDelegate().authenticate(username, password);
	
		boolean success = !ServiceResponseUtils.hasErrorMessages(response);
		
		//Jazz #110471 US REQ#3.6 RemoteID Proofing - For the re-authentication page, User experience shall be the same as current functionality on home page login for access attempt tries  all the way through to resetting.
		if(!success) {
			if(response.getMessages().hasErrorMessages()) {
				Message message = getError(response.getMessages(), "invalid.credentials");
				//Jazz #110471 US REQ#3.3 RemoteID Proofing - If the Password is incorrect, MHV system shall display following user friendly error message:    
				if (message != null) {
					req.getSession().setAttribute(DISPLAY_ERROR, INVALID_PASSWORD);
					res.sendRedirect(login_url);
					return;
				} else {
					List msgList = new ArrayList();
					msgList.add(response.getMessages().getErrorMessages());
					req.getSession().setAttribute(Messages.REQ_ATTR_ERROR_MSGS, msgList);
					res.sendRedirect(login_url);
				}
			}
			return;
		}
		
		String ahash = null;
		if(useExistingDslogonProcess) {
			ahash = (String) req.getSession().getAttribute(HASH_ATTR);
			if(ahash == null) {
				ahash = req.getHeader(HASH_PARAM);
				req.getSession().setAttribute(HASH_ATTR, ahash);
			}
	
			if (log.isDebugEnabled()) {
				log.debug("********** EAuthGatewayServlet.doGet() ahash:"+ahash);		
			}
		}
		
		if(isFourOrMoreTraitsUpdate(uProfile, req)) {
			gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserProfileServiceDelegate().changeActivationState(
				uProfile, UserProfileDeactivationUtils.findReason(UserProfileDeactivationUtils.MULTIPLE_ACCOUNTS_RESOLUTION_NAME), 
				"System", "Four or More Traits updat", null);
			res.sendRedirect(error_url);
			return;
		}
		
		String csid = (String)req.getSession().getAttribute(CSID_ATTR);
		if(csid == null) {
			csid = req.getHeader(CSID_PARAM);
			csid = URLDecoder.decode(csid, "UTF-8");
			req.getSession().setAttribute(CSID_ATTR, csid);
		}

		//Jazz #110471 US REQ#5.5.1 RemoteID Proofing - On the re-authentication page , when user selects CONTINUE button and 
		//the user is successfully authenticated, MHV system shall create an AAL entry to record the account was connected:  
		Auditor.auditDSLogonMappingEvent(uProfile.getId(), ActivityActorTypeEnumeration.SELF, csid, ACTION_DETAIL_DS_LOGON_ACCESS_VA);
		
		if(useExistingDslogonProcess) {
			populateEauthHash(uProfile, ahash, csid);
			res.sendRedirect("gateway");
			return;
		}
		
		String firstName = retrieveTraitFromHeader(req, FIRST_NAME);
		if(firstName != null && !isNotFound(firstName)) {
			uProfile.setFirstName(firstName);
		}
		
		String middleName = retrieveTraitFromHeader(req, MIDDLE_NAME);
		if(middleName != null && !isNotFound(middleName)) {
			uProfile.setMiddleName(middleName);
		}
		
		String lastName = retrieveTraitFromHeader(req, LAST_NAME);
		if(lastName != null && !isNotFound(lastName)) {
			uProfile.setLastName(lastName);
		}
		
		String gender = retrieveTraitFromHeader(req, GENDER);
		if(gender != null && !isNotFound(gender)) {
			if(gender.equalsIgnoreCase(GenderEnumeration.MALE)) {
				uProfile.setGender(GenderEnumeration.getEnum(GenderEnumeration.MALE));
			} else if(gender.equalsIgnoreCase(GenderEnumeration.FEMALE)) {
				uProfile.setGender(GenderEnumeration.getEnum(GenderEnumeration.FEMALE));
			}	
		}
		
		String ssn = retrieveTraitFromHeader(req, ID);
		if(ssn != null && !isNotFound(ssn)) {
			uProfile.setSsn(formatSSN(ssn));
		}
		
		String birthDate = retrieveTraitFromHeader(req, BIRTH_DATE);
		if(birthDate != null && !isNotFound(birthDate)) {
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
			try {
				Date dateParsed = sdf.parse(birthDate.substring(0, 10));
				uProfile.setBirthDate(dateParsed);
			} catch (ParseException e) {
				e.printStackTrace();
			}
		}
		
		String address1 = retrieveTraitFromHeader(req, ADDRESS1);
		if(address1 != null && !isNotFound(address1)) {
			uProfile.setAddressStreet1(address1);
		}
		
		String city = retrieveTraitFromHeader(req, CITY);
		if(city != null && !isNotFound(city)) {
			uProfile.setAddressCity(city);
		}
		
		String state = retrieveTraitFromHeader(req, STATE);
		if(city != null && !isNotFound(city)) {
			uProfile.setAddressCity(city);
		}
		
		String postalCode = retrieveTraitFromHeader(req, POSTAL_CODE);
		if(postalCode != null && !isNotFound(postalCode)) {
			uProfile.setAddressPostalCode(postalCode);
		}
		
		String country = retrieveTraitFromHeader(req, COUNTRY);
		if(country != null && !isNotFound(country)) {
			uProfile.setAddressCountry(CountryEnumeration.getEnum(Country.getMHVCountry(country)));
		}
		
		String email = retrieveTraitFromHeader(req, EMAIL);
		if(email != null && !isNotFound(email)) {
			uProfile.setContactInfoEmail(email);
		}
		
		gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createEntityMaintenanceServiceDelegate().save(uProfile);
		MviIntegrationServiceDelegate mviSyncServiceDelegate = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createMviIntegrationServiceDelegate();
		PatientServiceResponse mviResponse = mviSyncServiceDelegate.matchPersonInIPA(uProfile);
		
		if(mviResponse.getMessages().hasErrorMessages()) {
			res.sendRedirect(error_url);
			return;
		} else {
			Subject sub = null;
			String deepLink = (String)req.getSession().getAttribute(DEEP_LINK_ATTR);
			try {
				String id = uProfile.getUserName();
				sub = weblogic.security.services.Authentication.assertIdentity("EauthToken", buildToken(id).getBytes());
				if( log.isInfoEnabled()) {
					log.info("authentication successful for user: " + id);
				}

				if(useExistingDslogonProcess) {
					EAuthLogServiceDelegate elsd = ServiceDelegateFactory.createEAuthLogServiceDelegate();
			        VoidServiceResponse vsr = elsd.logEauthEvent(ahash, id, csid, "Authenticate");
			        if (vsr.getMessages().hasErrorMessages()) {
			          for (Object item:vsr.getMessages().getErrorMessages()) {
					    log.error("EAuth Log event error: " + item);
			          }
			        }
				}	
				//JAZZ Story#39187 - US REQ#4 MHV System shall make following AAL Entry updates for Login
		        Auditor.auditEBenefitsLoginEvent(uProfile.getId(), ActivityActorTypeEnumeration.SELF, Boolean.TRUE, deepLink, csid);
			} catch (LoginException le) {
				// authentication failed - redirect to mapping page
				// set hash on session
				log.error("error logging in ", le);
				res.sendRedirect(eauth_portal);
				return;
			}

			weblogic.servlet.security.ServletAuthentication.runAs(sub, req);

			if (null != sub && !SubjectUtils.isUserAnonymous(sub)) {
                //JAZZ # 111941 - Remote ID Proofing US REQ#3.11 - When the User lands into their MHV account already logged in,   
				//MHV system shall record this login as the last time the user logged in to their MHV account. 
				processLastLogin(uProfile);
				String portal_url = mhv_portal;
				if(null != deepLink){
					if(deepLink.equalsIgnoreCase("download_my_data"))
						portal_url = mhv_portal_custom + "?_nfpb=true&_nfls=false&_pageLabel=downloadData";
					else if(deepLink.equalsIgnoreCase("prescription_refill"))
						portal_url = mhv_portal_custom + "?_nfpb=true&_pageLabel=rxRefill&_nfls=false";
					else if(deepLink.equalsIgnoreCase("appointments"))
						portal_url = mhv_portal_custom + "?_nfpb=true&_pageLabel=appointments&_nfls=false";
					else if(deepLink.equalsIgnoreCase("secure_messaging"))
						portal_url = mhv_portal_custom + "?_nfpb=true&_nfto=false&_pageLabel=secureMessagingPage";
				}
				res.sendRedirect(portal_url);
			}
			
		}
	}
	
	private static String formatSSN(String ssn) {
		StringBuffer ssnBuff = new StringBuffer(ssn);
		ssnBuff.insert(3, '-');
		ssnBuff.insert(6, '-');
		return ssnBuff.toString();
	}
	
	private static String getCountryName(String countryCode) {
		String[] countris = Locale.getISOCountries();
		return null;
	}
	
	protected void doGet(HttpServletRequest req, HttpServletResponse res)
			throws ServletException, IOException {
		String cancel = req.getParameter("cancel");
		if(cancel !=null && cancel.equalsIgnoreCase("true")) {
			res.sendRedirect(splash_url);
			return;
		}
		res.sendRedirect(login_url);
	}

	private boolean isDeactivated(String username) {
		boolean result = true;

		UserProfileServiceDelegate userProfileDelegate = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserProfileServiceDelegate();
		UserProfileServiceResponse upResponse = userProfileDelegate.getProfileForUser(username);
		if (upResponse.getMessages().hasErrorMessages()) {
	          for (Object item:upResponse.getMessages().getErrorMessages()) {
			    log.error("UserProfile getUserProfileByUsername action Error: " + item);
	          }
	    }

		if( upResponse.getUserProfile() != null && upResponse.getUserProfile().getDeactivationReason() == null) {
			result = false;
		}
		if( result == true && log.isDebugEnabled() ) {
			log.debug("Found a deactivated status for user:: " + username + ", Deactivation Reason: " + upResponse.getUserProfile().getDeactivationReason() );
		}

		return result;
	}

	private UserProfile findUserProfile(String firstName, String lastName,
			String gender, String sSN, String birthDate) {

		GenderEnumeration genderEnum;
		if(gender.equalsIgnoreCase(GenderEnumeration.MALE)){
			genderEnum = GenderEnumeration.getEnum(GenderEnumeration.MALE);
		}else{
			genderEnum = GenderEnumeration.getEnum(GenderEnumeration.FEMALE);
		}

		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		Date date = null;
		try {
			date = sdf.parse(birthDate.substring(0, 10));
		} catch (ParseException e) {
			if(log.isDebugEnabled()){
				log.debug("Not able to parse date for the user:: " + firstName + " " + lastName + ", birthdate: " + birthDate);
			}
		}

		UserProfileServiceDelegate userProfileDelegate = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserProfileServiceDelegate();
		UserProfileServiceResponse upResponse = userProfileDelegate.getUserProfileForVAAFITraits(firstName, lastName, date, genderEnum);
		if (upResponse.getMessages().hasErrorMessages()) {
	          for (Object item:upResponse.getMessages().getErrorMessages()) {
			    log.error("UserProfile getUserProfileForVAAFITraits action Error: " + item);
	          }
	    }

		UserProfile uProfile = upResponse.getUserProfile();
		if( uProfile != null ) {
			if(log.isDebugEnabled()){
				log.debug("Found DSLogon user profile:: " + uProfile.getId() + " with user_name:: " + uProfile.getUserName());
			}

			/*
			 * Compare SSN right here (SSN could be null for a non patient from evault)
			 */
			if(uProfile.getSsn() == null || !uProfile.getSsn().replace("-", "").equals(sSN)){
				if(log.isDebugEnabled()){
					log.debug("SSN did not match for DSLogon CSID user for the user profile:: " + uProfile.getId());
				}
				uProfile = null;
			}
		}

		return uProfile;
	}

	/**
	 *
	 * @param userid
	 * @return String - encoded token used for IdentityAsserter
	 */
	private String buildToken(String userid) {
		String tokenValue = "username=" + encrypt(userid);
		return tokenValue;
	}

	private String encrypt(String s) {
		String result = null;
		if (s == null || s.trim().length() == 0) {
			throw new IllegalArgumentException(
					"unencrypted string was null or empty");
		}
		try {
			SecretKey key = keyFactory.generateSecret(keySpec);
			cipher.init(Cipher.ENCRYPT_MODE, key);
			byte[] sb = s.getBytes(UNICODE_FORMAT);
			byte[] csb = cipher.doFinal(sb);
			BASE64Encoder be = new BASE64Encoder();
			result = be.encodeBuffer(csb);
		} catch (InvalidKeySpecException iks) {
			log.error("error encrypting token", iks);
    } catch (InvalidKeyException ik) {
      log.error("error encrypting token", ik);
    } catch (UnsupportedEncodingException use) {
      log.error("error encrypting token", use);
    } catch (BadPaddingException bp) {
      log.error("Error encrypting token", bp);
    } catch (IllegalBlockSizeException ibs) {
      log.error("Error encrypting token", ibs);
		}
		return result;
	}

	public boolean hasERarAccess(String username) {
		return true;
		//return hasExtractAccess(AccessFeatureValueUtils.VAERAR.getName(), username);
	}
	
	public boolean hasExtractAccess(String extractFeature, String username) {

		boolean hasExtractAccess = false;
		AccessControlCollectionServiceResponse accessResponse =
			getAccessControlServiceDelegate().findAccessControls(username);

		hasExtractAccess = AccessControlUtils.hasAccess(AccessDomainUtils.PHR, extractFeature,
							AccessPermissionUtils.READ, accessResponse.getAccessControls());
		return hasExtractAccess;
	}

	public AccessControlServiceDelegate getAccessControlServiceDelegate() {
		return gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createAccessControlServiceDelegate();
	}
	
    private boolean isFourOrMoreTraitsUpdate(UserProfile uProfile, HttpServletRequest req) {
    	int traitUpdateCount = 0;
    	
        if (log.isDebugEnabled()) {
        	log.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() for user profile id:" + uProfile.getId());
        }
        
		String firstName = retrieveTraitFromHeader(req, FIRST_NAME);
        //If a trait is not the same, we increment the count. This applies to first name, middle name, last name, ssn, gender and DOB.
        if(!uProfile.getFirstName().equalsIgnoreCase(firstName)) {
            if (log.isDebugEnabled()) {
            	log.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() first name is different for user profile id:" + uProfile.getId());
            }
        	traitUpdateCount++;
        }
        
		String middleName = retrieveTraitFromHeader(req, MIDDLE_NAME);
		
        //Middle name can be null on both sides so we need to cover all permutations.
        if(uProfile.getMiddleName()==null && middleName!=null) {
            if (log.isDebugEnabled()) {
            	log.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() middle name is different for user profile id:" + uProfile.getId());
            }
        	traitUpdateCount++;
        } else if(uProfile.getMiddleName()!=null && middleName==null) {
            if (log.isDebugEnabled()) {
            	log.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() middle name is different for user profile id:" + uProfile.getId());
            }
        	traitUpdateCount++;
        } else if(uProfile.getMiddleName()==null && middleName==null) {
            if (log.isDebugEnabled()) {
            	log.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() no middle name for user profile id:" + uProfile.getId());
            }
        } else if(!uProfile.getMiddleName().equalsIgnoreCase(middleName)) {
            if (log.isDebugEnabled()) {
            	log.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() middle name is different for user profile id:" + uProfile.getId());
            }
        	traitUpdateCount++;
        }
        
		String lastName = retrieveTraitFromHeader(req, LAST_NAME);
		
        if(!uProfile.getLastName().equalsIgnoreCase(lastName)) {
            if (log.isDebugEnabled()) {
            	log.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() last name is different for user profile id:" + uProfile.getId());
            }
        	traitUpdateCount++;
        }
        
		String birthDate = retrieveTraitFromHeader(req, BIRTH_DATE);
		if(birthDate != null && !isNotFound(birthDate)) {
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
			try {
				Date dateParsed = sdf.parse(birthDate.substring(0, 10));
		        if(!uProfile.getBirthDate().equals(dateParsed)) {
		            if (log.isDebugEnabled()) {
		            	log.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() birth date is different for user profile id:" + uProfile.getId());
		            }
		        	traitUpdateCount++;
		        }
			} catch (ParseException e) {
				e.printStackTrace();
			}
		}
		
		String gender = retrieveTraitFromHeader(req, GENDER);

        
        if(!uProfile.getGender().getName().equalsIgnoreCase(gender)) {
            if (log.isDebugEnabled()) {
            	log.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() gender is different for user profile id:" + uProfile.getId());
            }
        	traitUpdateCount++;
        }
        
        String ssn = retrieveTraitFromHeader(req, ID);
		if(ssn.length()==10) {
			ssn = ssn.substring(1);
		}
        
        String ssnFromUProfile = uProfile.getSsn().replaceAll("-", "");

        if(!ssnFromUProfile.equals(ssn)) {
            if (log.isDebugEnabled()) {
            	log.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() ssn is different for user profile id:" + uProfile.getId());
            }
        	traitUpdateCount++;
        }
        
        if(traitUpdateCount>3) {
        	return true;
        }
        
        return false;
    }

}
