package gov.va.med.mhv.eauth;

import gov.va.med.mhv.eauth.service.delegate.EAuthHashServiceDelegate;
import gov.va.med.mhv.eauth.service.delegate.EAuthLogServiceDelegate;
import gov.va.med.mhv.eauth.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActionTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.AuthenticationDefermentReason;
import gov.va.med.mhv.usermgmt.enumeration.AuthenticationStatus;
import gov.va.med.mhv.usermgmt.enumeration.GenderEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.PatientCorrelationStatus;
import gov.va.med.mhv.usermgmt.service.AccessControlCollectionServiceResponse;
import gov.va.med.mhv.usermgmt.service.FacilityInfoServiceResponse;
import gov.va.med.mhv.usermgmt.service.InPersonAuthenticationServiceResponse;
import gov.va.med.mhv.usermgmt.service.PatientServiceResponse;
import gov.va.med.mhv.usermgmt.service.UserProfileServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.AccessControlServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.FacilityServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.InPersonAuthenticationServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.MviIntegrationServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.UserProfileServiceDelegate;
import gov.va.med.mhv.usermgmt.transfer.FacilityInfo;
import gov.va.med.mhv.usermgmt.transfer.InPersonAuthentication;
import gov.va.med.mhv.usermgmt.transfer.Patient;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;
import gov.va.med.mhv.usermgmt.util.AccessControlUtils;
import gov.va.med.mhv.usermgmt.util.AccessDomainUtils;
import gov.va.med.mhv.usermgmt.util.AccessPermissionUtils;
import gov.va.med.mhv.usermgmt.util.Auditor;
import gov.va.med.mhv.usermgmt.util.PatientCorrelationStatusUtils;
import gov.va.med.mhv.usermgmt.util.mvi.MviConstants;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.KeySpec;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Properties;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;
import javax.security.auth.Subject;
import javax.security.auth.login.LoginException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.messages.Messages;
import org.tigris.atlas.service.StringServiceResponse;
import org.tigris.atlas.service.VoidServiceResponse;

import com.bea.p13n.util.encoding.EncodingUtil;

import sun.misc.BASE64Encoder;
import weblogic.security.SubjectUtils;

public class EAuthGatewayServlet extends HttpServlet implements EauthConstants{

	Log log = LogFactory.getFactory().getInstance(EAuthGatewayServlet.class);

	private static final long serialVersionUID = 5589615804128963698L;

	private static final String ENCRYPTION_KEY = "098765432109876543210987654321";

	private static final String DES_ENCRYPTION_SCHEME = "DES";
	private static final String DISPLAY_ERROR = "displayError";	
	private static final String ORC_CSP = "2030004";
	private static final String UNICODE_FORMAT = "UTF-8";
	private static final String USER_NAME = "username";
	private static final String ERROR_DETAIL = "errorDetail";
	
	private static final String ACTION_DETAIL_DS_LOGON_ACCESS_VA = "";
	
	private static final String MULTIPLE_ACCOUNT_ERROR_ATTRIB = "multiAccountError";
	private static final String FEDERATED_CREDENTIAL_ATTRIB = "FCCX";
	private static final String DISPLAY_ONLINE_UPGRADE_ATTRIB = "displayOnlineUpgrade";
	private static final String DISPLAY_ERROR_ATTRIB = "displayError";
	private static final String VAPII_FORM_URL_ATTRIB = "vapiiformurl";
	private static final String HAS_ERAR_ACCESS_ATTRIB = "hasERarAccess";
	private static final String UPGRADE_OPTION_SELECTED = "upgradeOptionSelected";
	
	private static final String TRUE_VALUE = "true";
	private static final String FALSE_VALUE = "false";
	private static final String MVI_COMM_ISSUE_VALUE = "MVI_COMM_ISSUE";
	private static final String ERROR_VALUE = "ERROR:";
	private static final String VAPII_ERRIR_VALUE = "vapiiError";
	private static final String VAPII_SUBMITTED_VALUE = "Submitted";
	private static final String FORM_NOT_FOUND_VALUE = "formNotFound";
	private static final String AUTO_OPTION_VALUE = "auto";
	
	private static final String MULTIPLE_ACCOUNT_ERROR_MSG = "Multiple Account Error";
	private static final String DEACTIVATED_ACCOUNT_MSG = "Deactivated Account";
	private static final String EMPTY_CSID_MSG = "CSID is empty";
	private static final String EMPTY_ASSURANCE_LEVEL_MSG = "Empty VA Eauth Assurance Level";
	private static final String INVALID_ASSURANCE_LEVEL_MSG = "Invalid VA Eauth Assurance Level";
	private static final String INVALID_HASH_MSG = "Invalid Hash";
	private static final String NOT_FOUND_ERROR_MSG = "NOT_FOUND on first name, last name birth date or ssn";
	private static final String INVALID_SSN_MSG = "Invalid SSN";
	private static final String INVALID_DOB_MSG = "Invalid Birth Date";
	private static final String INVALID_ICN_MSG = "Invalid ICN";
	private KeySpec keySpec;

	private SecretKeyFactory keyFactory;

	private Cipher cipher;

	private String splash_url;
	
	private String error_url = null;

	private String mvi_comm_error_url = null;

	private String eauth_portal = null;

	private String mhv_portal = null;

	private String mhv_portal_custom = null;

	private String dslogon_value;

	private String usaa_value;

	private String va_eauth_assur_prop_level;

	private String realmName = null;

	private String anonymous_url;

	private String register_url;
	
	private String upgrade_url;

	private String login_url;
	
	private String auto_upgrade_url;
	
	private String auto_upgrade_vapii_url;
	
	private String manual_upgrade_url;

	private String yesNoUpgrade = null;

	private String orc_retired_url = null;
	
	private boolean useExistingDslogonProcess = false;
	
	private boolean useExistingSSO = false;
	
	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		try {
			byte[] keyAsBytes = ENCRYPTION_KEY.getBytes(UNICODE_FORMAT);
			keySpec = new DESKeySpec(keyAsBytes);
			keyFactory = SecretKeyFactory.getInstance(DES_ENCRYPTION_SCHEME);
			cipher = Cipher.getInstance(DES_ENCRYPTION_SCHEME);
	    } catch (NoSuchAlgorithmException nsa) {
	      log.error("init error", nsa);
	    } catch (InvalidKeyException ik) {
	      log.error("init error", ik);
	    } catch (UnsupportedEncodingException use) {
	      log.error("init error", use);
	    } catch (NoSuchPaddingException nsp) {
	      log.error("init error", nsp);
	    }

		Properties props = new Properties();
		String propertyFile = "eauth.properties";
		String path = System.getProperty("config.location");
		String anotherProperty = System.getProperty("Dmhv.config.filename");

		try {
			props.load(new FileInputStream(path + File.separator + propertyFile));

			// Now get the value from properties file.
			error_url = props.getProperty("eauth.error.url");
			splash_url = props.getProperty("eauth.splash.screen.url");
			mvi_comm_error_url = props.getProperty("eauth.dslogon.mvi.comm.error.url");
			eauth_portal = props.getProperty("eauth.portal.url");
			mhv_portal = props.getProperty("mhv.portal.url");
			mhv_portal_custom = props.getProperty("mhv.portal.url.custom");
			realmName = props.getProperty("weblogic.security.realm");
			dslogon_value = props.getProperty("eauth.dslogon.value", "DSLOGON");
			usaa_value = props.getProperty("eauth.usaa.value", "USAA");
			va_eauth_assur_prop_level = props.getProperty("eauth.va.assurance.level", "2");
			anonymous_url = props.getProperty("eauth.anonymous.url", "/anonymous.portal");
			upgrade_url = props.getProperty("eauth.upgrade.url", "eauth.portal?_nfpb=true&_nfto=false&_pageLabel=mhvUpgrade");
			register_url = props.getProperty("eauth.upgrade.url", "anonymous.portal?_nfpb=true&_nfto=false&_pageLabel=registrationHome");
			login_url = "eauth.portal?_nfpb=true&_nfto=false&_pageLabel=mhvEauthLogin&removeTabs=true";
			auto_upgrade_url = props.getProperty("eauth.auto.upgrade.url", "eauth.portal?_nfpb=true&_nfto=false&_pageLabel=mhvAutoUpgrade");
			auto_upgrade_vapii_url = props.getProperty("eauth.auto.upgrade.vapii_url", "eauth.portal?_nfpb=true&_nfto=false&_pageLabel=mhvAutoUpgradeVapii");
			manual_upgrade_url = props.getProperty("eauth.manual.upgrade.url", "eauth.portal?_nfpb=true&_nfto=false&_pageLabel=mhvManualUpgrade");
			orc_retired_url = props.getProperty("eauth.orc.retiring.url", "anonymous.portal");
			
			if(props.getProperty("use.existing.dslogon.process")!=null && props.getProperty("use.existing.dslogon.process").equalsIgnoreCase(TRUE_VALUE)) {
				useExistingDslogonProcess = true;
			}
			register_url = "anonymous.portal?_nfpb=true&_nfto=false&_pageLabel=registrationHome";
		} catch (IOException io) {
			log.error("error getting eauth related properties", io);
		}
	}

	private boolean isDsLogonUser(String csid) {
		if(csid.equalsIgnoreCase(dslogon_value)) {
			return true;
		}
		return false; 
	}
	
	private boolean goToStartTodayPage(String csid, InPersonAuthentication ipa, HttpServletRequest req, HttpServletResponse res) 
		throws ServletException, IOException {
		
		if(isDsLogonUser(csid)) {
			req.getSession().setAttribute(DISPLAY_ONLINE_UPGRADE_ATTRIB, TRUE_VALUE);
		} else {
			req.getSession().setAttribute(DISPLAY_ONLINE_UPGRADE_ATTRIB, FALSE_VALUE);
		}
		
		//JAZZ #32762 US01 REQ#1.3 The MHV system shall check if the user has signed the form or 
		//started the electronic upgrade process by checking the  "Signed 20-5345a-MHV form online" 
		//field or  "Date/Time Start upgrade online" field in the MHV database.
		StringServiceResponse vapiiStatusResponse = 
			gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserProfileServiceDelegate().getVapiiFormStatusForUser(
					ipa.getPatient().getUserProfile());
		//JAZZ #32762 US04 REQ#1.1.2 WhenVAPii is unavailable, refer US05: VAPii is Unavailable.
		if(vapiiStatusResponse.getMessages().hasErrorMessages()) {
			enterInfoLog("getVapiiFormStatus has errors:" + retrieveErrorMessages(vapiiStatusResponse.getMessages()), ipa.getPatient().getIcn());
			req.getSession().setAttribute(DISPLAY_ONLINE_UPGRADE_ATTRIB, FALSE_VALUE);
			req.getSession().setAttribute(DISPLAY_ERROR_ATTRIB, VAPII_ERRIR_VALUE);
			res.sendRedirect(upgrade_url);
		} else if(!VAPII_SUBMITTED_VALUE.equalsIgnoreCase(vapiiStatusResponse.getString())) {
			//JAZZ #32762 US02 REQ#5.2 If the User did not complete the form process or VAPii does not locate the form, the 
			//System shall provide a user-friendly message indicating the signed form was not located within VAPii at the top of the 
			//Start Upgrading Your My HealtheVet Account Today page. 
			if(ipa.getStartUpgradeOnlineDateTime()!=null || ipa.getSignedFormOnlineDateTime()!=null) {
				req.getSession().setAttribute(DISPLAY_ERROR_ATTRIB, FORM_NOT_FOUND_VALUE);
			} else {
				//JAZZ #32762 US01 REQ#2 If the users have not started the upgrade process or signed the eRAR form, 
				//MHV system shall display the Start Upgrading Your My HealtheVet Account Today page in MHV.
				req.getSession().removeAttribute(DISPLAY_ERROR_ATTRIB);
			}
			res.sendRedirect(upgrade_url);
		} else if(VAPII_SUBMITTED_VALUE.equalsIgnoreCase(vapiiStatusResponse.getString())) {
			enterInfoLog("Not Going to the Upgrade page. VAPii Status is 'SUBMITTED' for UserProfileId: " + 
				ipa.getPatient().getUserProfile().getUserName(), ipa.getPatient().getIcn());
		    return false;
		}
		return true;
	}

	private void setVapiiFormUrlInSession(HttpServletRequest req) {
		StringServiceResponse vapiiFormUrlResponse = 
			gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserProfileServiceDelegate().getVapiiFormUrl();
		req.getSession().setAttribute(VAPII_FORM_URL_ATTRIB, vapiiFormUrlResponse.getString());
	}
	
	private boolean isUserSignedeRARForm(UserProfile userProfile) {
		StringServiceResponse vapiiStatusResponse = 
			gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserProfileServiceDelegate().getVapiiFormStatusForUser(
					userProfile);
		if(VAPII_SUBMITTED_VALUE.equalsIgnoreCase(vapiiStatusResponse.getString())) {
			return true;
		}
		return false;
	}

	private FacilityInfo getAustinMHVFacilityInfo() {
		FacilityServiceDelegate facilityInfoDelegate = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createFacilityServiceDelegate();
		FacilityInfoServiceResponse facilityInfoResponse = facilityInfoDelegate.queryFacilityInfoByStationNumber("200MH");
		return facilityInfoResponse.getFacilityInfo();
	}
	
	private String populateEauthHash(UserProfile uProfile, String ahash, String csid) {
		EAuthLogServiceDelegate elsd = ServiceDelegateFactory.createEAuthLogServiceDelegate();
		EAuthHashServiceDelegate esd = ServiceDelegateFactory.createEAuthHashServiceDelegate();
		
		esd.saveHash(uProfile.getUserName(), ahash);
		VoidServiceResponse vsr = elsd.logEauthEvent(ahash, uProfile.getUserName(), csid, "MAP");
		//Write the mapping event to the Activity table
		if(uProfile != null && uProfile.getId() != null){
			//JAZZ #32762 US REQ#5 MHV System shall make following AAL Entry updates. 
			Auditor.auditDSLogonMappingEvent(uProfile.getId(), ActivityActorTypeEnumeration.SELF, csid, ACTION_DETAIL_DS_LOGON_ACCESS_VA);
    	}

		if (vsr.getMessages().hasErrorMessages()) {
	        enterInfoLog("Error logging to EAUTHLOG: " + retrieveErrorMessages(vsr.getMessages()), "userProfileId:"+uProfile.getId());  
	    }
		StringServiceResponse ssr1 = esd.findUserNameByHash(ahash);
		return ssr1.getString();
	}
	
	public String retrieveTraitFromHeaderAndDecode(HttpServletRequest req, String attribute) 
		throws IOException {
		String trait = (String) req.getSession().getAttribute(attribute);
		if(trait == null) {
			trait = req.getHeader(attribute);
			if(trait != null)      {
				trait = URLDecoder.decode(trait, UNICODE_FORMAT);
				req.getSession().setAttribute(attribute, trait);
			}
		}
		return trait;
	}
	
	public String retrieveTraitFromHeader(HttpServletRequest req, String attribute) {
		String trait = (String) req.getSession().getAttribute(attribute);
		if(trait == null) {
			trait = req.getHeader(attribute);
			req.getSession().setAttribute(attribute, trait);
		}
		return trait;
	}
	
	/**
	 * JAZZ # 39187: Story:Remote ID Proofing - Implementation - 
	 * US REQ#1.2 In the event VAAFI sends NOT_FOUND value for any of the user traits in header 
	 * (first name, middle name, last name, birth date, ssn or gender), MHV system shall display 
	 * a user friendly message Not able to recognize account information sent to My HealtheVet 
	 * as a third bullet on the We Cannot Connect Your Account page.
	 * 
	 * @param dateStr
	 * @return
	 */
	private boolean isNotFound(String str) {
		if(str==null || str.equalsIgnoreCase("NOT_FOUND")) {
			return true;
		}
		return false;
	}
	
	private boolean isDate(String dateStr) {
		if(dateStr.equalsIgnoreCase("NOT_FOUND")) {
			return false;
		}
		
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		Date date = null;
		try {
			date = sdf.parse(dateStr.substring(0, 10));
		} catch (Throwable t) {
			t.printStackTrace();
			return false;
		}
		return true;
	}
	
	private boolean isDigit(String number) {
		char[] numberArray = number.toCharArray();
		for (int i = 0; i < numberArray.length; i++) {
			if(!Character.isDigit(numberArray[i])) {
				return false;
			}
		}
		return true;
	}

	private boolean isEdipiExist(String edipi) {
		if((edipi!=null) && !isNotFound(edipi)){
			return true;
		}
		return false;
	}
	
	private boolean isEdipiValid(String edipi) {
		if((edipi!=null) && !isNotFound(edipi) && edipi.length()==10 && isDigit(edipi)){
			String[] edipis = edipi.split(",");
			if(edipis !=null && edipis.length > 1) {
				return false;
			}
			return true;
		}
		return false;
	}
	
	private boolean isEauthHashValid(String ahash) {
		if((ahash!=null) && !isNotFound(ahash) && ahash.length()==28 && ahash.endsWith("=")){
			return true;
		}
		return false;
	}
	
	private boolean isSsnValid(String ssn, String idType) {
		if(!isDigit(ssn) || ssn.length()!=9 || !idType.equalsIgnoreCase("SSN")) {
			return false;
		}
		return true;
	}	
	
	private String findUserNameByHash(String ahash) {
		EAuthHashServiceDelegate esd = ServiceDelegateFactory.createEAuthHashServiceDelegate();
		StringServiceResponse ssr = esd.findUserNameByHash(ahash);
		return ssr.getString();
	}
	
	/*
     * Jazz #110471 RemoteID Proofing - Use ICN
	 */
	private String retrieveICN(HttpServletRequest req) {
		String icn = (String) req.getSession().getAttribute(ICN_PARAM);
		if(icn == null) {
			icn = req.getHeader(ICN_PARAM);
			req.getSession().setAttribute(ICN_PARAM, icn);
		}
		
		return icn;
	}
	
	/*
     * Jazz #110471 RemoteID Proofing - Use ICN
	 */
	private boolean isICNAvailable(String icn) {
		if((icn!=null) && !isNotFound(icn)){
			return true;
		}
		return false;
	}
	
	private boolean isICNValid(String icn) {
		//Jazz #110471 RemoteID Proofing - Use ICN -US02 - If ICN exists then validate the syntax of ICN (17 character ICN. 
		//(Format is 10 digits, one alpha letter followed by 6 digits )* 
		//(per discussion with IAM on fully qualified ICN)
		if((icn!=null) && !isNotFound(icn) && icn.length()==17 && isDigit(icn.substring(0, 10)) && 
			!isDigit(icn.substring(10, 11)) && isDigit(icn.substring(11, 17))){
			return true;
		}
		return false;
	}
	
	private String retrieveErrorMessages(Messages messages) {
		StringBuffer buff = new StringBuffer();
		for (Object item:messages.getErrorMessages()) {
			buff.append(item+";");
		}
		return buff.toString();
	}

	private void sendToErrorPage(HttpServletRequest req, HttpServletResponse res, String icn, String errorDetail) 
		throws ServletException, IOException {
		System.out.println("********** EAuthGatewayServlet.doGet() "+ errorDetail +" for ICN:" + icn);
		if(log.isInfoEnabled()) {
			log.info("********** EAuthGatewayServlet.doGet() "+ errorDetail +" for ICN:" + icn);		
		}
		req.getSession().setAttribute(ERROR_DETAIL, errorDetail);
		res.sendRedirect(error_url);
		return;
	}
	
	private void enterInfoLog(String message, String id) {
		System.out.println("********** EAuthGatewayServlet.doGet() " + message + " for ICN:" + id);
		if (log.isInfoEnabled()) {
			log.info("********** EAuthGatewayServlet.doGet() " + message + " for ICN:" + id);		
		}
	}
	
	private void enterWarningLog(String message) {
		System.out.println("********** EAuthGatewayServlet.doGet() " + message);
		if (log.isWarnEnabled()) {
			log.warn("********** EAuthGatewayServlet.doGet() " + message);		
		}
	}
	
	private String processMiddleName(String middleName) {
		if(isNotFound(middleName)) {
			middleName = "";
		}
		return middleName;
	}
	
	private String processGender(String gender) {
		if(isNotFound(gender)) {
			gender = "";
		}
		return gender;
	}
	
	private String processSsn(String ssn) {
		if(ssn.length()==10) {
			ssn = ssn.substring(1);
		}
		return ssn;
	}
	
	private boolean isUserDeactivated(UserProfile uProfile) {
		if(uProfile.getDeactivationReason() != null) {
			return true;
		}
		return false;
	}
	
	protected void doGet(HttpServletRequest req, HttpServletResponse res)
			throws ServletException, IOException {
//if(true) {
//	res.sendRedirect(upgrade_url);
//	return;
//}
		
		
		if (!"https".equalsIgnoreCase(req.getScheme())) {
			enterWarningLog("Connection not secure");
		}
		
		EAuthLogServiceDelegate elsd = ServiceDelegateFactory.createEAuthLogServiceDelegate();
		String ahash = retrieveTraitFromHeader(req, HASH_ATTR);
		
		String csid = retrieveTraitFromHeaderAndDecode(req, CSID_ATTR);
		String icn = retrieveTraitFromHeader(req, ICN_PARAM);

//if(true) {
//	csid = "DSLOGON";
//}

		if(csid == null) {
			sendToErrorPage(req, res, icn, EMPTY_CSID_MSG);
			return;
		} else {
			if(useExistingDslogonProcess && csid.equalsIgnoreCase(dslogon_value)) {
				useExistingSSO = true;
			}
		}
		
		String firstName = retrieveTraitFromHeader(req, FIRST_NAME);
		String middleName = retrieveTraitFromHeader(req, MIDDLE_NAME);
		
		if(isNotFound(middleName)) {
			middleName = null;
			req.getSession().setAttribute(MIDDLE_NAME, null);
		}
		
		String lastName = retrieveTraitFromHeader(req, LAST_NAME);
		String gender = retrieveTraitFromHeader(req, GENDER);
		String ssn = retrieveTraitFromHeader(req, ID);
		String idType = retrieveTraitFromHeader(req, ID_TYPE);
		String birthDate = retrieveTraitFromHeader(req, BIRTH_DATE);
		String edipiValue = retrieveTraitFromHeader(req, EDIPI_PARAM);
		String userProfileId = retrieveTraitFromHeader(req, USER_PROFILE_ID);
		String cspOnly = retrieveTraitFromHeader(req, CSP_ONLY);
		String deepLink = retrieveTraitFromHeader(req, DEEP_LINK_ATTR);
//if(true) {		
//cspOnly = TRUE_VALUE;
//lastName = "mhvjtestf";
////lastName = "NOT_FOUND";
//firstName = "testfdelete";
//birthDate = "1958-01-30";
//ssn = "7050PORT6";
//idType = "SSN";
//gender = "MALE";
////userProfileId = "1854391";
//}		
		//JAZZ # 143252 - Federated Credential - adding more traits from the VAAFI header
		retrieveTraitFromHeader(req, ADDRESS1);
		retrieveTraitFromHeader(req, CITY);
		retrieveTraitFromHeader(req, STATE);
		retrieveTraitFromHeader(req, POSTAL_CODE);
		retrieveTraitFromHeader(req, COUNTRY);
		retrieveTraitFromHeader(req, EMAIL);

		enterInfoLog("userProfileId:"+userProfileId, icn);
		enterInfoLog("csid:"+csid, icn);
		enterInfoLog("firstName:"+firstName, icn);
		enterInfoLog("middleName:"+middleName, icn);
		enterInfoLog("lastName:"+lastName, icn);
		enterInfoLog("gender:"+gender, icn);
		enterInfoLog("ssn:"+ssn, icn);
		enterInfoLog("idType:"+idType, icn);
		enterInfoLog("birthDate:"+birthDate, icn);
		enterInfoLog("edipiValue:"+edipiValue, icn);
		enterInfoLog("ahash:"+ahash, icn);
		
		//JAZZ Story#39187 US REQ#1	Check if va_eauth_hash has NOT_FOUND OR is Null ; then show error msg
		//Check if va_eauth_hash is 28 characters and ends with a = character:
		//27 character that are in the ranges a-z, A-Z, 0-9 and the special characters + and /.  
		//These are followed by an = character for a total of 28 characters. Example: CpjrdSXMPb2P+YK2VOCV81I/iW0=
		//If above check (va_eauth_hash is 28 characters) fails then show error message
		
		//JAZZ Story #144874 Federated Credential - RemoteID Proofing as part of remote id proofing we do not store hash from IAM anymore.
		boolean ahashAlreadyMapped = false;
		String id = null; 
		
		if(useExistingSSO) {
			enterInfoLog("useExistingSSO", icn);
			
			id = findUserNameByHash(ahash);

			enterInfoLog("userName from Hash table:" + id, icn);
			
			if(id != null) {
				ahashAlreadyMapped = true;
			} else {
				if(ahash.length()>28) {
					ahash = EncodingUtil.decodeXSS(ahash);
				}
			}
		}

//if(true) {
//ahash = "123456789012345678901234567=";
//}
		
		//JAZZ Story #144874 Federated Credential - RemoteID Proofing - If Hash value is not valid, we will error out.
		if (!isEauthHashValid(ahash)) {
			sendToErrorPage(req, res, icn, INVALID_HASH_MSG);
			return;
		}
		
if(true) {
//edipiValue = "3214569870";
}
		if(edipiValue != null) {
			edipiValue = URLDecoder.decode(edipiValue, UNICODE_FORMAT);
		}
		
		String va_eauth_assurancelevel = retrieveTraitFromHeader(req, VA_EAUTH_ASSUR_LEVEL);

//if(true) {
//va_eauth_assurancelevel = "2";
//}
		
		if(va_eauth_assurancelevel == null) {
			sendToErrorPage(req, res, icn, EMPTY_ASSURANCE_LEVEL_MSG);
			return;
		}
		
		enterInfoLog("va_eauth_assurancelevel:"+va_eauth_assurancelevel, icn);
		
		// lookup userid in database
		if(useExistingSSO) {
			EAuthHashServiceDelegate esd = ServiceDelegateFactory.createEAuthHashServiceDelegate();
			StringServiceResponse ssr = esd.findUserNameByHash(ahash);
			id = ssr.getString();
			
			enterInfoLog("Hash associated username:"+id, icn);
			
			//Check if the account is deactivated
			if(null != id) {
				if(isDeactivated(id)) {
					sendToErrorPage(req, res, icn, DEACTIVATED_ACCOUNT_MSG);
					return;
				}
			}
		}	
		// Retire ORC Logon. Redirect to custom page.
		if(csid.equalsIgnoreCase(ORC_CSP)) {
			enterInfoLog("Sending to Anonymous Page for CSID:2030004", icn);
			res.sendRedirect(anonymous_url);
	        return;
		}
		
//TODO - Delete this code if not applicable		
// Throw CAC to Anonymous portal
//		if(csid.equalsIgnoreCase("CAC"))
//			res.sendRedirect(anonymous_url);

		UserProfile uProfile = null;
		PatientServiceResponse mviPatRes = null;
		Patient patient = null;
		boolean cancelUpgrade = false;
		
		// If va_eauth_assurancelevel is not 2 send them to anonymous url
		if(Integer.parseInt(va_eauth_assurancelevel) < Integer.parseInt(va_eauth_assur_prop_level)) {
			sendToErrorPage(req, res, icn, INVALID_ASSURANCE_LEVEL_MSG);
			return;
		} else {
			enterInfoLog("va_eauth_assurancelevel" + va_eauth_assurancelevel, icn);
			
if(useExistingSSO) {
			enterInfoLog("use existing DSLOGON Process for ICN:", icn);
			
			//JAZZ#32762 US01 REQ#1 When a user maps their DS Logon LOA 2 or CAC LOA 3 or greater credential 
			//to their MHV account during their SSO VAAFI session from eBenefits or AccessVA, MHV system 
			//shall make following checks --- we are checking leve 2 or higher
			//TODO: The User if mapped in MHV pulling the userProfile & patient
			if(id != null) {
				UserProfileServiceResponse userProfileSerRes = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserProfileServiceDelegate().getProfileForUser(id);
				uProfile = userProfileSerRes.getUserProfile();
				if (uProfile != null) {
					mviPatRes = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createPatientServiceDelegate().getPatientForUser(uProfile);
					patient = mviPatRes.getPatient();
				}
			} else {
				boolean checkEdipi = false;
				//Jazz #110471 RemoteID Proofing - REQ#1 - For SSO user, MHV system shall extract ICN from the VAAFI header
				icn = retrieveICN(req);
				//Jazz #110471 RemoteID Proofing - REQ#1 - and send it to MVI with getCorrespondingID call after making following validation checks:
				if(isICNAvailable(icn)) {
					if(!isICNValid(icn)) {
						sendToErrorPage(req, res, icn, INVALID_ICN_MSG);
						return;
					}
					mviPatRes = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createMviIntegrationServiceDelegate().searchPersonInMVIByICN(icn);
					
					req.getSession().removeAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB);
					if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().equals(MVI_COMM_ISSUE_VALUE)) {
						res.sendRedirect(mvi_comm_error_url);
						return;
					} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().equals(MviConstants.MULTIPLE_IDS_ERROR)) {
						//JAZZ Story#39187 - US REQ#2.0 When MHV system checks to see if the user has one active advanced account known to MVI, 
						//in the event multiple accounts are returned from MVI, the system shall display a user-friendly error message in a 
						//yellow box at the top of We Cannot Connect Your Account page.  
						req.getSession().setAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB, TRUE_VALUE);
						sendToErrorPage(req, res, icn, MULTIPLE_ACCOUNT_ERROR_MSG);
						return;
					} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().toString().startsWith(ERROR_VALUE)) {
						//JAZZ Story#39187 - US REQ#2.0 When MHV system checks to see if the user has one active advanced account known to MVI, 
						//in the event multiple accounts are returned from MVI, the system shall display a user-friendly error message in a 
						//yellow box at the top of We Cannot Connect Your Account page.  
						sendToErrorPage(req, res, icn, mviPatRes.getValue().toString());
						return;
					}
					
					if (mviPatRes != null && mviPatRes.getPatient() != null) {
						patient = mviPatRes.getPatient();
					}
					
					//NOTE: After the final discussion with Carnetta, we are not checking the ICN in our database but checking with MVI. 
					//Jazz #110471 RemoteID Proofing - REQ#1 - If ICN is valid, then MHV should check in our DB and see if user exists with this ICN
//					PatientServiceResponse patRes = 
//						gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createPatientServiceDelegate().getPatientByIcn(icn);
//					patient = patRes.getPatient();
					
					//Jazz #110471 RemoteID Proofing - REQ#1 - i.	If ICN is not found in MHV DB then MHV to check for EDIPI
					if(patient == null) {
						checkEdipi = true;
					} else if(patient.getUserProfile().getDeactivationReason()!=null) {
						//Jazz #110471 RemoteID Proofing - REQ#1 - For inactive accounts, throw existing error message.
						sendToErrorPage(req, res, icn, DEACTIVATED_ACCOUNT_MSG);
						return;
					}
					
				//JAZZ Story#39187 - US REQ#3.0 For the first time SSO user coming to MHV using any credential, MHV system shall use the EDIPI value if it exists or 
				//the user traits (firstName, middleName, lastName, birthdate, gender, ssn) to search in MVI.
				//Check if EDIPI is 10 numeric characters, if not then dont use the EDIPI and check user traits in VAAFI header.
			    } else {
			    	//Jazz #110471 RemoteID Proofing - REQ#1 - If ICN does not exists then check for EDIPI
			    	checkEdipi = true;
			    }
				
				if(checkEdipi) {
					if(isEdipiValid(edipiValue)) {
						mviPatRes = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createMviIntegrationServiceDelegate().searchPersonInMVIForDSLogon(edipiValue);
					} else {
						//If  "NOT_FOUND" is sent in "first name", "last name" , "birth date", "ssn" and "gender" , MHV will show error msg 
						if(isNotFound(firstName) || isNotFound(lastName) || isNotFound(birthDate) || isNotFound(ssn)) {
							sendToErrorPage(req, res, icn, NOT_FOUND_ERROR_MSG);
							return;
						}
						
						middleName = processMiddleName(middleName);	
						gender = processGender(gender);	
						ssn = processSsn(ssn);	
						
						//If SSN is not 9 digit, MHV will show error msg 
						if(!isSsnValid(ssn, idType)) {
							sendToErrorPage(req, res, icn, INVALID_SSN_MSG);
							return;
						}
						
						//birth date is not in Date format, MHV will show error msg  
						if(!isDate(birthDate)) {
							sendToErrorPage(req, res, icn, INVALID_DOB_MSG);
							return;
						}
						
						mviPatRes = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createMviIntegrationServiceDelegate().
							searchPersonInMVIForUSAA(firstName, middleName, lastName, birthDate, gender, ssn);
					}
				
					req.getSession().removeAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB);
					if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().equals(MVI_COMM_ISSUE_VALUE)) {
						res.sendRedirect(mvi_comm_error_url);
						return;
					} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().equals(MviConstants.MULTIPLE_IDS_ERROR)) {
						//JAZZ Story#39187 - US REQ#2.0 When MHV system checks to see if the user has one active advanced account known to MVI, 
						//in the event multiple accounts are returned from MVI, the system shall display a user-friendly error message in a 
						//yellow box at the top of We Cannot Connect Your Account page.  
						req.getSession().setAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB, TRUE_VALUE);
						sendToErrorPage(req, res, icn, MULTIPLE_ACCOUNT_ERROR_MSG);
						return;
					}
					
					if (mviPatRes != null && mviPatRes.getPatient() != null) {
						patient = mviPatRes.getPatient();
					}
				}
				
				if(patient != null) {
					uProfile = patient.getUserProfile();
					
					if(isDeactivated(uProfile.getUserName())) {
						sendToErrorPage(req, res, icn, DEACTIVATED_ACCOUNT_MSG);
						return;
					}
					
					//In the event MVI returns a different SSN value, MHV will show error msg.
					if(!ssn.equals(uProfile.getSsn().replaceAll("[\\s\\-()]", ""))) {
						sendToErrorPage(req, res, icn, INVALID_SSN_MSG);
						return;
					}
					
     				//Jazz #110471 US REQ#3 RemoteID Proofing - For the users who want to map their account, MHV system shall display 
					//updated content for the MHV Connect Page where user can select to go to MHV
					if(useExistingSSO && !ahashAlreadyMapped) {
						//Jazz #110471 US REQ#3 RemoteID Proofing - If the Correlation status = Pending Uncorrelation/Correlation Failed 
						//then throw existing 12.9 error message.
						enterInfoLog("Correlation Status:" + patient.getCorrelationStatus().getName(), icn);
				    	if(patient.getCorrelationStatus().equals(PatientCorrelationStatusUtils.PENDING_UNCORRELATION)||
				    	   patient.getCorrelationStatus().equals(PatientCorrelationStatusUtils.FAILED_CORRELATION)) {
							sendToErrorPage(req, res, icn, "Pending Uncorrelation or Failed Correlation Status");
							return;
				    	}
				    	
				    	//Jazz #110471 US REQ#3 RemoteID Proofing - b.	Then check for Matched Date:
				    	//i.	If there is no Match Date, MHV will match the record and set the match date to todays date (Note for Dev: use the code in admin portal for match button)
				    	//ii.	If matching fails MHV will display the 12.9 error message to the user.
				    	if(patient.getMatchedDateTime() == null) {
							enterInfoLog("Processed MVI Match for ICN:", icn);
				    		MviIntegrationServiceDelegate mviSyncServiceDelegate = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createMviIntegrationServiceDelegate();
				    		PatientServiceResponse response = mviSyncServiceDelegate.matchPersonInIPA(uProfile);
				    		patient = response.getPatient();
				    		if(response.getMessages().hasErrorMessages()) {
								sendToErrorPage(req, res, icn, retrieveErrorMessages(response.getMessages()));
								return;
				    		}
							enterInfoLog("MVI Match successful for ICN:", icn);
				    	}
				    	
				    	//Jazz #110471 US REQ#3 RemoteID Proofing - c.	Then check  for Correlated Status
				    	//i.	If there is no Correlation (Correlation status = UnCorrelated), MHV will correlate the user (Note for Dev: use the code in admin portal for Correlate button)
				    	//ii.	If correlation fails, MHV shall put the record in failed correlation state and user will see the existing 12.9 error message. 
				    	if(patient.getCorrelationStatus() == PatientCorrelationStatus.getEnum(PatientCorrelationStatus.UNCORRELATED) ||
				    	   patient.getCorrelationStatus() == PatientCorrelationStatus.getEnum(PatientCorrelationStatus.MVIDATAMATCH)) {
							enterInfoLog("Processed MVI Correlation for ICN:", icn);
				    		MviIntegrationServiceDelegate mviSyncServiceDelegate = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createMviIntegrationServiceDelegate();		
				    		PatientServiceResponse mviSyncServiceResponse = 
				    			mviSyncServiceDelegate.addPersonForCorrelationForIPA(patient, patient.getUserProfile(), "System");
				    		
				    		patient = mviSyncServiceResponse.getPatient();
				    		uProfile = patient.getUserProfile();
				    		
				    		if(mviSyncServiceResponse.getMessages().hasErrorMessages()) {
								sendToErrorPage(req, res, icn, retrieveErrorMessages(mviSyncServiceResponse.getMessages()));
								return;
				    		}
							enterInfoLog("MVI Correlation successful", icn);
				    	}
				    	
						req.getSession().removeAttribute(DISPLAY_ERROR);
						req.getSession().setAttribute(USER_NAME, uProfile.getUserName());
						req.getSession().removeAttribute("displayUpdateButton");
						res.sendRedirect(login_url);
						return;
					}
				}
			}
			
} else {
			
			//JAZZ Story #144874 Federated Credential - RemoteID Proofing - Check to see if IAM is providing CSP traits or Full PS traits
			if(cspOnly != null && cspOnly.equalsIgnoreCase(TRUE_VALUE)) {
				enterInfoLog("CSPONLY process", icn);
				
				//JAZZ Story #144874 Federated Credential - RemoteID Proofing - CSP Only Traits
				if(isEdipiExist(edipiValue)) {
					if(!isEdipiValid(edipiValue)) {
						sendToErrorPage(req, res, icn, "Invalid EDIPI");
						return;
					}
					
					//JAZZ Story #144874 Federated Credential - RemoteID Proofing - EDIPI is provided and valid
					//If the record is found, check to see if MHV IEN is provided.
					//If not provided, 
					mviPatRes = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createMviIntegrationServiceDelegate().searchPersonInMVIForDSLogon(edipiValue);
					req.getSession().removeAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB);
					if(mviPatRes.getMessages().hasErrorMessages()) {
						sendToErrorPage(req, res, icn, retrieveErrorMessages(mviPatRes.getMessages()));
						return;
					} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().equals(MVI_COMM_ISSUE_VALUE)) {
						res.sendRedirect(mvi_comm_error_url);
						return;
					} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().equals(MviConstants.MULTIPLE_IDS_ERROR)) {
						//JAZZ Story#39187 - US REQ#2.0 When MHV system checks to see if the user has one active advanced account known to MVI, 
						//in the event multiple accounts are returned from MVI, the system shall display a user-friendly error message in a 
						//yellow box at the top of We Cannot Connect Your Account page.  
						req.getSession().setAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB, TRUE_VALUE);
						sendToErrorPage(req, res, icn, MULTIPLE_ACCOUNT_ERROR_MSG);
						return;
					} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().toString().startsWith(ERROR_VALUE)) {
						//JAZZ Story#39187 - US REQ#2.0 When MHV system checks to see if the user has one active advanced account known to MVI, 
						//in the event multiple accounts are returned from MVI, the system shall display a user-friendly error message in a 
						//yellow box at the top of We Cannot Connect Your Account page.  
						sendToErrorPage(req, res, icn, mviPatRes.getValue().toString());
						return;
					}
					
					if(mviPatRes.getPatient() != null) {
						patient = mviPatRes.getPatient();
						uProfile = patient.getUserProfile();
						if(uProfile.getDeactivationReason()!=null) {
							//JAZZ Story #144874 Federated Credential - RemoteID Proofing - display an error message to the user indicating
							//MHV account is inactive and to call the helpdesk for assistance.
							sendToErrorPage(req, res, icn, DEACTIVATED_ACCOUNT_MSG);
							return;
						}
						processFacilitiesUpdate(patient);
					} else {
						//If  "NOT_FOUND" is sent in "first name", "last name" , "birth date", "ssn" and "gender" , MHV will show error msg 
						if(isNotFound(firstName) || isNotFound(lastName) || isNotFound(birthDate) || isNotFound(ssn)) {
							sendToErrorPage(req, res, icn, NOT_FOUND_ERROR_MSG);
							return;
						}
						
						middleName = processMiddleName(middleName);
						gender = processGender(gender);
						ssn = processSsn(ssn);
						
						//If SSN is not 9 digit, MHV will show error msg 
						if(!isSsnValid(ssn, idType)) {
							sendToErrorPage(req, res, icn, INVALID_SSN_MSG);
							return;
						}
						
						//birth date is not in Date format, MHV will show error msg  
						if(!isDate(birthDate)) {
							sendToErrorPage(req, res, icn, INVALID_DOB_MSG);
							return;
						}
						
						//JAZZ Story #144874 Federated Credential - RemoteID Proofing - Check MHV for existing accoutns with the same
						//first name, last name, SSN, and DOB.
						enterInfoLog("Finding the user with traits in MHV", icn);
						uProfile = findUserProfile(firstName, lastName, gender, ssn, birthDate);
						if(uProfile != null) {
							if(uProfile.getDeactivationReason()!=null) {
								//JAZZ Story #144874 Federated Credential - RemoteID Proofing - display an error message to the user indicating
								//MHV account is inactive and to call the helpdesk for assistance.
								sendToErrorPage(req, res, icn, DEACTIVATED_ACCOUNT_MSG);
								return;
							}
							enterInfoLog("Found an active user with traits in MHV", icn);
							//JAZZ Story #144874 Federated Credential - RemoteID Proofing - Notify User Existing MHV Account with
							//their Personal Data Already exists
							req.getSession().removeAttribute("displayUpdateButton");
							req.setAttribute("removeTabs", "true");
							res.sendRedirect(login_url);
							return;
						} else {
							if(isICNValid(icn)) {
								PatientServiceResponse patRes = 
									gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createPatientServiceDelegate().getPatientByIcn(icn);
								patient = patRes.getPatient();
								if(patient != null) {
									uProfile = patient.getUserProfile();
									req.getSession().removeAttribute("displayUpdateButton");
									res.sendRedirect(login_url);
									return;
								}
							}
 						}
						
						enterInfoLog("Calling get corresponding id to MVI with traits", icn);
						mviPatRes = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createMviIntegrationServiceDelegate().
							searchPersonInMVIForUSAA(firstName, middleName, lastName, birthDate, gender, ssn);
					
						req.getSession().removeAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB);
						if(mviPatRes.getMessages().hasErrorMessages()) {
							sendToErrorPage(req, res, icn, retrieveErrorMessages(mviPatRes.getMessages()));
							return;
						} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().equals(MVI_COMM_ISSUE_VALUE)) {
							res.sendRedirect(mvi_comm_error_url);
							return;
						} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().equals(MviConstants.MULTIPLE_IDS_ERROR)) {
							//JAZZ Story#39187 - US REQ#2.0 When MHV system checks to see if the user has one active advanced account known to MVI, 
							//in the event multiple accounts are returned from MVI, the system shall display a user-friendly error message in a 
							//yellow box at the top of We Cannot Connect Your Account page.  
							req.getSession().setAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB, TRUE_VALUE);
							sendToErrorPage(req, res, icn, MULTIPLE_ACCOUNT_ERROR_MSG);
							return;
						} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().toString().startsWith(ERROR_VALUE)) {
							//JAZZ Story#39187 - US REQ#2.0 When MHV system checks to see if the user has one active advanced account known to MVI, 
							//in the event multiple accounts are returned from MVI, the system shall display a user-friendly error message in a 
							//yellow box at the top of We Cannot Connect Your Account page.  
							req.getSession().setAttribute(ERROR_DETAIL, mviPatRes.getValue().toString());
							res.sendRedirect(error_url);
							return;
						}
						
						if(icn != null) {
							if(gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createMviIntegrationServiceDelegate().treatmentFacilitiesExist(icn).getBoolean()) {
								req.getSession().setAttribute("treatingFaciltiesExist", TRUE_VALUE);
							}
						}
						
						req.getSession().setAttribute(FEDERATED_CREDENTIAL_ATTRIB, TRUE_VALUE);
						req.getSession().setAttribute("displayUpdateButton", TRUE_VALUE);
						res.sendRedirect(login_url);
						return;
					}
				} else {
					//If  "NOT_FOUND" is sent in "first name", "last name" , "birth date", "ssn" and "gender" , MHV will show error msg 
					if(isNotFound(firstName) || isNotFound(lastName) || isNotFound(birthDate) || isNotFound(ssn)) {
						sendToErrorPage(req, res, icn, NOT_FOUND_ERROR_MSG);
						return;
					}
					
					processMiddleName(middleName);
					processGender(gender);
					processSsn(ssn);
					
					//If SSN is not 9 digit, MHV will show error msg 
					if(!isSsnValid(ssn, idType)) {
						sendToErrorPage(req, res, icn, INVALID_SSN_MSG);
						return;
					}
					
					//birth date is not in Date format, MHV will show error msg  
					if(!isDate(birthDate)) {
						sendToErrorPage(req, res, icn, INVALID_DOB_MSG);
						return;
					}
					
					//JAZZ Story #144874 Federated Credential - RemoteID Proofing - Initial MVI GetCorresponding IDs
					mviPatRes = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createMviIntegrationServiceDelegate().
						searchPersonInMVIForUSAA(firstName, middleName, lastName, birthDate, gender, ssn);
					
					req.getSession().removeAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB);
					if(mviPatRes.getMessages().hasErrorMessages()) {
						sendToErrorPage(req, res, icn, retrieveErrorMessages(mviPatRes.getMessages()));
						return;
					} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().equals(MVI_COMM_ISSUE_VALUE)) {
						res.sendRedirect(mvi_comm_error_url);
						return;
					} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().equals(MviConstants.MULTIPLE_IDS_ERROR)) {
						//JAZZ Story#39187 - US REQ#2.0 When MHV system checks to see if the user has one active advanced account known to MVI, 
						//in the event multiple accounts are returned from MVI, the system shall display a user-friendly error message in a 
						//yellow box at the top of We Cannot Connect Your Account page.  
						req.getSession().setAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB, TRUE_VALUE);
						sendToErrorPage(req, res, icn, MULTIPLE_ACCOUNT_ERROR_MSG);
						return;
					} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().toString().startsWith(ERROR_VALUE)) {
						//JAZZ Story#39187 - US REQ#2.0 When MHV system checks to see if the user has one active advanced account known to MVI, 
						//in the event multiple accounts are returned from MVI, the system shall display a user-friendly error message in a 
						//yellow box at the top of We Cannot Connect Your Account page.  
						sendToErrorPage(req, res, icn, mviPatRes.getValue().toString());
						return;
					}
					
					if(mviPatRes.getPatient() != null) {
						patient = mviPatRes.getPatient();
						//uProfile = patient.getUserProfile();
						if(isUserDeactivated(uProfile)) {
							sendToErrorPage(req, res, icn, DEACTIVATED_ACCOUNT_MSG);
							return;
						}
						processFacilitiesUpdate(patient);
					} else {
						//JAZZ Story #144874 Federated Credential - RemoteID Proofing - Check MHV for existing accoutns with the same
						//first name, last name, SSN, and DOB.
						uProfile = findUserProfile(firstName, lastName, gender, ssn, birthDate);
						if(uProfile != null) {
							if(isUserDeactivated(uProfile)) {
								//JAZZ Story #144874 Federated Credential - RemoteID Proofing - display an error message to the user indicating
								//MHV account is inactive and to call the helpdesk for assistance.
								sendToErrorPage(req, res, icn, DEACTIVATED_ACCOUNT_MSG);
								return;
							}
							//JAZZ Story #144874 Federated Credential - RemoteID Proofing - Notify User Existing MHV Account with
							//their Personal Data Already exists
							req.getSession().removeAttribute("displayUpdateButton");
							res.sendRedirect(login_url);
							return;
						} else {
							if(isICNValid(icn)) {
								PatientServiceResponse patRes = 
									gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createPatientServiceDelegate().getPatientByIcn(icn);
								patient = patRes.getPatient();
								if(patient != null) {
									uProfile = patient.getUserProfile();
									req.getSession().removeAttribute("displayUpdateButton");
									res.sendRedirect(login_url);
									return;
								}
							}
 						}
						
						mviPatRes = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createMviIntegrationServiceDelegate().
							searchPersonInMVIForUSAA(firstName, middleName, lastName, birthDate, gender, ssn);
					
						req.getSession().removeAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB);
						if(mviPatRes.getMessages().hasErrorMessages()) {
							sendToErrorPage(req, res, icn, retrieveErrorMessages(mviPatRes.getMessages()));
							return;
						} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().equals(MVI_COMM_ISSUE_VALUE)) {
							res.sendRedirect(mvi_comm_error_url);
							return;
						} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().equals(MviConstants.MULTIPLE_IDS_ERROR)) {
							//JAZZ Story#39187 - US REQ#2.0 When MHV system checks to see if the user has one active advanced account known to MVI, 
							//in the event multiple accounts are returned from MVI, the system shall display a user-friendly error message in a 
							//yellow box at the top of We Cannot Connect Your Account page.  
							req.getSession().setAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB, TRUE_VALUE);
							sendToErrorPage(req, res, icn, MULTIPLE_ACCOUNT_ERROR_MSG);
							return;
						} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().toString().startsWith(ERROR_VALUE)) {
							//JAZZ Story#39187 - US REQ#2.0 When MHV system checks to see if the user has one active advanced account known to MVI, 
							//in the event multiple accounts are returned from MVI, the system shall display a user-friendly error message in a 
							//yellow box at the top of We Cannot Connect Your Account page.  
							sendToErrorPage(req, res, icn, mviPatRes.getValue().toString());
							return;
						}
						
						req.getSession().setAttribute(FEDERATED_CREDENTIAL_ATTRIB, TRUE_VALUE);
						req.getSession().setAttribute("displayUpdateButton", TRUE_VALUE);
						res.sendRedirect(login_url);
						return;
					}
				}
			} else {
				try {
					//JAZZ Story #144874 Federated Credential - RemoteID Proofing - Check to see IAM is providing Full PS traits
					String[] uProfileIds = userProfileId.split(",");
					//JAZZ Story #144874 Federated Credential - RemoteID Proofing - If multiple user profile ids are provided, go to error.
					if(uProfileIds != null && uProfileIds.length > 1) {
						req.getSession().setAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB, TRUE_VALUE);
						sendToErrorPage(req, res, icn, MULTIPLE_ACCOUNT_ERROR_MSG);
						return;
					}
				} catch (Throwable t) {
					uProfile = null;
				}
				
				try {
					Long uProfileId = null;
					
					try {
						uProfileId = Long.parseLong(userProfileId);
						UserProfileServiceResponse userProfileSerRes = 
							gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserProfileServiceDelegate().getUserProfileById(
									uProfileId);
						uProfile = userProfileSerRes.getUserProfile();
					} catch (Throwable t) {
						uProfile = null;
					}
					
					if (uProfile != null) {
						if(uProfile.getDeactivationReason() != null) {
							sendToErrorPage(req, res, icn, DEACTIVATED_ACCOUNT_MSG);
							return;
						}
						
						mviPatRes = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createPatientServiceDelegate().getPatientForUser(uProfile);
						patient = mviPatRes.getPatient();
						processFacilitiesUpdate(patient);
					} else {
						//If  "NOT_FOUND" is sent in "first name", "last name" , "birth date", "ssn" and "gender" , MHV will show error msg 
						if(isNotFound(firstName) || isNotFound(lastName) || isNotFound(birthDate) || isNotFound(ssn)) {
							sendToErrorPage(req, res, icn, NOT_FOUND_ERROR_MSG);
							return;
						}
						
						middleName = processMiddleName(middleName);
						gender = processGender(gender);
						ssn = processSsn(ssn);
						
						//If SSN is not 9 digit, MHV will show error msg 
						if(!isSsnValid(ssn, idType)) {
							sendToErrorPage(req, res, icn, INVALID_SSN_MSG);
							return;
						}
						
						//birth date is not in Date format, MHV will show error msg  
						if(!isDate(birthDate)) {
							sendToErrorPage(req, res, icn, INVALID_DOB_MSG);
							return;
						}
						
						//JAZZ Story #144874 Federated Credential - RemoteID Proofing - Check for existing account with the user traits
						uProfile = findUserProfile(firstName, lastName, gender, ssn, birthDate);
						if(uProfile != null) {
							if(isUserDeactivated(uProfile)) {
								//JAZZ Story #144874 Federated Credential - RemoteID Proofing - display an error message to the user indicating
								//MHV account is inactive and to call the helpdesk for assistance.
								sendToErrorPage(req, res, icn, DEACTIVATED_ACCOUNT_MSG);
								return;
							}
							//JAZZ Story #144874 Federated Credential - RemoteID Proofing - Notify User Existing MHV Account with
							//their Personal Data Already exists
							req.getSession().removeAttribute("displayUpdateButton");
							res.sendRedirect(login_url);
							return;
						} else {
							if(isICNValid(icn)) {
								PatientServiceResponse patRes = 
									gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createPatientServiceDelegate().getPatientByIcn(icn);
								patient = patRes.getPatient();
								if(patient != null) {
									uProfile = patient.getUserProfile();
									req.getSession().removeAttribute("displayUpdateButton");
									res.sendRedirect(login_url);
									return;
								}
							}
 						}
						
						mviPatRes = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createMviIntegrationServiceDelegate().
							searchPersonInMVIForUSAA(firstName, middleName, lastName, birthDate, gender, ssn);
					
						req.getSession().removeAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB);
						if(mviPatRes.getMessages().hasErrorMessages()) {
							sendToErrorPage(req, res, icn, retrieveErrorMessages(mviPatRes.getMessages()));
							return;
						} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().equals(MVI_COMM_ISSUE_VALUE)) {
							res.sendRedirect(mvi_comm_error_url);
							return;
						} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().equals(MviConstants.MULTIPLE_IDS_ERROR)) {
							//JAZZ Story#39187 - US REQ#2.0 When MHV system checks to see if the user has one active advanced account known to MVI, 
							//in the event multiple accounts are returned from MVI, the system shall display a user-friendly error message in a 
							//yellow box at the top of We Cannot Connect Your Account page.  
							req.getSession().setAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB, TRUE_VALUE);
							sendToErrorPage(req, res, icn, MULTIPLE_ACCOUNT_ERROR_MSG);
							return;
						} else if(mviPatRes != null && mviPatRes.getValue() != null && mviPatRes.getValue().toString().startsWith(ERROR_VALUE)) {
							//JAZZ Story#39187 - US REQ#2.0 When MHV system checks to see if the user has one active advanced account known to MVI, 
							//in the event multiple accounts are returned from MVI, the system shall display a user-friendly error message in a 
							//yellow box at the top of We Cannot Connect Your Account page.  
							sendToErrorPage(req, res, icn, mviPatRes.getValue().toString());
							return;
						}
						
						req.getSession().setAttribute(FEDERATED_CREDENTIAL_ATTRIB, TRUE_VALUE);
						req.getSession().setAttribute("displayUpdateButton", TRUE_VALUE);
						res.sendRedirect(login_url);
						return;
					}
				} catch (Throwable t) {
					sendToErrorPage(req, res, icn, t.getMessage());
					return;
				}
			}
			
}

			if(uProfile == null){
				enterInfoLog("User not found", icn);
				res.sendRedirect(anonymous_url);
				return;
			} else {
				id = uProfile.getUserName();
				
				if(uProfile.getDeactivationReason() != null) {
					sendToErrorPage(req, res, icn, DEACTIVATED_ACCOUNT_MSG);
					return;
				} else {
					
					//JAZZ#32762 US01 REQ#1.1 MHV system shall check the user is a VA Patient and/or Veteran and has a correlated MHV Advanced account.
					if(patient != null) {
						id = patient.getUserProfile().getUserName();
						if(hasERarAccess(uProfile.getUserName())) {
							req.getSession().setAttribute(HAS_ERAR_ACCESS_ATTRIB, TRUE_VALUE);
						} else {
							req.getSession().setAttribute(HAS_ERAR_ACCESS_ATTRIB, FALSE_VALUE);
						}
						
						String upgradeOptionSelected = req.getParameter(UPGRADE_OPTION_SELECTED);
						
						if(upgradeOptionSelected!=null && upgradeOptionSelected.equalsIgnoreCase(AUTO_OPTION_VALUE)) {
							InPersonAuthenticationServiceDelegate ipaDelegate = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createInPersonAuthenticationServiceDelegate();
							InPersonAuthenticationServiceResponse ipaServiceResponse = ipaDelegate.getAuthenticationForPatient(patient.getId());
							
							InPersonAuthentication ipa = ipaServiceResponse.getInPersonAuthentication();
							//JAZZ#32762 US02 REQ#2.1 The MHV system shall set Prerequisite fields: "User has presented the required 
							//identification check box.	
							ipa.setIdentificationPresented(Boolean.TRUE);
							ipa.setParticipationFormSigned(Boolean.FALSE);
							ipa.setVideoViewed(Boolean.TRUE);

							//JAZZ#32762 US02 REQ#2.2 MHV System shall set the Authenticated Facility field to AUSTIN MHV.											
							ipa.setAuthenticatingFacility(getAustinMHVFacilityInfo());
							
							//JAZZ#32762 US02 REQ#2.3 The MHV system shall set the Reason for Delay field as "PREREQUISITESINCOMPLETE".
							AuthenticationDefermentReason reason = AuthenticationDefermentReason.getEnum(AuthenticationDefermentReason.PREREQUISITESINCOMPLETE);
							ipa.setDefermentReason(reason);
							
							//JAZZ#32762 US02 REQ#2.4 The MHV system shall set the status of this account to In Process											
							ipa.setStatus(AuthenticationStatus.getEnum(AuthenticationStatus.INPROCESS));
							
							//JAZZ#32762 US02 REQ#2.5 The MHV system shall set the NEW field Date/Time Start upgrade online with the current date/time											
							ipa.setStartUpgradeOnlineDateTime(new Timestamp(new Date().getTime()));
				            ipa.setApprovedForRecordsAccess(Boolean.FALSE);

							ipaDelegate.saveIPA(ipa);
							
							setVapiiFormUrlInSession(req);
							
							//JAZZ#32762 US02 REQ#12 The MHV System shall create an AAL entry for the in process action when a user selects Yes to start the upgrade process.   
							Auditor.auditOnlineAuthenticationEvent(uProfile.getId(), true, ActivityTypeEnumeration.ACCOUNT_UPGRADE, 
								ActivityActionTypeEnumeration.START_UPGRADE_ONLINE, "");  
							
							
							
							
							StringServiceResponse vapiiStatusResponse = 
								gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserProfileServiceDelegate().getVapiiFormStatusForUser(
								uProfile);
							if(vapiiStatusResponse.getMessages().hasErrorMessages()) {
								enterInfoLog("Error Retrieving Fomr Status:" + retrieveErrorMessages(vapiiStatusResponse.getMessages()), icn);
								req.getSession().setAttribute(DISPLAY_ERROR_ATTRIB, VAPII_ERRIR_VALUE);
								res.sendRedirect(upgrade_url);
						        return;
							} else {
								enterInfoLog("No Error, taking to Online Form page", icn);
								req.getSession().removeAttribute(DISPLAY_ERROR_ATTRIB);
								res.sendRedirect(auto_upgrade_url);
						        return;
							}
						}else if(upgradeOptionSelected!=null && upgradeOptionSelected.equalsIgnoreCase("manual")) {
							res.sendRedirect(manual_upgrade_url);
					        return;
						}else if(upgradeOptionSelected!=null && upgradeOptionSelected.equalsIgnoreCase("cancel")) {
							cancelUpgrade = true;
						}else if(upgradeOptionSelected!=null && upgradeOptionSelected.equalsIgnoreCase("home")) {
							res.sendRedirect(upgrade_url);
					        return;
						}

						enterInfoLog("Found User: " + uProfile.getId() + " is a patient/veteran", icn);
//						PatientServiceDelegate delegate = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createPatientServiceDelegate();
//						patient = delegate.getPatientForUser(uProfile).getPatient();

						InPersonAuthenticationServiceDelegate ipaDelegate = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createInPersonAuthenticationServiceDelegate();
						InPersonAuthenticationServiceResponse ipaServiceResponse = ipaDelegate.getAuthenticationForPatient(patient.getId());

						if (ipaServiceResponse.getMessages().hasErrorMessages()) {
					          for (Object item:ipaServiceResponse.getMessages().getErrorMessages()) {
							    log.error("IPA find action Error: " + item);
					          }
					          res.sendRedirect(anonymous_url);
					          return;
					    }

						AuthenticationStatus authStatus =ipaServiceResponse.getInPersonAuthentication().getStatus();
					    //JAZZ#32762 US01 REQ#1.2.1 For Authenticated Users, MHV system shall place the user on the MHV home with user logged in.	
						if(AuthenticationStatus.getEnum(AuthenticationStatus.AUTHENTICATED) == authStatus) {
							enterInfoLog("the username:" + uProfile.getUserName()+" has the authenticated status", icn);
							if( null == id ) {
								id = populateEauthHash(uProfile, ahash, csid);
							}
						} else if(AuthenticationStatus.getEnum(AuthenticationStatus.UNAUTHENTICATED) == authStatus ||
							   //JAZZ#32762 US01 REQ#1.2 MHV system shall check the users Authentication status 
							   //is either Unauthenticated or In Process or Pending Data Mismatch.	
							   AuthenticationStatus.getEnum(AuthenticationStatus.INPROCESS) == authStatus ||
							   AuthenticationStatus.getEnum(AuthenticationStatus.PENDINGDATAMISMATCH) == authStatus) {
									
									enterInfoLog("The user authStatus:" + authStatus, icn);
									
									yesNoUpgrade = req.getParameter("doUpgrade");
									
									if(null != yesNoUpgrade && yesNoUpgrade.equalsIgnoreCase("autostart")){
									InPersonAuthentication ipa = ipaServiceResponse.getInPersonAuthentication();
									//JAZZ#32762 US02 REQ#2.1 The MHV system shall set Prerequisite fields: "User has presented the required 
									//identification check box.	
									ipa.setIdentificationPresented(Boolean.TRUE);
									ipa.setParticipationFormSigned(Boolean.FALSE);
									ipa.setVideoViewed(Boolean.TRUE);

									//JAZZ#32762 US02 REQ#2.2 MHV System shall set the Authenticated Facility field to AUSTIN MHV.											
									ipa.setAuthenticatingFacility(getAustinMHVFacilityInfo());
									
									//JAZZ#32762 US02 REQ#2.3 The MHV system shall set the Reason for Delay field as "PREREQUISITESINCOMPLETE".
									AuthenticationDefermentReason reason = AuthenticationDefermentReason.getEnum(AuthenticationDefermentReason.PREREQUISITESINCOMPLETE);
									ipa.setDefermentReason(reason);
									
									//JAZZ#32762 US02 REQ#2.4 The MHV system shall set the status of this account to In Process											
									ipa.setStatus(AuthenticationStatus.getEnum(AuthenticationStatus.INPROCESS));
									
									//JAZZ#32762 US02 REQ#2.5 The MHV system shall set the NEW field Date/Time Start upgrade online with the current date/time											
									ipa.setStartUpgradeOnlineDateTime(new Timestamp(new Date().getTime()));
						            ipa.setApprovedForRecordsAccess(Boolean.FALSE);

									ipaDelegate.saveIPA(ipa);
									
									setVapiiFormUrlInSession(req);
									
									//JAZZ#32762 US02 REQ#12 The MHV System shall create an AAL entry for the in process action when a user selects Yes to start the upgrade process.   
									Auditor.auditOnlineAuthenticationEvent(uProfile.getId(), true, ActivityTypeEnumeration.ACCOUNT_UPGRADE, 
										ActivityActionTypeEnumeration.START_UPGRADE_ONLINE, "");  

									//JAZZ#32762 US02 REQ#3 The MHV System shall place the user on the VAPii page.
									res.sendRedirect(auto_upgrade_vapii_url);
							        return;
								} else if(null != yesNoUpgrade && yesNoUpgrade.equalsIgnoreCase(AUTO_OPTION_VALUE)){
									enterInfoLog("user: " + uProfile.getId() + " selected YES for Level 2 upgrade online", icn);
									if(patient != null) {
										StringServiceResponse vapiiStatusResponse = 
											gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserProfileServiceDelegate().getVapiiFormStatusForUser(
												patient.getUserProfile());
										
										enterInfoLog("VAPII Form Status:" + vapiiStatusResponse.getString() , icn);
										
										//JAZZ#32762 US02 REQ#6 if the user signed the form, VAPii will return with a Submitted status.
										if(VAPII_SUBMITTED_VALUE.equalsIgnoreCase(vapiiStatusResponse.getString())) {
											
											enterInfoLog("Online form is succussfully submitted", icn);
											
											InPersonAuthentication ipa = ipaServiceResponse.getInPersonAuthentication();
											
											//JAZZ#32762 US02 REQ#6.2 The MHV system shall set Prerequisite fields: 
											//User has presented the required identification and Signed Request to Receive Access to P
											//ersonal Health Record (PHR) check box.
											ipa.setVideoViewed(Boolean.TRUE);
											ipa.setIdentificationPresented(Boolean.TRUE);
											ipa.setParticipationFormSigned(Boolean.TRUE);
								            ipa.setApprovedForRecordsAccess(Boolean.TRUE);
											
											//JAZZ#32762 US02 REQ#6.3 MHV System shall set the Authenticating Facility field to AUSTIN MHV
											ipa.setAuthenticatingFacility(getAustinMHVFacilityInfo());
											
											//JAZZ#32762 US02 REQ#6.4 MHV system shall not set the Reason for Delay field. 
											ipa.setDefermentReason(null);
											
											//JAZZ#32762 US02 REQ#6.1 The MHV System shall set the NEW field Signed 10-5345a-MHV form online field to 
											//date and time the MHV form was signed online and confirmation is received that the user has a signed form 
											//on file; otherwise blank.
											ipa.setSignedFormOnlineDateTime(new Timestamp(new Date().getTime()));
											
											//JAZZ#32762 US02 REQ#6.7 The MHV system shall set the Name of Authenticator (VA Staff):   field to System
											ipa.setAuthenticatedBy("System");
//											ipa.setStatus(AuthenticationStatus.getEnum(AuthenticationStatus.AUTHENTICATED));
//											VoidServiceResponse saveResponse = ipaDelegate.saveIPA(ipa);
//											
//											if (saveResponse.getMessages().hasErrorMessages()) {
//												if(log.isInfoEnabled()){
//													log.info("********** EAuthGatewayServlet.doGet() ipaDelegate.saveIPA() response.getMessages().hasErrorMessages()");
//													Collection<Message> errorMessages = saveResponse.getMessages().getErrorMessages();
//													for (Message message : errorMessages) {
//														log.info("********** EAuthGatewayServlet.doGet() ipaDelegate.saveIPA() message.getKey():"+message.getKey());
//													}
//												}
//											}	
																						
//											ipaServiceResponse = ipaDelegate.getAuthenticationForPatient(patient.getId());
//											if(log.isInfoEnabled()){
//												log.info("********** EAuthGatewayServlet.doGet() ipaServiceResponse.getInPersonAuthentication().getStatus().getName()" + ipaServiceResponse.getInPersonAuthentication().getStatus().getName());
//											}

											
											//JAZZ#32762 US02 REQ#6 After confirming the Submitted form status from VAPii, MHV System shall upgrade the users account to a Premium account type.
											//JAZZ#32762 US02 REQ#6.5 The MHV system shall set the status of this account to Authenticated/Premium
											//JAZZ#32762 US02 REQ#6.6 The MHV system shall notify MVI the Release of Information (ROI) flag was set.
											InPersonAuthenticationServiceResponse response = ipaDelegate.authenticate(ipa);
											
											if (response.getMessages().hasErrorMessages()) {
												enterInfoLog("Error in IPA Authentication" + retrieveErrorMessages(response.getMessages()), icn);	
												//JAZZ#32762 US02 REQ#6.8 MHV System shall capture the following information in the Account Activity Log (AAL) table in case of failure:
												Auditor.auditOnlineAuthenticationEvent(uProfile.getId(), false, ActivityTypeEnumeration.ONLINE_FORM_AVAILABLE, 
														ActivityActionTypeEnumeration.ONLINE_10_5345A_MHV_FORM_FILED, "");  
										    } else {
												//JAZZ#32762 US02 REQ#6.8 MHV System shall capture the following information in the Account Activity Log (AAL) table in case of success:
												Auditor.auditOnlineAuthenticationEvent(uProfile.getId(), true, ActivityTypeEnumeration.ONLINE_FORM_AVAILABLE, 
													ActivityActionTypeEnumeration.ONLINE_10_5345A_MHV_FORM_FILED, "");  
										    }
										} else {
											//JAZZ#32762 US02 REQ#5.2 If VAPii does not retutn with a Submitted status, MHV System shall 
											//provide a user-friendly message indicating the signed form was not located within VAPii at the top 
											//of the Start Upgrading Your My HealtheVet Account Today page.
											//req.getSession().setAttribute("displayError", "formSubmissionError");
											//res.sendRedirect(upgrade_url);
											//JAZZ#32762 US02 REQ#6.8 MHV System shall capture the following information in the Account Activity Log (AAL) table in case of failure:
											Auditor.auditOnlineAuthenticationEvent(uProfile.getId(), true, ActivityTypeEnumeration.ACCOUNT_UPGRADE, 
													ActivityActionTypeEnumeration.CANCEL_UPGRADE_ONLINE, "");  
										}
									}
									else if (mviPatRes == null || mviPatRes.getMessages().hasInformationalMessages()){
										res.sendRedirect(upgrade_url);
									    return;
									}
									
									if(useExistingSSO) {
										//Log the upgrade value in EAUTHLOG table
										elsd.logEauthEvent(ahash, uProfile.getUserName(), csid, "Upgrade_Request_Yes");
										//Create the mapping for them since they have passed the business rules for qualifications and are not already mapped.
										if( null == id ) {
											id = populateEauthHash(uProfile, ahash, csid);
										}
									}
									//Write the upgrade event to the Activity table
									if(uProfile != null && uProfile.getId() != null){
							    		//Auditor.auditUpgradeEvent(uProfile.getId(), ActivityActorTypeEnumeration.SELF);
										//JAZZ #32762 US REQ#6 MHV System shall make following AAL Entry updates. 
										Auditor.auditUpgradeEvent(uProfile.getId(), ActivityActorTypeEnumeration.SELF, csid, ACTION_DETAIL_DS_LOGON_ACCESS_VA);
							    	}
									
								} else if(null != yesNoUpgrade && yesNoUpgrade.equalsIgnoreCase("manual")){
									enterInfoLog("selected YES for Level 2 upgrade manual", icn);
									if(patient != null) {
										// Upgrade them here
										InPersonAuthentication ipa = ipaServiceResponse.getInPersonAuthentication();
										//ipa.setParticipationFormSigned(Boolean.FALSE);
										
										//JAZZ#32762 US REQ#2.1 MHV system shall set Prerequisite field "
										//User has presented the required identification" check box
										ipa.setVideoViewed(Boolean.TRUE);
										ipa.setParticipationFormSigned(Boolean.FALSE);
										ipa.setIdentificationPresented(Boolean.TRUE);
							            ipa.setApprovedForRecordsAccess(Boolean.FALSE);

										//JAZZ#32762 US03 REQ#2.3 The MHV system shall not set the Reason for Delay field. 
										AuthenticationDefermentReason reason = AuthenticationDefermentReason.getEnum(AuthenticationDefermentReason.PREREQUISITESINCOMPLETE);
										ipa.setDefermentReason(reason);

										//JAZZ#32762 US03 REQ#2.2 MHV system shall  Authenticated Facility field to AUSTIN MHV
										ipa.setAuthenticatingFacility(getAustinMHVFacilityInfo());

										//JAZZ#32762 US02 REQ#2.5 The MHV system shall set the NEW field Date/Time Start upgrade manually with the current date/time.											
										ipa.setStartUpgradeManualDateTime(new Timestamp(new Date().getTime()));
										
										//JAZZ#32762 US03 REQ#2.4 The MHV system shall set the status of this account to In Process. 
										InPersonAuthenticationServiceResponse response = ipaDelegate.deferAuthentication(ipa);
										
										if (response.getMessages().hasErrorMessages()) {
											  enterInfoLog(retrieveErrorMessages(response.getMessages()), icn);
									          
											  //JAZZ#32762 US03 REQ#5 The MHV System shall create an AAL entry for the in process action when a user selects Yes to start the upgrade process.   
											  Auditor.auditManualAuthenticationEvent(uProfile.getId(), false, ActivityActionTypeEnumeration.START_UPGRADE_MANUALLY, "");  
									          res.sendRedirect(anonymous_url);
									          return;
									    } else {
											  //JAZZ#32762 US03 REQ#5 The MHV System shall create an AAL entry for the in process action when a user selects Yes to start the upgrade process.   
											  Auditor.auditManualAuthenticationEvent(uProfile.getId(), true, ActivityActionTypeEnumeration.START_UPGRADE_MANUALLY, "");  
									    }

										//Log the upgrade value in EAUTHLOG table
										elsd.logEauthEvent(ahash, uProfile.getUserName(), csid, "Upgrade_Request_Yes");
										//Create the mapping for them since they have passed the business rules for qualifications and are not already mapped.
										if( null == id ) {
											id = populateEauthHash(uProfile, ahash, csid);
										}
										//Write the upgrade event to the Activity table
										if(uProfile != null && uProfile.getId() != null){
											Auditor.auditUpgradeEvent(uProfile.getId(), ActivityActorTypeEnumeration.SELF, csid, ACTION_DETAIL_DS_LOGON_ACCESS_VA);
								    	}
									}
									else if (mviPatRes == null || mviPatRes.getMessages().hasInformationalMessages()){
										enterInfoLog("User is not a patient so sending them to anonymous page", icn);
										res.sendRedirect(anonymous_url);
								        return;
									}
								} else if(cancelUpgrade) {
									enterInfoLog(" selected NO for Level 2 upgrade", icn);

									// Forward the user to logged in page if the user is already mapped, if not, forward them to map page
									VoidServiceResponse vsr = elsd.logEauthEvent(ahash, uProfile.getUserName(), csid, "Upgrade_Request_No");
									enterInfoLog(" selected NO for Level 2 upgrade", icn);

									enterInfoLog("Found that user (Level 1) needs to be auto mapped for UserProfileId: " + uProfile.getId(), icn);
									//Create the mapping for them since they have passed the business rules for qualifications and are not already mapped.
//									if( null == id ) {
//										id = populateEauthHash(uProfile, ahash, csid);
//									}
								} else {
									InPersonAuthentication ipa = ipaServiceResponse.getInPersonAuthentication();
									if(goToStartTodayPage(csid, ipa, req, res)) {
										return;
									}
								}
							} else {
//								id = populateEauthHash(uProfile, ahash, csid);
								// The user could be in any other IPA status (Authenticated, PendingDataMismatch, In-process)
								//JAZZ#32762 US01 REQ#1.2.1 If the Authentication Status has a different value than listed, MHV system 
								//shall place the user on the MHV splash page NOT logged in.
								//Note: 
								//MHV status continues to be status other than "Unauthenticated, "Pending Data Mismatch", or "In Process" 
								//(Advanced account)
								res.sendRedirect(splash_url);
								return;
							}
					} 
					else {
						// End patient
						enterInfoLog("User is not a patient so sending them to anonymous page for UserProfileId: " + uProfile.getId(), icn);
						res.sendRedirect(anonymous_url);
				        return;
					}
				}
			}
		}
		
		
		if(useExistingSSO && patient != null && patient.getCorrelationStatus().equals(PatientCorrelationStatusUtils.CORRELATED)) {
			processFacilitiesUpdate(patient);
		}
		
		if (uProfile != null) {
			id = uProfile.getUserName();
			Subject sub = null;
			try {
				sub = weblogic.security.services.Authentication.assertIdentity("EauthToken", buildToken(id).getBytes());
				enterInfoLog("authentication successful for user: " + id, icn);
				
				if(useExistingSSO) {
			        VoidServiceResponse vsr = elsd.logEauthEvent(ahash, id, csid, "Authenticate");
			        enterInfoLog(retrieveErrorMessages(vsr.getMessages()), icn);
				}
//				//JAZZ Story#39187 - US REQ#4 MHV System shall make following AAL Entry updates for Login
//				req.getSession().setAttribute(MULTIPLE_ACCOUNT_ERROR_ATTRIB, TRUE_VALUE);
		        Auditor.auditEBenefitsLoginEvent(uProfile.getId(), ActivityActorTypeEnumeration.SELF, Boolean.TRUE, deepLink, csid);
			} catch (LoginException le) {
				// authentication failed - redirect to mapping page
				// set hash on session
		        enterInfoLog("error logging in:" + le.getMessage(), icn);
				res.sendRedirect(eauth_portal);
				return;
			}

			weblogic.servlet.security.ServletAuthentication.runAs(sub, req);

			if (null != sub && !SubjectUtils.isUserAnonymous(sub)) {
                //JAZZ # 111941 - Remote ID Proofing US REQ#3.11 - When the User lands into their MHV account already logged in,   
				//MHV system shall record this login as the last time the user logged in to their MHV account. 
				processLastLogin(uProfile);
				String portal_url = mhv_portal;
				if(null != deepLink){
					if(deepLink.equalsIgnoreCase("download_my_data"))
						portal_url = mhv_portal_custom + "?_nfpb=true&_nfls=false&_pageLabel=downloadData";
					else if(deepLink.equalsIgnoreCase("prescription_refill"))
						portal_url = mhv_portal_custom + "?_nfpb=true&_pageLabel=rxRefill&_nfls=false";
					else if(deepLink.equalsIgnoreCase("appointments"))
						portal_url = mhv_portal_custom + "?_nfpb=true&_pageLabel=appointments&_nfls=false";
					else if(deepLink.equalsIgnoreCase("secure_messaging"))
						portal_url = mhv_portal_custom + "?_nfpb=true&_nfto=false&_pageLabel=secureMessagingPage";
				}
				res.sendRedirect(portal_url);
			}
		} else {
			res.sendRedirect(eauth_portal);
		}
		
	}

	//JAZZ # 128616 - Process Facilities Update
    private boolean processFacilitiesUpdate(Patient patient) {
    	enterInfoLog("Proccesing Facilities Update", patient.getIcn());
		boolean sameDay = false;
		
		if(patient.getLastFacilitiesUpdateDateTime() != null) {
			Calendar now = Calendar.getInstance();
			//now.setTime(now.getTime() - 1000*60*60*24);
			Calendar lastFacilitiesUpdate = Calendar.getInstance();
			lastFacilitiesUpdate.setTime(patient.getLastFacilitiesUpdateDateTime());
			
			sameDay = now.get(Calendar.YEAR) == lastFacilitiesUpdate.get(Calendar.YEAR) &&
				now.get(Calendar.DAY_OF_YEAR) == lastFacilitiesUpdate.get(Calendar.DAY_OF_YEAR);
		}
		if(!sameDay) {
			PatientServiceResponse response = 
				gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createMviIntegrationServiceDelegate().updateTreatmentFacilitiesForExistingPatients(patient);
			if(response.getMessages().hasErrorMessages()) {
				return false;
			}
		}
		
		return true;
    }
    
	//JAZZ # 111941 - Remote ID Proofing - Last Login 
	private void processLastLogin(UserProfile uProfile) {
		if(uProfile.getCurrentLogin()!=null) {
			uProfile.setLastLogin(uProfile.getCurrentLogin());
		}
		//Jazz136316 Logout box date time change
		uProfile.setCurrentLogin(new Timestamp( new Date().getTime()));
		gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createEntityMaintenanceServiceDelegate().save(uProfile);
	}
	
	private boolean isDeactivated(String username) {
		boolean result = true;

		UserProfileServiceDelegate userProfileDelegate = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserProfileServiceDelegate();
		UserProfileServiceResponse upResponse = userProfileDelegate.getProfileForUser(username);
		if (upResponse.getMessages().hasErrorMessages()) {
	          for (Object item:upResponse.getMessages().getErrorMessages()) {
			    log.error("UserProfile getUserProfileByUsername action Error: " + item);
	          }
	    }

		if( upResponse.getUserProfile() != null && upResponse.getUserProfile().getDeactivationReason() == null) {
			result = false;
		}
		if( result == true && log.isInfoEnabled() ) {
			log.info("Found a deactivated status for user:: " + username + ", Deactivation Reason: " + upResponse.getUserProfile().getDeactivationReason() );
		}

		return result;
	}

	private UserProfile findUserProfile(String firstName, String lastName,
			String gender, String sSN, String birthDate) {

		GenderEnumeration genderEnum = null;
		if(gender != null) {
			if(gender.equalsIgnoreCase(GenderEnumeration.MALE)){
				genderEnum = GenderEnumeration.getEnum(GenderEnumeration.MALE);
			}else{
				genderEnum = GenderEnumeration.getEnum(GenderEnumeration.FEMALE);
			}
		}
		
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		Date date = null;
		try {
			date = sdf.parse(birthDate.substring(0, 10));
		} catch (ParseException e) {
			if(log.isInfoEnabled()){
				log.info("Not able to parse date for the user:: " + firstName + " " + lastName + ", birthdate: " + birthDate);
			}
		}

		UserProfileServiceDelegate userProfileDelegate = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserProfileServiceDelegate();
		UserProfileServiceResponse upResponse = userProfileDelegate.getUserProfileForVAAFITraits(firstName, lastName, date, genderEnum);
		if (upResponse.getMessages().hasErrorMessages()) {
	          for (Object item:upResponse.getMessages().getErrorMessages()) {
			    log.error("UserProfile getUserProfileForVAAFITraits action Error: " + item);
	          }
	    }

		UserProfile uProfile = upResponse.getUserProfile();
		if( uProfile != null ) {
			if(log.isInfoEnabled()){
				log.info("Found DSLogon user profile:: " + uProfile.getId() + " with user_name:: " + uProfile.getUserName());
			}

			/*
			 * Compare SSN right here (SSN could be null for a non patient from evault)
			 */
			if(uProfile.getSsn() == null || !uProfile.getSsn().replace("-", "").equals(sSN)){
				if(log.isInfoEnabled()){
					log.info("SSN did not match for DSLogon CSID user for the user profile:: " + uProfile.getId());
				}
				uProfile = null;
			}
		}

		return uProfile;
	}

	/**
	 *
	 * @param userid
	 * @return String - encoded token used for IdentityAsserter
	 */
	private String buildToken(String userid) {
		String tokenValue = "username=" + encrypt(userid);
		return tokenValue;
	}

	private String encrypt(String s) {
		String result = null;
		if (s == null || s.trim().length() == 0) {
			throw new IllegalArgumentException(
					"unencrypted string was null or empty");
		}
		try {
			SecretKey key = keyFactory.generateSecret(keySpec);
			cipher.init(Cipher.ENCRYPT_MODE, key);
			byte[] sb = s.getBytes(UNICODE_FORMAT);
			byte[] csb = cipher.doFinal(sb);
			BASE64Encoder be = new BASE64Encoder();
			result = be.encodeBuffer(csb);
		} catch (InvalidKeySpecException iks) {
			log.error("error encrypting token", iks);
    } catch (InvalidKeyException ik) {
      log.error("error encrypting token", ik);
    } catch (UnsupportedEncodingException use) {
      log.error("error encrypting token", use);
    } catch (BadPaddingException bp) {
      log.error("Error encrypting token", bp);
    } catch (IllegalBlockSizeException ibs) {
      log.error("Error encrypting token", ibs);
		}
		return result;
	}

	public boolean hasERarAccess(String username) {
		//11/21/2014 - decided to go national so this will always return true so everyone has ERAR access. 
		return true;
		//return hasExtractAccess(AccessFeatureValueUtils.VAERAR.getName(), username);
	}
	
	public boolean hasExtractAccess(String extractFeature, String username) {

		boolean hasExtractAccess = false;
		AccessControlCollectionServiceResponse accessResponse =
			getAccessControlServiceDelegate().findAccessControls(username);

		hasExtractAccess = AccessControlUtils.hasAccess(AccessDomainUtils.PHR, extractFeature,
							AccessPermissionUtils.READ, accessResponse.getAccessControls());
		return hasExtractAccess;
	}

	public AccessControlServiceDelegate getAccessControlServiceDelegate() {
		return gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createAccessControlServiceDelegate();
	}
}
