package gov.va.med.mhv.rxrefill.bizobj;

import java.util.ArrayList;
import java.util.List;
import java.util.Collection;

import gov.va.med.mhv.rxrefill.service.delegate.PrescriptionServiceDelegate;
import gov.va.med.mhv.rxrefill.service.delegate.RxTrackingServiceDelegate;
import gov.va.med.mhv.rxrefill.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.rxrefill.transfer.Prescription;
import gov.va.med.mhv.rxrefill.transfer.RxTracking;
import gov.va.med.mhv.rxrefill.enumeration.MhvStatusEnumeration;
import gov.va.med.mhv.rxrefill.enumeration.PrescriptionStatusEnumeration;

/**
 * Business object for the Prescription application entity
 *
 * Generated Skeleton Code
 */

public class PrescriptionBO extends PrescriptionBaseBO {

	//prescription mhv statuses
	// pending mhv statuses
	public static final String[] MHV_STATUS_ENUM_NAMES_PENDING = {
		MhvStatusEnumeration.PENDING,
		MhvStatusEnumeration.RESULTSBACKORNOTPICKEDUP
	};

	// invalid mhv statuses
	public static final String[] MHV_STATUS_ENUM_NAMES_INVALID = {
		MhvStatusEnumeration.UNABLETORESOLVEPATIENT,
		MhvStatusEnumeration.RESOLVEPRESCRIPTION,
		MhvStatusEnumeration.DFNNOTMATCHRX
	};

	//prescription statuses
	public static final PrescriptionStatusEnumeration STATUS_ACTIVE =
		PrescriptionStatusEnumeration.getEnum(PrescriptionStatusEnumeration.ACTIVE);
	public static final PrescriptionStatusEnumeration STATUS_SUBMITTED =
		PrescriptionStatusEnumeration.getEnum(PrescriptionStatusEnumeration.SUBMITTED);
	public static final PrescriptionStatusEnumeration STATUS_SUSPENDED =
		PrescriptionStatusEnumeration.getEnum(PrescriptionStatusEnumeration.SUSPENDED);
	public static final PrescriptionStatusEnumeration STATUS_REFILLINPROCESS =
		PrescriptionStatusEnumeration.getEnum(PrescriptionStatusEnumeration.REFILLINPROCESS);
	public static final PrescriptionStatusEnumeration STATUS_DISCONTINUED =
		PrescriptionStatusEnumeration.getEnum(PrescriptionStatusEnumeration.DISCONTINUED);
	public static final PrescriptionStatusEnumeration STATUS_DISCONTINUEDEDIT =
		PrescriptionStatusEnumeration.getEnum(PrescriptionStatusEnumeration.DISCONTINUEDEDIT);

	public static List getHistoricalPrescriptionsWithRequestStatuses(
			long userId) {

		List rxList1 = getHistoricalPrescriptionsWithLastRequestSuccessful(new Long(userId));
		setRxStatuses(rxList1, false, true);
		List rxList2 = getHistoricalPrescriptionsWithLastRequestUnsuccessful(new Long(userId));
		setRxStatuses(rxList2, false, false);

		return joinListsWithLatestPrescriptions(rxList1, rxList2);
	}

	public static List getTrackingPrescriptionsWithRequestStatuses(
			long userId) {

		List rxList1 = getTrackingPrescriptionsWithLastRequestSuccessful(new Long(userId));
		setRxStatuses(rxList1, false, true);
		List rxList2 = getTrackingPrescriptionsWithLastRequestUnsuccessful(new Long(userId));
		setRxStatuses(rxList2, false, false);

		return joinListsWithLatestTrackingPrescriptions(rxList1, rxList2);
	}

	public static List getActivePrescriptionsWithRequestStatuses(
			long userId) {

		List rxList1 = getActivePrescriptionsWithLastRequestSuccessful(new Long(userId));
		setRxStatuses(rxList1, true, true);
		List rxList2 = getActivePrescriptionsWithLastRequestUnsuccessful(new Long(userId));
		setRxStatuses(rxList2, true, false);

		return joinListsWithLatestPrescriptions(rxList1, rxList2);
	}

    /* TODO: move to presentation layer?
	 * isLastRequestSuccessful, isInvalidMhvStatus, isRefillable
	 * & status (submitted) are for UI and not to be persisted.
	 */
	private static void setRxStatuses(List rxList,
									  boolean activeRx,
									  boolean lastRequestSuccessful) {

		for (int i=0; i<rxList.size(); i++) {
			PrescriptionBO rx = (PrescriptionBO)rxList.get(i);

            // set default values
			rx.setIsInvalidMhvStatus(new Boolean(false));
            rx.setIsRefillable(new Boolean(false));
			rx.setIsLastRequestSuccessful(new Boolean(lastRequestSuccessful));

			// rename statuses
            if (STATUS_SUSPENDED.equals(rx.getStatus()))
                rx.setStatus(STATUS_REFILLINPROCESS);

            if (STATUS_DISCONTINUEDEDIT.equals(rx.getStatus()))
                rx.setStatus(STATUS_DISCONTINUED);

            if (checkMhvStatusesPending(rx.getMhvStatus())) {
				/* pending mhv status: checking submitted status
				 * when mhvStatus is 'pending' or 'processed',
				 * rx status is 'active', and last request is successful
				 * change rx status to 'submitted'
				 */
				if (STATUS_ACTIVE.equals(rx.getStatus())
						&& rx.getIsLastRequestSuccessful().booleanValue()) {
					rx.setStatus(STATUS_SUBMITTED);
				}
			}

			if (activeRx) {
                //checking MHV status
				if (checkMhvStatusesInvalid(rx.getMhvStatus())) {
					//invalid mhv status: setting isInvalidMhvStatus
					rx.setIsInvalidMhvStatus(new Boolean(true));

				} else if (STATUS_ACTIVE.equals(rx.getStatus())
                        && rx.getNumberOfRefills().intValue() > 0
                        && !checkMhvStatusesPending(rx.getMhvStatus())) {

                    rx.setIsRefillable(new Boolean(true));
				}
			}
		}

		return;
	}

	protected void complexValidation() {

		//default the inactive flag to false:
		if (getIsInactive() == null) {
			setIsInactive(Boolean.FALSE);
		}

	}

	private static List joinListsWithLatestPrescriptions(List list1, List list2) {

        ArrayList list = new ArrayList();

        if (list1 == null || list1.isEmpty()) return list2;
        if (list2 == null || list2.isEmpty()) return list1;

        // add list1 items first
        for (int i=0; i < list1.size(); i++) {
            PrescriptionBO rx1 = (PrescriptionBO) list1.get(i);

            // loop through list2 to find the same id
            for (int j=0; j < list2.size(); j++) {
                PrescriptionBO rx2 = (PrescriptionBO) list2.get(j);

                if (rx1.getPrescriptionId().equals(rx2.getPrescriptionId())) {

                    // check which modified date is later
                    if (rx1.getModifiedDate().compareTo(rx2.getModifiedDate()) < 0)
                        rx1 = rx2; //rx2 replaces rx1

                    list2.remove(j); // remove the dup record
                    break; // done with this list1 item
                }
            }
            list.add(rx1);
        }

        // add all unmatched list2 items
        list.addAll(list2);

        return list;
    }

	public static boolean checkTrackingPrescriptions(PrescriptionBO rx1) {

		PrescriptionServiceDelegate service = ServiceDelegateFactory.createPrescriptionServiceDelegate();
		Prescription p = service.getPrescriptionDetailsForPrescriptionID(rx1.getPrescriptionId()).getPrescription();
		String stationNumber = p.getInstitution().getStationNumber();
		String rxnumber = p.getPrescriptionNumber();

		RxTrackingServiceDelegate trackingService = ServiceDelegateFactory.createRxTrackingServiceDelegate();
		Collection trackingNumbers = trackingService.getPrescriptionTrackingNumbers(rxnumber, stationNumber).getRxTrackings();
		// Check if there are rx_tracking records for the prescription. If no, return false.
		if (trackingNumbers.size() > 0) {
			//System.out.println(" 1001 >>>>>>>>>>> coount is > 0 :: "+rx1.getPrescriptionId());
			return true;
		}
		else
		{
			//System.out.println(" 1001 >>>>>>>>>>> coount is <= 0 :: "+rx1.getPrescriptionId());
			return false;
		}
	}

	private static List joinListsWithLatestTrackingPrescriptions(List list1, List list2) {

        ArrayList list = new ArrayList();
        ArrayList list1remove = new ArrayList();
        ArrayList list2remove = new ArrayList();

        for (Object rx1 : list1) {
			if (!checkTrackingPrescriptions((PrescriptionBO)rx1)) {
				//remove prescription from the list
				//System.out.println(" 1011 >>>>>>>>>>> Removing row :: "+((PrescriptionBO)rx1).getPrescriptionId());
				list1remove.add(rx1);
			}
		}
		list1.removeAll(list1remove);

        for (Object rx1 : list2) {
			if (!checkTrackingPrescriptions((PrescriptionBO)rx1)) {
				//remove prescription from the list
				//System.out.println(" 1012 >>>>>>>>>>> Removing row :: "+((PrescriptionBO)rx1).getPrescriptionId());
				list2remove.add(rx1);
			}
		}
		list2.removeAll(list2remove);

        if (list1 == null || list1.isEmpty()) return list2;
        if (list2 == null || list2.isEmpty()) return list1;

        // add list1 items first
        for (int i=0; i < list1.size(); i++) {
            PrescriptionBO rx1 = (PrescriptionBO) list1.get(i);

            // loop through list2 to find the same id
            for (int j=0; j < list2.size(); j++) {
                PrescriptionBO rx2 = (PrescriptionBO) list2.get(j);

                if (rx1.getPrescriptionId().equals(rx2.getPrescriptionId())) {

                    // check which modified date is later
                    if (rx1.getModifiedDate().compareTo(rx2.getModifiedDate()) < 0)
                        rx1 = rx2; //rx2 replaces rx1

                    list2.remove(j); // remove the dup record
                    break; // done with this list1 item
                }
            }
            list.add(rx1);
        }

        // add all unmatched list2 items
        list.addAll(list2);

        return list;
    }

	public static boolean checkMhvStatusesPending(MhvStatusEnumeration status) {
		return checkMhvStatuses(MHV_STATUS_ENUM_NAMES_PENDING, status);
	}

	public static boolean checkMhvStatusesInvalid(MhvStatusEnumeration status) {
		return checkMhvStatuses(MHV_STATUS_ENUM_NAMES_INVALID, status);
	}

	private static boolean checkMhvStatuses(String[] names,
			MhvStatusEnumeration status)
	{

		for (int i=0; i<names.length; i++) {

			if (MhvStatusEnumeration.getEnum(names[i]).equals(status))
				return true;
		}
		return false;
	}

}