package gov.va.med.mhv.rxrefill.bizobj;

import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import gov.va.med.mhv.rxrefill.enumeration.RequestFunctionEnumeration;
import gov.va.med.mhv.rxrefill.enumeration.RequestStatusEnumeration;
import gov.va.med.mhv.rxrefill.service.impl.util.RxRequestStrategy;

/**
 * Business object for the Request application entity
 *
 * Generated Skeleton Code
 */
 
public class RequestBO extends RequestBaseBO {
	
	//	request statuses
	public static final RequestStatusEnumeration STATUS_SUCCESSFUL = 
		RequestStatusEnumeration.getEnum(RequestStatusEnumeration.SUCCESSFUL);
	
	public static final RequestStatusEnumeration STATUS_FAILED = 
		RequestStatusEnumeration.getEnum(RequestStatusEnumeration.FAILED);
	
	//	request functions
	public static final RequestFunctionEnumeration FUNCTION_PROFILE = 
		RequestFunctionEnumeration.getEnum(RequestFunctionEnumeration.GETPRESCRIPTIONPROFILE);
	public static final RequestFunctionEnumeration FUNCTION_STATUS = 
		RequestFunctionEnumeration.getEnum(RequestFunctionEnumeration.GETPRESCRIPTIONSTATUS);
	public static final RequestFunctionEnumeration FUNCTION_UPDATE = 
		RequestFunctionEnumeration.getEnum(RequestFunctionEnumeration.GETPRESCRIPTIONUPDATE);
	public static final RequestFunctionEnumeration FUNCTION_REFILL = 
		RequestFunctionEnumeration.getEnum(RequestFunctionEnumeration.REFILLPRESCRIPTION);
	
	protected void complexValidation() {
		//default the inactive flag to false:
		if (getIsInactive() == null) {
			setIsInactive(Boolean.FALSE);
		}
	}

	public static List getFacilityRequestStatusesForHistoricalPrescriptions(
			Long userId) {
		
		List requestList = getLatestHistoricalRequestsByFacility(userId);
		return getFacilityRequestStatusList(requestList);
	}
	
	public static List getFacilityRequestStatusesForActivePrescriptions(
			long userId) {
		
		List requestList = getLatestActiveRequestsByFacility(new Long(userId));
		return getFacilityRequestStatusList(requestList);
	}
	
	public static List getFacilityRequestStatusesAfterRefill(
			long userId) {
		
		List requestList = getLatestRefillRequestsByFacility(new Long(userId));
		return getFacilityRequestStatusList(requestList);
	}
	
	private static List getFacilityRequestStatusList(List requestList) {

		for (int i=0; i<requestList.size(); i++) {
			RequestBO request = (RequestBO)requestList.get(i);	
			request.setExceededMaxFailedAttempts(
					new Boolean(checkMaxDailyFailedAttemptsExceeded(request)));
		}
		
		return requestList;
	}
	
	private static boolean checkMaxDailyFailedAttemptsExceeded(RequestBO request) {
		
        int failureCount = 0;
        
        Calendar today = Calendar.getInstance();
        
        // set time to the beginning of today
        today.set(Calendar.HOUR_OF_DAY, 0);
        today.set(Calendar.MINUTE, 0);
        today.set(Calendar.SECOND, 0);
        today.set(Calendar.MILLISECOND, 0);
                
        // only check today's request
        if (request.getCreatedDate().before(today.getTime()))
            return false;
		
		if (FUNCTION_STATUS.equals(request.getRequestFunction())
                || FUNCTION_UPDATE.equals(request.getRequestFunction())) {
			
            Set requestAttemptSet = request.getRequestAttempts();
			RequestAttemptBO attempt;
			Iterator i = requestAttemptSet.iterator();
			while (i.hasNext()) {
				attempt = (RequestAttemptBO)i.next();
				if (STATUS_FAILED.equals(attempt.getStatus()))
					failureCount++;
			}
			
			if (failureCount >= RxRequestStrategy.getMaxConsecutiveRxFailures())
				return true;
			else
				return false;
                
		} else // FUNCTION_REFILL & FUNCTION_PROFILE
            return false;
            /** 
             * profile request can be sent out only once a day
             * To see if max failures exceeded for historical rx, 
             * check update requests instead.
             */                
	}
}