package gov.va.med.mhv.rxrefill.service.impl;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.StopWatch;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.service.VoidServiceResponse;
import org.tigris.atlas.transfer.PrimaryKey;

import gov.va.med.mhv.rxrefill.bizobj.BusinessObjectFactory;
import gov.va.med.mhv.rxrefill.bizobj.InstitutionBO;
import gov.va.med.mhv.rxrefill.bizobj.PharmacyPatientBO;
import gov.va.med.mhv.rxrefill.bizobj.PharmacyPatientChangeBO;
import gov.va.med.mhv.rxrefill.bizobj.PrescriptionAssembler;
import gov.va.med.mhv.rxrefill.bizobj.PrescriptionBO;
import gov.va.med.mhv.rxrefill.bizobj.RequestAssembler;
import gov.va.med.mhv.rxrefill.bizobj.RequestBO;
import gov.va.med.mhv.rxrefill.service.PharmacyPatientServiceResponse;
import gov.va.med.mhv.rxrefill.service.PrescriptionCollectionServiceResponse;
import gov.va.med.mhv.rxrefill.service.PrescriptionService;
import gov.va.med.mhv.rxrefill.service.PrescriptionServiceResponse;
import gov.va.med.mhv.rxrefill.service.RequestCollectionServiceResponse;
import gov.va.med.mhv.rxrefill.service.RequestServiceResponse;
import gov.va.med.mhv.rxrefill.service.ServiceFactory;
import gov.va.med.mhv.rxrefill.service.impl.util.RxRequestDispatcher;
import gov.va.med.mhv.rxrefill.service.impl.util.RxRequestHelper;
import gov.va.med.mhv.rxrefill.transfer.Institution;
import gov.va.med.mhv.rxrefill.transfer.PharmacyPatient;
import gov.va.med.mhv.rxrefill.transfer.PharmacyPatientChange;
import gov.va.med.mhv.rxrefill.transfer.PharmacyPatientPK;
import gov.va.med.mhv.rxrefill.transfer.Prescription;
import gov.va.med.mhv.rxrefill.transfer.Request;
import gov.va.med.mhv.rxrefill.util.PrescriptionUtils;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActionTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityTypeEnumeration;
import gov.va.med.mhv.usermgmt.service.delegate.ActivityAuditServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.transfer.Activity;
import gov.va.med.mhv.usermgmt.transfer.Facility;
import gov.va.med.mhv.usermgmt.transfer.Patient;
import gov.va.med.mhv.usermgmt.transfer.PatientRegistryChange;
import gov.va.med.mhv.usermgmt.transfer.PatientRegistryChangePK;
import gov.va.med.mhv.usermgmt.transfer.TransferObjectFactory;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;


/**
 * Service implementation class for the Prescription service
 */
public class PrescriptionServiceImpl implements PrescriptionService {

	private static final Log LOG = LogFactory.getLog(PrescriptionServiceImpl.
		class);

	private static final String UNKNOWN = "<?>";

	/**
	 * Gest the PharamcyPatient TO and converts it into a BO
	 * @param patient The patient to get it for
	 * @return The PharamcyPatient BO
	 */
	private static PharmacyPatientBO getPharmacyPatientBO(
		PharmacyPatient pharmacyPatient)
	{
		PharmacyPatientBO pharmacyPatientBO = null;
		Patient patient = pharmacyPatient.getPatient();
		if (patient != null) {
			PharmacyPatientPK pk = new PharmacyPatientPK(patient
				.getUserProfile().getId());
			pharmacyPatientBO = PharmacyPatientBO.findByPrimaryKey(pk);
		}

		return pharmacyPatientBO;
	}

	private static PharmacyPatient getLatest(PharmacyPatient pharmacyPatient) {
		assert pharmacyPatient != null : "A pharamcyPatient is required";
		PharmacyPatientBO pharmacyPatientBO = getPharmacyPatientBO(
			pharmacyPatient);
		assert pharmacyPatientBO != null : "Could not find pharmacyPatient ["
			+ pharmacyPatient.getKey() + "]";
		return pharmacyPatientBO.getPharmacyPatientValues();
	}

	private static long toUserId(PharmacyPatient pharmacyPatient) {
		return pharmacyPatient.getPatient().getUserProfile().getId().
			longValue();
	}

	private static String describe(PharmacyPatient pharmacyPatient) {
		Patient patient = null;
		if (pharmacyPatient != null) {
			patient = pharmacyPatient.getPatient();
		}
		return describe(patient);
	}

	private static String describe(Patient patient) {
		StringBuilder builder = new StringBuilder("Patient[");
		if (patient != null) {
			UserProfile userProfile = patient.getUserProfile();
			if (userProfile != null) {
				builder.append("UserName=").append(userProfile.getUserName());
			} else {
				builder.append("PatientID=").append(patient.getId());
			}
			builder.append(";ICN=").append(patient.getIcn());
		} else {
			builder.append(UNKNOWN);
		}
		builder.append("]");
		return builder.toString();
	}

	private static String describePrescriptions(Collection prescriptions) {
		StringBuilder builder = new StringBuilder("prescriptions: {");
		if (prescriptions != null) {
			for (Iterator i = prescriptions.iterator(); i.hasNext(); ) {
				builder.append("\n\tPrescription(");
				Prescription prescription = (Prescription) i.next();
				if (prescription != null) {
					builder.append("ID=");
					builder.append(prescription.getPrescriptionId());
					builder.append(";Number=");
					builder.append(prescription.getPrescriptionNumber());
				} else {
					builder.append("NULL");
				}
				builder.append(")");
			}
		}
		builder.append("}");
		return builder.toString();
	}

	/**
	 * Execute the GetFacilityRequestStatusesForHistoricalPrescriptions service
	 * @return RequestCollectionServiceResponse
	 */
	public RequestCollectionServiceResponse
		getFacilityRequestStatusesForHistoricalPrescriptions(
		PharmacyPatient pharmacyPatient)
	{
		RequestCollectionServiceResponse response =
			new RequestCollectionServiceResponse();
		pharmacyPatient = getLatest(pharmacyPatient);

		Collection boList = RequestBO
			.getFacilityRequestStatusesForHistoricalPrescriptions(
			pharmacyPatient.getPatient().getUserProfile().getId());
		Collection requestList = RequestAssembler.getRequestCollection(boList);

		response.addRequests(requestList);
		return response;
	}

	/**
	 * Execute the SendRequest service
	 * @return PrescriptionCollectionServiceResponse
	 */
	public PrescriptionCollectionServiceResponse sendRequest(Patient patient,
		Request request)
	{
		return (new RxRequestHelper()).sendRequest(patient, request);
	}

	/**
	 * Execute the GetHistoricalPrescriptions service
	 * @return PrescriptionCollectionServiceResponse
	 */
	public PrescriptionCollectionServiceResponse getHistoricalPrescriptions(
		PharmacyPatient pharmacyPatient)
	{
		final PrescriptionCollectionServiceResponse response =
			new PrescriptionCollectionServiceResponse();

		new MonitoredRequest("Retrieve Historical Prescriptions",
			pharmacyPatient)
		{
			@Override
			protected void doRun() {
				PatientChanges patientChanges = processPatientChanges(
					getPharmacyPatient());
				VoidServiceResponse refreshResponse =
					refreshHistoricalPrescriptions(getPharmacyPatient());
				response.getMessages().addMessages(refreshResponse.
						getMessages());
				Collection prescriptionBOs = PrescriptionBO
					.getHistoricalPrescriptionsWithRequestStatuses(getUserId());
				response.addPrescriptions(PrescriptionAssembler
					.getPrescriptionCollection(prescriptionBOs));
				postProcessPatientChanges(patientChanges, response);
			}

		}.run();

		return response;
	}

	/**
	 * Execute the GetTrackingPrescriptions service
	 * @return PrescriptionCollectionServiceResponse
	 */
	public PrescriptionCollectionServiceResponse getTrackingPrescriptions(
		PharmacyPatient pharmacyPatient)
	{
		final PrescriptionCollectionServiceResponse response =
			new PrescriptionCollectionServiceResponse();

		new MonitoredRequest("Retrieve Historical Prescriptions",
			pharmacyPatient)
		{
			@Override
			protected void doRun() {
				PatientChanges patientChanges = processPatientChanges(
					getPharmacyPatient());
				VoidServiceResponse refreshResponse =
					refreshHistoricalPrescriptions(getPharmacyPatient());
				response.getMessages().addMessages(refreshResponse.
						getMessages());
				Collection prescriptionBOs = PrescriptionBO
					.getTrackingPrescriptionsWithRequestStatuses(getUserId());
				response.addPrescriptions(PrescriptionAssembler
					.getPrescriptionCollection(prescriptionBOs));
				postProcessPatientChanges(patientChanges, response);
			}

		}.run();

		return response;
	}

	/**
	 * Execute the GetTrackingPrescriptionsForEmails operation
	 *
	 * @param userId
	 * @return PrescriptionCollectionServiceResponse
	 */
	public PrescriptionCollectionServiceResponse getTrackingPrescriptionsForEmails(Long userId) {

		PrescriptionCollectionServiceResponse response = new PrescriptionCollectionServiceResponse();
		Collection prescriptionBOs = PrescriptionBO.getTrackingPrescriptionsWithRequestStatuses(userId);
		response.addPrescriptions(PrescriptionAssembler.getPrescriptionCollection(prescriptionBOs));
		return response;
	}


	/**
	 * Execute the RefillPrescriptions service
	 * @return VoidServiceResponse
	 */
	@SuppressWarnings("unchecked")
	public VoidServiceResponse refillPrescriptions(
		PharmacyPatient pharmacyPatient, Collection prescriptions)
	{
		final VoidServiceResponse response = new VoidServiceResponse();
		final List<Prescription> refills = new ArrayList<Prescription>();
		for (Object r: prescriptions) {
			refills.add((Prescription) r);
		}

		new MonitoredRequest("Refill Prescriptions", pharmacyPatient) {
			@Override
			protected void doRun() {
				int size = refills.size();
				if (size == 0) {
					return;
				}
				Patient patient = getPharmacyPatient().getPatient();
				if (patient == null) {
					return;
				}
				if (LOG.isDebugEnabled()) {
					LOG.debug("Received request to refill " + size);
				}
				Prescription[] refillArray = toArray(refills);
				assert refillArray.length > 0;
				RxRequestDispatcher dispatcher = new RxRequestDispatcher();
				dispatcher.dispatchRefillRequests(patient, refillArray,
					response);
				auditRefillrequest(patient, refillArray,	!response.getMessages().hasErrorMessages());
			}
			@Override
			protected String describeOnError() {
				return describePrescriptions(refills) + " "
					+ super.describeOnError();
			}

			private Prescription[] toArray(List<Prescription> prescriptions) {
				return refills.toArray(new Prescription[0]);
			}
		}.run();

		return response;


	}

	/**
	 * Execute the RefillPrescriptions service
	 * @return VoidServiceResponse
	 */
	@SuppressWarnings("unchecked")
	public VoidServiceResponse refillPrescriptionsAPI(PharmacyPatient pharmacyPatient, Collection prescriptions)
	{
		final VoidServiceResponse response = new VoidServiceResponse();
		final List<Prescription> refills = new ArrayList<Prescription>();
		for (Object r: prescriptions) {
			refills.add((Prescription) r);
		}
		new MonitoredRequest("Refill Prescriptions", pharmacyPatient) {
			@Override
			protected void doRun() {
				int size = refills.size();
				if (size == 0) {
					return;
				}
				Patient patient = getPharmacyPatient().getPatient();
				if (patient == null) {
					return;
				}
				if (LOG.isDebugEnabled()) {
					LOG.debug("Received request to refill " + size);
				}
				Prescription[] refillArray = toArray(refills);
				assert refillArray.length > 0;
				RxRequestDispatcher dispatcher = new RxRequestDispatcher();
				dispatcher.dispatchRefillRequests(patient, refillArray,	response);
			}
			@Override
			protected String describeOnError() {
				return describePrescriptions(refills) + " "
					+ super.describeOnError();
			}

			private Prescription[] toArray(List<Prescription> prescriptions) {
				return refills.toArray(new Prescription[0]);
			}
		}.run();

		return response;
	}


	private void auditRefillrequest(Patient patient,
		Prescription[] prescriptionArray, boolean success)
	{
		ActivityAuditServiceDelegate delegate =ServiceDelegateFactory.
			createActivityAuditServiceDelegate();
		Activity activity = TransferObjectFactory.createActivity();
		activity.setAction(ActivityActionTypeEnumeration.getEnum(
			ActivityActionTypeEnumeration.REQUEST_REFILL));
		activity.setActionType(ActivityTypeEnumeration.RXREFILL);
		activity.setCompletionTime(new Timestamp(new Date().getTime()));
		activity.setPerformerType(ActivityActorTypeEnumeration.getEnum(
			ActivityActorTypeEnumeration.SELF));
		activity.setSubjectId(patient.getUserProfile().getId());
		StringBuffer prescriptionIds = new StringBuffer(100);
		for (int i = 0; i < prescriptionArray.length; i++) {
			Prescription p = prescriptionArray[i];
			prescriptionIds.append(p.getDrugName());
			if (i + 1 < prescriptionArray.length) {
				prescriptionIds.append(",");
			}
		}
		activity.setDetailValue(prescriptionIds.toString());
		activity.setStatus(success);
		delegate.audit(activity);
	}

	/**
	 * Execute the GetPharmacyPatient service
	 * @return PharmacyPatientServiceResponse
	 */
	public PharmacyPatientServiceResponse getPharmacyPatient(Patient patient) {
		PharmacyPatientBO pharmacyPatient = null;
		if (patient != null) {
			PharmacyPatientPK pk = new PharmacyPatientPK(patient
				.getUserProfile().getId());
			pharmacyPatient = PharmacyPatientBO.findByPrimaryKey(pk);
			if (pharmacyPatient == null) {
				// did not previsouly exist, so create it now:
				pharmacyPatient = BusinessObjectFactory
					.createPharmacyPatientBO();
				pharmacyPatient.setPatient(patient);
				pharmacyPatient.setKey(pk);
				pharmacyPatient.save();
			}
			PharmacyPatientServiceResponse response =
				new PharmacyPatientServiceResponse();
			response.setPharmacyPatient(pharmacyPatient
				.getPharmacyPatientValues());
			return response;
		}

		return null;
	}

	/**
	 * Execute the RemovePrescriptionsForStations service
	 * @return VoidServiceResponse
	 */
	public VoidServiceResponse removePrescriptionsForStations(
		PharmacyPatient pharmacyPatient, Collection stations)
	{
		VoidServiceResponse response = new VoidServiceResponse();
		// interface implementation
		if (pharmacyPatient == null || pharmacyPatient.getPatient() == null
			|| pharmacyPatient.getPatient().getUserProfile() == null
			|| pharmacyPatient.getPatient().getUserProfile().getId() == null
			|| stations == null)
		{
			return response;
		}
		for (Object s: stations) {
			String stationNumber = (String) s;
			if (stationNumber != null) {
				List institutions = InstitutionBO.getInstitutionForStationNumber(
					stationNumber);
				if ((institutions != null) && (institutions.size() > 0)) {
					Institution institution = (Institution) institutions.get(0);
					deletePrescriptionsForUserInstitutions(
					toUserId(pharmacyPatient), institution.getInstitutionId());
				}
			}
		}
		return response;
	}


	/**
	 * Execute the RefreshActivePrescriptions service
	 * @return VoidServiceResponse
	 */
	public VoidServiceResponse refreshActivePrescriptions(
		PharmacyPatient pharmacyPatient)
	{
		VoidServiceResponse response = new VoidServiceResponse();
		Patient patient = pharmacyPatient.getPatient();
		if (patient != null) {
			RxRequestDispatcher dispatcher = new RxRequestDispatcher();
			dispatcher.dipatchStatusRequests(patient, response);
		}
		return response;
	}

	/**
	 * Execute the GetActivePrescriptions service
	 * @return PrescriptionCollectionServiceResponse
	 */
	public PrescriptionCollectionServiceResponse getActivePrescriptions(
		PharmacyPatient pharmacyPatient)
	{
		final PrescriptionCollectionServiceResponse response =
			new PrescriptionCollectionServiceResponse();

		new MonitoredRequest("Retrieve Active Prescriptions", pharmacyPatient) {
			@Override
			protected void doRun() {
				PatientChanges patientChanges = processPatientChanges(
					getPharmacyPatient());
				VoidServiceResponse refreshResponse =
					refreshActivePrescriptions(getPharmacyPatient());
				response.getMessages().addMessages(refreshResponse.
						getMessages());
				Collection prescriptionBOs = PrescriptionBO
					.getActivePrescriptionsWithRequestStatuses(getUserId());
				response.addPrescriptions(PrescriptionAssembler.
					getPrescriptionCollection(prescriptionBOs));
				postProcessPatientChanges(patientChanges, response);
			}
		}.run();

		return response;
	}

	/**
	 * Execute the RefreshHistoricalPrescriptions service
	 * @return VoidServiceResponse
	 */
	public VoidServiceResponse refreshHistoricalPrescriptions(
		PharmacyPatient pharmacyPatient)
	{
		VoidServiceResponse response = new VoidServiceResponse();
		Patient patient = pharmacyPatient.getPatient();
		if (patient != null) {
			RxRequestDispatcher dispatcher = new RxRequestDispatcher();
			dispatcher.dipatchExtractRequests(patient, response);
		}
		return response;
	}

	/**
	 * Execute the GetLastSuccessfulExtractRequestForUser service
	 * @return RequestServiceResponse
	 */
	public RequestServiceResponse getLastSuccessfulExtractRequestForUser(
		PharmacyPatient pharmacyPatient)
	{
		RequestServiceResponse response = new RequestServiceResponse();
		response.setEntity(null);

		pharmacyPatient = getLatest(pharmacyPatient);
		List requests = RequestBO.getLatestSuccessfulExtractRequests(
				pharmacyPatient.getPatient().getUserProfile().getId());
		for (Object r: requests) {
			Request request = ((RequestBO) r).getRequestValues();
			if (response.getEntity() != null) {
				if (((Request) response.getEntity()).getModifiedDate()
					.compareTo(request.getModifiedDate()) < 0)
				{
					response.setEntity(request);
				}
			} else {
				response.setEntity(request);
			}
		}

		return response;
	}

	/**
	 * Execute the GetFacilityRequestStatusesAfterRefill service
	 * @return RequestCollectionServiceResponse
	 */
	public RequestCollectionServiceResponse getFacilityRequestStatusesAfterRefill(
		PharmacyPatient pharmacyPatient)
	{
		RequestCollectionServiceResponse response =
			new RequestCollectionServiceResponse();
		pharmacyPatient = getLatest(pharmacyPatient);

		Collection boList = RequestBO
			.getFacilityRequestStatusesAfterRefill(pharmacyPatient.getPatient()
				.getUserProfile().getId().longValue());
		Collection requestList = RequestAssembler.getRequestCollection(boList);

		response.addRequests(requestList);

		return response;
	}

	/**
	 * Execute the GetFacilityRequestStatusesForActivePrescriptions service
	 * @return RequestCollectionServiceResponse
	 */
	public RequestCollectionServiceResponse
		getFacilityRequestStatusesForActivePrescriptions(
		PharmacyPatient pharmacyPatient)
	{
		RequestCollectionServiceResponse response =
			new RequestCollectionServiceResponse();
		pharmacyPatient = getLatest(pharmacyPatient);

		Collection boList = RequestBO
			.getFacilityRequestStatusesForActivePrescriptions(pharmacyPatient
				.getPatient().getUserProfile().getId().longValue());
		Collection requestList = RequestAssembler.getRequestCollection(boList);

		response.addRequests(requestList);

		return response;
	}

	private PatientChanges processPatientChanges(
		PharmacyPatient pharmacyPatient)
	{
		PharmacyPatientChangeBO mostRecentChange = addNewPharmacyChanges(
			pharmacyPatient);
		if (mostRecentChange == null) {
            LOG.debug("All patient registration changes has been accounted for "
                + "in pharmacy data");
			return null;
		}

        Patient patient = pharmacyPatient.getPatient();
		PatientChanges patientChanges = new PatientChanges();
		PatientRegistryChange patientChange = mostRecentChange.
			getPatientRegistryChange();
		if (StringUtils.isBlank(patientChange.getOldIcn())) {
			markRemovedFacilityRecordsAsInactive(patient, patientChange);
		} else {
			markAllFacilityRecordsAsInactive(patient, patientChanges);
		}
		return patientChanges;
	}

	/**
	 *
	 * @param pharmacyPatient
	 * @param changeSet
	 * @param pharmacyChangeSet
	 * @return The most recent change
	 */
	private PharmacyPatientChangeBO addNewPharmacyChanges(
		PharmacyPatient pharmacyPatient)
	{
		PharmacyPatientChangeBO mostRecentChange = null;
		Date mostRecentPharmacyChange = null;
		Map<PrimaryKey, PharmacyPatientChange> pharmacyChangesByKey =
				new HashMap<PrimaryKey, PharmacyPatientChange>();
		for (Object c: pharmacyPatient.getPharmacyPatientChanges()) {
			PharmacyPatientChange change = (PharmacyPatientChange) c;
			pharmacyChangesByKey.put(change.getPatientRegistryChange().getKey(),
					change);
			Date changeDate = change.getPatientRegistryChange().
				getRecordedOnDate();
			if ((mostRecentPharmacyChange == null)
				|| changeDate.before(mostRecentPharmacyChange))
			{
				mostRecentPharmacyChange = changeDate;
			}
		}

		for (Object pc: pharmacyPatient.getPatient().
                           getPatientRegistryChanges())
        {
			PatientRegistryChange patientChange = (PatientRegistryChange) pc;
			PatientRegistryChangePK pk = patientChange.
				getPatientRegistryChangePK();
			if (pharmacyChangesByKey.containsKey(pk)) {
				// Already exists in the pharmacy changes
				continue;
			}
			// Create a new instance
			PharmacyPatientChangeBO changeBO = BusinessObjectFactory.
				createPharmacyPatientChangeBO();
			changeBO.setPatientRegistryChange(patientChange);
			PharmacyPatientBO patientBO = getPharmacyPatientBO(pharmacyPatient);
			changeBO.setPharmacyPatient(patientBO);
			patientBO.addPharmacyPatientChange(changeBO);
			Date changeDate = patientChange.getRecordedOnDate();
			if (((mostRecentChange == null) || changeDate.after(mostRecentChange.
					getPatientRegistryChange().getRecordedOnDate()))
				// a new change must be the most recent patient registry change
			   &&
			   ((mostRecentPharmacyChange == null)
				 || mostRecentPharmacyChange.before(changeDate)))
				// a new change must be newer than the most recent pharmacy
				// change that has already applied
			{
				mostRecentChange = changeBO;
				if (LOG.isDebugEnabled()) {
					LOG.debug("Found a new/more recent registration change "
						+ describe(mostRecentChange));
				}
			}
		}

		return mostRecentChange;
	}

	private String describe(PharmacyPatientChangeBO pharmacyChange) {
		StringBuilder builder = new StringBuilder("PharmacyPatientChange[");
		if (pharmacyChange != null) {
			PatientRegistryChange change = pharmacyChange.
				getPatientRegistryChange();
			if (change != null) {
				builder.append("RecordedAt=");
				builder.append(change.getRecordedOnDate());
				builder.append(";OldIcn=");
				builder.append(change.getOldIcn());
				builder.append(";OldFacilityCount=");
				builder.append(change.getOldFacilityCount());
			} else {
				builder.append("RegistrationChange=").append(UNKNOWN);
			}
		}
		builder.append("]");
		return builder.toString();
	}

	/**
	 * Sets all prescriptions and requests to a status of "deleted" for the
	 * given patient. At no time are these records actually deleted from the
	 * database.
	 * @param patient The <tt>Patient</tt> for which to mark existing records
	 *        as "deleted"
	 */
	private void markAllFacilityRecordsAsInactive(Patient patient,
		PatientChanges patientChanges)
	{
		if (LOG.isDebugEnabled()) {
			LOG.debug("Patient's (" + describe(patient) + ") registration"
				+ " has been updated since the last request:"
				+ " marking records from all facilities as inactive");
		}
		Long userId = patient.getUserProfile().getId();
		for (Object p: RequestBO.getRequestsForPatient(userId)) {
			((RequestBO) p).setIsInactive(true);
		}
		for (Object p: PrescriptionBO.getPrescriptionsForPatient(userId)) {
			PrescriptionBO prescription = (PrescriptionBO) p;
			prescription.setIsInactive(true);
			patientChanges.addInactivePrescription(prescription.
				getPrescriptionValues());
		}
	}

	private void markRemovedFacilityRecordsAsInactive(Patient patient,
		PatientRegistryChange patientRegChange)
	{
		if (LOG.isDebugEnabled()) {
			LOG.debug("Patient's (" + describe(patient) + ") registration"
				+ " has been updated since the last request:"
				+ " marking only records from removed facilities as inactive");
		}
		Long userId = patient.getUserProfile().getId();
		Map<String, InstitutionBO> oldInstitutionMap =
			new HashMap<String, InstitutionBO>(10);
		for (Object r: RequestBO.getLatestRequestsByFacility(userId)) {
			RequestBO request = (RequestBO) r;
			InstitutionBO institution = request.getInstitution();
			if (institution != null) {
				oldInstitutionMap.put(institution.getStationNumber(),
					institution);
			}
		}

		// compare new institutions to the old institutions
		Set<InstitutionBO> validStationNumberSet = new HashSet<InstitutionBO>();
		for (Object f: patient.getFacilitys()) {
			Facility facility = (Facility) f;
			InstitutionBO institution = oldInstitutionMap.get(facility.getName());
			if (institution != null) {
				validStationNumberSet.add(institution);
			}
		}

		// next, remove valid institutions from the old set. This set will then
		// be used to mark old facilities as deleted.
		for (Object i: validStationNumberSet) {
			InstitutionBO institution = (InstitutionBO) i;
			InstitutionBO removed = oldInstitutionMap.remove(institution.
				getStationNumber());
			assert (removed != null) : "Failed to remove " + institution.
				getStationNumber() + " from oldInstitutionMap";
			if (LOG.isDebugEnabled()) {
				LOG.debug("Keep requests and prescriptions for user '"
					+ patient.getUserProfile().getUserName()
					+ "' and current facility " + institution.getStationNumber()
					+ "'");
			}
		}

		// finally, mark invalid institutions as deleted:
		for (Object i: oldInstitutionMap.values()) {
			InstitutionBO institution = (InstitutionBO) i;
			if (LOG.isDebugEnabled()) {
				LOG.debug("Delete requests and prescriptions for user '"
					+ patient.getUserProfile().getUserName()
					+ "' and old facility " + institution.getStationNumber()
					+ "'");
			}
			deletePrescriptionsForUserInstitutions(userId, institution.getId());
		}
	}

	@SuppressWarnings("unchecked")
	private void deletePrescriptionsForUserInstitutions(Long userId,
		Long institutionId)
	{
		List requestsToDelete = new ArrayList(RequestBO.
			getRequestsForPatientAndInstitution(userId, institutionId));
		for (Object r: requestsToDelete) {
			((RequestBO) r).setIsInactive(Boolean.TRUE);
		}
		List prescriptionsToInactivate = PrescriptionBO.
			getPrescriptionsForInactivation(userId,institutionId);
		for (Object p: prescriptionsToInactivate) {
			((PrescriptionBO) p).setIsInactive(Boolean.TRUE);
		}
	}

	private void postProcessPatientChanges(PatientChanges patientChanges,
		PrescriptionCollectionServiceResponse response)
	{
		if (patientChanges == null) {
			return;
		}
		for (Object p: response.getPrescriptions()) {
			Prescription prescription = (Prescription) p;
			Prescription deletedPrescription = patientChanges.
				findInactivePrescription(prescription.getPrescriptionNumber());
			if (PrescriptionUtils.isSubmitted(deletedPrescription)) {
				// This is necessary, because otherwise the last submitted
				// date is lost, as it is MHV only data
				prescription.setLastRefillSubmittedDate(deletedPrescription.
					getLastRefillSubmittedDate());
				PrescriptionServiceResponse psResponse = ServiceFactory.
					createEntityMaintenanceService().save(prescription);
				response.getMessages().addMessages(psResponse.getMessages());
			}
		}
	}


	/**
	 * Handles timing the doRun() execution and captures runtime exception
	 * in log.
	 * Additionally the pharmacy Patient is updated to the latests values
	 * from the business object.
	 */
	private static abstract class MonitoredRequest {

		private final String name;
		private final PharmacyPatient pharmacyPatient;
		protected MonitoredRequest(String name,
			PharmacyPatient pharmacyPatient)
		{
			this.name = name;
			this.pharmacyPatient = getLatest(pharmacyPatient);
		}

		protected String getName() {
			return name;
		}

		protected PharmacyPatient getPharmacyPatient() {
			return pharmacyPatient;
		}

		protected long getUserId() {
			return toUserId(pharmacyPatient);
		}

		public void run() {
			StopWatch stopWatch = null;
			if (LOG.isDebugEnabled()) {
				stopWatch = new StopWatch();
				stopWatch.start();
			}
			try {
				doRun();
			} catch (RuntimeException e) {
				onRuntimeException(e);
				throw e;
			}
			if (LOG.isDebugEnabled() && (stopWatch != null)) {
				stopWatch.stop();
				LOG.debug("PERFORMANCE - " + getName() + " in "
					+ stopWatch.getTime() + "ms");
			}
		}

		protected abstract void doRun();

		protected void onRuntimeException(RuntimeException e) {
			// Exception (stacktrace) will be logged in the ExceptionHandler
			// Do not log it here
			LOG.error("Unexpected exception occurred while " + getName() + " "
				+ describeOnError());
		}

		protected String describeOnError() {
			return "for " + describe(getPharmacyPatient());
		}
	}

	/**
	 * PatientChanges holds changes (within this service only) made
	 * to the pharmacy records that must be used for post processing.
	 * Changes can be added while processing any patient changes and passed
	 * to post processing via a PatientChanges instance
	 * E.g. the LastRefillSubmitDate of prescriptions must be perserved as it
	 * MHV only data.
	 */
	private static final class PatientChanges {

		Map<String, Prescription> inactivePrescriptions =
			new HashMap<String, Prescription>();

		public PatientChanges() {
		}

		public void addInactivePrescription(Prescription prescription) {
			if (prescription == null) {
				return;
			}
			inactivePrescriptions.put(prescription.getPrescriptionNumber(),
				prescription);
		}

		public Map<String, Prescription> getInactivePrescriptions() {
			return inactivePrescriptions;
		}

		public Prescription findInactivePrescription(String prescriptionNumber) {
			return inactivePrescriptions.get(prescriptionNumber);
		}
}


	@SuppressWarnings("unchecked")
	public PrescriptionServiceResponse getPrescriptionDetailsForPrescriptionID(
			Long prescriptionId) {
		PrescriptionServiceResponse response =
			new PrescriptionServiceResponse();
		List prescriptionsList = PrescriptionBO.getPrescriptionDetailsForPrescriptionId(prescriptionId);

			if (!prescriptionsList.isEmpty()) {
				PrescriptionBO currentPrescription = ((PrescriptionBO)
						prescriptionsList.get(0));
				response.setPrescription(currentPrescription.getPrescriptionValues());
				return response;
			}
			return response;
		}

	/**
	 * Execute the GetPrescriptionDetailsForPrescriptionNumber operation
	 *
	 * @param prescriptionNumber
	 * @return PrescriptionServiceResponse
	 */
	public PrescriptionServiceResponse getPrescriptionDetailsForPrescriptionNumber(String prescriptionNumber, String stationNumber) {

		PrescriptionServiceResponse response = new PrescriptionServiceResponse();
		List prescriptionsList = PrescriptionBO.getPrescriptionDetailsForPrescriptionNumber(prescriptionNumber,stationNumber);

			if (!prescriptionsList.isEmpty()) {
				PrescriptionBO currentPrescription = ((PrescriptionBO)
						prescriptionsList.get(0));
				response.setPrescription(currentPrescription.getPrescriptionValues());
				return response;
			}
			return response;
	}
}

