package gov.va.med.mhv.rxrefill.service.impl;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import gov.va.med.mhv.core.service.delegate.EmailServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.UserProfileServiceDelegate;
import gov.va.med.mhv.rxrefill.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.rxrefill.service.delegate.PrescriptionServiceDelegate;
import gov.va.med.mhv.rxrefill.service.delegate.RxTrackingServiceDelegate;
import gov.va.med.mhv.rxrefill.service.delegate.RxTrackingServiceDelegateImpl;


import gov.va.med.mhv.rxrefill.bizobj.RxTrackingAssembler;
import gov.va.med.mhv.rxrefill.bizobj.RxTrackingBO;
import gov.va.med.mhv.rxrefill.bizobj.PrescriptionBO;
import gov.va.med.mhv.rxrefill.service.RxTrackingCollectionServiceResponse;
import gov.va.med.mhv.rxrefill.service.RxTrackingService;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;
import gov.va.med.mhv.rxrefill.transfer.Prescription;
import gov.va.med.mhv.rxrefill.transfer.RxTracking;


import org.tigris.atlas.config.ConfigurationManager;
import org.tigris.atlas.messages.Messages;
import org.tigris.atlas.service.VoidServiceResponse;

/**
 * Service implementation class for the RxTracking service
 * @see gov.va.med.mhv.rxrefill.service.RxTrackingService
 */
public class RxTrackingServiceImpl implements RxTrackingService {

    private static final Log LOG = LogFactory.getLog(RxTrackingServiceImpl.class);
	private static final Properties PROPS = ConfigurationManager.getConfiguration("/mailCfg.properties");
	private static final String RXTRACKING_EMAILS = "rxtracking_emails";
	private static final String EMAIL_SUBJECT = "VA RX Shipment Notification";
	private static final String EMAIL_MESSAGE_CONTENT_TYPE = "text/html";
	private static final String BREAK = "<br/>";

	/**
	 * Execute the GetPrescriptionTrackingNumbers service
	 * @see gov.va.med.mhv.rxrefill.service.RxTrackingService#GetPrescriptionTrackingNumbers()
	 */
	public RxTrackingCollectionServiceResponse getPrescriptionTrackingNumbers(String prescriptionNumber, String stationNumber) {
		RxTrackingCollectionServiceResponse response = new RxTrackingCollectionServiceResponse();

		Collection presTrackingNumbers = RxTrackingBO.getPrescriptionTrackingBySiteAndRxNumber(prescriptionNumber, stationNumber);
		response.addRxTrackings(RxTrackingAssembler.getRxTrackingCollection(presTrackingNumbers));

		return response;
	}

	/**
	 * Execute the GetPrescriptionDataByTrackingNumber operation
	 *
	 * @param trackingNumber
	 * @return RxTrackingCollectionServiceResponse
	 */
	public RxTrackingCollectionServiceResponse getPrescriptionDataByTrackingNumber(String trackingNumber) {
		RxTrackingCollectionServiceResponse response = new RxTrackingCollectionServiceResponse();

		Collection presNumbersFromRxTracking = RxTrackingBO.getPrescriptionDataByTrackingNumber(trackingNumber);
		response.addRxTrackings(RxTrackingAssembler.getRxTrackingCollection(presNumbersFromRxTracking));

		return response;
	}

/*
    **
	 * Execute the ScheduleAppointmentReminders service
	 * @see gov.va.med.mhv.phr.service.AppointmentReminderBusinessService#ScheduleAppointmentReminders()
	 *
    @SuppressWarnings("unchecked")
	public VoidServiceResponse scheduleAppointmentReminders() {
		logDebugMessages("Got into AppointmentReminderBusinessServiceImpl's schedulerAppointmentReminders method");
		return ServiceDelegateFactory.createAppointmentReminderServiceDelegate().scheduleAsyncAppointmentReminders();
	}



	**
	 * Execute the ScheduleAsyncAppointmentReminders service
	 * @see gov.va.med.mhv.phr.service.AppointmentReminderBusinessService#ScheduleAsyncAppointmentReminders()
	 *
    @SuppressWarnings("unchecked")
	public VoidServiceResponse scheduleAsyncAppointmentReminders() {

		VoidServiceResponse response = new VoidServiceResponse();
		logDebugMessages("Got into AppointmentReminderBusinessServiceImpl's scheduleAsyncAppointmentReminders method");

		try{
			// The end date for the appointment reminders is 14 days
			//endDate14CurTime holds the date 14 days in future with the current time
			//endDate holds the date 14 days in the future with time set to the end of the day
			//11:59:59.  This end date will include all appointments that occur for the rest of the 14th
			//day....including appts after the current time.
			Timestamp endDate14CurTime = new Timestamp(AppointmentReminderUtil.getCurrentDate().getTime() + 14 * 24 * 60 * 60 * 1000);
			Timestamp endDate = AppointmentReminderUtil.get14DayEndDate(endDate14CurTime);

			// Calls scheduleAppointmentRemindersForDateRange method
			response = ServiceDelegateFactory.createAppointmentReminderServiceDelegate().scheduleAppointmentRemindersForDateRange(AppointmentReminderUtil.getCurrentTimeStamp(), endDate);
		}
		catch (Exception ex){
			logErrorMessages("Exception in scheduleAsyncAppointmentReminders(): " + ex.getLocalizedMessage());
		}
		finally{
            // Logging messages here, because this operation is intended to
            // be called asynchronously, such that messages would get lost
			if(response!=null){
				Messages scheduleAsyncAppointmentReminderMessages = response.getMessages();

				if(scheduleAsyncAppointmentReminderMessages != null && scheduleAsyncAppointmentReminderMessages.hasErrorMessages()){
					logErrorMessages("Got into scheduleAsyncAppointmentReminders() finally block");
					logMessages(response);
				}
			}

		} // end of finally block

		return response;

	}

*/

    public VoidServiceResponse processRxTrackingEmailNotifications() {
		logInfoMessages("Got into RxTrackingServiceImpl's processRxTrackingEmailNotifications method");
		return gov.va.med.mhv.rxrefill.service.delegate.ServiceDelegateFactory.createRxTrackingServiceDelegate().processAsyncRxTrackingEmailNotifications();
    }

	public VoidServiceResponse processAsyncRxTrackingEmailNotifications() {
		VoidServiceResponse response = new VoidServiceResponse();
		int maxCount = 0;

		logInfoMessages("Got into RxTrackingServiceImpl's processAsyncRxTrackingEmailNotifications method");

		try{
			UserProfileServiceDelegate userProfileService = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserProfileServiceDelegate();
			Collection userProfiles = userProfileService.getUserProfilesForRxTracking().getUserProfiles();

			if(userProfiles != null) {
				maxCount = userProfiles.size();
				logInfoMessages("processRxTrackingEmailNotifications number of users with rxtracking email pref set:" + maxCount);
			}

			if(userProfiles != null && !userProfiles.isEmpty()) {
				logInfoMessages("processRxTrackingEmailNotifications number of users with rxtracking email pref set:" + userProfiles.size());
				Timestamp startTime = new Timestamp(new Date().getTime());

				for (Object e: userProfiles) {
					UserProfile userProfile = (UserProfile) e;
					logInfoMessages("UserProfile Info:: ID:-" + userProfile.getId()+":: Email Address:-"+userProfile.getContactInfoEmail());


					PrescriptionServiceDelegate prescriptionService = gov.va.med.mhv.rxrefill.service.delegate.ServiceDelegateFactory.createPrescriptionServiceDelegate();
					Collection prescriptions = prescriptionService.getTrackingPrescriptionsForEmails(userProfile.getId()).getPrescriptions();
					for (Object ep: prescriptions) {
						Prescription prescription = (Prescription) ep;
						logInfoMessages("Prescription Info:: RxNumber:-" + prescription.getPrescriptionNumber()+":: Station Number:-"+prescription.getInstitution().getStationNumber());

						RxTrackingServiceDelegate trackingService = gov.va.med.mhv.rxrefill.service.delegate.ServiceDelegateFactory.createRxTrackingServiceDelegate();
						Collection trackingNumbers = trackingService.getPrescriptionTrackingNumbers(prescription.getPrescriptionNumber(), prescription.getInstitution().getStationNumber()).getRxTrackings();
						for (Object et: trackingNumbers) {
							RxTracking rx = (RxTracking) et;
							logInfoMessages("Tracking Info:: RxNumber:-" + rx.getTrackingNumber()+":: Date Loaded:-"+rx.getDateLoaded());
						}
					}

					/*
					* Calls the email service
					*/
					EmailServiceDelegate service = gov.va.med.mhv.core.service.delegate.ServiceDelegateFactory.createEmailServiceDelegate();

					//String bodyMessage = AppointmentReminderUtil.buildEmailBody(tableContent, reminderType, dateKey);
					//String bodyMessage = "Test Message";
					String bodyMessage = buildEmailBody();

					/*
					* Calls the sendMessageWithContentType method in the Email service (which is in mhv_core service)
					*/
					logInfoMessages("Sending email Notification");
					response = service.sendMessageWithContentType(PROPS.getProperty("rxTrackingNotifications.from.email"), EMAIL_SUBJECT, bodyMessage, userProfile.getContactInfoEmail().toString(), EMAIL_MESSAGE_CONTENT_TYPE);

					//emailSent = true;
				}
			}
		}
		catch (Exception ex){
			logErrorMessages("Exception in processAsyncRxTrackingEmailNotifications(): " + ex.getLocalizedMessage());
		}
		finally{
            // Logging messages here, because this operation is intended to
            // be called asynchronously, such that messages would get lost
			if(response!=null){
				Messages processAsyncRxTrackingEmailNotificationMessages = response.getMessages();

				if(processAsyncRxTrackingEmailNotificationMessages != null && processAsyncRxTrackingEmailNotificationMessages.hasErrorMessages()){
					logErrorMessages("Got into processAsyncRxTrackingEmailNotifications() finally block");
					//logMessages(response);
				}
			}

		} // end of finally block

		return response;

	}

    public static String buildEmailBody()
    {
        StringBuffer emailBody = new StringBuffer();

        emailBody.append("Dear VA Patient,");
        emailBody.append(BREAK);
        emailBody.append(BREAK);
        emailBody.append("This is a notification of prescription package(s) shipped from the VA Mail Order Pharmacy. Additional information can be found by logging into your <u>My Health<em><strong>e</strong></em>Vet</u> account and going to the Pharmacy menu.");
        emailBody.append(BREAK);
        emailBody.append(BREAK);
        emailBody.append("<b>Package(s) Shipped from VA Mail Order Pharmacy:</b>");
        //emailBody.append(tableContent.toString());
        emailBody.append(BREAK);
        emailBody.append(BREAK);
        emailBody.append(BREAK);
        emailBody.append("*Tracking Information last updated in My HealtheVet on <<DATE HERE>> [when data came from CDW]");
        emailBody.append(BREAK);
        emailBody.append(BREAK);
        emailBody.append("**If you have already received your package, please disregard this message.");
        emailBody.append(BREAK);
        emailBody.append(BREAK);
        emailBody.append(BREAK);
		emailBody.append("<table border='1' cellpadding='2' cellspacing='2'>");
		emailBody.append("<tr>");
        emailBody.append("<td>If you have questions about your prescription(s)</td>");
        emailBody.append("<td>Please contact your local VA Pharmacy.</td>");
		emailBody.append("</tr>");
		emailBody.append("<tr>");
        emailBody.append("<td>If you received this message in error or need technical assistance</td>");
        emailBody.append("<td>Please contact the My HealtheVet Help Desk <a href='https://www.myhealth.domain/mhv-portal-web/anonymous.portal?_nfpb=true&_nfto=false&_pageLabel=contactMHVHome' target='_blank'> online</a> or by phone at 1-877-327-0022 Monday - Friday, 7:00 a.m. - 7:00 p.m. CT or 1-800-877-8339 (TTY)</td>");
		emailBody.append("</tr>");
		emailBody.append("</table>");
        emailBody.append(BREAK);

        return emailBody.toString();
    }


    public static void logInfoMessages(String msg)
    {
    	LOG.info(msg);
    	System.out.println(">>>>>>>>>>"+msg);
    }

    public static void logDebugMessages(String msg)
    {
    	LOG.debug(msg);
    	System.out.println(">>>>>>>>>>"+msg);
    }

    public static void logErrorMessages(String msg)
    {
    	LOG.error(msg);
    	System.out.println(">>>>>>>>>>"+msg);
    }
}