package gov.va.med.mhv.rxrefill.service.impl.util;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Hashtable;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.messages.MessageUtils;
import org.tigris.atlas.service.ServiceResponse;

import gov.va.med.mhv.rxrefill.bizobj.RequestBO;
import gov.va.med.mhv.rxrefill.enumeration.RequestFunctionEnumeration;
import gov.va.med.mhv.rxrefill.enumeration.RequestStatusEnumeration;
import gov.va.med.mhv.rxrefill.service.RequestServiceResponse;
import gov.va.med.mhv.rxrefill.service.ServiceFactory;
import gov.va.med.mhv.rxrefill.transfer.Institution;
import gov.va.med.mhv.rxrefill.transfer.Prescription;
import gov.va.med.mhv.rxrefill.transfer.PrescriptionRequest;
import gov.va.med.mhv.rxrefill.transfer.Request;
import gov.va.med.mhv.rxrefill.transfer.RequestAttempt;
import gov.va.med.mhv.rxrefill.transfer.TransferObjectFactory;
import gov.va.med.mhv.rxrefill.util.RequestUtils;
import gov.va.med.mhv.usermgmt.transfer.Facility;
import gov.va.med.mhv.usermgmt.transfer.Patient;

public class RxRequestDispatcher {

	private static Log LOG = LogFactory.getLog(RxRequestDispatcher.class);

	private static boolean testInstitution(Institution institution) {
		boolean result = ((institution != null)
				&& institution.getIsTreatment().equalsIgnoreCase("Y")
				&& institution.getIsActive().equalsIgnoreCase("Y"));
		return result;
	}

	public int dispatchRefillRequests(Patient patient,
		Prescription[] prescriptions, ServiceResponse response)
    {
		//System.out.println(" 70 >>>>>>>>>>>> in side dispatchRefillRequests - begining");
		RxRequestStrategy strategy = new RxRequestStrategy();
		Hashtable<Long, List<Prescription>> mappedPrescriptions =
			new Hashtable<Long, List<Prescription>>();
		for (Prescription prescription: prescriptions) {
			Long institutionId = prescription.getInstitution().
				getInstitutionId();
			if (!mappedPrescriptions.keySet().contains(institutionId)) {
				mappedPrescriptions.put(institutionId,
					new ArrayList<Prescription>());
			}
			mappedPrescriptions.get(institutionId).add(prescription);
		}
		List<Request> requests = new ArrayList<Request>();
		for (Long institutionId: mappedPrescriptions.keySet()) {
			List<Prescription> list = mappedPrescriptions.get(institutionId);
			Prescription prescription = list.iterator().next();
			Institution institution = prescription.getInstitution();
			if (testInstitution(institution)
				&& strategy.checkSendRefillRequest(patient, institution))
			{
				Timestamp now = UtilityMethods.dateToTimestamp(new Date());
				Request request = createRefillRequest(now, list);
				requests.add(request);
				//System.out.println(" 71 >>>>>>>>>>>> in side dispatchRefillRequests - calling dispatchRequest - 2");
				dispatchRequest(patient, request);
			} else if (institution == null) {
				LOG.error("Request information for prescription '"
					+ prescription.getPrescriptionNumber()
					+ "', but no associated institution."
					+ " Please check INSTITUTION(S) table.");
				response.getMessages().addMessage(MessageUtils.
					createErrorMessage("error.vista.systemfailure", null, null));
			} else {
				// User cannot change situation, so not alerting user,
				// just ignroring.
				LOG.error("Request information for patient " + patient.getId()
					+ " from institution (Name: "  + institution.getName()
					+ "; SN:" + institution.getStationNumber()
					+ ") which is not a treatment nor an active facility");
			}
		}
		collectRequestStatuses(requests, response);
		return requests.size();
	}

	/**
	 * @param createdDate
	 * @param prescriptions
	 * @return
	 */
	private Request createRefillRequest(Timestamp now,
		List<Prescription> prescriptions)
	{
		Prescription prescription = prescriptions.get(0);

		Institution institution = prescription.getInstitution();
		Long userId = prescription.getUserId();

		Timestamp fromDate = UtilityMethods.dateToTimestamp(UtilityMethods.
			getDateWithoutTime(now));
		Timestamp toDate = UtilityMethods.dateToTimestamp(
			new Date(fromDate.getTime() + 24 * 60 * 60 * 1000));

		List requests = RequestBO.getRequests(userId, RequestUtils.
			REFILL_FUNCTION, institution.getInstitutionId(), fromDate, toDate);
		Request request = (requests.isEmpty())
			? createRefillRequest(userId, now, institution)  // create new
			: ((RequestBO) requests.get(0)).getRequestValues(); // use existing

		request.setStatus(RequestUtils.INPROGRESS_STATUS);
		request.setModifiedDate(now);
		addAttempt(request, now, RequestUtils.INPROGRESS_STATUS);
		addPrescriptions(request, prescriptions, now,
			RequestUtils.INPROGRESS_STATUS);

		RequestServiceResponse response = ServiceFactory.
			createEntityMaintenanceService().save(request);
		if (response.getMessages().hasErrorMessages()) {
			LOG.debug("Error messages in response.");
		}
		request = response.getRequest();
		return request;
	}

	private Request createRefillRequest(Long userId, Timestamp now,
		Institution institution)
	{
		// create request
		Request request = TransferObjectFactory.createRequest();
		request.setUserId(userId);
		request.setCreatedDate(now);
		request.setInstitution(institution);
		request.setRequestFunction(RequestUtils.REFILL_FUNCTION);
		return request;
	}

	private void addAttempt(Request request, Timestamp now,
		RequestStatusEnumeration status)
	{
		// create prescription request attempt
		RequestAttempt attempt = TransferObjectFactory.createRequestAttempt();
		attempt.setRequest(request);
		attempt.setStatus(status);
		attempt.setCreatedDate(now);
		attempt.setModifiedDate(now);
		request.addRequestAttempt(attempt);
	}

	private void addPrescriptions(Request request, List prescriptions,
		Timestamp now, RequestStatusEnumeration status)
	{
		for (Object p: prescriptions) {
			Prescription prescription = (Prescription) p;
			// create prescription request
			PrescriptionRequest prescriptionRequest = TransferObjectFactory.
				createPrescriptionRequest();
			prescriptionRequest.setRequest(request);
			prescriptionRequest.setCreatedDate(now);
			prescriptionRequest.setModifiedDate(now);
			prescriptionRequest.setPrescription(prescription);
			prescriptionRequest.setStatus(status);
			request.addPrescriptionRequest(prescriptionRequest);
		}
	}

	private Request[] multiplexExtractRequests(Patient patient,
			RequestFunctionEnumeration function, ServiceResponse response)
	{
		//System.out.println(" 100 >>>>>>>>>>>> in side multiplexExtractRequests - begining");
		RxRequestStrategy strategy = new RxRequestStrategy();
		List<Request> requestList = new ArrayList<Request>();
		Request[] result = new Request[0];
		for (Object f: patient.getFacilitys()) {
			Facility facility = (Facility) f;
			Institution institution = UtilityMethods.getInstitution(facility
					.getName());
			if (testInstitution(institution)) {
				boolean exhaustedTryes = true;
				Request request = null;
				final Date now = new Date();
				if ((RequestUtils.PROFILE_FUNCTION.equals(function)
					  && strategy.checkSendHistoricalUpdateRequest(patient,
							institution))
					|| (RequestUtils.STATUS_FUNCTION.equals(function)
						&& strategy.checkSendActiveUpdateRequest(patient,
							 institution)))
				{
					// if there is prior profile or active extract and
					// corresponding function requested - create update request
					Date cutoffDate = strategy.checkRxExtractCutOffDate(
							patient, institution);
					//System.out.println(" 101 >>>>>>>>>>>> in side multiplexExtractRequests - calling createExtractRequest -1");
					request = createExtractRequest(patient, institution, now,
						cutoffDate, RequestUtils.UPDATE_FUNCTION);
				} else if (RequestUtils.STATUS_FUNCTION.equals(function)
						&& strategy.checkSendStatusRequest(patient,
						institution))
				{
					// if request for active and there is no prior profile or
					// active extracts
					//System.out.println(" 102 >>>>>>>>>>>> in side multiplexExtractRequests - calling createExtractRequest -2");
					request = createExtractRequest(patient, institution, now,
						null, function);
				}
				if (request != null) {
					requestList.add(request);
					exhaustedTryes = false;
				}
				if (strategy.checkSendProfileRequest(patient, institution)) {
					// if there is no profile extract - create profile request
					// for any function
					//System.out.println(" 103 >>>>>>>>>>>> in side multiplexExtractRequests - calling createExtractRequest -3");
					request = createExtractRequest(patient, institution, now,
						null, RequestUtils.PROFILE_FUNCTION);
					if (request != null) {
						requestList.add(request);
						exhaustedTryes = false;
					}
				}
				if (exhaustedTryes) {
					addExhaustedTriesMessage(response, patient, now,
						institution, function);
				}
			}
			else if (institution == null) {
				LOG.error("Building request information for facility '"
					+ facility.getName()
					+ "', but not registered as an institution. "
					+ "Please check INSTITUTION(S) table.");
				response.getMessages().addMessage(MessageUtils.
					createErrorMessage("error.vista.systemfailure", null, null));
			} else {
				// User cannot change situation, so not alerting user,
				// just ignroring.
				LOG.warn("Request information for patient " + patient.getId()
					+ " from institution (Name: "  + institution.getName()
					+ "; SN:" + institution.getStationNumber()
					+ ") which is not a treatment nor an active facility");
			}
		}
		result = requestList.toArray(new Request[0]);
		return result;
	}

	private void addExhaustedTriesMessage(ServiceResponse response,
		Patient patient, Date createdDate, Institution institution,
		RequestFunctionEnumeration function)
	{
		//non-persistent request for uniformity
		Request nonPersistedRequest = TransferObjectFactory.createRequest();
		nonPersistedRequest.setUserId(patient.getUserProfile().getId());
		nonPersistedRequest.setCreatedDate(UtilityMethods.dateToTimestamp(
			createdDate));
		nonPersistedRequest.setInstitution(institution);
		nonPersistedRequest.setRequestFunction(function);
		nonPersistedRequest.setExceededMaxFailedAttempts(Boolean.TRUE);
        // addProperty so new messages are allowed to the HashSet
		// of InformationalMessages
		response.getMessages().addMessage(MessageUtils.
			createInformationalMessage(RxRequestMessages.REQUEST_STATUS,
				new String[] {nonPersistedRequest.getInstitution().getName()},
				new Object[] {nonPersistedRequest}));
	}

	/**
	 * @param patient
	 * @param institution
	 * @param createdDate
	 * @return
	 */
	private Request createExtractRequest(Patient patient,
			Institution institution, Date createdDate, Date cutOffDate,
			RequestFunctionEnumeration function)
	{
		//System.out.println(" 110 >>>>>>>>>>>> in side createExtractRequest - begining");
		if (LOG.isDebugEnabled()) {
			LOG.debug("Adding " + function.getName() + " request for site "
				+ institution.getStationNumber() + "...");
		}

		final Timestamp createdAt = UtilityMethods.dateToTimestamp(createdDate);
		final Timestamp cutoffAt = UtilityMethods.dateToTimestamp(cutOffDate);
		Long userId = (patient.getUserProfile().getId());
		final Timestamp fromDate = UtilityMethods.dateToTimestamp(
			UtilityMethods.getDateWithoutTime(createdDate));
		final Timestamp toDate = UtilityMethods.dateToTimestamp(
			new Date(fromDate.getTime() + 24 * 60 * 60 * 1000));
		List requests = RequestBO.getRequests(userId, function, institution.
			getInstitutionId(), fromDate, toDate);
		Request request = null;
		if (requests.isEmpty()) {
			// create request
			request = TransferObjectFactory.createRequest();
			request.setUserId(userId);
			request.setCreatedDate(createdAt);
			request.setInstitution(institution);
			request.setRequestFunction(function);
		} else {
			// use existing
			request = ((RequestBO) requests.get(0)).getRequestValues();
		}

		request.setStatus(RequestUtils.INPROGRESS_STATUS);
		//System.out.println(" 111 >>>>>>>>>>>> in side createExtractRequest - calling setModifiedDate :: "+createdAt);
		request.setModifiedDate(createdAt);
		//System.out.println(" 112 >>>>>>>>>>>> in side createExtractRequest - calling setStartDate :: "+cutoffAt);
		request.setStartDate(cutoffAt);
		// create prescription request attempt
		RequestAttempt attempt = TransferObjectFactory.createRequestAttempt();
		attempt.setRequest(request);
		attempt.setStatus(RequestUtils.INPROGRESS_STATUS);
		attempt.setCreatedDate(createdAt);
		attempt.setModifiedDate(createdAt);
		request.addRequestAttempt(attempt);

		RequestServiceResponse response = ServiceFactory.
			createEntityMaintenanceService().save(request);
		if (LOG.isDebugEnabled() && response.getMessages().hasErrorMessages()) {
			LOG.debug("Error messages in response "
				+ response.getMessages().getErrorMessages());
		}
		request = response.getRequest();
		return request;
	}

	public int dipatchExtractRequests(Patient patient, ServiceResponse response) {
		//System.out.println(" 80 >>>>>>>>>>>> in side dipatchExtractRequests - profile");
		return dispatchRequest(patient, response, RequestUtils.PROFILE_FUNCTION);
	}

	public int dipatchStatusRequests(Patient patient, ServiceResponse response) {
		//System.out.println(" 81 >>>>>>>>>>>> in side dipatchExtractRequests - status");
		return dispatchRequest(patient, response, RequestUtils.STATUS_FUNCTION);
	}

	private int dispatchRequest(Patient patient, ServiceResponse response,
		RequestFunctionEnumeration function)
	{
		//System.out.println(" 90 >>>>>>>>>>>> in side dispatchRequest - calling multiplexExtractRequests");
		Request[] requests = multiplexExtractRequests(patient, function,
			response);
		for (Request request: requests) {
			dispatchRequest(patient, request);
		}
		collectRequestStatuses(Arrays.asList(requests), response);
		return requests.length;
	}

	private void dispatchRequest(Patient patient, Request request) {
		//System.out.println(" 91 >>>>>>>>>>>> in side dispatchRequest - calling sendRequest");
        new RxRequestHelper().sendRequest(patient, request);
	}

	private void collectRequestStatuses(List<Request> requests,
		ServiceResponse response)
	{
		if (requests.isEmpty()) {
			return;
		}
		for (Request cachedRequest: requests) {
			if (!RequestUtils.isInProgress(cachedRequest)) {
				continue;
			}
			RequestBO requestBO = RequestBO.findByPrimaryKey(cachedRequest.
				getRequestPK());
			if (requestBO != null) {
				Request request = requestBO.getRequestValues();
				if (request != null) {
					if (!RequestUtils.isInProgress(request)) {
						cachedRequest.setStatus(request.getStatus());
                    }
					cachedRequest.setModifiedDate(request.getModifiedDate());
				}
			}
		}

		for (Request request: requests) {
			response.getMessages().addMessage(MessageUtils.
				createInformationalMessage(RxRequestMessages.REQUEST_STATUS,
		            // add property so messages are allowed for various
					// institutions
					new String[] { request.getInstitution().getName() },
					new Object[] { request } ));
		}
	}

}
