package gov.va.med.mhv.rxrefill.service.impl.util;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.messages.Message;
import org.tigris.atlas.messages.Messages;

import gov.va.med.mhv.rxrefill.bizobj.PrescriptionBO;
import gov.va.med.mhv.rxrefill.enumeration.RequestStatusEnumeration;
import gov.va.med.mhv.rxrefill.service.PrescriptionCollectionServiceResponse;
import gov.va.med.mhv.rxrefill.service.ServiceFactory;
import gov.va.med.mhv.rxrefill.transfer.Institution;
import gov.va.med.mhv.rxrefill.transfer.Prescription;
import gov.va.med.mhv.rxrefill.transfer.PrescriptionRequest;
import gov.va.med.mhv.rxrefill.transfer.Request;
import gov.va.med.mhv.rxrefill.transfer.RequestAttempt;
import gov.va.med.mhv.rxrefill.util.PrescriptionUtils;
import gov.va.med.mhv.rxrefill.util.RequestUtils;
import gov.va.med.mhv.usermgmt.transfer.Patient;

public class RxRequestHelper {

	private static Log log = LogFactory.getLog(RxRequestHelper.class);

	private void postProcessRefill(Prescription inPrescription,
		Prescription storedPrescription, PrescriptionRequest prescriptionRequest)
	{
		final int MHV_STATUS_FILLED = 1;
		final int MHV_STATUS_PROCESSED = -1;

		int mhvStatus = inPrescription.getMhvStatus().getValue();
		if (mhvStatus == MHV_STATUS_FILLED || mhvStatus == MHV_STATUS_PROCESSED)
		{
			prescriptionRequest.setStatus(RequestUtils.SUCCESSFUL_STATUS);
		} else {
			prescriptionRequest.setStatus(RequestUtils.REJECTED_STATUS);
			if (log.isDebugEnabled()) {
				log.debug("Prescription refill request for prescription "
					+ storedPrescription.getPrescriptionNumber()
					+ " at facility "
					+ storedPrescription.getInstitution().getInstitutionId()
					+ " was rejected with a status of "
					+ inPrescription.getIen() + " and mhv status code of "
					+ mhvStatus);
			}
		}
	}

	private void cloneRefillPrescriptionProperties(
		Prescription fromPrescription, Prescription toPrescription)
	{
		toPrescription.setMhvStatus(fromPrescription.getMhvStatus());
		if (fromPrescription.getStatus() != null) {
			toPrescription.setStatus(fromPrescription.getStatus());
			if (PrescriptionUtils.isSubmitted(fromPrescription)) {
				toPrescription.setLastRefillSubmittedDate(new Date());
			}
		}
	}

	private void cloneExtractPrescriptionProperties(
		Prescription fromPrescription, PrescriptionBO toPrescription)
	{
		toPrescription.setArchivedDate(fromPrescription.getArchivedDate());
		toPrescription.setDaysSupply(fromPrescription.getDaysSupply());
		toPrescription.setDivision(fromPrescription.getDivision());
		toPrescription.setDivisionName(fromPrescription.getDivisionName());
		toPrescription.setDrugName(fromPrescription.getDrugName());
		toPrescription.setExpirationCancelDate(fromPrescription
			.getExpirationCancelDate());
		toPrescription.setIen(fromPrescription.getIen());
		toPrescription.setIssueDateTime(fromPrescription.getIssueDateTime());
		toPrescription.setLastFillDate(fromPrescription.getLastFillDate());
		toPrescription.setMailWindow(fromPrescription.getMailWindow());
		toPrescription.setMhvStatus(fromPrescription.getMhvStatus());
		if (fromPrescription.getMhvStatusDate() != null) {
			int mhvStatus = (fromPrescription.getMhvStatus() != null
				? fromPrescription.getMhvStatus().getValue() : 0);
			if (mhvStatus == 1 || mhvStatus == 2) {
				toPrescription.setMhvStatusDate(fromPrescription
					.getMhvStatusDate());
			}
		}
		toPrescription.setNumberOfRefills(fromPrescription.getNumberOfRefills());
		toPrescription.setPlacerOrderNumber(fromPrescription
			.getPlacerOrderNumber());
		// toPrescription.setPrescriptionId(fromPrescription.getPrescriptionId());
		toPrescription.setProviderFirstName(fromPrescription
			.getProviderFirstName());
		toPrescription.setProviderId(fromPrescription.getProviderId());
		toPrescription.setProviderLastName(fromPrescription
			.getProviderLastName());
		toPrescription.setQuantity(fromPrescription.getQuantity());
		toPrescription.setReleaseDateTime(fromPrescription.getReleaseDateTime());
		toPrescription.setRemarks(fromPrescription.getRemarks());
		toPrescription.setSig(fromPrescription.getSig());
		if (PrescriptionUtils.isMHVPending(fromPrescription)
			&& PrescriptionUtils.isActive(fromPrescription))
		{
			// prescription is waiting in mhv->vista queue but status shows as
			// active - then it is submitted
			toPrescription.setStatus(PrescriptionUtils.SUBMITTED_STATUS);
		} else {
			toPrescription.setStatus(fromPrescription.getStatus());
		}
	}

	private List processResponse(Request request, RequestAttempt attempt,
		Collection<Prescription> inPrescriptions)
	{
		//System.out.println(" 1 >>>>>>>>>>>>> in side sendRequest of RxRequestHelper - begining");

		return (RequestUtils.isRefillRequest(request))
			? processRefillResults(request, attempt, inPrescriptions)
		    : processExtractResults(request, attempt, inPrescriptions);
	}

	private List<Prescription> processRefillResults(Request request,
		RequestAttempt attempt, Collection<Prescription> inPrescriptions)
	{
		final Timestamp now  = UtilityMethods.dateToTimestamp(new Date());
		List<Prescription> outPrescriptions = new ArrayList<Prescription>();
		if (log.isDebugEnabled()) {
			log.debug("Process " + (inPrescriptions != null ? inPrescriptions.
				size() : 0) + " refill results");
		}
		for (Object p: inPrescriptions) {
			Prescription inPrescription = (Prescription) p;
			for (Object r: request.getPrescriptionRequests()) {
				PrescriptionRequest prescriptionRequest =
					(PrescriptionRequest) r;
				Prescription storedPrescription = prescriptionRequest
					.getPrescription();
				if (inPrescription.getPrescriptionNumber().equals(
					storedPrescription.getPrescriptionNumber()))
				{
					// match found
					cloneRefillPrescriptionProperties(inPrescription,
						storedPrescription);
					storedPrescription.setModifiedDate(now);
					storedPrescription = ServiceFactory.
						createEntityMaintenanceService().save(
						storedPrescription).getPrescription();
					postProcessRefill(inPrescription, storedPrescription,
						prescriptionRequest);
					outPrescriptions.add(storedPrescription);
				}
			}
		}
		attempt.setModifiedDate(now);
		attempt.setStatus(RequestUtils.SUCCESSFUL_STATUS);
		request.setModifiedDate(now);
		request.setStatus(RequestUtils.SUCCESSFUL_STATUS);

		ServiceFactory.createEntityMaintenanceService().save(request);
		return outPrescriptions;
	}

	private void checkForAbsentActive(Request request,
		Collection<Prescription> inPrescriptions)
	{
		Map<String, Prescription> incomingPrescriptions =
			new HashMap<String, Prescription>();
		for (Prescription prescription : inPrescriptions) {
			incomingPrescriptions.put(prescription.getPrescriptionNumber(),
				prescription);
		}

		List userActivePrescriptions = PrescriptionBO
			.getActivePatientPrescriptions(request.getUserId());
		for (Object p: userActivePrescriptions) {
			PrescriptionBO prescription = (PrescriptionBO) p;
			if (hasSameInstitution(request, prescription)
				&& !incomingPrescriptions.containsKey(prescription.
					getPrescriptionId()))
			{
				prescription.setStatus(PrescriptionUtils.UNKNOWN_STATUS);
			}
		}
	}

	private boolean hasSameInstitution(Request request,
		PrescriptionBO prescription)
	{
		return (prescription != null) && (request != null)
		    && (prescription.getInstitution() != null)
		    && (prescription.getInstitution().getInstitutionId() != null)
		    && (request.getInstitution() != null)
		    && prescription.getInstitution().getInstitutionId().equals(
				request.getInstitution().getInstitutionId());
	}

	private List<Prescription> processExtractResults(Request request,
		RequestAttempt attempt, Collection<Prescription> inPrescriptions)
	{
		//System.out.println(" 2 >>>>>>>>>>>>> in side processExtractResults of RxRequestHelper - begining");
		Timestamp now = UtilityMethods.dateToTimestamp(new Date());
		int inPrescriptionCount = inPrescriptions.size();
		List<Prescription> outPrescriptions = new ArrayList<Prescription>(
			inPrescriptionCount);
		if (log.isDebugEnabled()) {
			log.debug("ExtractResults contains " + inPrescriptionCount
				+ " records");
		}
		Long userId = request.getUserId();
		Institution institution = request.getInstitution();
		checkForAbsentActive(request, inPrescriptions);
		for (Prescription inPrescription: inPrescriptions) {
			List currentPrescriptions = PrescriptionBO.
				getPrescriptionByPatientSiteNumber(userId, institution.getId(),
				inPrescription.getPrescriptionNumber());
			if (!currentPrescriptions.isEmpty()) {
				//System.out.println(" 3 >>>>>>>>>>>>> in side processExtractResults of RxRequestHelper - if block");
				PrescriptionBO currentPrescription = ((PrescriptionBO)
					currentPrescriptions.get(0));
				cloneExtractPrescriptionProperties(inPrescription,
					currentPrescription);
				currentPrescription.setModifiedDate(now);
				outPrescriptions.add(currentPrescription.getPrescriptionValues());
			} else {
				//System.out.println(" 4 >>>>>>>>>>>>> in side processExtractResults of RxRequestHelper - else block ");
				inPrescription.setCreatedDate(now);
				inPrescription.setModifiedDate(now);
				inPrescription.setUserId(userId);
				inPrescription.setInstitution(institution);
				outPrescriptions.add(ServiceFactory.
					createEntityMaintenanceService().save(inPrescription).
					getPrescription());
			}
		}
		attempt.setModifiedDate(now);
		attempt.setStatus(RequestUtils.SUCCESSFUL_STATUS);
		//System.out.println(" 5 >>>>>>>>>>>>> in side processExtractResults of RxRequestHelper ::"+ now);
		request.setModifiedDate(now);
		request.setStatus(RequestUtils.SUCCESSFUL_STATUS);

		request = ServiceFactory.createEntityMaintenanceService().save(request)
			.getRequest();
		return outPrescriptions;
	}

	private void processBadResponse(Request request, RequestAttempt attempt,
		RequestStatusEnumeration newStatus)
	{
		final Timestamp now = UtilityMethods.dateToTimestamp(new Date());
		if (attempt != null) {
			attempt.setStatus(newStatus);
			attempt.setModifiedDate(now);
		}
		request.setStatus(newStatus);
		//System.out.println(" 6 >>>>>>>>>>>>> in side processBadResponse of RxRequestHelper ::"+ now);
		request.setModifiedDate(now);
		ServiceFactory.createEntityMaintenanceService().save(request);
		if (RequestUtils.isRefillRequest(request)) {
			// update prescription requests
			for (Object r: request.getPrescriptionRequests()) {
				PrescriptionRequest prescriptionRequest =
					(PrescriptionRequest) r;
				prescriptionRequest.setStatus(newStatus);
				ServiceFactory.createEntityMaintenanceService().save(
					prescriptionRequest);
			}
		}
	}

	@SuppressWarnings("unchecked")
	public PrescriptionCollectionServiceResponse sendRequest(Patient patient,
		Request request)
	{
		//System.out.println(" 0 >>>>>>>>>>>>> in side sendRequest of RxRequestHelper - begining");
		RequestAttempt attempt = null;
		for (Object a: request.getRequestAttempts()) {
			RequestAttempt tmp = (RequestAttempt) a;
			if (RequestUtils.isInProgress(tmp)) {
				// there may be only one in-progress attempt
				attempt = tmp;
				break;
			}
		}

		PrescriptionCollectionServiceResponse response = null;
		PrescriptionCollectionServiceResponse re = ServiceFactory
			.createRxRequestVistaService().sendRxRequest(patient, request);
		response = new PrescriptionCollectionServiceResponse();
		Messages messages = re.getMessages();
		if (messages.getInformationalMessageCount() > 0) {
			for (Object m: messages.getInformationalMessages()) {
				Message message = (Message) m;
				if (RxRequestMessages.isMessagingError(message)) {
					processBadResponse(request, attempt, RequestUtils.
						FAILED_STATUS);
					return response;
				} else if (RxRequestMessages.isStatusError(message)) {
					processBadResponse(request, attempt, RequestUtils.
						REJECTED_STATUS);
					return response;
				}
			}
		}
		response.getMessages().addMessages(messages);

		List resultPrescriptions = processResponse(request, attempt, re
			.getPrescriptions());
		for (Object o: resultPrescriptions) {
			Prescription prescription = (Prescription) o;
			response.addPrescription(prescription);
		}
		return response;
	}

}
