/**
 * 
 */
package gov.va.med.mhv.rxrefill.service.impl.util;

import gov.va.med.mhv.rxrefill.transfer.Request;

import org.tigris.atlas.messages.Message;
import org.tigris.atlas.messages.MessageUtils;

/**
 * @author Rob Proper
 *
 */
public final class RxRequestMessages {
	
	public static final String REQUEST_STATUS = "prescriptionservice.request";

	public static final String MESSAGING_ERROR = 
		"rxrequestvistaservice.messaging.error";
	
	public static final String STATUS_ERROR = 
		"rxrequestvistaservice.status.error";
	
	public static final String UNEXPECTED_ERROR =
		"rxrequestvistaservice.unexpected.error";

	/**
	 * 
	 */
	private RxRequestMessages() {
		// Hide for Utility class
	}

	public static boolean isMessagingError(Message message) {
		return (message != null) 
			&& (MESSAGING_ERROR.equalsIgnoreCase(message.getKey())
		       || UNEXPECTED_ERROR.equalsIgnoreCase(message.getKey()));
	}

	public static boolean isStatusError(Message message) {
		return (message != null) && STATUS_ERROR.equalsIgnoreCase(message.
			getKey());
	}

	public static Message createStatusErrorMessage(Request request) {
		return MessageUtils.createInformationalMessage(
				RxRequestMessages.STATUS_ERROR,
				new String[] { identify(request) },
				new Object[] { request });
	}
	

	public static Message createMessagingErrorMessage(Request request) {
		return MessageUtils.createInformationalMessage(
				RxRequestMessages.MESSAGING_ERROR,
				new String[] { identify(request) },
				new Object[] { request });
	}
	
	public static Message createUnexpectedErrorMessage(Request request) {
		return MessageUtils.createInformationalMessage(
				RxRequestMessages.UNEXPECTED_ERROR,
				new String[] { identify(request) },
				new Object[] { });
	}
	
	private static String identify(Request request) {
		return (request != null) ? "" + request.getRequestId() : "<unknown>";
	}
	
}
