package gov.va.med.mhv.rxrefill.service.impl.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.Hashtable;
import java.util.List;

import gov.va.med.mhv.rxrefill.bizobj.RequestBO;
import gov.va.med.mhv.rxrefill.transfer.Institution;
import gov.va.med.mhv.rxrefill.transfer.Request;
import gov.va.med.mhv.rxrefill.transfer.RequestAttempt;
import gov.va.med.mhv.rxrefill.util.RequestUtils;
import gov.va.med.mhv.rxrefill.util.RxRefillProperties;
import gov.va.med.mhv.usermgmt.transfer.Facility;
import gov.va.med.mhv.usermgmt.transfer.Patient;

public class RxRequestStrategy {

	public static final String requests_per_day_mpi_policy = "requests-per-day";

	public static final String request_time_window_mpi_policy = "request-time-window";

	private Hashtable<Long, Boolean> htHadSuccessfulActiveExtract;
	private Hashtable<Long, Boolean> htHadSuccessfulProfileExtract;
	private Hashtable<Long, Long> htTodayRxRequests;
	private Hashtable<Long, Long> htTodayRxConsecutiveFailures;
	private Hashtable<Long, Date> htSuccessfulExtractRequests;
	private Hashtable<Long, Long> htTodayRxProfileFailures;
	
	public static long getMaxTotalRxRequests() {
		return RxRefillProperties.getInstance().getMaxConsecutiveRequests();
	}

	public static long getMaxConsecutiveRxFailures() {
		return RxRefillProperties.getInstance().getMaxConsecutiveFailures();
	}
	
	public static long getMaxRxProfileFailures() {
		// TODO - write implementation
		return 1;
	}

	public void initializeRxLogicData(Patient patient) {
		initializeRxLogicData(patient, false);
	}

	@SuppressWarnings("unchecked")
	public void initializeRxLogicData(Patient patient, boolean force) {
		if ((htTodayRxRequests != null) 
			&& (htTodayRxConsecutiveFailures != null)
			&& (htHadSuccessfulProfileExtract != null)
			&& (htSuccessfulExtractRequests != null)
			&& (htTodayRxProfileFailures != null)  
			&& !force) 
		{
			return;
		}

		htHadSuccessfulActiveExtract = new Hashtable<Long, Boolean>();
		htHadSuccessfulProfileExtract = new Hashtable<Long, Boolean>();
		htTodayRxRequests = new Hashtable<Long, Long>();
		htTodayRxConsecutiveFailures = new Hashtable<Long, Long>();
		htSuccessfulExtractRequests = new Hashtable<Long, Date>();
		htTodayRxProfileFailures = new Hashtable<Long, Long>();
		
		for (Object r: RequestBO.getRequestsForPatientByFunction(patient.
				getUserProfile().getId(), RequestUtils.STATUS_FUNCTION)) {
			Request request = ((RequestBO) r).getRequestValues();
			Long institutionId = request.getInstitution().getInstitutionId();
			if (RequestUtils.hasSucceeded(request)) {
				if (!htHadSuccessfulActiveExtract.containsKey(institutionId)) {
					htHadSuccessfulActiveExtract.put(institutionId,
						new Boolean(true));
				}
			}
		}

		for (Object r: RequestBO.getRequestsForPatientByFunction(
					patient.getUserProfile().getId(), 
					RequestUtils.PROFILE_FUNCTION)) 
		{
			Request request = ((RequestBO) r).getRequestValues();
			Long institutionId = request.getInstitution().getInstitutionId();
			if (RequestUtils.hasSucceeded(request)) {
				if (!htHadSuccessfulProfileExtract.containsKey(institutionId)) {
					htHadSuccessfulProfileExtract.put(institutionId,
						new Boolean(true));
				}
			}
		}

		Date date = new Date();
		Date fromDate = UtilityMethods.getDateWithoutTime(date);
		Date toDate = new Date(fromDate.getTime() + 24 * 60 * 60 * 1000);
		List<Request> requestList = new ArrayList<Request>();
		List allAttempts = new ArrayList();
		String facilityName;
		for (Object f: patient.getFacilitys()) {
			Facility facility = (Facility) f;
			facilityName = facility.getName();
			Institution institution = UtilityMethods.getInstitution(
				facilityName);
			if (institution ==null)
			{	
				continue;
			}
			Long institutionId = institution.getInstitutionId();
			List profileRequests = RequestBO.getRequests(patient.
				getUserProfile().getId(), RequestUtils.PROFILE_FUNCTION, 
				institutionId, UtilityMethods.dateToTimestamp(fromDate), 
				UtilityMethods.dateToTimestamp(toDate));
			for (Object r: profileRequests) {
				Request request = ((RequestBO) r).getRequestValues();
				allAttempts.addAll(request.getRequestAttempts());
				requestList.add(request);
			}

			List statusRequests = RequestBO.getRequests(patient.
				getUserProfile().getId(), RequestUtils.STATUS_FUNCTION, 
				institutionId, UtilityMethods.dateToTimestamp(fromDate), 
				UtilityMethods.dateToTimestamp(toDate));
			for (Object r: statusRequests) {
				Request request = ((RequestBO) r).getRequestValues();
				allAttempts.addAll(request.getRequestAttempts());
				requestList.add(request);
			}

			List updateRequests = RequestBO.getRequests(patient.
				getUserProfile().getId(), RequestUtils.UPDATE_FUNCTION, 
				institutionId, UtilityMethods.dateToTimestamp(fromDate), 
				UtilityMethods.dateToTimestamp(toDate));
			for (Object r: updateRequests) {
				Request request = ((RequestBO) r).getRequestValues();
				allAttempts.addAll(request.getRequestAttempts());
				requestList.add(request);
			}

			Comparator comparator = getRequestAttemptCompator();
			Collections.sort(allAttempts, comparator);
			long totalRequests = 0;
			long successiveFailures = 0;
			long profileFailures = 0;
			boolean prevFailure = false;
			boolean currFailure = false;
			for (Object ra: allAttempts) {
				RequestAttempt attempt = (RequestAttempt) ra;
				currFailure = RequestUtils.hasFailed(attempt);
				if (RequestUtils.isProfileRequest(attempt) && currFailure) {
					profileFailures++;
				}
				if (currFailure) {
					if (prevFailure)
						successiveFailures++;
					else
						successiveFailures = 1;
				} else {
					successiveFailures = 0;
					totalRequests++;
				}
				prevFailure = currFailure;
			}
			htTodayRxProfileFailures.put(institutionId, new Long(profileFailures));
			htTodayRxRequests.put(institutionId, new Long(totalRequests));
			htTodayRxConsecutiveFailures.put(institutionId, new Long(
					successiveFailures));
		}

		for (Object r: RequestBO.getLatestSuccessfulExtractRequests(patient.
				          getUserProfile().getId()))
		{
			Request request = ((RequestBO) r).getRequestValues();
			Long institutionId = request.getInstitution().getInstitutionId();
			for (Object a: request.getRequestAttempts()) {
				RequestAttempt attempt = (RequestAttempt) a;
				if (htSuccessfulExtractRequests.containsKey(institutionId)) {
					Date dt = htSuccessfulExtractRequests.get(institutionId);
					if (attempt.getCreatedDate().after(dt))
						htSuccessfulExtractRequests.put(institutionId, 
							attempt.getCreatedDate());
				} else {
					htSuccessfulExtractRequests.put(institutionId, 
						attempt.getCreatedDate());
				}
			}
		}
	}

	private Comparator getRequestAttemptCompator() {
		Comparator comparator = new Comparator() {
			public int compare(Object obj1, Object obj2) {
				if ((obj1 instanceof RequestAttempt)
						&& (obj2 instanceof RequestAttempt)) {
					RequestAttempt ra1 = (RequestAttempt) obj1;
					RequestAttempt ra2 = (RequestAttempt) obj2;
					return ra1.getCreatedDate().compareTo(ra2.getCreatedDate());
				} else if (obj1 instanceof RequestAttempt) {
					return -1;
				} else if (obj2 instanceof RequestAttempt) {
					return 1;
				}
				return 0;
			}

			public boolean equals(Object obj) {
				return obj.equals(this);
			}
		};
		return comparator;
	}

	private long getTodayRxRequests(Institution institution) {
		long todayRxRequests = 0;
		if (htTodayRxRequests.containsKey(institution.getInstitutionId())) {
			todayRxRequests = htTodayRxRequests.get(institution.
				getInstitutionId()).longValue();
		}
		return todayRxRequests;
	}

	private long getTodayConsecutiveRxFailures(Institution institution) {
		long todaySuccessiveRxFailures = 0;
		if (htTodayRxConsecutiveFailures.containsKey(institution
				.getInstitutionId())) {
			todaySuccessiveRxFailures = htTodayRxConsecutiveFailures.
				get(institution.getInstitutionId()).longValue();
		}
		return todaySuccessiveRxFailures;
	}

	private boolean isBelowRequestLimit(Institution institution) {
		return ((getMaxTotalRxRequests() <= 0) 
				|| (getTodayRxRequests(institution) < getMaxTotalRxRequests()))
			&& ((getMaxConsecutiveRxFailures() <= 0) 
			     || (getTodayConsecutiveRxFailures(institution) < 
			    		 getMaxConsecutiveRxFailures()));
	}

	public boolean checkSendActiveUpdateRequest(Patient patient,
			Institution institution) {

		initializeRxLogicData(patient);

		// check if user had successful profile extract or active extract
		boolean result = 
			(htHadSuccessfulProfileExtract.containsKey(institution.
				getInstitutionId()) 
			|| htHadSuccessfulActiveExtract.containsKey(institution.
				getInstitutionId()));
		// check request conditions
		result = result && isBelowRequestLimit(institution);

		return result;
	}
	
	public boolean checkSendHistoricalUpdateRequest(Patient patient,
			Institution institution) {

		initializeRxLogicData(patient);

		// check if user had successful profile extract
		boolean result = 
			(htHadSuccessfulProfileExtract.containsKey(institution.
				getInstitutionId()));		
		// check request conditions
		result = result && isBelowRequestLimit(institution);

		return result;
	}


	public boolean checkSendStatusRequest(Patient patient, 
		Institution institution) 
	{
		verifyPatientAndInstitution(patient, institution);
		
		initializeRxLogicData(patient);

		// check if user had successful full extract or active extract
		Long institutionId = institution.getInstitutionId();
		boolean result = 
			(institutionId != null
					&&
			 !htHadSuccessfulProfileExtract.containsKey(institutionId) 
					&&
			 !htHadSuccessfulActiveExtract.containsKey(institutionId));		
		// check request conditions
		result = result && isBelowRequestLimit(institution);

		return result;
	}

	private void verifyPatientAndInstitution(Patient patient, Institution institution) {
		if ((patient == null) || (institution == null)) {
			throw new IllegalArgumentException("patient and institution must BOTH be not null! patient=" + 
				patient + ", institution=" + institution);
		}
	}

	public boolean checkSendProfileRequest(Patient patient,	Institution institution) {
		verifyPatientAndInstitution(patient, institution);
		initializeRxLogicData(patient);

		// check if user had successful full extract
		boolean result = !htHadSuccessfulProfileExtract.containsKey(institution
				.getInstitutionId());
		// check request conditions
		result = result && isBelowRequestLimit(institution);

		// check max profile requests for today reached
		if(htTodayRxProfileFailures.containsKey(institution.getInstitutionId()))
		{
			long currProfileFailures = htTodayRxProfileFailures.get(institution.
				getInstitutionId()).longValue();
			result = result && (currProfileFailures < getMaxRxProfileFailures());
		}
		return result;
	}

	public boolean checkSendRefillRequest(Patient patient,
			Institution institution) {
		return true;
	}

	public Date checkRxExtractCutOffDate(Patient patient,
			Institution institution) 
	{
		initializeRxLogicData(patient);
		return htSuccessfulExtractRequests.get(institution.getInstitutionId());
	}
}
