package gov.va.med.mhv.rxrefill.util;

import gov.va.med.mhv.rxrefill.enumeration.RequestFunctionEnumeration;
import gov.va.med.mhv.rxrefill.enumeration.RequestStatusEnumeration;
import gov.va.med.mhv.rxrefill.transfer.Request;
import gov.va.med.mhv.rxrefill.transfer.RequestAttempt;

public final class RequestUtils {
	
	public static final RequestStatusEnumeration FAILED_STATUS = 
		RequestStatusEnumeration.getEnum(RequestStatusEnumeration.FAILED);
	
	public static final RequestStatusEnumeration REJECTED_STATUS = 
		RequestStatusEnumeration.getEnum(RequestStatusEnumeration.REJECTED);
	
	public static final RequestStatusEnumeration SUCCESSFUL_STATUS = 
		RequestStatusEnumeration.getEnum(RequestStatusEnumeration.SUCCESSFUL);
	
	public static final RequestStatusEnumeration INPROGRESS_STATUS = 
		RequestStatusEnumeration.getEnum(RequestStatusEnumeration.INPROGRESS);
	
	public static final RequestFunctionEnumeration REFILL_FUNCTION =  
		RequestFunctionEnumeration.getEnum(RequestFunctionEnumeration.
		REFILLPRESCRIPTION);

	public static final RequestFunctionEnumeration PROFILE_FUNCTION =  
		RequestFunctionEnumeration.getEnum(RequestFunctionEnumeration.
		GETPRESCRIPTIONPROFILE);

	public static final RequestFunctionEnumeration STATUS_FUNCTION =  
		RequestFunctionEnumeration.getEnum(RequestFunctionEnumeration.
		GETPRESCRIPTIONSTATUS);

	public static final RequestFunctionEnumeration UPDATE_FUNCTION =  
		RequestFunctionEnumeration.getEnum(RequestFunctionEnumeration.
		GETPRESCRIPTIONUPDATE);

	private RequestUtils() {
		// Hide constructor for utility class
	}
	
	
	public static boolean isInProgress(Request request) {
		return (request != null) && INPROGRESS_STATUS.equals(request.
			getStatus()); 
	}

	public static boolean hasSucceeded(Request request) {
		return (request != null) && SUCCESSFUL_STATUS.equals(request.
			getStatus()); 
	}

	public static boolean hasFailed(Request request) {
		return (request != null) && FAILED_STATUS.equals(request.
			getStatus()); 
	}

	public static boolean hasBeenRejected(Request request) {
		return (request != null) && REJECTED_STATUS.equals(request.
			getStatus()); 
	}

	public static boolean isRefillRequest(Request request) {
		return (request != null) && REFILL_FUNCTION.equals(request.
			getRequestFunction());
	}

	public static boolean isStatusRequest(Request request) {
		return (request != null) && STATUS_FUNCTION.equals(request.
			getRequestFunction()); 
	}

	public static boolean isProfileRequest(Request request) {
		return (request != null) && PROFILE_FUNCTION.equals(request.
			getRequestFunction()); 
	}

	public static boolean hasUpdateFunction(Request request) {
		return (request != null) && UPDATE_FUNCTION.equals(request.
			getRequestFunction()); 
	}

	
	
	public static boolean isInProgress(RequestAttempt attempt) {
		return (attempt != null) && INPROGRESS_STATUS.equals(attempt.
			getStatus());
	}

	public static boolean hasSucceeded(RequestAttempt attempt) {
		return (attempt != null) && hasSucceeded(attempt.getRequest());
	}

	public static boolean hasFailed(RequestAttempt attempt) {
		return (attempt != null) && hasFailed(attempt.getRequest());
	}

	public static boolean hasBeenRejected(RequestAttempt attempt) {
		return (attempt != null) && hasBeenRejected(attempt.getRequest());
	}

	public static boolean isStatusRequest(RequestAttempt attempt) {
		return (attempt != null) && isStatusRequest(attempt.getRequest());
	}

	public static boolean isProfileRequest(RequestAttempt attempt) {
		return (attempt != null) && isProfileRequest(attempt.getRequest());
	}

	public static boolean hasUpdateFunction(RequestAttempt attempt) {
		return (attempt != null) && hasUpdateFunction(attempt.getRequest());
	}

	public static boolean isRefillRequest(RequestAttempt attempt) {
		return (attempt != null) && isRefillRequest(attempt.getRequest());
	}


}
