package gov.va.med.mhv.rxrefill.portlet;

import java.util.List;
import java.net.URL;
import java.util.Date;
import java.sql.Timestamp;
import org.apache.beehive.netui.pageflow.Forward;
import org.apache.beehive.netui.pageflow.annotations.Jpf;
import org.tigris.atlas.messages.Severity;

import gov.va.med.mhv.rxrefill.portlet.controller.PrescriptionPageFlowController;
import gov.va.med.mhv.rxrefill.service.delegate.PrescriptionServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.ActivityAuditServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActionTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityTypeEnumeration;
import gov.va.med.mhv.usermgmt.transfer.TransferObjectFactory;
import gov.va.med.mhv.usermgmt.transfer.Activity;
import gov.va.med.mhv.rxrefill.transfer.Prescription;
import gov.va.med.mhv.rxrefill.web.form.ActivePrescriptionViewData;
import gov.va.med.mhv.rxrefill.web.form.PrescriptionForm;
import gov.va.med.mhv.rxrefill.web.form.PrescriptionViewData;

/**
 * @jpf:catch type="Exception" method="handleException"
 * @jpf:controller struts-merge="/WEB-INF/struts-override.xml"
 *  */
@SuppressWarnings("serial")
@Jpf.Controller(strutsMerge = "/WEB-INF/struts-override.xml",
                catches = {
                    @Jpf.Catch(type = Exception.class,
                               method = "handleException")
                })
public class ActivePrescriptionFlowController extends PrescriptionPageFlowController
{

    protected final String TEXT_RXREFILL = "rxrefill";
    protected final String PAGE_RXREFILL_HEALTH_TIP = "rxrefill_healthtip_active_rx.htm";
    protected final String MESSAGE_CODE_ERROR_SELECTED_REFILL_PAGINATION = "rxrefill.error.submitrefillwarning.pagination";
    protected final String TEXT_ONLINE_INSTRUCTION_GUIDE = "online instruction guide";
    protected final String URL_ENCODED_FORWARD_SLASH = "%2F";
    protected final String LINK_TARGET_MHV_HELP = "_mhvHelp";
    protected final String LINK_REFILL_INSTRUCTION
            = "/resources/jsp/help.jsp?helpDirectRequest=rxrefill_training_vets" +
              URL_ENCODED_FORWARD_SLASH +
              "index.htm";

    private boolean selectionWarned = false;

    /**
     * This method represents the point of entry into the pageflow
     * @jpf:action
     * @jpf:forward name="success" path="index.jsp"
     * @jpf:forward name="failure" path="index.jsp"
     * @jpf:forward name="nonPatient" path="nonPatient.jsp"
     * @jpf:forward name="agreement" path="agreement.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success",
                     path = "index.jsp"),
        @Jpf.Forward(name = "failure",
                     path = "index.jsp"),
        @Jpf.Forward(name = "nonPatient",
                     path = "nonPatient.jsp"),
        @Jpf.Forward(name = "agreement",
                     path = "agreement.jsp")
    })
    public Forward begin()
    {
        return super.begin();
    }

    @Jpf.Action(forwards = {
        	@Jpf.Forward(name = "success", path = "index.jsp"),
        	@Jpf.Forward(name = "failure", path = "index.jsp"),
            @Jpf.Forward(name = "agreement", path="agreement.jsp")
    	}
    )
    public Forward refreshAction()
    {
        return super.refreshAction();
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="begin.do"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success",
                     path = "begin.do")
    })
    protected Forward acceptAgreementAction()
    {
        return super.acceptAgreementAction();
    }

    /**
     * @jpf:action
     * @jpf:forward name="warning" return-to="currentPage"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "warning",
                     navigateTo = Jpf.NavigateTo.currentPage)
    })
    protected Forward declineAgreementAction()
    {
        return super.declineAgreementAction();
    }

    /**
     * @jpf:action
     */
    @Jpf.Action()
    protected Forward contactUsAction()
    {
        return super.contactUsAction();
    }

    /**
     * @jpf:action
     */
    @Jpf.Action()
    protected Forward forwardToProfiles()
    {
        return super.forwardToProfiles();
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="index.jsp"
     * @jpf:forward name="failure" path="index.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success",
                     path = "index.jsp"),
        @Jpf.Forward(name = "failure",
                     path = "index.jsp")
    })
    public Forward resetViewListAction()
    {
        return super.resetViewListAction();
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="view.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success", path = "view.jsp"),
        @Jpf.Forward(name = "failure", action = "begin")
    })
    protected Forward viewAction(PrescriptionForm form)
    {
        return super.viewAction(form);
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="trackingView.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success", path = "trackingView.jsp"),
        @Jpf.Forward(name = "failure", action = "begin")
    })
    protected Forward viewTrackingAction(PrescriptionForm form)
    {
        return super.viewTrackingAction(form);
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="indexPF.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success",
                     path = "indexPF.jsp")
    })
    public Forward printerFriendlyListAction()
    {
        return super.printerFriendlyListAction();
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="viewPF.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success",
                     path = "viewPF.jsp")
    })
    public Forward printerFriendlyDetailAction(PrescriptionForm form)
    {
        return super.printerFriendlyDetailAction(form);
    }
    /**
     * @jpf:action
     * @jpf:forward name="success" path="viewPF.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success",
                     path = "trackingViewPF.jsp")
    })
    public Forward printerFriendlyTrackingDetailAction(PrescriptionForm form)
    {
        return super.printerFriendlyTrackingDetailAction(form);
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="trackingViewMedicationImage.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success", path = "trackingViewMedicationImage.jsp")
    })
    public Forward viewTrackingImageAction()
    {
    	String imageTracking = (String)getRequest().getParameter("imageTracking");
    	String imageTrackingAAL = (String)getRequest().getParameter("imageTrackingAAL");
    	getRequest().getSession().setAttribute("imageTracking", imageTracking);
    	getRequest().getSession().setAttribute("imageTrackingAAL", imageTrackingAAL);
    	return super.viewTrackingImageAction();
    	//return new Forward("success");
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="trackingViewCarrier.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success", path = "trackingViewCarrier.jsp")
    })
    public Forward viewTrackingCarrierAction()
    {
    	String carrierTracking = (String)getRequest().getParameter("carrierTracking");
    	String carrierTrackingAAL = (String)getRequest().getParameter("carrierTrackingAAL");

    	getRequest().getSession().setAttribute("carrierTracking", carrierTracking);
    	getRequest().getSession().setAttribute("carrierTrackingAAL", carrierTrackingAAL);
    	//return super.ViewTrackingCarrierAction(form);
    	return new Forward("success");
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="trackingView.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success", path = "trackingView.jsp")
    })
    public Forward continueAction() throws Exception
    {
		String urlString = "";
		String carrierTracking = (String)getRequest().getSession().getAttribute("carrierTracking");
		String carrierTrackingAAL = (String)getRequest().getSession().getAttribute("carrierTrackingAAL");

		String[] carrierTrackingAALInfo = carrierTrackingAAL.split("::");
		String rxNumber = carrierTrackingAALInfo[0];
		String rxName = carrierTrackingAALInfo[1];
    	String trackStringAAL = "RX#: "+rxNumber+"    RX Name: "+rxName;

    	// AAL Code here ********
    	ActivityAuditServiceDelegate delegate = ServiceDelegateFactory.createActivityAuditServiceDelegate();
    	Activity activity = TransferObjectFactory.createActivity();
		activity.setAction(ActivityActionTypeEnumeration.getEnum(ActivityActionTypeEnumeration.PRESCRIPTION));
		activity.setActionType(ActivityTypeEnumeration.TRACK);
		activity.setPerformerType(ActivityActorTypeEnumeration.getEnum(ActivityActorTypeEnumeration.SELF));
		activity.setSubjectId(getCurrentUserId());
		activity.setCompletionTime(new Timestamp(new Date().getTime()));
		activity.setDetailValue(trackStringAAL);
		activity.setStatus(Boolean.TRUE);
		delegate.audit(activity);

		String[] carrierTrackingInfo = carrierTracking.split("::");
		String carrier = carrierTrackingInfo[0];
		String trackingNumber = carrierTrackingInfo[1];

		if (carrier.compareTo("UPS") == 0)
		{
			String ups_link="http://wwwapps.ups.com/WebTracking/processInputRequest?HTMLVersion=5.0&loc=en_US&Requester=UPSHome&tracknum=";
			urlString = ups_link+trackingNumber;
		}
		else if (carrier.compareTo("USPS") == 0)
		{
			String usps_link="https://tools.usps.com/go/TrackConfirmAction_input?qtc_tLabels1=";
			urlString = usps_link+trackingNumber;
		}
		else if (carrier.compareTo("DHL") == 0)
		{
			String dhl_link="http://webtrack.dhlglobalmail.com/?id=462&trackingnumber=";
			urlString = dhl_link+trackingNumber;
		}
		else
		{
			// Need to fix this. It should not come here if there is no defoined carrier.
			urlString = trackingNumber;
		}
		// Clean attributes.
		getRequest().getSession().removeAttribute("carrierTracking");
		getRequest().getSession().removeAttribute("carrierTrackingAAL");

		// redirect to the carrier site url for tracking information.
    	return new Forward(new URL(urlString));
    }

    // Landing Page - > Click on Continue button -> continueAction() -> parameters -> AAL Action-> send the page to Carrier site ->
    // check in MHV_PHR or MHV_USR

    /**
     * @jpf:action
     * @jpf:forward name="success" path="trackingView.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success", path = "trackingView.jsp")
    })
    public Forward cancelAction()
    {
		// Clean attributes.
		getRequest().getSession().removeAttribute("carrierTracking");
		getRequest().getSession().removeAttribute("carrierTrackingAAL");

    	return new Forward("success");
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="trackingView.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success", path = "trackingView.jsp")
    })
    public Forward returnToTrackingViewAction()
    {
		// Clean attributes.
		getRequest().getSession().removeAttribute("imageTracking");
		getRequest().getSession().removeAttribute("imageTrackingAAL");

    	return new Forward("success");
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="view.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success",
                     path = "view.jsp")
    })
    public Forward returnToViewAction(PrescriptionForm form)
    {
		getRequest().getSession().setAttribute("currentDetailActionName", "viewAction");
        return super.returnToViewAction(form);
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="view.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success",
                     path = "trackingView.jsp")
    })
    public Forward returnToViewTrackingAction(PrescriptionForm form)
    {
		getRequest().getSession().setAttribute("currentDetailActionName", "viewTrackingAction");
        return super.returnToViewAction(form);
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="index.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success",
                     path = "index.jsp")
    })
    public Forward returnToListAction(PrescriptionForm form)
    {
        return super.returnToListAction();
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="index.jsp"
     * @jpf:forward name="failure" path="index.jsp"
     */
	//@Jpf.Action(
	//	forwards = {
	//		@Jpf.Forward(name = "success", path = "index.jsp"),
	//		@Jpf.Forward(name = "failure", path = "index.jsp")
	//	}
	//)
    @Jpf.Action(
        	forwards = {
        		@Jpf.Forward(name = "success", path = "index.jsp")
        	},
            validationErrorForward = @Jpf.Forward(name = "failure",
            	navigateTo = Jpf.NavigateTo.currentPage),
            	preventDoubleSubmit = true
    )
    protected Forward submitRefillAction() {
        return super.submitRefillAction();
    }

    /**
     * @jpf:exception-handler
     * @jpf:forward name="error" path="../include/error.jsp"
     */
    @Jpf.ExceptionHandler(
		forwards = {
			@Jpf.Forward(name = "error", path = "../include/error.jsp")
		}
	)
    protected Forward handleException(Exception ex, String actionName,
                String message, Object form) {

        return super.handleException(ex, actionName, message);
    }

    // implementing rxrefill-specific methods
    protected org.tigris.atlas.service.EntityCollectionServiceResponse lookupPrescriptions(
            PrescriptionServiceDelegate service)
    {
        return service.getActivePrescriptions(pharmacyPatient);
    }

    /* the link removed from message per defect 1354
    protected String getNoPrescriptionsMessage() {
        Map refillInstructionLink = new HashMap(1);
        refillInstructionLink.put(MSG_VAR_REFILL_INSTRUCTION_LINK,
                getHtmlLink(getRequest().getContextPath()+LINK_REFILL_INSTRUCTION,
                LINK_TARGET_MHV_HELP,
                TEXT_ONLINE_INSTRUCTION_GUIDE));

        return NotificationUtil.message(
                getMessageIdPrefix()+MESSAGE_CODE_UNAVAILABLE, refillInstructionLink);
    }
    */

    protected PrescriptionViewData createPrescriptionViewData() {
        return new ActivePrescriptionViewData(DEFAULT_PAGE_SIZE);
    }

    protected String getMessageIdPrefix() {
        return TEXT_RXREFILL;
    }

    protected String getHealthTipPage() {
        return PAGE_RXREFILL_HEALTH_TIP;
    }

    protected boolean warningCurrentSeletion() {

        if (!selectionWarned) {

            List refillList = getViewData().populateRefillList(false);

            if (refillList != null && refillList.size() > 0) {

            	super.addSummaryMessage(Severity.getSeverity(Severity.ERROR),
            		MESSAGE_CODE_ERROR_SELECTED_REFILL_PAGINATION, null);
                // FROM: addNotificationMessage(NOTI_ID_RX_ERROR,
                //        MESSAGE_CODE_ERROR_SELECTED_REFILL_PAGINATION,
                //        null, null);

                selectionWarned = true;
                return true;
            }
        }

        return false;
    }

    @Override
    public Class getFormClass() {
		// TODO: Check if this is the correct class
		return PrescriptionForm.class;
    }

	@Override
	protected Class getTransferObjectClass() {
		// TODO: Check if this is the correct class
    	return Prescription.class;
	}

  /* (non-Javadoc)
   * @see gov.va.med.mhv.beehive.EntityPagingListPageFlowController#getTableUID()
   */
  @Override
  public String getTableUID() {
    return "activePrescriptions";
  }

  	protected String getPortletLabel() {
  		return "activePrescription";
  	}

  	protected String getPageLabel() {
  		return "refillPrescriptions";
  	}
}
