package gov.va.med.mhv.rxrefill.web.form;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import javax.servlet.http.HttpServletRequest;


import gov.va.med.mhv.rxrefill.enumeration.MailWindowEnumeration;
import gov.va.med.mhv.rxrefill.transfer.Prescription;
import gov.va.med.mhv.rxrefill.web.form.ViewData.TableCell;
import gov.va.med.mhv.rxrefill.web.form.ViewData.TableRow;

/**
 * FormData get and set methods may be overwritten by the Form Bean editor.
 */
public class ActivePrescriptionViewData extends PrescriptionViewData {


	public final String HEADER_RX_REFILL = "Refill Prescription";
    public final String TYPE_RX_REFILL_HELP = "rxrefill";
    public final String TYPE_RX_REFILL_LIST = "Active Prescriptions of";
    public final String TYPE_RX_REFILL_DETAIL = "Active Prescription of";
    public final String TYPE_RX_REFILL_DETAIL1 = "Active Prescription";
    public final String HEADER_RX_TRACKING = "View Prescription Tracking";
    public final String TYPE_RX_TRACKING_DETAIL = "Prescription Delivery";
    public final String TYPE_RX_TRACKING_HELP = "rxtrackinghistory";
    public final String TITLE_SELECT_TO_REFILL = "Select to Refill";
    public final String TRACK_DELIVERY = "Track Delivery";

    // narrow titles to avoid overlapping with sorting arrows
    public final String TAG_BR = "<br>";
    public final String TITLE_NARROW_REFILL_STATUS = "Refill" + TAG_BR + "Status";
    public final String TITLE_NARROW_SUBMIT_DATE = "Refill" + TAG_BR + "Submit Date";

    public final String TITLE_NARROW_RX_REMAINING = "Refills" + TAG_BR + "Remaining";

    public final String TITLE_NARROW_FILL_DATE = "Fill" + TAG_BR + "Date";
    public final String TITLE_NARROW_RX_NUMBER = "Prescription"; // + TAG_BR + "Number";
    public final String TITLE_NARROW_LAST_UPDATED = "Last" + TAG_BR + "Updated";
    public final String TITLE_NARROW_SELECT_TO_REFILL = "Select" + TAG_BR + "to Refill";

    public ActivePrescriptionViewData(int pageSize) {
        super(pageSize);
    }

    public String getHeaderType() {
        return HEADER_RX_REFILL;
    }

    public String getHeader1Type() {
        return HEADER_RX_REFILL;
    }

    public String getTrackingHeaderType() {
        return HEADER_RX_TRACKING;
    }

    public String getHelpType() {
        return TYPE_RX_REFILL_HELP;
    }

    public String getTrackingHelpType() {
        return TYPE_RX_TRACKING_HELP;
    }

    public String getListType() {
        return TYPE_RX_REFILL_LIST;
    }

    public String getDetailType() {
        return TYPE_RX_REFILL_DETAIL;
    }

    public String getDetail1Type() {
        return TYPE_RX_REFILL_DETAIL1;
    }

    public String getTrackingDetailType() {
        return TYPE_RX_TRACKING_DETAIL;
    }

    protected String getStatusTitle() {
        return TITLE_REFILL_STATUS;
    }

    protected String getTrackingTitle() {
        return TITLE_TRACKING;
    }

    public String getDefaultSortColumn() {
	    return COLUMN_FILL_DATE;
	    //return COLUMN_DISPENSE_DATE;
	}

    public String getDefaultSortColumn2() {
        return COLUMN_RX_Number;
        //return COLUMN_DRUG_NAME;
    }

    protected void populateListTableHeaderRow() {
        listHeaderRow = new TableRow();
        listHeaderRow.addSortableHeaderCellNarrow(TITLE_NARROW_REFILL_STATUS, COLUMN_STATUS, TITLE_REFILL_STATUS);
        listHeaderRow.addSortableHeaderCellNarrow(TITLE_NARROW_SUBMIT_DATE, COLUMN_SUBMIT_DATE, TITLE_SUBMIT_DATE);
        listHeaderRow.addSortableHeaderCellNarrow(TITLE_NARROW_FILL_DATE, COLUMN_FILL_DATE, TITLE_FILL_DATE);

        listHeaderRow.addSortableHeaderCellNarrow(TITLE_NARROW_RX_REMAINING, COLUMN_RX_REMAINING, TITLE_RX_REMAINING);

        listHeaderRow.addSortableHeaderCellNarrow(TITLE_NARROW_RX_NUMBER, COLUMN_DRUG_NAME, TITLE_RX);
        //listHeaderRow.addSortableHeaderCellNarrow(TITLE_NARROW_LAST_UPDATED, COLUMN_LAST_UPDATED, TITLE_LAST_UPDATED);
        listHeaderRow.addSortableHeaderCell(TITLE_FACILITY, COLUMN_FACILITY);
        listHeaderRow.addSortableHeaderCellPFHiddenNarrow(TITLE_NARROW_SELECT_TO_REFILL, COLUMN_IS_REFILLABLE, TITLE_SELECT_TO_REFILL);
        listHeaderRow.addHeaderCellCenter(TITLE_TRACKING, COLUMN_TRACKING);
    }

    protected boolean createPrescriptionTableRowContent(TableRow tr,
    	PrescriptionForm rxForm, String contextPath, HttpServletRequest request)
    {
        addCellRefillStatusAndSubmitDate(tr, rxForm, contextPath);
        addCellPrescriptionFillDate(tr, rxForm, contextPath);
        addCellPrescriptionRxRemaining(tr, rxForm, contextPath);
        addCellMedication(tr, rxForm, contextPath);
        addCellFacility(tr, rxForm, contextPath);
        addCellRefillCheckbox(tr, rxForm);
		addCellTracking(tr, rxForm, contextPath, request);
		return true;
    }

    protected boolean checkRefillable(Prescription rx) {
        if (rx.getIsRefillable() != null && rx.getIsRefillable().booleanValue())
            return true;
        else
            return false;
    }

    protected void addCellRefillStatusAndSubmitDate(
            TableRow tr, PrescriptionForm rxForm, String contextPath) {

        // adding refill status
        TableCell cell = new TableCell();
        cell.content = rxForm.getPrescriptionStatus();

        // adding warning icon?
        if (rxForm.getPrescriptionIsInvalidMhvStatus().equalsIgnoreCase(TEXT_TRUE)) {
            cell.setImg(contextPath+IMAGE_FILE_ALERT, ALT_IMAGE_INVALID_STATUS);
        }

        tr.addCell(cell);

        // adding refill submit date
        tr.addDateCell(
                rxForm.getPrescriptionLastRefillSubmittedDateMonth(),
                rxForm.getPrescriptionLastRefillSubmittedDateDay(),
                rxForm.getPrescriptionLastRefillSubmittedDateYear());
    }

    protected void addCellRefillCheckbox(TableRow tr, PrescriptionForm rxForm) {
        // adding refill checkbox
        tr.checkboxColumnIndex = tr.getCells().size();

        TableCell cell = new TableCell();
        cell.pfHidden = true;
        if (rxForm.getPrescriptionIsRefillable().equalsIgnoreCase(TEXT_TRUE)) {
            cell.hasCheckbox = true;
            cell.checkboxAlt = ALT_CHECKBOX_REFILL;
            cell.styleClass = STYLE_CELL_CENTER;
        }

        tr.addCell(cell);
    }

	@Override
	protected boolean checkTrackable(Prescription rx) {
		// TODO Auto-generated method stub
		return false;
	}

}