package gov.va.med.mhv.rxrefill.web.form;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import javax.servlet.http.HttpServletRequest;

import gov.va.med.mhv.rxrefill.enumeration.MailWindowEnumeration;
import gov.va.med.mhv.rxrefill.transfer.Prescription;
import gov.va.med.mhv.rxrefill.web.form.ViewData.TableCell;
import gov.va.med.mhv.rxrefill.web.form.ViewData.TableRow;

/**
 * FormData get and set methods may be overwritten by the Form Bean editor.
 */
public class HistoricalPrescriptionViewData extends PrescriptionViewData {


	public final String HEADER_RX_HISTORY= "View Prescription History";
    public final String TYPE_RX_HISTORY_HELP = "rxhistory";
    public final String TYPE_RX_HISTORY_LIST = "Historical Prescriptions of";
    public final String TYPE_RX_HISTORY_DETAIL = "Historical Prescription of";
    public final String TYPE_RX_HISTORY_DETAIL1 = "Historical Prescription";
    public final String HEADER_RX_TRACKING = "View Prescription Tracking";
    public final String TYPE_RX_TRACKING_DETAIL = "Prescription Delivery";
    public final String TYPE_RX_TRACKING_HELP = "rxtrackinghistory";
    public final String WINDOW = "window";
    public final String TRACK_DELIVERY = "Track Delivery";


    public HistoricalPrescriptionViewData(int pageSize) {
        super(pageSize);
    }

    public String getHeaderType() {
        return HEADER_RX_HISTORY;
    }

    public String getHeader1Type() {
        return HEADER_RX_HISTORY;
    }

    public String getTrackingHeaderType() {
        return HEADER_RX_TRACKING;
    }

    public String getHelpType() {
        return TYPE_RX_HISTORY_HELP;
    }

    public String getTrackingHelpType() {
        return TYPE_RX_TRACKING_HELP;
    }

    public String getListType() {
        return TYPE_RX_HISTORY_LIST;
    }

    public String getDetailType() {
        return TYPE_RX_HISTORY_DETAIL;
    }

    public String getDetail1Type() {
        return TYPE_RX_HISTORY_DETAIL1;
    }

    public String getTrackingDetailType() {
        return TYPE_RX_TRACKING_DETAIL;
    }

    protected String getStatusTitle() {
        return TITLE_STATUS;
    }

    protected String getTrackingTitle() {
        return TITLE_TRACKING;
    }

    public String getDefaultSortColumn() {
        return COLUMN_FILL_DATE;
        //return COLUMN_DISPENSE_DATE;
    }

    public String getDefaultSortColumn2() {
	        return COLUMN_RX_Number;
	        //return COLUMN_DRUG_NAME;
    }
    protected void populateListTableHeaderRow() {
        listHeaderRow = new TableRow();
        listHeaderRow.addSortableHeaderCell(TITLE_FILL_DATE, COLUMN_FILL_DATE);
        listHeaderRow.addSortableHeaderCell(TITLE_RX, COLUMN_DRUG_NAME);
        listHeaderRow.addSortableHeaderCell(TITLE_STATUS, COLUMN_STATUS);
        //listHeaderRow.addSortableHeaderCell(TITLE_LAST_UPDATED, COLUMN_LAST_UPDATED);
        listHeaderRow.addSortableHeaderCell(TITLE_FACILITY, COLUMN_FACILITY);
        //listHeaderRow.addSortableHeaderCell(TITLE_TRACKING, COLUMN_TRACKING);
        listHeaderRow.addHeaderCellCenter(TITLE_TRACKING, COLUMN_TRACKING);
    }

    protected boolean createPrescriptionTableRowContent(TableRow tr,
    	PrescriptionForm rxForm, String contextPath, HttpServletRequest request)
    {
        addCellPrescriptionFillDate(tr, rxForm, contextPath);
        addCellMedication(tr, rxForm, contextPath);
        addCellHistoricalRxStatus(tr, rxForm, contextPath);
        addCellFacility(tr, rxForm, contextPath);
        addCellTracking(tr, rxForm, contextPath, request);
        return true;
    }

    protected boolean checkRefillable(Prescription rx) {
        return false;
    }

    protected void addCellHistoricalRxStatus(
            TableRow tr, PrescriptionForm rxForm, String contextPath) {
        // adding Status
        TableCell cell = new TableCell();
        cell.content = rxForm.getPrescriptionStatus();

        // adding warning icon?
        if (rxForm.getPrescriptionIsInvalidMhvStatus().equalsIgnoreCase(TEXT_TRUE)) {
            cell.setImg(contextPath+IMAGE_FILE_ALERT, ALT_IMAGE_INVALID_STATUS);
        }
        tr.addCell(cell);
    }

	@Override
	protected boolean checkTrackable(Prescription rx) {
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DAY_OF_YEAR , -46);
		Date daysAgo = cal.getTime();

		if (rx.getMailWindow().getName().equalsIgnoreCase(WINDOW) && (rx.getReleaseDateTime().after(daysAgo))|| rx.getReleaseDateTime().equals(daysAgo) ){
			//System.out.println("**** checkTrackable::true:: rx release date: " + rx.getReleaseDateTime() + " 46 daysAgo: " + daysAgo + " Mail/Window: " + rx.getMailWindow().getName());
			return true;
		}else{
			//System.out.println("**** checkTrackable::false:: rx release date: " + rx.getReleaseDateTime() + " 46 daysAgo: " + daysAgo + " Mail/Window: " + rx.getMailWindow().getName());
			return false;
		}
	}

}