package gov.va.med.mhv.rxrefill.web.form;

//import gov.va.med.mhv.core.web.form.InputForm;
import gov.va.med.mhv.rxrefill.enumeration.MailWindowEnumeration;
import gov.va.med.mhv.rxrefill.form.RxTrackingForm;
import gov.va.med.mhv.rxrefill.service.delegate.PrescriptionServiceDelegate;
import gov.va.med.mhv.rxrefill.service.delegate.RxTrackingServiceDelegate;
import gov.va.med.mhv.rxrefill.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.rxrefill.transfer.Prescription;
import gov.va.med.mhv.rxrefill.transfer.RxTracking;
import gov.va.med.mhv.rxrefill.util.TimerUtils;
import gov.va.med.mhv.rxrefill.util.RxRefillProperties;
import gov.va.med.mhv.rxrefill.web.form.ViewData.TableCell;
import gov.va.med.mhv.rxrefill.web.form.ViewData.TableRow;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import java.net.HttpURLConnection;
import java.net.URL;
import java.io.IOException;

import org.tigris.atlas.web.struts.InputForm;

/**
 * FormData get and set methods may be overwritten by the Form Bean editor.
 */
public abstract class PrescriptionViewData extends ViewData {

	private static final Log LOG = LogFactory.getLog(PrescriptionViewData.class);

	public final String COLUMN_STATUS = "status";
    public final String COLUMN_SUBMIT_DATE = "lastRefillSubmittedDate";
    public final String COLUMN_FILL_DATE = "lastFillDate";
    public final String COLUMN_DISPENSE_DATE = "releaseDateTime";

    public final String COLUMN_RX_REMAINING = "numberOfRefills";

    public final String COLUMN_RX_Number = "prescriptionNumber";
    public final String COLUMN_LAST_UPDATED = "modifiedDate";
    public final String COLUMN_FACILITY = "institution.name";
    public final String COLUMN_IS_REFILLABLE = "isRefillable";

	public final String COLUMN_DRUG_NAME = "drugName";

    public final String TITLE_REFILL_STATUS = "Refill Status";
    public final String TITLE_STATUS = "Status";
    public final String TITLE_SUBMIT_DATE = "Refill Submit Date";
    public final String TITLE_FILL_DATE = "Fill Date";
	public final String TITLE_DISPENSE_DATE = "Dispensed Date";

    public final String TITLE_RX_REMAINING = "Refills Remaining";

    public final String TITLE_RX = "Prescription";
    public final String TITLE_RX_NAME = "Prescription Name";
    public final String TITLE_RX_NUMBER = "Prescription Number";
    public final String TITLE_LAST_UPDATED = "Last Updated";
    public final String TITLE_FACILITY = "Facility";

    public final String TITLE_VIEW_DETAIL = "View Detail";
    public final String TITLE_NUMBER_OF_REFILLS = "Refills Remaining";
    public final String TITLE_QUANTITY = "Quantity";
    public final String TITLE_ISSUE_DATE = "Ordered On";
    public final String TITLE_RELEASE_DATE = "Dispensed On";
    public final String TITLE_EXPIRATION_CANCEL_DATE = "Expiration Date";

    public final String ALT_IMAGE_INVALID_STATUS = "Invalid Status";
    public final String ALT_CHECKBOX_REFILL = "Indicates Refill Requested";
    public final String ALT_BUTTON_REFILL = "Track Delivery";
    public final String TEXT_VIEW_DETAILS = "View Details";

    public final String COLUMN_TRACKING = "prescriptionId";
    public final String TITLE_TRACKING = "Prescription Tracking";
	public final String TITLE_TRACKING1 = "Tracking";
    public final String TITLE_TRACKING_HEADING = "Tracking Information";
    public final String TITLE_TRACKING_HEADING_SEPARATOR_PART1 = "In the last 45 days ";
    public final String TITLE_TRACKING_HEADING_SEPARATOR_PART2 = " was also mailed on the following dates";
    public final String TITLE_TRACKING_DATE_SHIPPED = "Date Shipped";
    public final String TITLE_RX_HEADING = "Prescription Information";
    public final String TITLE_TRACKING_FACILITY = "Facility Name";
    public final String TITLE_FACILITY_PHONE = "Rx Information Phone Number";
    public final String TITLE_TRACKING_DELIVERY_SERVICE = "Delivery Service";
    public final String TITLE_TRACKING_NUMBER = "Select Tracking Number";
    public final String TITLE_TRACKING_OTHERPRESCRIPTIONS = "Other Prescriptions included in This Package";
    public final String TRACK_DELIVERY = "Track Delivery";

    private boolean hasRefillable;
    private boolean isDuplicateRefillSubmitted;
    private boolean hasTrackable;

     // abstract methods to be implemented in active & historical rxViewData
    abstract protected String getStatusTitle();
    abstract protected boolean checkRefillable(Prescription rx);
    abstract protected boolean checkTrackable(Prescription rx);
    abstract protected String getDefaultSortColumn2();
    // abstract methods end

    public PrescriptionViewData(int pageSize) {
        super(pageSize);
    }

    //public String getDefaultSortColumn() {
    //    return COLUMN_FILL_DATE;
    //    //return COLUMN_DISPENSE_DATE;
    //}

    public void populateDetailTableRows(org.tigris.atlas.web.struts.InputForm form) {
        PrescriptionForm rxForm = (PrescriptionForm) form;
        detailTableRows = new ArrayList();

        addDetailHeaderRow(TITLE_RX_NUMBER + ": " + rxForm.getPrescriptionPrescriptionNumber());

        // Removed IPA check as per CR 6671
        addDetailRow(TITLE_RX_NAME, rxForm.getPrescriptionDrugName());

        addDetailDateRow(TITLE_FILL_DATE,
                rxForm.getPrescriptionLastFillDateMonth(),
                rxForm.getPrescriptionLastFillDateDay(),
                rxForm.getPrescriptionLastFillDateYear());

        addDetailDateRow(TITLE_RELEASE_DATE,
                rxForm.getPrescriptionReleaseDateTimeMonth(),
                rxForm.getPrescriptionReleaseDateTimeDay(),
                rxForm.getPrescriptionReleaseDateTimeYear());


        addDetailRow(getStatusTitle(), rxForm.getPrescriptionStatus());

        addDetailRow(TITLE_FACILITY, rxForm.getFacilityName());

        addDetailDateRow(TITLE_ISSUE_DATE,
                rxForm.getPrescriptionIssueDateTimeMonth(),
                rxForm.getPrescriptionIssueDateTimeDay(),
                rxForm.getPrescriptionIssueDateTimeYear());

        addDetailRow(TITLE_QUANTITY, rxForm.getPrescriptionQuantity());

        addDetailRow(TITLE_NUMBER_OF_REFILLS, rxForm.getPrescriptionNumberOfRefills());

        addDetailDateRow(TITLE_EXPIRATION_CANCEL_DATE,
                rxForm.getPrescriptionExpirationCancelDateMonth(),
                rxForm.getPrescriptionExpirationCancelDateDay(),
                rxForm.getPrescriptionExpirationCancelDateYear());
    }


	public Collection fetchTrackingDetailTableRows(long rxId, HttpServletRequest request) {

        PrescriptionServiceDelegate service = ServiceDelegateFactory.createPrescriptionServiceDelegate();
        Prescription p = service.getPrescriptionDetailsForPrescriptionID(Long.valueOf(rxId)).getPrescription();
        String stationNumber = p.getInstitution().getStationNumber();
        String rxnumber = p.getPrescriptionNumber();
        Date releaseDateTime = p.getReleaseDateTime();

        if (LOG.isInfoEnabled()) {
	        LOG.info("For:: "+request.getRemoteUser()+" fetchTrackingDetailTableRows: instId: stationNumber: releaseDateTime: " + stationNumber + " : " + rxnumber + " : " + releaseDateTime);
		}
        RxTrackingServiceDelegate trackingService = ServiceDelegateFactory.createRxTrackingServiceDelegate();
        Collection trackingNumbers = trackingService.getPrescriptionTrackingNumbers(rxnumber, stationNumber).getRxTrackings();
        return trackingNumbers;
	}


	public void populateTrackingDetailTableRows(InputForm form, HttpServletRequest request) {

		String divisionPhone = "";
		String ndc = "";
		String ndc1 = "";
		String completeDateTime = "";
		String firstDateShipped = "";
		String carrier = "";
		String trackingString = "";

		Boolean headerSeparatorDisplayed = false;

		TimerUtils timer = new TimerUtils();
		timer.start();
        PrescriptionForm rxForm = (PrescriptionForm) form;

        long rxId = rxForm.getPrescriptionId();

        PrescriptionServiceDelegate service = ServiceDelegateFactory.createPrescriptionServiceDelegate();
        Prescription p = service.getPrescriptionDetailsForPrescriptionID(Long.valueOf(rxId)).getPrescription();
        String mainStationNumber = p.getInstitution().getStationNumber();
        String rxnumber = p.getPrescriptionNumber();
        Date releaseDateTime = p.getReleaseDateTime();

        if (LOG.isInfoEnabled()) {
	        LOG.info("For:: "+request.getRemoteUser()+" populateTrackingDetailTableRows: instId: mainStationNumber: releaseDateTime: " + mainStationNumber + " : " + rxnumber + " : " + releaseDateTime);
		}

        RxTrackingServiceDelegate trackingService = ServiceDelegateFactory.createRxTrackingServiceDelegate();
        Collection trackingNumbers = trackingService.getPrescriptionTrackingNumbers(rxnumber, mainStationNumber).getRxTrackings();

        if (LOG.isInfoEnabled()) {
	        LOG.info("For:: "+request.getRemoteUser()+" populateTrackingDetailTableRows: Number of rx_tracking Records " + trackingNumbers.size());
		}

		for (Object e: trackingNumbers) {
			RxTracking rx = (RxTracking) e;
			ndc1 = rx.getNdc();
			if ((ndc1 == null) || (ndc1.equals(""))) {
				ndc = "NA";
				if (LOG.isInfoEnabled()) {
					LOG.info("For:: "+request.getRemoteUser()+"RX_Number:: "+rxForm.getPrescriptionPrescriptionNumber()+" populateTrackingDetailTableRows: Main NDC is Empty");
				}
			}
			else {
				ndc = ndc1.replaceAll(" ","");
				if (LOG.isInfoEnabled()) {
					LOG.info("For:: "+request.getRemoteUser()+"RX_Number:: "+rxForm.getPrescriptionPrescriptionNumber()+" populateTrackingDetailTableRows: Main NDC value (After removing Spaces)::"+ndc);
				}
			}
			divisionPhone = rx.getDivisionPhone();
			if ((divisionPhone == null) || (divisionPhone.equals("")))
				divisionPhone = "N/A";
			break;
		}

        trackingDetailTableRows = new ArrayList();

        addTrackingDetailHeaderRow(TITLE_RX_HEADING);
        if ((ndc == null) || (ndc.equals("")) || (ndc.compareTo("NA") == 0) || (ndc.length() < 11)) {
        	addTrackingDetailRow(TITLE_RX_NAME, rxForm.getPrescriptionDrugName());
        	if (LOG.isInfoEnabled()) {
        		LOG.info("For:: "+request.getRemoteUser()+"RX_Number:: "+rxForm.getPrescriptionPrescriptionNumber()+" populateTrackingDetailTableRows: Main NDC value is Empty or Less Than 11 Chars.(After removing Spaces)::"+ndc);
			}
		}
        else {
        	addTrackingDetailImageAnchorRow(TITLE_RX_NAME, rxForm.getPrescriptionDrugName(), rxForm.getPrescriptionPrescriptionNumber()+"::"+rxForm.getPrescriptionDrugName()+"::"+ndc,"");
        	if (LOG.isInfoEnabled()) {
        		LOG.info("For:: "+request.getRemoteUser()+"RX_Number:: "+rxForm.getPrescriptionPrescriptionNumber()+" populateTrackingDetailTableRows: Main NDC value (After removing Spaces)::"+ndc);
			}
		}

        addTrackingDetailRow(TITLE_RX_NUMBER, rxForm.getPrescriptionPrescriptionNumber());
        addTrackingDetailRow(TITLE_TRACKING_FACILITY, rxForm.getFacilityName());

        addTrackingDetailRow(TITLE_FACILITY_PHONE, divisionPhone);

		int j=0;
        for (Object e: trackingNumbers) {
            RxTracking rx = (RxTracking) e;

			if (LOG.isInfoEnabled()) {
				LOG.info("For:: "+request.getRemoteUser()+" populateTrackingDetailTableRows : trackingNumber :: " + rx.getTrackingNumber());
			}

			completeDateTime = new SimpleDateFormat("MM/dd/yyyy").format(rx.getCompleteDateTime());
			carrier = rx.getCarrier();

			// for the first row in rx-tracking
			if (j == 0) {
				request.getSession().setAttribute("lastUpdatedDate", new SimpleDateFormat("MM/dd/yyyy 'at' HHmm").format(rx.getDateLoaded()));
				firstDateShipped = completeDateTime;
			}

			// check if the Tracking Heading Separator already displayed, if not check if it is the same date shipped. All first Date Shipped tracking numbers will be displayed above the Heading Separator.
			if (headerSeparatorDisplayed == false) {
				if (firstDateShipped.compareTo(completeDateTime) != 0) {
					addTrackingDetailHeaderRow(TITLE_TRACKING_HEADING_SEPARATOR_PART1+rxForm.getPrescriptionDrugName()+TITLE_TRACKING_HEADING_SEPARATOR_PART2);
					headerSeparatorDisplayed = true;
				}
			}

			addTrackingDetailHeaderRow(TITLE_TRACKING_HEADING);
        	addTrackingDetailRow(TITLE_TRACKING_DATE_SHIPPED, completeDateTime);
        	addTrackingDetailRow(TITLE_TRACKING_DELIVERY_SERVICE, carrier);
			addTrackingDetailAnchorRow(TITLE_TRACKING_NUMBER, rx.getTrackingNumber(), carrier+"::"+rx.getTrackingNumber(), rxForm.getPrescriptionPrescriptionNumber()+"::"+rxForm.getPrescriptionDrugName());
			j++;

			// Get other prescription info for the tracking number.
			Collection prescriptionNumbers = trackingService.getPrescriptionDataByTrackingNumber(rx.getTrackingNumber()).getRxTrackings();

			ArrayList<String> otherPrescriptionInfo = new ArrayList<String>();
			for (Object e1: prescriptionNumbers) {
				String addtionalText = "";
				String drugName = "";
				RxTracking rx1 = (RxTracking) e1;

				if (LOG.isInfoEnabled()) {
					LOG.info("For:: "+request.getRemoteUser()+" Other Prescription Info : prescriptionNumber ::" + rx1.getRxNumber());
				}

				if (rx1.getRxNumber().compareTo(rxForm.getPrescriptionPrescriptionNumber()) != 0)
				{
					// Now get the prescription drug name for this prescription number and the stattion number.
					Prescription p1 = service.getPrescriptionDetailsForPrescriptionNumber(rx1.getRxNumber(),rx1.getStationNumber()).getPrescription();
					//Check for Null p1
					if (p1 != null)
						drugName = p1.getDrugName();
					else
						drugName = "N/A";
					String stationNumber = rx1.getStationNumber();

					if (LOG.isInfoEnabled()) {
						LOG.info("For:: "+request.getRemoteUser()+" Other Prescription Info : drugName :: " + drugName+"  stationNumber :: " +stationNumber);
					}

					if (stationNumber.compareTo(mainStationNumber) == 0)
						addtionalText = "";
					else
						addtionalText = " (Please Note: This prescription is from another facility, please contact that Pharmacy at "+ (((rx1.getDivisionPhone() == null) || (rx1.getDivisionPhone().equals("")))?"N/A":rx1.getDivisionPhone())+" if you have any questions.)";
					ndc1 = rx1.getNdc();
					if ((ndc1 == null) || (ndc1.equals(""))) {
						ndc = "NA";
						if (LOG.isInfoEnabled()) {
							LOG.info("For:: "+request.getRemoteUser()+"RX_Number:: "+rx1.getRxNumber()+" populateTrackingDetailTableRows: Other NDC is Empty");
						}
					}
					else {
						ndc = ndc1.replaceAll(" ","");
						if (LOG.isInfoEnabled()) {
							LOG.info("For:: "+request.getRemoteUser()+"RX_Number:: "+rx1.getRxNumber()+" populateTrackingDetailTableRows: Other NDC value (After removing Spaces)::"+ndc);
						}
					}
					otherPrescriptionInfo.add(drugName+"::"+rx1.getRxNumber()+"::"+addtionalText+"::"+ndc);
				}
			}
			Collections.sort(otherPrescriptionInfo);
			Iterator<String> itr = otherPrescriptionInfo.iterator();
			int i=0;
			while(itr.hasNext()){
				if (i == 0)
				{
					addTrackingDetailHeaderRow(TITLE_TRACKING_OTHERPRESCRIPTIONS);
					i++;
				}
				String[] prescInfo = itr.next().split("::");
				if ((prescInfo[3] == null) || (prescInfo[3].equals("")) || (prescInfo[3].compareTo("NA") == 0) || (prescInfo[3].length() < 11)) {
					addTrackingDetailRow(TITLE_RX_NAME, prescInfo[0]);
					if (LOG.isInfoEnabled()) {
						LOG.info("For:: "+request.getRemoteUser()+"RX_Number:: "+prescInfo[1]+" populateTrackingDetailTableRows: Other NDC value is Empty or Less Than 11 Chars.(After removing Spaces)::"+ndc);
					}
				}
				else {
					addTrackingDetailImageAnchorRow(TITLE_RX_NAME, prescInfo[0], prescInfo[1]+"::"+prescInfo[0]+"::"+prescInfo[3],"");
					if (LOG.isInfoEnabled()) {
        				LOG.info("For:: "+request.getRemoteUser()+"RX_Number:: "+prescInfo[1]+" populateTrackingDetailTableRows: Main NDC value (After removing Spaces)::"+ndc);
					}
				}
				addTrackingDetailRow(TITLE_RX_NUMBER, prescInfo[1]+prescInfo[2]);
				if (LOG.isInfoEnabled()) {
					LOG.info("For Other Prescriptions included in This Package:: Rx_name :: " + prescInfo[0]+"  rx_number :: " +prescInfo[1]);
				}
				if (LOG.isInfoEnabled()) {
					LOG.info("For Other Prescriptions included in This Package:: Rx_name :: " + prescInfo[0]+"  rx_number :: " +prescInfo[1]);
				}
			}
        }

        timer.stop();
		if (LOG.isInfoEnabled()) {
			LOG.info("For:: "+request.getRemoteUser()+" populateTrackingDetailTableRows :: Elapsed time in Seconds : " + timer.getElapsedTimeSecs());
		}
	}

	/**
	 * Pings a HTTP URL. This effectively sends a HEAD request and returns <code>true</code> if the response code is in
	 * the 200-399 range.
	 * @param url The HTTP URL to be pinged.
	 * @param timeout The timeout in millis for both the connection timeout and the response read timeout. Note that
	 * the total timeout is effectively two times the given timeout.
	 * @return <code>true</code> if the given HTTP URL has returned response code 200-399 on a HEAD request within the
	 * given timeout, otherwise <code>false</code>.
	 */
	public static boolean ping(String url, int timeout) {

		//System.out.println("URL is (Before)::"+url);

		//url = url.replaceFirst("https", "http"); // Otherwise an exception may be thrown on invalid SSL certificates.

		//System.out.println("URL is (After)::"+url);

		try {
			java.net.HttpURLConnection connection = (java.net.HttpURLConnection) new java.net.URL(url).openConnection();
			connection.setConnectTimeout(timeout);
			connection.setReadTimeout(timeout);
			connection.setRequestMethod("HEAD");
			int responseCode = connection.getResponseCode();
			//System.out.println("Response code is::"+responseCode);
			return (200 <= responseCode && responseCode <= 399);
		} catch (IOException exception) {
			//System.out.println("In the exception catch");
			System.out.println(exception);
			return false;
		}
	}

	public void populateTrackingImageTableRows(HttpServletRequest request) {

        trackingImageTableRows = new ArrayList();
		String imageTracking = (String)request.getSession().getAttribute("imageTracking");
		//String imageTrackingAAL = (String)getRequest().getSession().getAttribute("imageTrackingAAL");
		String[] imageTrackingInfo = imageTracking.split("::");
		String rx_number = imageTrackingInfo[0];
		String rx_name = imageTrackingInfo[1];
		String ndc = imageTrackingInfo[2].replaceAll(" ",""); // Remove all spaces in NDC
		String folderName = ndc.replaceFirst("^0+(?!$)","").substring(0,1);
		String fileName = "NDC"+ndc+".jpg";

		List folderNames = Arrays.asList("1", "2", "3", "4", "5", "6", "7", "8", "9");
		// check if the foldername is one of the listed folders. if not, set folder name to "other";
		if (!folderNames.contains(folderName))
			folderName = "other";

        //addTrackingImageHeaderRow(TITLE_RX_HEADING);
        addTrackingImageRow(TITLE_RX_NAME, rx_name);
        addTrackingImageRow(TITLE_RX_NUMBER, rx_number);

		//System.out.println("getRequestURL returns::"+request.getRequestURL());
		//getRequestURL returns::https://abc-serv.myhealth.domain:443/mhv-portal-web/gov/va/med/mhv/rxtracking/portlet/viewTrackingImageAction.do

		String[] parts = request.getRequestURL().toString().split("/mhv-portal-web");
		String serverString = parts[0];

        //https://abc-serv.myhealth.domain/mhv-portal-web/ShowBinary/BEA%20Repository/MILDrugImages/6/NDC00006074931.jpg
        String uriString = request.getContextPath()+"/ShowBinary/BEA%20Repository/MILDrugImages/"+folderName+"/"+fileName;
        //System.out.println("parts[0] is::"+parts[0]);
        //System.out.println("uriString is::"+uriString);
        if(ping(parts[0]+uriString,1000)) {
        	addTrackingShowImageRow("","",uriString,fileName, true);
        	LOG.info("For:: "+request.getRemoteUser()+"RX_Number:: "+rx_number+" populateTrackingImageTableRows: Image Is Available. Image URL::"+parts[0]+uriString);
		}
        else {
        	//addTrackingShowImageRow("","No Image Available",uriString,fileName, false);
        	addTrackingShowImageRow("","",request.getContextPath()+"/resources/images/NoImageAvailable.jpg","No Image Available", true);
        	LOG.info("For:: "+request.getRemoteUser()+"RX_Number:: "+rx_number+" populateTrackingImageTableRows: Image Is NOT Available. Image URL::"+parts[0]+uriString);
		}
	}

    protected void addCellTracking(
            TableRow tr, PrescriptionForm rxForm, String contextPath, HttpServletRequest request) {

		int noOfDaysToShowTrackingDetails = RxRefillProperties.getInstance().getNoOfDaysToShowTrackingDetails();

		if (LOG.isInfoEnabled()) {
			LOG.info("For:: "+request.getRemoteUser()+" noOfDaysToShowTrackingDetails :: " + noOfDaysToShowTrackingDetails);
		}

        Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DAY_OF_YEAR , -1*noOfDaysToShowTrackingDetails);
		//return RxRefillProperties.getInstance().getMaxConsecutiveFailures();
		Date daysAgo = cal.getTime();

        if(rxForm.getPrescriptionReleaseDateTimeDay()!= null && rxForm.getPrescriptionMailWindow() != null){
	        String releasedDateString = rxForm.getPrescriptionReleaseDateTimeMonth() + "/" +
			   							rxForm.getPrescriptionReleaseDateTimeDay() + "/" +
			   							rxForm.getPrescriptionReleaseDateTimeYear();

			if (LOG.isInfoEnabled()) {
				LOG.info("For:: "+request.getRemoteUser()+" addCellTracking :: releasedDateString : " + releasedDateString);
			}

	        DateFormat df = new SimpleDateFormat("MM/dd/yyyy");

	        Date releasedDate = null;

	        try{
	        	releasedDate = df.parse(releasedDateString);

				if (LOG.isInfoEnabled()) {
					LOG.info("For:: "+request.getRemoteUser()+" addCellTracking :: releasedDate : " + releasedDate);
				}

	        	// adding refill tracking button if certain criteria is met
	        if((MailWindowEnumeration.MAIL.equalsIgnoreCase(rxForm.getPrescriptionMailWindow()) && releasedDate!= null && (releasedDate.after(daysAgo))|| releasedDate.equals(daysAgo) )){
				Collection trackingNumbers = fetchTrackingDetailTableRows(rxForm.getPrescriptionId(), request);
				if (trackingNumbers.size() > 0) {
					if (LOG.isInfoEnabled()) {
						LOG.info("For:: "+request.getRemoteUser()+" addCellTracking :: true :: rx release date : " + releasedDate + " 51 daysAgo: " + daysAgo + " Mail/Window: " + rxForm.getPrescriptionMailWindow());
					}

	        		tr.addContentRedButtonCellWithTitle(TRACK_DELIVERY, rxForm.getPrescriptionId().toString(), TEXT_VIEW_TRACKING_ITEM_DETAILS);
	        	}else{
					tr.addContentCell(" ");
	        	}
			}else{
					if (LOG.isInfoEnabled()) {
						LOG.info("For:: "+request.getRemoteUser()+" addCellTracking::false:: rx release date: " + releasedDate + " 51 daysAgo: " + daysAgo + " Mail/Window: " + rxForm.getPrescriptionMailWindow());
					}

					tr.addContentCell(" ");
			}
	        }catch (java.text.ParseException e) {
				if (LOG.isErrorEnabled()) {
					LOG.error("For:: "+request.getRemoteUser()+" addCellTracking::releasedDate parse exception: " + e);
				}

	        	e.printStackTrace();
			}

        }
        else
        {
        	tr.addContentCell(" ");
		}
    }

    protected boolean checkAddTrackingRow(PrescriptionForm rxForm, String contextPath, HttpServletRequest request) {

		boolean retVal = false;

		int noOfDaysToShowTrackingDetails = RxRefillProperties.getInstance().getNoOfDaysToShowTrackingDetails();

		if (LOG.isInfoEnabled()) {
			LOG.info("For:: "+request.getRemoteUser()+" noOfDaysToShowTrackingDetails :: " + noOfDaysToShowTrackingDetails);
		}

        Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DAY_OF_YEAR , -1*noOfDaysToShowTrackingDetails);
		Date daysAgo = cal.getTime();

        if(rxForm.getPrescriptionReleaseDateTimeDay()!= null && rxForm.getPrescriptionMailWindow() != null){
	        String releasedDateString = rxForm.getPrescriptionReleaseDateTimeMonth() + "/" +
			   							rxForm.getPrescriptionReleaseDateTimeDay() + "/" +
			   							rxForm.getPrescriptionReleaseDateTimeYear();

			if (LOG.isInfoEnabled()) {
				LOG.info("For:: "+request.getRemoteUser()+" addCellTracking :: releasedDateString : " + releasedDateString);
			}

	        DateFormat df = new SimpleDateFormat("MM/dd/yyyy");

	        Date releasedDate = null;

	        try{
	        	releasedDate = df.parse(releasedDateString);

				if (LOG.isInfoEnabled()) {
					LOG.info("For:: "+request.getRemoteUser()+" addCellTracking :: releasedDate : " + releasedDate);
				}

	        	// adding refill tracking button if certain criteria is met
	        if((MailWindowEnumeration.MAIL.equalsIgnoreCase(rxForm.getPrescriptionMailWindow()) && releasedDate!= null && (releasedDate.after(daysAgo))|| releasedDate.equals(daysAgo) )){
				Collection trackingNumbers = fetchTrackingDetailTableRows(rxForm.getPrescriptionId(), request);
				if (trackingNumbers.size() > 0) {
					if (LOG.isInfoEnabled()) {
						LOG.info("For:: "+request.getRemoteUser()+" addCellTracking :: true :: rx release date : " + releasedDate + " 51 daysAgo: " + daysAgo + " Mail/Window: " + rxForm.getPrescriptionMailWindow());
					}

	        		retVal = true;
	        	}else{
					retVal = false;
	        	}
			}else{
					if (LOG.isInfoEnabled()) {
						LOG.info("For:: "+request.getRemoteUser()+" addCellTracking::false:: rx release date: " + releasedDate + " 51 daysAgo: " + daysAgo + " Mail/Window: " + rxForm.getPrescriptionMailWindow());
					}

					retVal = false;
			}
	        }catch (java.text.ParseException e) {
				if (LOG.isErrorEnabled()) {
					LOG.error("For:: "+request.getRemoteUser()+" addCellTracking::releasedDate parse exception: " + e);
				}

	        	e.printStackTrace();
	        	retVal = false;
			}
        }
        else
        {
        	retVal = false;
		}

		return retVal;
    }



    protected void populateListTableRows(HttpServletRequest request) {
        if (viewHelper == null) return;

        // re-sort the entity list
        if (doSort)
            viewHelper.sortEntityList(
                    sortColumn,
                    sortAscending,
                    getDefaultSortColumn2(),
                    isDefaultSortAscending(getDefaultSortColumn2()));

        // get the paginated list
        pageEntityList = viewHelper.getPageEntityList(pageSize, pageNumber, request);

        // re-populated the view table
        addListTableRows(pageEntityList, request.getContextPath(), request);
    }

    private void addListTableRows(List tableEntityList, String requestContextPath, HttpServletRequest request) {

        listTableRows = new ArrayList();
        hasRefillable = false;
        boolean oddRow = true;

        for (int i=0; i<tableEntityList.size(); i++) {
            Prescription rx = (Prescription)pageEntityList.get(i);
            PrescriptionForm rxForm = new PrescriptionForm();
            rxForm.copyFromEntity(rx);

            // show submit refill button if at least one rx is refillable
            if (checkRefillable(rx)) hasRefillable = true;

			TableRow row = new TableRow();
            boolean retVal = createPrescriptionTableRow(row, rxForm, requestContextPath, request);

            if (retVal) {

            // setting odd/even row style
            row.rowStyle = oddRow? STYLE_ROW_ODD:STYLE_ROW_EVEN;
            // toggle for the next row
            oddRow = oddRow? false:true;

            // adding a new row to the table
            listTableRows.add(row);
			}
        }
    }

    protected boolean isDefaultSortAscending(String sortColumn) {
        // time columns: descending sort as the default
        if (COLUMN_FILL_DATE.equals(sortColumn)
        		|| COLUMN_DISPENSE_DATE.equals(sortColumn)
                || COLUMN_SUBMIT_DATE.equals(sortColumn)
                || COLUMN_LAST_UPDATED.equals(sortColumn)
                || COLUMN_IS_REFILLABLE.equals(sortColumn)
                || COLUMN_TRACKING.equals(sortColumn))
            return false;
        else
            return true;
    }

    public boolean getHasRefillable() {
        return hasRefillable;
    }

    public boolean getHasTrackable() {
        return hasTrackable;
    }

    public synchronized List populateRefillList(boolean toRefill) {

        List refillList = new ArrayList();
        isDuplicateRefillSubmitted = false;

        for (int i=0; i< listTableRows.size(); i++) {

            TableRow row = (TableRow)listTableRows.get(i);
            Prescription rx = (Prescription)pageEntityList.get(i);

            if (row.getId() != null
                    && row.getId().equals(rx.getPrescriptionId())
                    && row.isCheckboxSelected()) {

                if (rx.getIsRefillable().booleanValue()) {

                    refillList.add(rx);

                    // to prevent user from requesting the same refills twice
                    // by double-clicking the submit button
                    if (toRefill) rx.setIsRefillable(new Boolean(false));

                } else
                   isDuplicateRefillSubmitted = true;
            }
        }

        return refillList;
    }

    public boolean isDuplicateRefillSubmitted() {
        return isDuplicateRefillSubmitted;
    }

    protected void addDetailRow(String header, String content) {
        TableRow tr = new TableRow();
        tr.addDetailHeaderCell(header);
        tr.addContentCell(content);
        detailTableRows.add(tr);
    }

    protected void addTrackingShowImageRow(String header, String content, String imagePath, String altString, boolean hasImg) {
        TableRow tr = new TableRow();
        TableCell cell = new TableCell();
        //tr.addDetailHeaderCell(header);
        if(hasImg) {
			cell.setImg(imagePath, altString);
			tr.addCell(cell);
		}
		else
		{
			tr.addContentCell(content);
		}
        trackingImageTableRows.add(tr);
    }

    protected void addTrackingImageRow(String header, String content) {
        TableRow tr = new TableRow();
        tr.addDetailHeaderCell(header);
        tr.addContentCell(content);
        trackingImageTableRows.add(tr);
    }

    protected void addTrackingDetailRow(String header, String content) {
        TableRow tr = new TableRow();
        tr.addDetailHeaderCell(header);
        tr.addContentCell(content);
        trackingDetailTableRows.add(tr);
    }

    protected void addDetailAnchorRow(String header, String content, String anchorParameter, String trackingAnchorParameter) {
        TableRow tr = new TableRow();
        tr.addDetailHeaderCell(header);
        tr.addAnchorCell(content,anchorParameter,trackingAnchorParameter);
        detailTableRows.add(tr);
    }

    protected void addTrackingDetailAnchorRow(String header, String content, String anchorParameter, String trackingAnchorParameter) {
        TableRow tr = new TableRow();
        tr.addDetailHeaderCell(header);
        tr.addAnchorCell(content,anchorParameter,trackingAnchorParameter);
        trackingDetailTableRows.add(tr);
    }

    protected void addTrackingDetailImageAnchorRow(String header, String content, String anchorParameter, String trackingAnchorParameter) {
        TableRow tr = new TableRow();
        tr.addDetailHeaderCell(header);
        tr.addImageAnchorCell(content,anchorParameter,trackingAnchorParameter);
        trackingDetailTableRows.add(tr);
    }

    private void addDetailHeaderRow(String header) {
        TableRow tr = new TableRow();
        TableCell td = new TableCell();
        td.isHeaderCell = true;
        td.hasTwoColumns = true;
        td.content = header;
        tr.addCell(td);
        detailTableRows.add(tr);
    }

    private void addTrackingImageHeaderRow(String header) {
        TableRow tr = new TableRow();
        TableCell td = new TableCell();
        td.styleClass = STYLE_ROW_TRACK_DETAIL_HEADER;
        tr.rowStyle = STYLE_ROW_TRACK_DETAIL_HEADER;
        td.isHeaderCell = true;
        td.hasTwoColumns = true;
        td.content = header;
        tr.addCell(td);
        trackingImageTableRows.add(tr);
    }

    private void addTrackingDetailHeaderRow(String header) {
        TableRow tr = new TableRow();
        TableCell td = new TableCell();
        td.styleClass = STYLE_ROW_TRACK_DETAIL_HEADER;
        tr.rowStyle = STYLE_ROW_TRACK_DETAIL_HEADER;
        td.isHeaderCell = true;
        td.hasTwoColumns = true;
        td.content = header;
        tr.addCell(td);
        trackingDetailTableRows.add(tr);
    }

    private void addDetailDateRow(String header, String month, String day, String year) {
        TableRow tr = new TableRow();
        tr.addDetailHeaderCell(header);
        tr.addDateCell(month, day, year);
        detailTableRows.add(tr);
    }

    private void addTrackingDetailDateRow(String header, String month, String day, String year) {
        TableRow tr = new TableRow();
        tr.addDetailHeaderCell(header);
        tr.addDateCell(month, day, year);
        trackingDetailTableRows.add(tr);
    }

    //private String getDefaultSortColumn2() {
    //    return COLUMN_RX_Number;
    //    //return COLUMN_DRUG_NAME;
    //}

    /**
     * Added cell to add number of prescriptions remaining cell on perscription form.
     * @param tr
     * @param rxForm
     * @param contextPath
     */
    protected final void addCellPrescriptionRxRemaining(TableRow tr,
        	PrescriptionForm rxForm, String contextPath)
    {
        tr.addContentCell(rxForm.getPrescriptionNumberOfRefills());
    }

    protected final void addCellPrescriptionDispenseDate(TableRow tr,
        	PrescriptionForm rxForm, String contextPath)
    {
        tr.addDateCell(
           rxForm.getPrescriptionReleaseDateTimeMonth(),
           rxForm.getPrescriptionReleaseDateTimeDay(),
           rxForm.getPrescriptionReleaseDateTimeYear());
    }

    protected final void addCellPrescriptionFillDate(TableRow tr,
        	PrescriptionForm rxForm, String contextPath)
    {
        tr.addDateCell(
           rxForm.getPrescriptionLastFillDateMonth(),
           rxForm.getPrescriptionLastFillDateDay(),
           rxForm.getPrescriptionLastFillDateYear());
    }

    protected final void addCellMedication(TableRow tr,
    	PrescriptionForm rxForm, String contextPath)
    {
        tr.addContentAnchorCellWithTitle(
        		rxForm.getPrescriptionDrugName() + "<br/>RX#" + rxForm.getPrescriptionPrescriptionNumber(),
                rxForm.getPrescriptionId().toString(),
                TEXT_VIEW_ITEM_DETAILS);
    }

    protected final void addCellLastUpdated(TableRow tr,
    	PrescriptionForm rxForm, String contextPath)
    {
        tr.addDateCell(
           rxForm.getPrescriptionModifiedDateMonth(),
           rxForm.getPrescriptionModifiedDateDay(),
           rxForm.getPrescriptionModifiedDateYear());
    }

    protected final void addCellFacility(TableRow tr,
    	PrescriptionForm rxForm, String contextPath)
    {
        tr.addContentCell(rxForm.getFacilityName());
    }

    protected abstract boolean createPrescriptionTableRowContent(TableRow tr,
    	PrescriptionForm rxForm, String contextPath, HttpServletRequest request);

    private boolean createPrescriptionTableRow(TableRow tr, PrescriptionForm rxForm,
    		String contextPath, HttpServletRequest request)
    {
        //TableRow tr = new TableRow();
        tr.id = rxForm.getPrescriptionId();
        if (rxForm.getPrescriptionIsInvalidMhvStatus().equalsIgnoreCase(
        	TEXT_TRUE))
        {
            tr.boldFont = true;
        }
        boolean retVal = createPrescriptionTableRowContent(tr, rxForm, contextPath, request);
        //return tr;
        return retVal;
    }

}