package gov.va.med.mhv.rxrefill.web.form;

import gov.va.med.mhv.rxrefill.web.util.ListViewHelper;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * FormData get and set methods may be overwritten by the Form Bean editor.
 */
public abstract class ViewData
// From: extends com.bea.wlw.netui.pageflow.FormData
	// The beehive FormData is deprecated. Suggested is to just implement
	// Serializable
	implements Serializable
{
	private static final Log LOG = LogFactory.getLog(ViewData.class);

    public final String PARAMETER_PAGE_NUM = "pageNum";
    public final String PARAMETER_PAGE_SIZE = "pageSize";
    public final String PARAMETER_SORT_COLUMN = "sortColumn";

    public final String STYLE_ROW_ODD = "mhv-list-row-odd";
    public final String STYLE_ROW_EVEN = "mhv-list-row-even";
    public final String STYLE_ROW_TRACK_DETAIL_HEADER = "mhv-heading-table-row";
    public final String STYLE_CELL = "mhv-list-cell";
    public final String STYLE_CELL_BOLD = "mhv-list-cell-bold";
    public final String STYLE_CELL_CENTER = "mhv-list-cell-center";
    public final String STYLE_CELL_HEADER_SORTABLE = "mhv-list-col-sortable";
    public final String STYLE_CELL_HEADER_SORTED_ASC = "mhv-list-col-sorted mhv-list-col-sortable mhv-list-col-sorted-asc";
    public final String STYLE_CELL_HEADER_SORTED_DES = "mhv-list-col-sorted mhv-list-col-sortable mhv-list-col-sorted-des";

    public final String IMAGE_FOLDER = "/resources/images";
    public final String IMAGE_FILE_ALERT = IMAGE_FOLDER + "/icons15/icoAlert15.gif";

    public final String TEXT_ASCENDING = "ascending";
    public final String TEXT_DESCENDING = "descending";
    public final String TEXT_VIEW_ITEM_DETAILS = "View item details";
    public final String TEXT_VIEW_TRACKING_ITEM_DETAILS = "View tracking details";
    public final String TEXT_SORT_BY = "Sort by ";
    public final String TEXT_IN = " in ";
    public final String TEXT_ORDER = " order.";
    public final String TEXT_CURRENT_SORT_IS = "  The current sort order for this column is ";
    public final String TEXT_PERIOD = ".";

    public final String TEXT_TRUE = "true";
    public final String TEXT_FALSE = "false";
    public final String DATE_FORMAT_SEPERATOR = "/";

    protected ListViewHelper viewHelper;
    protected boolean doSort;
    protected int pageSize;
    protected int pageNumber = 1;
    protected String sortColumn;
    protected boolean sortAscending = true;
    protected String sortColumn2;
    protected boolean sortAscending2 = true;
    protected List entityList;
    protected List pageEntityList;
    protected TableRow listHeaderRow;
    protected List listTableRows;
    protected List detailTableRows;
    protected List trackingDetailTableRows;
    protected List trackingImageTableRows;

    abstract public String getHeaderType();
    abstract public String getHeader1Type();
    abstract public String getListType();
    abstract public String getDetailType();
    abstract public String getDetail1Type();
    abstract public String getHelpType();

    abstract protected String getDefaultSortColumn();
    abstract protected void populateListTableHeaderRow();
    abstract protected void populateListTableRows(HttpServletRequest request);
    abstract protected void populateDetailTableRows(org.tigris.atlas.web.struts.InputForm form);
    abstract protected void populateTrackingDetailTableRows(org.tigris.atlas.web.struts.InputForm form, HttpServletRequest request);
	abstract protected void populateTrackingImageTableRows(HttpServletRequest request);
    abstract protected boolean isDefaultSortAscending(String sortColumn);

    public ViewData(int pageSize) {
        this.pageSize = pageSize;
    }

    public TableRow getListHeaderRow() {
        return listHeaderRow;
    }

    public List getListTableRows() {
        return listTableRows;
    }

    public boolean isEmptyListTableRows() {
    	if (LOG.isDebugEnabled()) {
    		LOG.debug("isEmptyListTableRows: " + listTableRows
    			+ (listTableRows != null ? listTableRows.size() : 0));
    	}
    	// Added to fix JSP expression problem
        return (listTableRows == null) || (listTableRows.size() == 0);
    }

    public List getDetailTableRows() {
        return detailTableRows;
    }

    public List getTrackingDetailTableRows() {
        return trackingDetailTableRows;
    }

    public List getTrackingImageTableRows() {
        return trackingImageTableRows;
    }


    public void setEntityList(Collection entities, HttpServletRequest request) {

        // populate table header
        populateListTableHeaderRow();

        // set default sort
        setSorting(getDefaultSortColumn());

        if (entities != null && entities.size() > 0) {

            entityList = new ArrayList();
            entityList.addAll(entities);
            viewHelper = new ListViewHelper(entityList);

            // populate table rows
            populateListTableRows(request);
        }
    }

    public List getPageEntityList() {
        return pageEntityList;
    }

    public int getPageNumber() {
        return pageNumber;
    }

    public void resetListTableRows(HttpServletRequest request) {
        setViewOptionsFromRequest(request);
        populateListTableRows(request);
    }

    public String getPageNavigationBar(String baseUrl, String portletLabel,
    	PageContext ctx)
    {
        return (viewHelper != null)
        	? viewHelper.getPageNavigationBar(PARAMETER_PAGE_SIZE,
        		PARAMETER_PAGE_NUM, baseUrl, portletLabel,ctx)
        	: null;
    }

    private void setSortingIndicators() {

        for (int i=0; i<listHeaderRow.cells.size(); i++) {

            TableCell th = (TableCell)listHeaderRow.cells.get(i);

            if (!th.hasAnchor) continue; // unsortable column

            if (th.anchorParameter != null
                    && th.anchorParameter.equals(sortColumn)) {

                // current sort column
                th.title = getHeaderCellSortingTitle(th, true);

                if (sortAscending) {
                    th.styleClass = STYLE_CELL_HEADER_SORTED_ASC;
                } else
                    th.styleClass = STYLE_CELL_HEADER_SORTED_DES;
            } else {

                // unsorted but sortable column
                th.styleClass = STYLE_CELL_HEADER_SORTABLE;
                th.title = getHeaderCellSortingTitle(th, false);
            }
        }
    }

    private String getHeaderCellSortingTitle(TableCell th, boolean currentSort) {

        // default sort Ascending
        String currentOrder = TEXT_ASCENDING;
        String newOrder = TEXT_DESCENDING;

         if (currentSort) {
            if (!sortAscending) {
                currentOrder = TEXT_DESCENDING;
                newOrder = TEXT_ASCENDING;
            }
        } else {
            // not currently sorted
            if (isDefaultSortAscending(th.anchorParameter))
                newOrder = TEXT_ASCENDING;
            else
                newOrder = TEXT_DESCENDING;
        }

        StringBuffer sf = new StringBuffer();

        if (th.contentNarrow != null && th.contentNarrow.length() > 0)
            sf.append(TEXT_SORT_BY + th.contentNarrow + TEXT_IN + newOrder + TEXT_ORDER);
        else
            sf.append(TEXT_SORT_BY + th.content + TEXT_IN + newOrder + TEXT_ORDER);

        if (currentSort) {
            sf.append(TEXT_CURRENT_SORT_IS + currentOrder + TEXT_PERIOD);
        }
        return sf.toString();
    }

    private void setSorting(String sortColumn) {
        doSort = true;
        if (sortColumn.equals(this.sortColumn)) {
            // change sort order only
            sortAscending = (sortAscending? false:true);

        } else {
            this.sortColumn = sortColumn;
            sortAscending = isDefaultSortAscending(sortColumn);
        }

        setSortingIndicators();
        pageNumber = 1;
    }

    private void setViewOptionsFromRequest(HttpServletRequest request) {
        boolean reSort = false;

        // check sort column
        if (request.getParameter(PARAMETER_SORT_COLUMN) != null) {
            String sortColumn = (String)request.getParameter(PARAMETER_SORT_COLUMN);
            setSorting(sortColumn);
            reSort = true;
        }

        // check page number
        if (request.getParameter(PARAMETER_PAGE_NUM) != null) {
            pageNumber = Integer.parseInt(request.getParameter(PARAMETER_PAGE_NUM));
        }

        // check page size
        if (request.getParameter(PARAMETER_PAGE_SIZE) != null) {
            pageSize = Integer.parseInt(request.getParameter(PARAMETER_PAGE_SIZE));
            pageNumber = 1;
        }

        this.doSort = reSort;
    }

    // class TableRow represent HTML <tr> data
    public class TableRow {

        public Long id;
        public String rowStyle;
        public boolean boldFont;
        public List cells = new ArrayList();
        public int checkboxColumnIndex;

        public Long getId() {
            return this.id;
        }

        public void addCell(TableCell cell) {
            if (boldFont && !cell.hasCheckbox) {
                cell.styleClass = STYLE_CELL_BOLD;
            }
            cells.add(cell);
        }

        public List getCells() {
            return cells;
        }

        public void clear() {
            rowStyle = null;
            boldFont = false;
            cells = new ArrayList();
        }

        public void addDetailHeaderCell(String content) {
            TableCell cell = new TableCell(content, null);
            cell.isHeaderCell = true;
            cells.add(cell);
        }

        public void addContentCell(String content) {
            TableCell cell = new TableCell(content, null);
            cells.add(cell);
        }

        public void addAnchorCell(String content, String anchorParameter, String trackingAnchorParameter) {
            TableCell cell = new TableCell(content, anchorParameter);
            cell.trackingAnchorParameter = trackingAnchorParameter;
            cells.add(cell);
        }

        public void addImageAnchorCell(String content, String anchorParameter, String trackingAnchorParameter) {
            TableCell cell = new TableCell(content, anchorParameter);
            cell.setImageAnchor(anchorParameter);
            cell.trackingAnchorParameter = trackingAnchorParameter;
            cells.add(cell);
        }

        public void addContentAnchorCellWithTitle(String content, String anchorParameter, String title) {
            TableCell cell = new TableCell(content, anchorParameter);
            cell.title = title;
            cells.add(cell);
        }

        public void addContentRedButtonCellWithTitle(String content, String anchorParameter, String title) {
            TableCell cell = new TableCell(content, anchorParameter);
            cell.title = title;
            cell.styleClass = STYLE_CELL_CENTER;
            cell.hasRedButton = true; //false;
            cells.add(cell);
        }

        public void addSortableHeaderCell(String content, String anchorParameter) {
            TableCell cell = new TableCell(content, anchorParameter);
            cells.add(cell);
        }

        public void addSortableHeaderCellCenter(String content, String anchorParameter) {
            TableCell cell = new TableCell(content, anchorParameter);
            cell.styleClass = STYLE_CELL_CENTER;
            cells.add(cell);
        }

        public void addHeaderCellCenter(String content, String anchorParameter) {
            TableCell cell = new TableCell(content, null);
            cell.styleClass = STYLE_CELL_CENTER;
            cells.add(cell);
        }

        public void addSortableHeaderCellNarrow(String content, String anchorParameter, String contentNarrow) {
            TableCell cell = new TableCell(content, anchorParameter);
            cell.contentNarrow = contentNarrow;
            cells.add(cell);
        }

        public void addSortableHeaderCellPFHidden(String content, String anchorParameter) {
            TableCell cell = new TableCell(content, anchorParameter);
            cell.pfHidden = true;
            cells.add(cell);
        }

        public void addSortableHeaderCellPFHiddenNarrow(String content, String anchorParameter, String contentNarrow) {
            TableCell cell = new TableCell(content, anchorParameter);
            cell.pfHidden = true;
            cell.contentNarrow = contentNarrow;
            cells.add(cell);
        }

        public void addDateCell(String month, String day, String year) {
            TableCell cell = new TableCell(month, day, year);
            addCell(cell);
        }

        public int getColumnCount() {
            return cells.size();
        }

        public boolean isCheckboxSelected() {
            TableCell cell = (TableCell)cells.get(checkboxColumnIndex);
            return cell.checkboxSelected;
        }
    }

    // class TableCell represent HTML <td> data
    public class TableCell {

        public boolean isHeaderCell;
        public boolean pfHidden; // hide in printer friendly pages
        public String styleClass = STYLE_CELL;
        public String title;
        public String content;
        public String contentNarrow;
        public boolean hasAnchor;
        public boolean hasImageAnchor;
        public boolean hasRedButton;
        public boolean hasTrackingAnchor;
        public String anchorParameter;
        public String trackingAnchorParameter;
        public boolean hasImg;
        public String imgSrc;
        public String imgAlt;
        public boolean hasCheckbox;
        public String checkboxAlt;
        public String submitButtonAlt;
        public boolean checkboxSelected;
        public boolean hasTwoColumns;
        public boolean hasTrackingButton;

        public TableCell() {};

        public TableCell(String content, String anchorParameter) {
            this.content = content;
            setAnchor(anchorParameter);
        }

        public TableCell(String month, String day, String year) {
            addDateContent(month, day, year);
        }

        public void addDateContent(String month, String day, String year) {
            StringBuffer date = new StringBuffer();
            if (month != null)
                date.append(month);
            if (day != null)
                date.append(DATE_FORMAT_SEPERATOR + day);
            if (year != null)
                date.append(DATE_FORMAT_SEPERATOR + year);

            this.content = date.toString();
        }

        public void setAnchor(String anchorParameter) {
            if (anchorParameter == null || anchorParameter.length() == 0)
                return;

            this.hasAnchor = true;
            this.anchorParameter = anchorParameter;
        }

        public void setImageAnchor(String anchorParameter) {
            if (anchorParameter == null || anchorParameter.length() == 0)
                return;

            this.hasImageAnchor = true;
            this.anchorParameter = anchorParameter;
        }

        public void setTrackingAnchor(String anchorParameter) {
            if (anchorParameter == null || anchorParameter.length() == 0)
                return;

            //this.hasAnchor = true;
            this.hasTrackingButton = true;
            this.anchorParameter = anchorParameter;
        }

        public void setImg(String imgSrc, String imgAlt) {
            this.hasImg = true;
            this.imgSrc = imgSrc;
            this.imgAlt = imgAlt;
        }
    }
}