package gov.va.med.mhv.rxrefill.web.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;

import org.displaytag.model.Row;
import org.displaytag.model.RowSorter;
import org.displaytag.properties.TableProperties;
import org.displaytag.util.DefaultHref;

import gov.va.med.mhv.core.web.util.ComparableComparator;

public class ListViewHelper {

	private final List entityList;
	private RxRefillSmartListHelper listHelper;

	public ListViewHelper(List entityList) {
		this.entityList = entityList;
	}

	// view helper functions (sorting, paging) from here
	public List getPageEntityList(int pageSize, int pageNumber,
		HttpServletRequest request)
	{
		listHelper = new RxRefillSmartListHelper(entityList, entityList.size(),
			pageSize, TableProperties.getInstance(request), false);
		listHelper.setCurrentPage(pageNumber);
		return listHelper.getListForCurrentPage();
	}

	public String getPageNavigationBar(String pageSizeParameter, 
		String pageNumberParameter, String baseUrl, String portletLabel, 
		PageContext ctx)
	{
		if (listHelper == null) {
			return null;
		}

		StringBuffer navigationBar = new StringBuffer();
		navigationBar.append(listHelper.getSearchResultsSummary());
		navigationBar.append(listHelper.getPageNavigationBar(baseUrl, 
			portletLabel, pageNumberParameter));
		navigationBar.append(listHelper.getPageSizes(baseUrl, portletLabel, 
			pageSizeParameter));

		return navigationBar.toString();
	}

	public void sortEntityList(String sortColumn, boolean isAscending,
		String sortColumn2, boolean isAscending2)
	{

		if (sortColumn == null || entityList == null || entityList.isEmpty())
			return;

		// convert entityList to row objects
		ArrayList<Row> rowList = new ArrayList<Row>();
		for (int i = 0; i < entityList.size(); i++) {
			Row row = new Row(entityList.get(i), i);
			rowList.add(row);
		}

		// sort rows using display tag's RowSorter Comparator
		/*
		 * RowSorter(int sortedColumnIndex, (optional) String beanProperty,
		 * TableDecorator tableDecorator, (optional) boolean ascendingOrder)
		 */

		// do the secondary sort first
		if (sortColumn2 != null && !sortColumn2.equals(sortColumn))
			Collections.sort(rowList, new RowSorter(0, sortColumn2, null,
				isAscending2, new ComparableComparator()));

		// do the primary sort last
		Collections.sort(rowList, new RowSorter(0, sortColumn, null,
			isAscending, new ComparableComparator()));

		// re-populate the entityList
		for (int i = 0; i < entityList.size(); i++) {
			Row row = rowList.get(i);
			entityList.set(i, row.getObject());
		}
	}

}
