package gov.va.med.mhv.rxrefill.web.util;

import java.util.List;

import org.displaytag.pagination.SmartListHelper;
import org.displaytag.properties.TableProperties;

public class RxRefillSmartListHelper extends SmartListHelper {

	// NOTE: that this class violates the Liskov-substitution principle,
	// but the class was created to avoid having to rewrite major portions
	// of the RxRefill application
	// It purpose is to circumvent the inability of the superclass's use
	// of Href to do proper URL rewriting necessary for WebLOgic 9.2
	
	// These page sizes must be in ascending order
	private static final int[] PAGE_SIZES = { 10, 25, 50, 100 };
	
	/*
	 * TODO: These should be visible from parent class, but since they are not,
	 * keep a local copy
	 */
	private int fullListSize;
	private int pageSize;

	public RxRefillSmartListHelper(List list, int fullSize, int itemsInPage,
		TableProperties tableProperties, boolean partialList)
	{
		super(list, fullSize, itemsInPage, tableProperties, partialList);
		this.fullListSize = fullSize;
		this.pageSize = itemsInPage;
	}

	public String getPageSizes(String baseUrl, String portletLabel, 
		String pageSizeParameter) 
	{
		if (fullListSize <= PAGE_SIZES[0]) {
			// Don't display anything when there are less items 
			// than smallest page size
			return "";
		}
		StringBuilder builder = new StringBuilder(
			"\n<div class=\"displaytag-pagesizes\">Number of rows to display per page: ");
		for (int i = 0; i < PAGE_SIZES.length; i++) {
			appendPageSize(builder, baseUrl, portletLabel, i, pageSizeParameter);
		}
		builder.append("\n</div>\n");

		return builder.toString();
	}

	/**
	 * Adds page size denotes by pageSizeIndex to page size list.
	 * The page size becomes an anchor if the current page size does not equal
	 * the given page size and the number of current list elements exceeds  
	 * @param builder
	 * @param pageSizeIndex
	 */
	private void appendPageSize(StringBuilder builder, String baseUrl, 
		String portletLabel, int pageSizeIndex, String pageSizeParameter) 
	{
		int pageSize = PAGE_SIZES[pageSizeIndex];
		int prevPageSize = (pageSizeIndex > 0) 
			? PAGE_SIZES[pageSizeIndex-1] : 0; 
		boolean showAnchor =
			// not the current page size
			(this.pageSize != pageSize)
			// list actually has enough elements to increase page size
			&& (fullListSize > prevPageSize);
		if (showAnchor) {
			builder.append("<a href=\"").append(baseUrl).append("&").
				append(portletLabel).append(pageSizeParameter).append("=").
				append(pageSize).append("\" title=\"Display ").append(pageSize).
				append(" items per page\">");
		}
		builder.append(pageSize);
		if (showAnchor) {
			builder.append("</a>");
		}
		builder.append(" ");
	}

	public String getPageNavigationBar(String baseUrl, String portletLabel, 
		String pageNumberParameter) 
	{
		int pageCount = computedPageCount();
		if (pageCount == 1) {
			return "";
		}

		int index = getFirstIndexForCurrentPage();
		int currentPage = (index/pageSize) + (index%pageSize==0 ? 1 : 0);

		StringBuilder builder = new StringBuilder(
			"\n\n<div class=\"displaytag-pagelinks\">");

		appendPageNumber(builder, baseUrl, portletLabel, 1, currentPage, 
			"First", pageNumberParameter);
		builder.append("/");
		appendPageNumber(builder, baseUrl, portletLabel, 
			((currentPage > 1) ? currentPage-1 : currentPage), 
			currentPage, "Prev", pageNumberParameter);
		builder.append(" ");
		for (int i = 1; i <= pageCount; i++) {
			appendPageNumber(builder, baseUrl, portletLabel, i, currentPage, 
				null, pageNumberParameter);
			builder.append(" ");
		}
		appendPageNumber(builder, baseUrl, portletLabel, 
			((currentPage < pageCount) ? currentPage+1 : currentPage), 
			currentPage, "Next", pageNumberParameter);
		builder.append("/");
		appendPageNumber(builder, baseUrl, portletLabel, pageCount, currentPage, 
			"Last", pageNumberParameter);
		builder.append("</div>");

		return builder.toString();
	}


	private void appendPageNumber(StringBuilder builder, String baseUrl, 
		String portletLabel, int pageNumber, int currentPage, 
		String anchorLabel, String pageNumberParameter) 
	{
		// Show anchor if not referring to the current page
		boolean showAnchor = currentPage != pageNumber;
		// Showing a page number? Or a label, such as First, Next etc.?
		boolean hasPageNumber = (anchorLabel == null);
		if (showAnchor) {
			String title = "Go to " + ((hasPageNumber) ? "page " + pageNumber 
				: anchorLabel + " page");
			builder.append("<a href=\"").append(baseUrl).
				append("&").append(portletLabel).append(pageNumberParameter).
				append("=").append(pageNumber).append("\" title=\"").
				append(title).append("\">");
		} else if (hasPageNumber) {
			builder.append("<strong>");
		}
		builder.append((hasPageNumber) ? Integer.toString(pageNumber) 
			: anchorLabel); 
		if (showAnchor) {
			builder.append("</a>");
		} else if (hasPageNumber) {
			builder.append("</strong>");
		}
	}

}
