package gov.va.med.mhv.usermgmt.persist.ldap.sunone;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.tigris.atlas.bizobj.BusinessObject;
import org.tigris.atlas.transfer.PrimaryKey;

import gov.va.med.mhv.usermgmt.persist.UserEntryDao;
import gov.va.med.mhv.usermgmt.persist.ldap.LdapAttributeModification;
import gov.va.med.mhv.usermgmt.persist.ldap.LdapEntryModification;
import gov.va.med.mhv.usermgmt.persist.ldap.LdapEnvironment;
import gov.va.med.mhv.usermgmt.persist.ldap.LdapUpdate;
import gov.va.med.mhv.usermgmt.persist.ldap.LdapUtil;

/**
 * UserDao implementation that writes to a directory server.
 *
 * @author Jon Crater Apr 4, 2006 3:51:19 PM
 */
public class UserSunOneDaoImpl implements UserEntryDao {
	/** The directory environment against which we're operating. */
	private LdapEnvironment env = null;
	
	private LdapEnvironment getEnv() {
		if( env == null ) {
			env = LdapEnvironment.getInstance();
		}
		
		return env;
	}
	
	/**
	 * Reset the password of the user represented by the userName parameter.
	 * @param userName
	 * @param newPassword
	 */
	public void resetPassword(String userName, String newPassword) {		
		LdapUpdate update = new LdapUpdate(LdapUtil.buildUserDn(userName));
		update.addCommand(LdapAttributeModification.wipeAndReplace(
			getEnv().getUserPasswordAttribute(), newPassword));
		update.executeUpdate();
	}
	
	/**
	 * Change the userName of the user represented by the oldUserName parameter.
	 * @param oldUserName
	 * @param newUserName
	 */
	public void changeUserName(String oldUserName, String newUserName) {
		LdapUpdate update = new LdapUpdate(LdapUtil.buildUserDn(newUserName));
		update.addCommand(LdapEntryModification.wipeAndReplace(LdapUtil.buildUserDn(oldUserName), Collections.EMPTY_MAP));
		update.executeUpdate();
	}

	/**
	 * Create a user entry in the directory.
	 * @param userName The value that will be written to the uid, cn and sn attributes of the newly created entry.
	 * @param password The value that will be encrypted and written to the password attribute of the newly created entry.
	 */
	public void createUser(String userName, String password) {
		LdapUpdate update = new LdapUpdate(LdapUtil.buildUserDn(userName));
		
		Map values = new HashMap(5);
		values.put(getEnv().getObjectClassAttribute(), getEnv().getObjectClassValues());
		values.put(getEnv().getUserNameAttribute(), userName);
		values.put(getEnv().getUserPasswordAttribute(), password);
		
		// until and unless we pass first name and last name, these attributes have to be set
		// to comply with the schema rules.  for now, set them both to the userName
		values.put(getEnv().getCommonNameAttribute(), userName);
		values.put(getEnv().getSurNameAttribute(), userName);
		
		// persist it
		update.addCommand(LdapEntryModification.add(values));
		update.executeUpdate();
	}

	/**
	 * Remove the user with the specified userName from the directory.
	 * @param userName The username to delete.
	 */
	public void removeUser(String userName) {
		LdapUpdate update = new LdapUpdate(LdapUtil.buildUserDn(userName));
		update.addCommand(LdapEntryModification.delete());
		update.executeUpdate();
	}

	public BusinessObject delete(PrimaryKey arg0) {
		throw new UnsupportedOperationException("delete not implemented for this DAO");
	}

	public BusinessObject findByPrimaryKey(PrimaryKey arg0) {
		throw new UnsupportedOperationException("findByPrimaryKey not implemented for this DAO");
	}
}