package gov.va.med.mhv.usermgmt.service.impl;

import gov.va.med.mhv.core.util.DescriptionBuilder;
import gov.va.med.mhv.core.util.MessagesStringBuilder;
import gov.va.med.mhv.core.util.Precondition;
import gov.va.med.mhv.core.util.TimestampUtils;
import gov.va.med.mhv.integration.registry.transfer.PersonalInfo;
import gov.va.med.mhv.service.MHVAbstractService;
import gov.va.med.mhv.usermgmt.bizobj.BatchReportBO;
import gov.va.med.mhv.usermgmt.bizobj.BusinessObjectFactory;
import gov.va.med.mhv.usermgmt.bizobj.InPersonAuthenticationAssembler;
import gov.va.med.mhv.usermgmt.bizobj.InPersonAuthenticationBO;
import gov.va.med.mhv.usermgmt.bizobj.PatientAssembler;
import gov.va.med.mhv.usermgmt.bizobj.PatientBO;
import gov.va.med.mhv.usermgmt.bizobj.UserProfileBO;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.AuthenticationStatus;
import gov.va.med.mhv.usermgmt.enumeration.ContactMethodEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.CountryEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.GenderEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.MaritalStatusEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.PatientCorrelationStatus;
import gov.va.med.mhv.usermgmt.enumeration.StateEnumeration;
import gov.va.med.mhv.usermgmt.integration.adapter.PatientCorrelationServiceAdapter;
import gov.va.med.mhv.usermgmt.messages.UserManagementMessages;
import gov.va.med.mhv.usermgmt.service.EntityMaintenanceService;
import gov.va.med.mhv.usermgmt.service.InPersonAuthenticationService;
import gov.va.med.mhv.usermgmt.service.InPersonAuthenticationServiceResponse;
import gov.va.med.mhv.usermgmt.service.PatientCollectionServiceResponse;
import gov.va.med.mhv.usermgmt.service.PatientService;
import gov.va.med.mhv.usermgmt.service.PatientServiceResponse;
import gov.va.med.mhv.usermgmt.service.ServiceFactory;
import gov.va.med.mhv.usermgmt.service.UserProfileServiceResponse;
import gov.va.med.mhv.usermgmt.service.adapter.AdapterException;
import gov.va.med.mhv.usermgmt.service.adapter.DuplicatePatientException;
import gov.va.med.mhv.usermgmt.service.adapter.LookupPatientAdapter;
import gov.va.med.mhv.usermgmt.service.adapter.LookupPatientAdapterWithMiddleName;
import gov.va.med.mhv.usermgmt.service.adapter.MpiProperties;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.service.handler.MviProperties;
import gov.va.med.mhv.usermgmt.transfer.BatchReport;
import gov.va.med.mhv.usermgmt.transfer.Facility;
import gov.va.med.mhv.usermgmt.transfer.InPersonAuthentication;
import gov.va.med.mhv.usermgmt.transfer.Patient;
import gov.va.med.mhv.usermgmt.transfer.PatientPK;
import gov.va.med.mhv.usermgmt.transfer.PatientRegistryChange;
import gov.va.med.mhv.usermgmt.transfer.TransferObjectFactory;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;
import gov.va.med.mhv.usermgmt.transfer.UserProfileDeactivationReason;
import gov.va.med.mhv.usermgmt.util.InPersonAuthenticationStatusUtils;
import gov.va.med.mhv.usermgmt.util.MessageKeys;
import gov.va.med.mhv.usermgmt.util.PHRAccessControl;
import gov.va.med.mhv.usermgmt.util.PatientCorrelationStatusUtils;
import gov.va.med.mhv.usermgmt.util.RegistryDescriptionBuilder;
import gov.va.med.mhv.usermgmt.util.UserProfileDeactivationUtils;

//import java.awt.Font;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.BooleanUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.enums.Enum;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.messages.Message;
import org.tigris.atlas.service.BooleanServiceResponse;
import org.tigris.atlas.service.EntityServiceResponse;
import org.tigris.atlas.service.ServiceResponse;
import org.tigris.atlas.service.StringServiceResponse;
import org.tigris.atlas.service.VoidServiceResponse;

import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Chunk;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;

/**
 * Service implementation class for the Patient service
 */
public class PatientServiceImpl extends MHVAbstractService implements
	PatientService
{

	private static final Log LOG = LogFactory.getLog(PatientServiceImpl.class);
	private static Font subFont = new Font(Font.FontFamily.TIMES_ROMAN, 9, Font.NORMAL);
	private static Font tableFont = new Font(Font.FontFamily.TIMES_ROMAN, 9, Font.BOLD);
	private static Font titleFont = new Font(Font.FontFamily.TIMES_ROMAN, 11, Font.BOLD);
	private static Font titleFont1 = new Font(Font.FontFamily.TIMES_ROMAN, 11, Font.NORMAL);

    private static String INVALIDATED_ICN_DELIMITER = "A";
	//JAZZ Story 112935 - ICN Search in Admin Portal HelpDesk
	private String WILDCARD = "%";


    private PatientCorrelationServiceAdapter adapter =
        new PatientCorrelationServiceAdapter();

    @Override
    protected Log getLog() {
        return LOG;
    }

    /*
     * (non-Javadoc)
     * @see gov.va.med.mhv.usermgmt.service.PatientService#getPatientForUser(
     * gov.va.med.mhv.usermgmt.transfer.UserProfile)
     */
    public PatientServiceResponse getPatientForUser(UserProfile userProfile) {
        PatientServiceResponse response = new PatientServiceResponse();

        if ((userProfile != null)) {
            response.setPatient(getPatientForUserProfileId(userProfile.
                getId()));
        }

        return response;
    }

    /*
     * (non-Javadoc)
     * @see gov.va.med.mhv.usermgmt.service.PatientService#getPatientByIcn(
     * java.lang.String)
     */
    public PatientServiceResponse getPatientByIcn(String icn) {
        Precondition.assertNotBlank("icn", icn);
        PatientServiceResponse response = new PatientServiceResponse();

        List patientBOs = PatientBO.getPatientByIcn(icn);
        Collection patients = PatientAssembler.getPatientCollection(patientBOs);
        if (patients.size() > 0) {
            response.setPatient((Patient) patients.iterator().next());
        }

        return response;
    }

	//JAZZ Story 112935 - ICN Search in Admin Portal HelpDesk
    public PatientCollectionServiceResponse getPatientByPartialIcn(String icn) {
        Precondition.assertNotBlank("icn", icn);
		icn = StringUtils.lowerCase(icn) + WILDCARD;
		PatientCollectionServiceResponse response = new PatientCollectionServiceResponse();

        List patientBOs = PatientBO.getPatientByPartialIcn(icn);
        Collection patients = PatientAssembler.getPatientCollection(patientBOs);
        response.addItems(patients);
        return response;
    }
    
    /*
     * (non-Javadoc)
     * @see gov.va.med.mhv.usermgmt.service.PatientService#getPatientById(
     * java.lang.String)
     */
    public PatientServiceResponse getPatientById(Long id) {
        PatientServiceResponse response = new PatientServiceResponse();

        PatientBO patientBO = PatientBO.findByPrimaryKey(new PatientPK(id));
        Patient patient = patientBO.getPatientValues();
        response.setPatient(patient);

        return response;
    }

    /*
     * (non-Javadoc)
     * @see gov.va.med.mhv.usermgmt.service.PatientService
     * #createPatientForUser(gov.va.med.mhv.usermgmt.transfer.Patient,
     * gov.va.med.mhv.usermgmt.transfer.UserProfile)
     */
    public PatientServiceResponse createPatientForUser(Patient patient,
        UserProfile userProfile)
    {
        PatientServiceResponse response = new PatientServiceResponse();

        patient.setUserProfile(userProfile);
        PatientServiceResponse psr = ServiceFactory.
            createEntityMaintenanceService().save(patient);

        response.setPatient(psr.getPatient());
        response.getMessages().addMessages(psr.getMessages());

        return response;
    }

	/**
	 * Execute the AddFacilityForPatient service
	 * @return AddFacilityForPatientServiceResponse
	 */
	public PatientServiceResponse addFacilityForPatient(Patient patient,
		Facility facility)
	{
		PatientServiceResponse response = new PatientServiceResponse();

		PatientPK patientKey = (PatientPK) patient.getKey();
		EntityMaintenanceService ems = ServiceFactory
			.createEntityMaintenanceService();
		Patient p = ems.findByPrimaryKey(patientKey).getPatient();
		p.addFacility(facility);
		PatientServiceResponse patientServiceResponse = ems.save(p);

		response.setPatient(patientServiceResponse.getPatient());
        copyMessages(response, patientServiceResponse);

		return response;
	}

	/**
	 * This operation gets a patient and facilities from the patient registry
	 * system (MPI).
	 * @param firstName
	 * @param lastName
	 * @param birthDate
	 * @param ssn
	 * @return GetPatientFromRegistryServiceResponse contains the patient
	 */
	public PatientServiceResponse getPatientFromRegistry(String firstName,
		String lastName, Date birthDate, String ssn, Long userId)
	{
		PatientServiceResponse response = new PatientServiceResponse();

		String numberOnlySsn = StringUtils.replace(ssn, "-", "");
		LookupPatientAdapter adapter = new LookupPatientAdapter(firstName,
			lastName, birthDate, numberOnlySsn, userId);
		try {
			Patient patient = adapter.lookupPatient();
			response.setPatient(patient);
			if (patient == null) {
				LOG.error("No patient found in MPI for first name: "
					+ firstName + " last name: " + lastName + " birth date: "
					+ formatDate(birthDate) + " ssn: " + ssn);
			} else if (LOG.isDebugEnabled()) {
				LOG.info("Patient successfully matched with MPI for first name: "
					+ firstName + " last name: " + lastName + " birth date: "
					+ formatDate(birthDate) + " last four digits of ssn: "
					+ StringUtils.substring(numberOnlySsn, 5));
			}
		} catch (DuplicatePatientException e) {
			LOG.error("More than one patient found in MPI for first name: "
				+ firstName + " last name: " + lastName + " birth date: "
				+ formatDate(birthDate) + " last four digits of ssn: "
				+ StringUtils.substring(numberOnlySsn, 5) + "; "
				+ e.getLocalizedMessage());
			addError(response, MessageKeys.PATIENT_DUPLICATE_ENTRIES);
		} catch (AdapterException e) {
			LOG.error("MPI connection or timeout exception: Host="
				+ e.getHost() + ":" + String.valueOf(e.getPort()));
			addError(response,
				UserManagementMessages.PATIENT_REGISTRY_ACCESS_ERROR);
		}

		return response;
	}

	/**
	 * This operation gets a patient and facilities from the patient registry
	 * system (MPI).
	 * @param firstName
	 * @param lastName
	 * @param middleName
	 * @param gender
	 * @param birthDate
	 * @param ssn
	 * @return GetPatientFromRegistryWithMiddleNameServiceResponse contains the patient
	 */
	public PatientServiceResponse getPatientFromRegistryWithMiddleName(String firstName,
		String lastName, String middleName, String gender, Date birthDate, String ssn, Long userId)
	{
		PatientServiceResponse response = new PatientServiceResponse();

		String numberOnlySsn = StringUtils.replace(ssn, "-", "");
		LookupPatientAdapterWithMiddleName adapter = new LookupPatientAdapterWithMiddleName(firstName,
			lastName, middleName, gender, birthDate, numberOnlySsn, userId);
		try {
			Patient patient = adapter.lookupPatient();
			response.setPatient(patient);
			if (patient == null) {
				LOG.error("No patient found in MPI for FIRST name: "
					+ firstName + " LAST name: " + lastName + " MIDDLE name: " + middleName +" birth date: "
					+ formatDate(birthDate) + " GENDER: " + gender  + " last four digits of ssn: "
					+ StringUtils.substring(numberOnlySsn, 5));
			} else if (LOG.isInfoEnabled()) {
				LOG.info("Patient successfully matched with MPI for first name: "
					+ firstName + " last name: " + lastName + " middle name: " + middleName + " birth date: "
					+ formatDate(birthDate) + " gender: " + gender + " last four digits of ssn: "
					+ StringUtils.substring(numberOnlySsn, 5));
			}
		} catch (DuplicatePatientException e) {
			LOG.error("More than one patient found in MPI for first name: "
				+ firstName + " last name: " + lastName + " middle name: " + middleName + " birth date: "
				+ formatDate(birthDate) + " gender: " + gender + " last four digits of ssn: "
				+ StringUtils.substring(numberOnlySsn, 5) + "; "
				+ e.getLocalizedMessage());
			addError(response, MessageKeys.PATIENT_DUPLICATE_ENTRIES);
		} catch (AdapterException e) {
			LOG.error("MPI connection or timeout exception: Host="
				+ e.getHost() + ":" + String.valueOf(e.getPort()));
			addError(response,
				UserManagementMessages.PATIENT_REGISTRY_ACCESS_ERROR);
		}

		return response;
	}

	public PatientServiceResponse getPatientFromMVI(UserProfile userProfile, Boolean atRegistration) {
		return ServiceDelegateFactory.createMviIntegrationServiceDelegate().searchPersonInMVI(userProfile, atRegistration);
	}
	
    /*
     *  (non-Javadoc)
     * @see gov.va.med.mhv.usermgmt.service.PatientService#
     * updatePatientFacilities(java.lang.String)
     */
	public PatientServiceResponse updatePatientFacilities(String userName) {
		PatientServiceResponse response = new PatientServiceResponse();

        try {
    		UserProfile userProfile = ServiceFactory.createUserProfileService().
                getProfileForUser(userName).getUserProfile();
    		if (userProfile == null) {
    			addError(response, UserManagementMessages.USER_DOES_NOT_EXIST,
                    new String[] { userName });
    			return response;
    		}
            if (!BooleanUtils.isTrue(userProfile.getIsPatient())) {
    			return response;
    		}
            response = getPatientForUser(userProfile);
            if (hasErrorMessages(response)) {
                return response;
            }
            Patient patient = response.getPatient();
            if (patient == null) {
                return response;
            }
            BooleanServiceResponse accessResponse = PHRAccessControl.
                hasPhrAccess(userName);
            if (hasErrorMessages(accessResponse)) {
                copyMessages(response, accessResponse);
                return response;
            }
            boolean isLinkEnabled = MpiProperties.getInstance().isLinkEnabled()
                && BooleanUtils.isTrue(accessResponse.getBoolean());
            // If not linking to MPI, then we must keep refreshing
            // the registry information upon login

            if (isLinkEnabled && !PatientCorrelationStatusUtils.isUncorrelated(
                patient))
            {
                if (LOG.isInfoEnabled()) {
                    LOG.info("Not updating facilities of patient '" + userName
                        + "', because patient is not uncorrelated (status=" +
                        patient.getCorrelationStatus().getName() + ")");
                }
            } else if (needsInvalidation(patient)) {
                if (LOG.isInfoEnabled()) {
                    LOG.info("Invalidating patient '" + userName
                        + "', because patient has invalid ICN (icn=" +
                        patient.getIcn()
                        + ") and has not been invalidated yet");
                }
            	UserProfileDeactivationReason reason = 
            		UserProfileDeactivationUtils.findReason(UserProfileDeactivationUtils.MULTIPLE_ACCOUNTS_RESOLUTION_NAME);
                invalidatePatient(patient, null, reason);
            } else if (isLinkEnabled && isPendingSynchronization(patient)) {
                if (LOG.isInfoEnabled()) {
                    LOG.info("Correlate patient '" + userName +
                        ", because patient is pending synchronization");
                }
                // NOTE correlate also calls updatePatientRegistryInformation()
                correlate(patient);
            } else {
                if (LOG.isInfoEnabled()) {
                    LOG.info("Update patient facilities for '" + userName +
                        "' in updatePatientRegistryInformation()");
                }
                //Correlated patients do have PatientRegistry Information invoked during login
                updatePatientRegistryInformation(userProfile);
            }
            return response;
        } finally {
            // Logging messages here, because this operation is intended to
            // be called asynchronously, such that messages could get lost
            if (response.getMessages().hasErrorMessages()) {
                logMessages(response);
            }
        }
	}

    /* (non-Javadoc)
     * @see gov.va.med.mhv.usermgmt.service.PatientService#updatePatient(
     * java.lang.String)
     */
    public PatientServiceResponse updatePatientRegistryInformation(
        Patient patient)
    {
        if (LOG.isDebugEnabled()) {
            LOG.debug("updatePatientRegistryInformation(" + patient.
                getUserProfile().getUserName() + ")");
        }

        PatientServiceResponse response = new PatientServiceResponse();
        Patient patientToUpdate = getPatientForUserProfileId(patient.
            getUserProfile().getId());
        if (patientToUpdate == null) {
            addError(response, MessageKeys.PATIENT_NOT_FOUND);
        } else {
            // The patientToUpdate is the original patient that is updated
            // with the new information to avoid loss of data and
            // avoid updates to non-registry information.
            RegistryChangeProcessor processor =
                new RegistryChangeProcessor(patientToUpdate, patient);
            if (processor.hasAddedRegistryChange()) {
                if (!hasErrorMessages(response)) {
                    response = savePatient(patientToUpdate, processor.
                        hasDemographicInfoChange());
                }
            } else {
                response.setPatient(patientToUpdate);
            }
        }

        return response;
    }

	//MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - Adding new method calls for MVI 
    public BooleanServiceResponse mviAuthenticate(Patient patient) {
        return	ServiceFactory.createMviIntegrationService().mviAuthenticate(patient, patient.getUserProfile(), true, false);
    }
    
    /*
     *  (non-Javadoc)
     * @see gov.va.med.mhv.usermgmt.service.PatientService#correlate(
     * gov.va.med.mhv.usermgmt.transfer.Patient)
     */
    public VoidServiceResponse correlate(Patient patient) {
        if (!existsPatient(patient)) {
            return createPatientNotFoundResponse();
        }
        BooleanServiceResponse accessResponse = PHRAccessControl.hasPhrAccess(
            patient);
        if (hasErrorMessages(accessResponse)) {
            return createVoidServiceResponseFrom(accessResponse);
        }
        boolean hasPhrAccess = accessResponse.getBoolean();
        boolean isLinkEnabled = MpiProperties.getInstance().isLinkEnabled();

        if (!(isLinkEnabled  && hasPhrAccess)) {
            if (LOG.isDebugEnabled()) {
                String reason = (!isLinkEnabled)
                    ? "correlation is disabled (mpi.properties)"
                    : "patient access to PHR is disabled (access controls)";
                LOG.debug("Not correlating patient " +
                    RegistryDescriptionBuilder.describe(patient) +
                    ", because " + reason +
                    ". Mark for synchronization instead.");
            }
            //patient have phr access means they are authenticated so I guess they are 
            //			getting added in patient_synchronization table. 
            patient.addPatientSynchronization(TransferObjectFactory.
                createPatientSynchronization());
            VoidServiceResponse response = new VoidServiceResponse();
            copyMessages(response, savePatient(patient, false));
            return response;
        }

        if (!PatientCorrelationStatusUtils.isUncorrelated(patient)
            // Allow retries
            && !PatientCorrelationStatusUtils.isFailedCorrelation(patient))
        {
            return createPatientNotUncorrelatedResponse();
        }
        // Synchronize information with MPI (via the query adapter),
        // such that the patient registration is up-to-date
        // and we do not get matching failures upon correlation
        // TODO: Refactor so this can be called directly and the patient
        // gets saved only in this method
        PatientServiceResponse queryResponse = updatePatientRegistryInformation(
            patient.getUserProfile());
        if (hasErrorMessages(queryResponse)) {
            return createVoidServiceResponseFrom(queryResponse);
        }
        if (queryResponse.getPatient() == null) {
            return createPatientRegistryAccessResponse();
        }
        patient = queryResponse.getPatient();

        patient.setCorrelationStatus(PatientCorrelationStatusUtils.
            PENDING_CORRELATION);
        PatientServiceResponse response = savePatient(patient, true);

        if (hasErrorMessages(response)) {
            return createVoidServiceResponseFrom(response);
        }

        if (LOG.isDebugEnabled()) {
            LOG.debug("Request correlation with MPI for patient "
                + RegistryDescriptionBuilder.describe(patient));
        }
        return adapter.correlate(patient);
    }

    /*
     * (non-Javadoc)
     * @see gov.va.med.mhv.usermgmt.service.PatientService#finalizeCorrelation(
     * gov.va.med.mhv.usermgmt.transfer.Patient, java.lang.String)
     */
    public VoidServiceResponse finalizeCorrelation(Patient patient,
        String error)
    {
        if (!existsPatient(patient)) {
            return createPatientNotFoundResponse();
        }
        if (!PatientCorrelationStatusUtils.isPendingCorrelation(patient)) {
            return createPatientNotPendingCorrelationResponse();
        }
        VoidServiceResponse response = new VoidServiceResponse();
        boolean hasError = !StringUtils.isBlank(error);
        if (hasError) {
            patient.setCorrelationStatus(PatientCorrelationStatusUtils.
                FAILED_CORRELATION);
            patient.setCorrelationErrorCode(error);
            // Do not add an 'error' message, as it will rollback the
            // transaction and the error must be captured in the database
            addInfo(response, MessageKeys.PATIENT_MPI_CORRELATION_FAILED,
                new Object[] { patient.getIcn(), error });
        } else {
            patient.setCorrelationErrorCode(null);
            patient.setCorrelationStatus(PatientCorrelationStatusUtils.
                CORRELATED);
            // Remove any patient synchronizations, avoiding further
            // synchronization attempts upon login
            // NOTE This code could be removed once all prior authenticated
            // patients are synchronized (i.e. correlated)
            patient.setPatientSynchronizations(null);
        }
        patient.getUserProfile().setIsMPIControlled(true);
        PatientServiceResponse patientServiceResponse = savePatient(patient,
            true);
        if (hasErrorMessages(patientServiceResponse)) {
            addError(response, MessageKeys.PATIENT_CORRELATION_UPDATE_FAILED,
                new Object[] { patient.getIcn()});
        } else if (!hasError) {
            addInfo(response, MessageKeys.PATIENT_MPI_CORRELATION_SUCCEEDED,
                new Object[] { patient.getIcn(), "finalize correlation" });
        }
        copyMessages(response, patientServiceResponse);
        return response;
    }

    /*
     * (non-Javadoc)
     * @see gov.va.med.mhv.usermgmt.service.PatientService#uncorrelate(
     * gov.va.med.mhv.usermgmt.transfer.Patient)
     */
    public VoidServiceResponse uncorrelate(Patient patient) {
        if (!existsPatient(patient)) {
            return createPatientNotFoundResponse();
        }
        boolean isEnabled = MpiProperties.getInstance().isUnlinkEnabled();
        boolean canUncorrelate =
            PatientCorrelationStatusUtils.isCorrelated(patient)
            // Allow retries
            || PatientCorrelationStatusUtils.isFailedUncorrelation(patient);
        boolean mustUnlink = isEnabled && canUncorrelate;

        patient.setCorrelationStatus((mustUnlink)
            ? PatientCorrelationStatusUtils.PENDING_UNCORRELATION
            : PatientCorrelationStatusUtils.UNCORRELATED);
        patient.setPatientSynchronizations(null);
        PatientServiceResponse response = savePatient(patient, true);
        if (hasErrorMessages(response) || !mustUnlink) {
            return createVoidServiceResponseFrom(response);
        }
        if (LOG.isInfoEnabled()) {
            LOG.info("Request uncorrelation from MPI for patient " +
                RegistryDescriptionBuilder.describe(patient));
        }
        return adapter.uncorrelate(patient);
    }

	//MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - Adding new method calls for MVI 
    public BooleanServiceResponse mviUnauthenticate(Patient patient) {
        return	ServiceFactory.createMviIntegrationService().mviAuthenticate(patient, patient.getUserProfile(), false, false);
    }
    
    /*
     * (non-Javadoc)
     * @see gov.va.med.mhv.usermgmt.service.PatientService#
     * finalizeUncorrelation(gov.va.med.mhv.usermgmt.transfer.Patient,
     * java.lang.String)
     */
    public VoidServiceResponse finalizeUncorrelation(Patient patient,
        String error)
    {
        if (!existsPatient(patient)) {
            return createPatientNotFoundResponse();
        }
        if (!PatientCorrelationStatusUtils.isPendingUncorrelation(patient)) {
            return createPatientNotPendingUncorrelationResponse();
        }
        VoidServiceResponse response = new VoidServiceResponse();
        boolean hasError = !StringUtils.isBlank(error);
        if (hasError) {
            patient.setCorrelationStatus(PatientCorrelationStatusUtils.
                FAILED_UNCORRELATION);
            patient.setCorrelationErrorCode(error);
            // Do not use an error message, as it will rollback the
            // transaction and the error must be captured in the database
            addInfo(response, MessageKeys.PATIENT_MPI_UNCORRELATION_FAILED,
                new Object[] { patient.getIcn(), error });
        } else {
            patient.setCorrelationStatus(PatientCorrelationStatusUtils.
                UNCORRELATED);
        }
        // NOTE: We cannot set isMPICorrelated back to false, as opposed to
        // setting it to true in the finalizeCorrelation, because it turns the
        // validation back and the MPI controlled values may no longer be valid
        // (the reason to use the isMPIControlled flag)
        // The flag needs to be set to false, once the user modifies the
        // profile and saves it
        PatientServiceResponse patientServiceResponse = savePatient(patient,
            false);
        copyMessages(response, patientServiceResponse);
        if (hasErrorMessages(patientServiceResponse)) {
            addError(response, MessageKeys.PATIENT_CORRELATION_UPDATE_FAILED,
                new Object[] { patient.getIcn(), "finalize uncorrelation"});
        } else {
            addInfo(response, MessageKeys.PATIENT_MPI_UNCORRELATION_SUCCEEDED,
                new Object[] { patient.getIcn() });
        }
        return response;
    }

    /*
     *  (non-Javadoc)
     * @see gov.va.med.mhv.usermgmt.service.PatientService#removeCorrelation(
     * gov.va.med.mhv.usermgmt.transfer.Patient)
     */
    public VoidServiceResponse removeCorrelation(Patient patient) {
        if (!existsPatient(patient)) {
            return createPatientNotFoundResponse();
        }
        VoidServiceResponse response = new VoidServiceResponse();

        patient.setCorrelationStatus(PatientCorrelationStatusUtils.
            UNCORRELATED);
        PatientServiceResponse patientServiceResponse = savePatient(
            patient, false);
        copyMessages(response, patientServiceResponse);
        if (hasErrorMessages(patientServiceResponse)) {
            addError(response, MessageKeys.PATIENT_CORRELATION_UPDATE_FAILED,
                new Object[] { patient.getIcn(), "remove correlation"});
        } else {
            addInfo(response, MessageKeys.PATIENT_MPI_REMOVAL_SUCCEEDED,
                new Object[] { patient.getIcn() });
        }
        return response;
    }
    
    public PatientServiceResponse invalidatePatient(Patient patient,
        String referenceId, UserProfileDeactivationReason reason)
    {
        PatientServiceResponse response = new PatientServiceResponse();
        Precondition.assertNotNull("patient", patient);
        Precondition.assertNotBlank("patient.icn", patient.getIcn());

		PatientServiceResponse mviSyncServiceResponse = 
			ServiceFactory.createMviIntegrationService().deletePersonFromCorrelationForHelpDesk(
					patient, patient.getUserProfile(), "");
        if (hasErrorMessages(mviSyncServiceResponse)) {
            logMessages(mviSyncServiceResponse);
            LOG.error("Unable to deactivate account of DFN ID: " +
                    patient.getUserProfile().getId() + 
                    " because MVI uncorrelate was not successful ");
            return response;
        }
        
        String invalidatedICN = null;
        if (!hasInvalidatedIcn(patient)) {
            // Only invalidate, if ICN Not yet invalidated
            invalidatedICN = patient.getIcn();
            patient.setIcn(createInvalidIcn(patient.getIcn()));
            response = updatePatientRegistryInformation(patient);
            patient = response.getPatient();
        }
        if (hasErrorMessages(response)) {
            logMessages(response);
            return response;
        }
        
        // Update all non-registry patient information
        patient.setFacilitys(null);
        patient.setCorrelationStatus(PatientCorrelationStatusUtils.CORRELATION_INVALID);
        if (!StringUtils.isBlank(invalidatedICN)) {
            patient.setInvalidatedIcn(invalidatedICN);
        }
        patient.setInvalidationReferenceId(StringUtils.isBlank(referenceId)
            ? createReferenceId(patient) : referenceId);
        patient.setPatientSynchronizations(null);
        response = savePatient(patient, false);
        patient = response.getPatient();

        if (hasErrorMessages(response)) {
            logMessages(response);
        } else if (reason == null) {
            // We do not want to fail this action
            LOG.error("Unable to deactivate account of patient " +
                patient.getUserProfile().getUserName() + ", because the " +
                " deactivation reason instance is unknown (referenceId: " +
                referenceId + ")");
             // TODO Add info message
        } else {
            if (LOG.isInfoEnabled()) {
                LOG.info("Invalidated patient '" + patient.getUserProfile().
                    getUserName() + "'. Changed ICN from " + patient.
                    getInvalidatedIcn() + " to " + patient.getIcn());
            }
            // We do not want account deactivation failure to prevent
            // this transaction from succeeding.
            // The PHR access has already been disabled, by changing the ICN
            logMessages(ServiceFactory.createUserProfileService().
                changeActivationState(patient.getUserProfile(), reason,
                UserProfileDeactivationUtils.STATE_CHANGED_BY_SYSTEM, reason.getName(), null));
        }
        return response;
    }
    
    //Admin Portal_CodeCR1959 - React/Deact Impl
	public PatientServiceResponse reactivatePatientInHelpDesk(Patient patient,
        String adminNetworkId, UserProfileDeactivationReason reason, String note) {
		
		PatientServiceResponse response = new PatientServiceResponse();
		if(patient.getInvalidatedIcn() == null) {
            addError(response, "mvi.helpdesk.revalidate.error.patient.active", new Object[] {});
		}
		
        if (reason == null) {
	   		addError(response, "mvi.helpdesk.revalidate.error.reason.missing", new String[]{"reactivateReason"});
	        if (LOG.isDebugEnabled()) {
	           LOG.debug("No deactivation reason for user:" + patient.getUserProfile().getUserName());
	
	        }
	    }
	       
	    if ((note == null)||(note.length()==0)) {
	    	addError(response, "mvi.helpdesk.revalidate.error.description.missing", new String[]{"reactivateNote"});
	    }
        
        if (hasErrorMessages(response)) {
            logMessages(response);
            return response;
        }
        
		patient.setIcn(patient.getInvalidatedIcn().trim());
		patient.setInvalidatedIcn(null);
		patient.setInvalidationReferenceId(createReferenceId(patient));
        response = savePatient(patient, false);
	
        VoidServiceResponse vResponse = ServiceFactory.createUserProfileService().
        reactivateUserInHelpDesk(patient.getUserProfile(), reason, adminNetworkId, note);
        if (!hasErrorMessages(vResponse)) {
            logMessages(vResponse);
        } else {
	    	addError(response, "mvi.helpdesk.revalidate.fail", new String[]{getFullName(patient.getUserProfile())});
        }
    	return response;
    }
    
	private String getFullName(UserProfile userProfile) {
		String firstName = userProfile.getFirstName();
		String middleName = userProfile.getMiddleName();
		String lastName = userProfile.getLastName();
		
		StringBuffer buff = new StringBuffer();
		buff.append(firstName);
		if (middleName != null) {
			buff.append(' ');
			buff.append(middleName);
		}
		buff.append(' ');
		buff.append(lastName);
		
		return(buff.toString());
	}

    private boolean isFourOrMoreTraitsUpdate(gov.va.med.mhv.usermgmt.transfer.Patient patient, PersonalInfo personalInfo) {
    	int traitUpdateCount = 0;
    	
        UserProfile userProfile = patient.getUserProfile();
        
        if (LOG.isDebugEnabled()) {
            LOG.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() for user profile id:" + userProfile.getId());
        }
        
        if(!userProfile.getFirstName().equalsIgnoreCase(personalInfo.getName().getFirstName())) {
            if (LOG.isDebugEnabled()) {
                LOG.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() first name is different for user profile id:" + userProfile.getId());
            }
        	traitUpdateCount++;
        }
        
        if(userProfile.getMiddleName()==null && personalInfo.getName().getMiddleName()!=null) {
            if (LOG.isDebugEnabled()) {
                LOG.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() middle name is different for user profile id:" + userProfile.getId());
            }
        	traitUpdateCount++;
        } else if(userProfile.getMiddleName()!=null && personalInfo.getName().getMiddleName()==null) {
            if (LOG.isDebugEnabled()) {
                LOG.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() middle name is different for user profile id:" + userProfile.getId());
            }
        	traitUpdateCount++;
        } else if(userProfile.getMiddleName()==null && personalInfo.getName().getMiddleName()==null) {
            if (LOG.isDebugEnabled()) {
                LOG.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() no middle name for user profile id:" + userProfile.getId());
            }
        } else if(!userProfile.getMiddleName().equalsIgnoreCase(personalInfo.getName().getMiddleName())) {
            if (LOG.isDebugEnabled()) {
                LOG.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() middle name is different for user profile id:" + userProfile.getId());
            }
        	traitUpdateCount++;
        }
        
        if(!userProfile.getLastName().equalsIgnoreCase(personalInfo.getName().getLastName())) {
            if (LOG.isDebugEnabled()) {
                LOG.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() last name is different for user profile id:" + userProfile.getId());
            }
        	traitUpdateCount++;
        }
        
        if(!userProfile.getBirthDate().equals(personalInfo.getDateOfBirth())) {
            if (LOG.isDebugEnabled()) {
                LOG.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() birth date is different for user profile id:" + userProfile.getId());
            }
        	traitUpdateCount++;
        }
        
        if(!userProfile.getGender().toString().equalsIgnoreCase(personalInfo.getGender().toString())) {
            if (LOG.isDebugEnabled()) {
                LOG.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() gender is different for user profile id:" + userProfile.getId());
            }
        	traitUpdateCount++;
        }

        if(!userProfile.getSsn().equals(personalInfo.getSsn())) {
            if (LOG.isDebugEnabled()) {
                LOG.debug("PatientIdentityService.isFourOrMoreTraitsUpdate() ssn is different for user profile id:" + userProfile.getId());
            }
        	traitUpdateCount++;
        }
        
        if(traitUpdateCount>3) {
            if (LOG.isDebugEnabled()) {
                LOG.debug("@@@@@@@@@@@@@@@@@@@@@@@@@@@@ traitUpdateCount>3");
            }
        	return true;
        }
        
        return false;
    }
	
    //JAZZ Defect# 20309: If there four or more traits update, DO NOT Update the user and deactivate the user.
	public PatientServiceResponse deactivatePatientForMultiTraitUpdateError(Patient patient,
            String referenceId, String adminNetowrkId, UserProfileDeactivationReason reason, String note) {
        Precondition.assertNotNull("patient", patient);
        Precondition.assertNotBlank("patient.icn", patient.getIcn());

        PatientServiceResponse response = new PatientServiceResponse();
        
        if (reason == null) {
        	reason = UserProfileDeactivationUtils.findReason(UserProfileDeactivationUtils.A31_POSSIBLE_ICN_MISMATCH_DO_NOT_REACTIVATE_NAME);
//	   		addError(response, "mvi.multi.trait.update.error.invalidate.error.reason.missing", new String [] {"deactivateReason"});
//	        if (LOG.isDebugEnabled()) {
//	           LOG.debug("No deactivation reason for user:" + patient.getUserProfile().getUserName());
//	
//	        }
	    }

System.out.println("***************************************************** reason.getName():" + reason.getName());        
System.out.println("***************************************************** note:" + note);        
        
	    if ((note == null)||(note.length()==0)) {
	    	addError(response, "mvi.multi.trait.update.error.invalidate.error.reason.missing", new String [] {"deactivateNote"});
	    }
        
        if (hasErrorMessages(response)) {
            logMessages(response);
            return response;
        }
        
        String invalidatedICN = null;

        if (!hasInvalidatedIcn(patient)) {
            // Only invalidate, if ICN Not yet invalidated
            invalidatedICN = patient.getIcn();
            patient.setIcn(createInvalidIcn(patient.getIcn()));
            response = updatePatientRegistryInformation(patient);
            patient = response.getPatient();
        }
        
        //JAZZ: 19292 - Take out matched date, set mpi controlled flag to null during deactivation.
        patient.setMatchedDateTime(null);
        
        if (hasErrorMessages(response)) {
            logMessages(response);
            return response;
        }

        // Update all non-registry patient information
        patient.setFacilitys(null);
        if (!StringUtils.isBlank(invalidatedICN)) {
            patient.setInvalidatedIcn(invalidatedICN);
        }
        patient.setInvalidationReferenceId(StringUtils.isBlank(referenceId)
            ? createReferenceId(patient) : referenceId);
        patient.setPatientSynchronizations(null);
        response = savePatient(patient, false);
        patient = response.getPatient();

        if (hasErrorMessages(response)) {
            logMessages(response);
        } else if (reason == null) {
            // We do not want to fail this action
            LOG.error("Unable to deactivate account of patient " +
                patient.getUserProfile().getUserName() + ", because the " +
                " deactivation reason instance is unknown (referenceId: " +
                referenceId + ")");
             // TODO Add info message
        } else {
            if (LOG.isInfoEnabled()) {
                LOG.info("Invalidated patient '" + patient.getUserProfile().
                    getUserName() + "'. Changed ICN from " + patient.
                    getInvalidatedIcn() + " to " + patient.getIcn());
            }
            // We do not want account deactivation failure to prevent
            // this transaction from succeeding.
            // The PHR access has already been disabled, by changing the ICN
            VoidServiceResponse vResponse = ServiceFactory.createUserProfileService().
                changeActivationState(patient.getUserProfile(), reason, adminNetowrkId, note, null);
            if (hasErrorMessages(vResponse)) {
System.out.println("***************************************************** deactivatePatientForMultiTraitUpdateError Deactivation failed");         
Iterator iterator = vResponse.getMessages().getErrorMessages().iterator();
while(iterator.hasNext()) {
	StringBuffer str = new StringBuffer();
	Message message = (Message)iterator.next();
	str.append(message.getKey());
	if (message.getInserts().size() > 0) {
		str.append("; inserts=[").append(message.getInserts()).
			append("]");
	}
	if (message.getProperties().size() > 0) {
		str.append("; properties=[").append(message.getProperties()).
			append("]");
	}
	if(LOG.isErrorEnabled()) {
		LOG.error(str);
	}
}

    	    	addError(response, "mvi.multi.trait.update.error.invalidate.fail", new String[]{getFullName(patient.getUserProfile())});
            } else {
System.out.println("***************************************************** deactivatePatientForMultiTraitUpdateError Deactivation success");

    	    	//logMessages(vResponse);
				response.setPatient(getPatientForUserProfileId(patient.getUserProfile().getId()));
            }
        }
        
printOutErrors(response);        
        return response;
	}
	
	private void printOutErrors(ServiceResponse response) {
		Iterator iterator = response.getMessages().getErrorMessages().iterator();
		while(iterator.hasNext()) {
			StringBuffer str = new StringBuffer("**************************************** PatientServiceImpl");
			Message message = (Message)iterator.next();
			str.append(message.getKey());
			if (message.getInserts().size() > 0) {
				str.append("; inserts=[").append(message.getInserts()).
					append("]");
			}
			if (message.getProperties().size() > 0) {
				str.append("; properties=[").append(message.getProperties()).
					append("]");
			}
			if(LOG.isErrorEnabled()) {
				LOG.error(str);
			}
		}
	}
	
    private InPersonAuthenticationService getIPAService() {
        return ServiceFactory.createInPersonAuthenticationService();
    }   
    
    private void handleMessages(String description, EntityServiceResponse response) {
//        if (response.getMessages().hasInformationalMessages()) {
        	StringBuffer buffer = new StringBuffer();
        	buffer.append(new MessagesStringBuilder().append(response.getMessages(), getClass()).getInfoString());
        	throw new RuntimeException(description + buffer.toString());
        	//This is an old code that didn't have any description. Left it commented for research purpse            	
        	//throw new RuntimeException(MessageKeys.UNKNOWN_EXCEPTION_OCCURRED);
//        }
    }

    public PatientServiceResponse deactivatePatientInHelpDesk(Patient patient,
            String referenceId, String adminNetowrkId, UserProfileDeactivationReason reason, String note, Date deathDate) {
        Precondition.assertNotNull("patient", patient);
        Precondition.assertNotBlank("patient.icn", patient.getIcn());
        
        PatientServiceResponse response = new PatientServiceResponse();
        
        if (reason == null) {
	   		addError(response, "mvi.helpdesk.invalidate.error.reason.missing", new String [] {"deactivateReason"});
	        if (LOG.isDebugEnabled()) {
	           LOG.debug("No deactivation reason for user:" + patient.getUserProfile().getUserName());
	
	        }
	    }
	       
	    if ((note == null)||(note.length()==0)) {
	    	addError(response, "mvi.helpdesk.invalidate.error.description.missing", new String [] {"deactivateNote"});
	    }
        
        if (hasErrorMessages(response)) {
            logMessages(response);
            return response;
        }
        
        String invalidatedICN = null;

        if (!hasInvalidatedIcn(patient)) {
            // Only invalidate, if ICN Not yet invalidated
            invalidatedICN = patient.getIcn();
            patient.setIcn(createInvalidIcn(patient.getIcn()));
            response = updatePatientRegistryInformation(patient);
            patient = response.getPatient();
        }
        
        //JAZZ: 19292 - Take out matched date, set mpi controlled flag to null during deactivation.
        patient.setMatchedDateTime(null);
        
        if (hasErrorMessages(response)) {
            logMessages(response);
            return response;
        }

        // Update all non-registry patient information
        patient.setFacilitys(null);
        if (!StringUtils.isBlank(invalidatedICN)) {
            patient.setInvalidatedIcn(invalidatedICN);
        }
        patient.setInvalidationReferenceId(StringUtils.isBlank(referenceId)
            ? createReferenceId(patient) : referenceId);
        patient.setPatientSynchronizations(null);
        response = savePatient(patient, false);
        patient = response.getPatient();

        if (hasErrorMessages(response)) {
            logMessages(response);
        } else if (reason == null) {
            // We do not want to fail this action
            LOG.error("Unable to deactivate account of patient " +
                patient.getUserProfile().getUserName() + ", because the " +
                " deactivation reason instance is unknown (referenceId: " +
                referenceId + ")");
             // TODO Add info message
        } else {
            if (LOG.isInfoEnabled()) {
                LOG.info("Invalidated patient '" + patient.getUserProfile().
                    getUserName() + "'. Changed ICN from " + patient.
                    getInvalidatedIcn() + " to " + patient.getIcn());
            }
            // We do not want account deactivation failure to prevent
            // this transaction from succeeding.
            // The PHR access has already been disabled, by changing the ICN
            VoidServiceResponse vResponse = ServiceFactory.createUserProfileService().
                changeActivationState(patient.getUserProfile(), reason, adminNetowrkId, note, deathDate);
            if (hasErrorMessages(vResponse)) {
    	    	addError(response, "mvi.helpdesk.invalidate.fail", new String[]{getFullName(patient.getUserProfile())});
            } else {
    	    	logMessages(vResponse);
            }
        }
        return response;
    }

    private String createReferenceId(Patient patient) {
        final SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd");
        return format.format(new Date()) + StringUtils.substring(patient.
            getUserProfile().getSsn(),5);
    }

    private boolean needsInvalidation(Patient patient) {
        return (hasInvalidatedIcn(patient)
            && !PatientCorrelationStatusUtils.isCorrelationInvalid(
                    patient));
    }

    private boolean hasInvalidatedIcn(Patient patient) {
        assert patient != null;
        assert !StringUtils.isBlank(patient.getIcn());
        return !StringUtils.isBlank(patient.getInvalidatedIcn())
            || (patient.getIcn().indexOf(INVALIDATED_ICN_DELIMITER) > 0);

    }

    private String createInvalidIcn(String icn) {
        assert !StringUtils.isBlank(icn);
        
        final String icnPrefix = StringUtils.substring(icn, 0, 10)
            + INVALIDATED_ICN_DELIMITER;
        for (int i = 1; i < 999999; i++) {
            String newIcn = icnPrefix + String.format("%1$06d", i);
            if (LOG.isDebugEnabled()) {
                LOG.debug("Assign new icn '" + newIcn + "'");
            }
            List patients = PatientBO.getPatientByIcn(newIcn);
            if ((patients == null) || patients.isEmpty()) {
                return newIcn;
            }
        }
        Precondition.fail("Cannot invalidate icn '" + icn +
            "', because too many invalidated already");
        return null;
    }

    private PatientServiceResponse updatePatientRegistryInformation(
        UserProfile userProfile)
    {
        PatientServiceResponse response = new PatientServiceResponse();
        PatientServiceResponse patientRegistryResponse = null;

        //Patient does not exist yet or is uncorrelated

		/*
		 * Checking the property file to see if we need to call getPatientFromRegistryWithMiddleName or getPatientFromRegistry
		 */
        boolean isUseMiddleNameAndGender = MpiProperties.getInstance().isUseMiddleNameAndGender();
        if(isUseMiddleNameAndGender){
	        if (LOG.isDebugEnabled()) {
	            LOG.debug("Middle name: " + userProfile.getMiddleName()
	            		+ "gender: " + userProfile.getGender().getName()
	                + " for user id: " + userProfile.getId()
	                + " and useMiddleNameAndGender property value: " + isUseMiddleNameAndGender + " from mpi.properties in updatePatientRegistryInformation method");
	        }
			String gender  = userProfile.getGender().getName();
			if(gender.equalsIgnoreCase("Male")){
				gender = "M";
			}else{
				gender = "F";
			}

	        patientRegistryResponse =
	        	getPatientFromRegistryWithMiddleName(userProfile.getFirstName(),
	            userProfile.getLastName(), userProfile.getMiddleName(), gender, userProfile.getBirthDate(),
	            userProfile.getSsn(), userProfile.getId());
		}else{
	        if (LOG.isDebugEnabled()) {
	            LOG.debug("isUseMiddleNameAndGender: " + isUseMiddleNameAndGender + " in PatientServiceImpl::updatePatientRegistryInformation for user id: " + userProfile.getId() + " not calling middle name service");
	        }
	        patientRegistryResponse =
	            getPatientFromRegistry(userProfile.getFirstName(),
	            userProfile.getLastName(), userProfile.getBirthDate(),
	            userProfile.getSsn(), userProfile.getId());
		}



        if (hasErrorMessages(patientRegistryResponse)) {
            copyMessages(response, patientRegistryResponse);
            return response;
        }
        if (patientRegistryResponse.getPatient() == null) {
            addError(response, MessageKeys.PATIENT_NOT_FOUND);
            return response;
        }

        Patient patient = patientRegistryResponse.getPatient();
        patient.setUserProfile(userProfile);
        response = updatePatientRegistryInformation(patient);
        return response;
    }

    /**
     * Determine if a given patient is pending synchronization.
     * @param patient The patient to check
     * @return True, only if the given patient is not null, has a
     * PatientSynchronization associated with it and is still uncorrelated.
     */
    private boolean isPendingSynchronization(
        gov.va.med.mhv.usermgmt.transfer.Patient patient)
    {
        return (patient != null)
            && (patient.getPatientSynchronizations() != null)
            && !patient.getPatientSynchronizations().isEmpty()
            // Correlation could already be in progress
            && PatientCorrelationStatusUtils.isUncorrelated(patient);
    }

    private Patient getPatientForUserProfileId(Long id) {
        Patient patient = null;
        if (id != null) {
            List patientBOs = PatientBO.getPatientForUserProfile(id);
            Collection patients = PatientAssembler.getPatientCollection(
                patientBOs);
            if (patients.size() > 0) {
                patient = (Patient) patients.iterator().next();
            }
        }
        return patient;
    }

    private PatientServiceResponse savePatient(Patient patient,
        boolean saveUserProfile)
    {
        PatientServiceResponse patientServiceResponse = ServiceFactory.
            createEntityMaintenanceService().save(patient);
        if (!hasErrorMessages(patientServiceResponse) && saveUserProfile) {
            UserProfileServiceResponse userProfileServiceResponse =
                ServiceFactory.createEntityMaintenanceService().save(
                    patient.getUserProfile());
            copyMessages(patientServiceResponse, userProfileServiceResponse);
        }
        if (hasErrorMessages(patientServiceResponse)) {
            logMessages(patientServiceResponse);
        } else if (LOG.isDebugEnabled()) {
            LOG.debug("Saved patient " +
                ((saveUserProfile) ? "and user profile " : "") +
                "changes: " + RegistryDescriptionBuilder.describe(
                   patientServiceResponse.getPatient()));
        }
        return patientServiceResponse;
    }

    private VoidServiceResponse createVoidServiceResponseFrom(
        ServiceResponse copyFrom)
    {
        VoidServiceResponse response = new VoidServiceResponse();
        copyMessages(response, copyFrom);
        return response;
    }
    private VoidServiceResponse createVoidServiceResponseFrom(
        EntityServiceResponse copyFrom)
    {
        VoidServiceResponse response = new VoidServiceResponse();
        copyMessages(response, copyFrom);
        return response;
    }

    private VoidServiceResponse createPatientNotFoundResponse() {
        VoidServiceResponse response = new VoidServiceResponse();
        addError(response, MessageKeys.PATIENT_NOT_FOUND);
        return response;
    }

    private VoidServiceResponse createPatientRegistryAccessResponse() {
        VoidServiceResponse response = new VoidServiceResponse();
        addError(response, UserManagementMessages.PATIENT_REGISTRY_ACCESS_ERROR);
        return response;
    }

    private VoidServiceResponse createPatientNotUncorrelatedResponse() {
        VoidServiceResponse response = new VoidServiceResponse();
        addError(response, MessageKeys.PATIENT_IS_NOT_UNCORRELATED);
        return response;
    }

    private VoidServiceResponse createPatientNotPendingCorrelationResponse() {
        VoidServiceResponse response = new VoidServiceResponse();
        addError(response, MessageKeys.PATIENT_IS_NOT_PENDING_CORRELATION);
        return response;
    }

    private VoidServiceResponse createPatientNotPendingUncorrelationResponse() {
        VoidServiceResponse response = new VoidServiceResponse();
        addError(response, MessageKeys.PATIENT_IS_NOT_PENDING_UNCORRELATION);
        return response;
    }

    private boolean existsPatient(Patient patient) {
        assertPatientIds(patient);
        return PatientBO.findByPrimaryKey(patient.getPatientPK()) != null;
    }

    private void assertPatientIds(Patient patient) {
        Precondition.assertNotNull("patient", patient);
        Precondition.assertNotNull("patient.id", patient.getId());
        Precondition.assertNotNull("patient.userProfile", patient.
            getUserProfile());
        Precondition.assertNotNull("patient.userProfile.id", patient.
            getUserProfile().getId());
    }

    private String formatDate(Date d) {
		if(d==null) return "null";
		String bdate = d.toString();
		try {
		  SimpleDateFormat df = new SimpleDateFormat("yyyyMMdd");
	          bdate = df.format(d);
		} catch(Exception e) {}
		return bdate;
    }


    private final class RegistryChangeProcessor {

        private final Patient oldPatient;
        private final Patient newPatient;
        private final UserProfile newUserProfile;
        private final UserProfile oldUserProfile;
        private PatientRegistryChange changeRecord = null;
        private boolean hasDemographicChange = false;

        public RegistryChangeProcessor(Patient oldPatient, Patient newPatient) {
            this.oldPatient = oldPatient;
            this.newPatient = newPatient;
            this.oldUserProfile = oldPatient.getUserProfile();
            this.newUserProfile = newPatient.getUserProfile();
            processIcnChange();
            processNameChange();
            processBirthDateChange();
            processSsnChange();
            processGenderChange();
            // Not processing address changes until all information is sent
            // by MPI; Once implemented, remove suppress unused
            //processAddressChange();
            processContactInfoChange();
            processFacilityChanges();
            if (LOG.isDebugEnabled()){
                LOG.debug("hasChanges(" + oldUserProfile.getUserName() + ") = "
                    + hasAddedRegistryChange());
            }
            if (hasAddedRegistryChange()) {
                oldPatient.addPatientRegistryChange(changeRecord);
                LOG.debug("addedChange to " + DescriptionBuilder.describe(
                    oldPatient));
            }

        }

        public boolean hasAddedRegistryChange() {
            return changeRecord != null;
        }

        private void setDemographicInfoChange() {
            hasDemographicChange = true;
        }
        public boolean hasDemographicInfoChange() {
            return (changeRecord != null) && hasDemographicChange;
        }

        private PatientRegistryChange getChangeRecord(String forProperty) {
            if (LOG.isDebugEnabled()) {
                LOG.debug("found " + forProperty + " change for "
                    + oldUserProfile.getUserName());
            }
            if (changeRecord == null) {
                changeRecord = TransferObjectFactory
                    .createPatientRegistryChange();
                changeRecord.setRecordedOnDate(TimestampUtils.
                    createCurrentTime());
            }
            return changeRecord;
        }

        private void processIcnChange() {
            if (!StringUtils.equals(newPatient.getIcn(), oldPatient.getIcn())) {
                getChangeRecord("ICN").setOldIcn(oldPatient.getIcn());
                oldPatient.setIcn(newPatient.getIcn());
            }
        }

        private void processNameChange() {
            // Remove suppress unused from method once activating
            // processing of middle name, prefix and suffix,
            processFirstNameChange();
            processMiddleNameChange();
            processLastNameChange();
            processNamePrefixChange();
            processNameSuffixChange();
        }
        private void processFirstNameChange() {
            if (!StringUtils.equals(newUserProfile.getFirstName(),
                oldUserProfile.getFirstName()))
            {
                getChangeRecord("FirstName").setOldFirstName(oldUserProfile.
                    getFirstName());
                oldUserProfile.setFirstName(newUserProfile.getFirstName());
                setDemographicInfoChange();
            }
        }

        private void processMiddleNameChange() {
            if (!StringUtils.equals(newUserProfile.getMiddleName(),
                oldUserProfile.getMiddleName()))
            {
                getChangeRecord("MiddleName").setOldMiddleName(oldUserProfile.
                    getMiddleName());
                oldUserProfile.setMiddleName(newUserProfile.getMiddleName());
                setDemographicInfoChange();
            }
        }

        private void processLastNameChange() {
            if (!StringUtils.equals(newUserProfile.getLastName(),
                oldUserProfile.getLastName()))
            {
                getChangeRecord("LastName").setOldLastName(oldUserProfile.
                    getLastName());
                oldUserProfile.setLastName(newUserProfile.getLastName());
                setDemographicInfoChange();
            }
        }

        private void processNamePrefixChange() {
            if (!equals(newUserProfile.getTitle(), oldUserProfile.getTitle())) {
                getChangeRecord("Title").setOldNamePrefix(oldUserProfile.
                    getTitle());
                oldUserProfile.setTitle(newUserProfile.getTitle());
                setDemographicInfoChange();
            }
        }
        private void processNameSuffixChange() {
            if (!equals(newUserProfile.getSuffix(), oldUserProfile.getSuffix())) {
                getChangeRecord("Suffix").setOldNameSuffix(oldUserProfile.
                    getSuffix());
                oldUserProfile.setSuffix(newUserProfile.getSuffix());
                setDemographicInfoChange();
            }
        }


        private void processBirthDateChange() {
            if (!newUserProfile.getBirthDate().equals(oldUserProfile.
                getBirthDate()))
            {
                getChangeRecord("DOB").setOldBirthDate(oldUserProfile.
                    getBirthDate());
                oldUserProfile.setBirthDate(newUserProfile.getBirthDate());
                setDemographicInfoChange();
            }
        }

        private void processSsnChange() {
            if (!StringUtils.equals(newUserProfile.getSsn(), oldUserProfile.
                getSsn()))
            {
                getChangeRecord("SSN").setOldSsn(oldUserProfile.getSsn());
                oldUserProfile.setSsn(newUserProfile.getSsn());
                setDemographicInfoChange();
            }
        }

        private void processGenderChange() {
            if (!equals(newUserProfile.getGender(), oldUserProfile.
                getGender()))
            {
                getChangeRecord("Gender").setOldGender(oldUserProfile.
                    getGender());
                oldUserProfile.setGender(newUserProfile.getGender());
                setDemographicInfoChange();
            }
        }

        @SuppressWarnings("unused")
		private void processAddressChange() {
            processAddressStreet1Change();
            processAddressStreet2Change();
            processAddressCityChange();
            processAddressPostalCodeChange();
            processAddressStateChange();
            processAddressProvinceChange();
            processAddressCountryChange();
        }
        private void processAddressStreet1Change() {
            if (!StringUtils.equals(newUserProfile.getAddressStreet1(),
                oldUserProfile.getAddressStreet1()))
            {
                getChangeRecord("AddressStreet1").setOldAddressStreet1(
                    oldUserProfile.getAddressStreet1());
                oldUserProfile.setAddressStreet1(newUserProfile.
                    getAddressStreet1());
                setDemographicInfoChange();
            }
        }
        private void processAddressStreet2Change() {
            if (!StringUtils.equals(newUserProfile.getAddressStreet2(),
                oldUserProfile.getAddressStreet2()))
            {
                getChangeRecord("AddressStreet2").setOldAddressStreet2(
                    oldUserProfile.getAddressStreet2());
                oldUserProfile.setAddressStreet2(newUserProfile.
                    getAddressStreet2());
                setDemographicInfoChange();
            }
        }
        private void processAddressCityChange() {
            if (!StringUtils.equals(newUserProfile.getAddressCity(),
                oldUserProfile.getAddressCity()))
            {
                getChangeRecord("StreetCity").setOldAddressCity(oldUserProfile.
                    getAddressCity());
                oldUserProfile.setAddressCity(newUserProfile.
                    getAddressCity());
                setDemographicInfoChange();
            }
        }
        private void processAddressPostalCodeChange() {
            if (!StringUtils.equals(newUserProfile.getAddressPostalCode(),
                oldUserProfile.getAddressPostalCode()))
            {
                getChangeRecord("StreetPostalCode").setOldAddressPostalCode(
                    oldUserProfile.getAddressPostalCode());
                oldUserProfile.setAddressPostalCode(newUserProfile.
                    getAddressPostalCode());
                setDemographicInfoChange();
            }
        }
        private void processAddressStateChange() {
            if (!equals(newUserProfile.getAddressState(), oldUserProfile.
                getAddressState()))
            {
                getChangeRecord("AddressState").setOldAddressState(
                    oldUserProfile.getAddressState());
                oldUserProfile.setAddressState(newUserProfile.
                    getAddressState());
                setDemographicInfoChange();
            }
        }
        private void processAddressProvinceChange() {
            if (!StringUtils.equals(newUserProfile.getAddressProvince(),
                oldUserProfile.getAddressProvince()))
            {
                getChangeRecord("AddressProvince").setOldAddressProvince(
                    oldUserProfile.getAddressProvince());
                oldUserProfile.setAddressProvince(newUserProfile.
                    getAddressProvince());
                setDemographicInfoChange();
            }
        }
        private void processAddressCountryChange() {
            if (!equals(newUserProfile.getAddressCountry(), oldUserProfile.
                getAddressCountry()))
            {
                getChangeRecord("StreetCountry").setOldAddressCountry(
                    oldUserProfile.getAddressCountry());
                oldUserProfile.setAddressCountry(newUserProfile.
                    getAddressCountry());
                setDemographicInfoChange();
            }
        }

        private void processContactInfoChange() {
            processHomePhoneChange();
        }

        private void processHomePhoneChange() {

              // Must ignore home phone number until the phone number can
        	  // handle international phone numbers
//            if (!StringUtils.equals(newUserProfile.getContactInfoHomePhone(),
//                oldUserProfile.getContactInfoHomePhone()))
//            {
//                if (LOG.isDebugEnabled()) {
//                    LOG.debug("Ignoring HomePhone change for patient '" +
//                        oldPatient.getUserProfile().getUserName() +
//                        "', because unable to handle international " +
//                        "phone numbers, so not processing home phone " +
//                        "number changes at all");
//                }
//                getChangeRecord("ContactInfoHomePhone").
//                    setOldContactInfoHomePhone(oldUserProfile.
//                    getContactInfoHomePhone());
//                oldUserProfile.setContactInfoHomePhone(newUserProfile.
//                    getContactInfoHomePhone());
//                setDemographicInfoChange();
//            }
        }


        private void processFacilityChanges() {
            Map<String, Facility> oldFacilities = collectStationNumbers(
                oldPatient);
            Map<String, Facility> newFacilities = collectStationNumbers(
                newPatient);
            if (!CollectionUtils.isEqualCollection(newFacilities.keySet(),
                    oldFacilities.keySet()))
            {
                getChangeRecord("Facilities").setOldFacilityCount(oldPatient.
                    getFacilitys().size());
                Set<Facility> facilities = new HashSet<Facility>();
                for (String stationNumber: newFacilities.keySet()) {
                    Facility facility = oldFacilities.get(stationNumber);
                    if (facility == null) {
                        if (LOG.isDebugEnabled()) {
                            LOG.debug("Adding facility '" + stationNumber
                                + "'");
                        }
                        facility = newFacilities.get(stationNumber);
                    } else {
                        if (LOG.isDebugEnabled()) {
                            LOG.debug("Keeping facility '" + stationNumber +
                                "' (" + facility.getId() + ")");
                        }
                    }
                    facilities.add(facility);
                }
                if (LOG.isDebugEnabled()) {
                    for (String stationNumber: oldFacilities.keySet()) {
                        Facility facility = newFacilities.get(stationNumber);
                        if (facility == null) {
                            LOG.debug("Removing facilities '" + stationNumber
                                + "'");
                        }
                    }
                }
                oldPatient.setFacilitys(facilities);
            }
        }

        private Map<String, Facility> collectStationNumbers(Patient patient) {
            assert patient != null : "Must provide a patient";
            Map<String, Facility> facilityNames =
                new HashMap<String, Facility>();
            for (Object f: patient.getFacilitys()) {
                Facility facility = (Facility) f;
                facilityNames.put(facility.getName(), facility);
            }
            return facilityNames;
        }

        private boolean equals(Enum newValue, Enum oldValue) {
            return ((newValue == null) && (oldValue == null))
                || ((newValue != null) && newValue.equals(oldValue));
        }
    }

    private InPersonAuthentication saveIPA(InPersonAuthentication ipa,
            AuthenticationStatus status){
        InPersonAuthenticationBO bo = BusinessObjectFactory
            .createInPersonAuthenticationBO();
        ipa = bo.saveAndUpdateStatus(ipa, status);
        return ipa;
    }
    
	/*
	 * JAZZ# 128619 - Admin Portal Enhancements - Correlate Matched account(s) Background Job
	 */
    public VoidServiceResponse correlateAsyncMatchedPatientsBatch() {
    	return correlateMatchedPatientsBatch();
    }
    
	/*
	 * JAZZ# 128619 - Admin Portal Enhancements - Correlate Matched account(s) Background Job
	 */
	public VoidServiceResponse correlateMatchedPatientsBatch() {
		VoidServiceResponse voidResponse = new VoidServiceResponse();
		int maxCount = 0;
		List<PatientBO> bos = PatientBO.getMatchedUncorrelatedPatients();
		if(bos != null) {
			maxCount = bos.size();
System.out.println("maxCount = " + maxCount);		
            if (LOG.isInfoEnabled()) {
                LOG.info("*********** PatientServiceImpl.correlateMatchedPatientsBatch()() number of matched uncorrelated patients:" + maxCount);
            }
		}
		
		if(bos != null && !bos.isEmpty()) {
        	
            if (LOG.isInfoEnabled()) {
                LOG.info("*********** PatientServiceImpl.correlateMatchedPatientsBatch() number of correlated patients:" + bos.size());
            }
        	
	        Timestamp startTime = new Timestamp(new Date().getTime());
	        int numberReviewed = 0;
	        int numberUpdated = 0;
	        int numberError = 0;
	        
        	for (int i = 0; i < maxCount; i++) {
        		PatientBO patientBO = bos.get(i);
        		Patient patient = patientBO.getPatientValues();
        		PatientServiceResponse response = 
        			ServiceFactory.createMviIntegrationService().addPersonForCorrelationForIPA(patient, patient.getUserProfile(), ActivityActorTypeEnumeration.SYSTEM);
        		numberReviewed ++;
        		
                if (LOG.isInfoEnabled()) {
                    LOG.info("*********** PatientServiceImpl.correlateMatchedPatientsBatch() processing patient icn:" + patient.getIcn());
                }
        		if(response.getMessages().hasErrorMessages()) {
	                if (LOG.isInfoEnabled()) {
	                    LOG.info("*********** PatientServiceImpl.correlateMatchedPatientsBatch() error processing patient icn:" + patient.getIcn());
	                }
                    System.out.println("*********** PatientServiceImpl.correlateMatchedPatientsBatch() error processing patient icn:" + patient.getIcn());
                    numberError++;
        		} else {
                    System.out.println("*********** PatientServiceImpl.correlateMatchedPatientsBatch() success processing patient icn:" + patient.getIcn());
	                if (LOG.isInfoEnabled()) {
	                    LOG.info("*********** PatientServiceImpl.correlateMatchedPatientsBatch() success processing patient icn:" + patient.getIcn());
	                }
	                numberUpdated++;
        		}
			}
        }
		return voidResponse;
		
	}

	/*
	 * JAZZ #128616 - Update Treatment facilities for existing users that are in the following categories
	 */
    public VoidServiceResponse updateAsyncTreatmentFacilitiesBatch() {
    	return updateTreatmentFacilitiesBatch();
    }

	/*
	 * JAZZ #128616 - Update Treatment facilities for existing users that are in the following categories
	 */
	public VoidServiceResponse updateTreatmentFacilitiesBatch(){
		VoidServiceResponse voidResponse = new VoidServiceResponse();
		MviProperties properties  = MviProperties.getInstance();
		int maxCount = properties.getMviFacilitiesUpdateMaxCount();
System.out.println("maxCount = " + maxCount);		
		List<PatientBO> bos = PatientBO.getCorrelatedPatients();
        if(bos != null && !bos.isEmpty()) {
        	
            if (LOG.isInfoEnabled()) {
                LOG.info("*********** PatientServiceImpl.updateTreatmentFacilitiesBatch() number of correlated patients:" + bos.size());
            }
        	
	        BatchReportBO bo = BusinessObjectFactory.createBatchReportBO();
	        BatchReport batchReport = new BatchReport();
	        batchReport.setBatchType("Facilities Update");
	        batchReport.setPerformerType(ActivityActorTypeEnumeration.getEnum(ActivityActorTypeEnumeration.SYSTEM));
	        batchReport.setStartTime(new Timestamp(new Date().getTime()));
	        batchReport.setCompletionTime(new Timestamp(new Date().getTime()));
	        batchReport.setTotalNumberError(0);
	        batchReport.setTotalNumberReviewed(0);
	        batchReport.setTotalNumberUpdated(0);
	        batchReport.setCompletionTime(new Timestamp(new Date().getTime()));
	        bo.setBatchReportValues(batchReport);
	        bo.save();
	        if( bo.getAllMessages( ).hasErrorMessages( ) ) {	     
	            if (LOG.isInfoEnabled()) {
	                LOG.info("*********** PatientServiceImpl.updateTreatmentFacilitiesBatch() error saving report 1");
	                System.out.println("*********** PatientServiceImpl.updateTreatmentFacilitiesBatch() error saving report 1");
	            }
	        }	

            if (LOG.isInfoEnabled()) {
                LOG.info("*********** PatientServiceImpl.updateTreatmentFacilitiesBatch() iniial report created:");
            }
	        
        	if(maxCount == -1) {
        		maxCount = bos.size();
        	}
        	
        	for (int i = 0; i < maxCount; i++) {
        		PatientBO patientBO = bos.get(i);
        		Patient patient = patientBO.getPatientValues();
        		PatientServiceResponse response = 
        			ServiceFactory.createMviIntegrationService().updateTreatmentFacilitiesForExistingPatients(patient);
        		if(bo.getTotalNumberReviewed() == null) {
        			bo.setTotalNumberReviewed((1));
        		} else {
        			bo.setTotalNumberReviewed((bo.getTotalNumberReviewed() + 1));
        		}
        		
                if (LOG.isInfoEnabled()) {
                    LOG.info("*********** PatientServiceImpl.updateTreatmentFacilitiesBatch() processing patient icn:" + patient.getIcn());
                }
        		if(response.getMessages().hasErrorMessages()) {
	                if (LOG.isInfoEnabled()) {
	                    LOG.info("*********** PatientServiceImpl.updateTreatmentFacilitiesBatch() error processing patient icn:" + patient.getIcn());
	                }
        			if(bo.getTotalNumberError()==null) {
        				bo.setTotalNumberError((1));
        			} else {
        				bo.setTotalNumberError((bo.getTotalNumberError() + 1));
        			}
        		} else {
	                if (LOG.isInfoEnabled()) {
	                    LOG.info("*********** PatientServiceImpl.updateTreatmentFacilitiesBatch() success processing patient icn:" + patient.getIcn());
	                }

        			if(bo.getTotalNumberUpdated()==null) {
        				bo.setTotalNumberUpdated((1));
        			} else {
        				bo.setTotalNumberUpdated((bo.getTotalNumberUpdated() + 1));
        			}
        		}
			}
	        batchReport.setCompletionTime(new Timestamp(new Date().getTime()));
	        bo.save();
	        if( bo.getAllMessages( ).hasErrorMessages( ) ) {	     
	            if (LOG.isInfoEnabled()) {
	                LOG.info("*********** PatientServiceImpl.updateTreatmentFacilitiesBatch() error saving report 2");
	                System.out.println("*********** PatientServiceImpl.updateTreatmentFacilitiesBatch() error saving report 2");
	            }
	        }	
        }
		return voidResponse;
	}
    
    public VoidServiceResponse processBatchMviAuthenticate(String auth) {
        VoidServiceResponse response = new VoidServiceResponse();
        List<InPersonAuthenticationBO> bos = InPersonAuthenticationBO.getMVIPendingAuthPatients();
        if(bos != null && !bos.isEmpty()) {
	        Collection<InPersonAuthentication> ipas = InPersonAuthenticationAssembler.getInPersonAuthenticationCollection(bos);
	        for (InPersonAuthentication ipa : ipas) {
	        	Patient patient = ipa.getPatient();
                if (LOG.isDebugEnabled()) {
                    LOG.debug("PatientServiceImpl.batchMviAuthenticate():authenticating patient with ICN:" + patient.getIcn());
                }
                BooleanServiceResponse patientServiceResponse = 
                	ServiceFactory.createMviIntegrationService().mviAuthenticate(patient, patient.getUserProfile(), true, true);
	            if (patientServiceResponse.getBoolean()) {
	                AuthenticationStatus status = InPersonAuthenticationStatusUtils.AUTHENTICATED;
	            	ipa.setMviAuthenticationStatus("OK");
	                ipa = saveIPA(ipa, status);
	            }
			}
        }
        return response;
    }
    
    public VoidServiceResponse processAsyncBatchMviAuthenticate(String auth) {
    	return processBatchMviAuthenticate(auth);
    }
    
    public VoidServiceResponse processBatchMviUnauthenticate(String unauth) {
        VoidServiceResponse response = new VoidServiceResponse();
        List<InPersonAuthentication> bos = InPersonAuthenticationBO.getMVIPendingUnauthPatients();
        if(bos != null && !bos.isEmpty()) {
	        Collection<InPersonAuthentication> ipas = InPersonAuthenticationAssembler.getInPersonAuthenticationCollection(bos);
	        for (InPersonAuthentication ipa : ipas) {
	        	Patient patient = ipa.getPatient();
                if (LOG.isDebugEnabled()) {
                    LOG.debug("PatientServiceImpl.processBatchMviUnauthenticate():unauthenticating patient with ICN:" + patient.getIcn());
                }
                BooleanServiceResponse patientServiceResponse = 
                	ServiceFactory.createMviIntegrationService().mviUnauthenticateUncorrelateForBatch(patient, patient.getUserProfile(), "SYSTEM");
	            if (patientServiceResponse.getBoolean()) {
	                AuthenticationStatus status = InPersonAuthenticationStatusUtils.UNAUTHENTICATED;
    				ipa.setMviAuthenticationStatus("OK");
	                ipa = saveIPA(ipa, status);
	            } else {
	                if (LOG.isDebugEnabled()) {
	                    LOG.debug("PatientServiceImpl.processBatchMviUnauthenticate(): Error unauthenticating patient with ICN:" + patient.getIcn());
	                }
	            }
			}
        }
        return response;
    }
    public VoidServiceResponse processAsyncBatchMviUnauthenticate(String auth) {
    	return processBatchMviUnauthenticate(auth);
    }
    
    @SuppressWarnings("unchecked")
	public VoidServiceResponse processAccountSyncronization() {
    	System.out.println("*** Process Account Synchronization Method***");
    	LOG.info("*** Process Account Synchronization Method***");
    	VoidServiceResponse voidResponse = new VoidServiceResponse();
    	List<Patient> requiredFieldsNotEmptyList = new ArrayList<Patient>();
    	List<Patient> recordNotMatchedToMVIList = new ArrayList<Patient>();
    	List<PatientBO> requiredFieldsNotEmptyListBO = new ArrayList<PatientBO>();
    	List<PatientBO> recordNotMatchedToMVIListBO = new ArrayList<PatientBO>();
    	//List<Patient> mhvmviMisMatchList = new ArrayList<Patient>();
    	//List<Patient> icnMissingList = new ArrayList<Patient>();
    	System.out.println("*** Getting Records Not Matched To MVI***");
    	LOG.info("*** Getting Records Not Matched To MVI***");
    	List<PatientBO> recordsNotMatchedToMVI = PatientBO.getRecordsNotMatchedToMVI();
    	System.out.println("*** Finished Getting Records Not Matched To MVI***");
    	LOG.info("*** Finished Getting Records Not Matched To MVI***");
    	System.out.println("*** Getting Required Fields Not Empty***");
    	LOG.info("*** Getting Required Fields Not Empty***");
    	List<PatientBO> requiredFieldsNotEmpty = PatientBO.getRequiredFieldsNotEmpty();
    	System.out.println("*** Finished Getting Required Fields Not Empty***");
    	LOG.info("*** Finished Getting Required Fields Not Empty***");
    	String requiredFieldsNotEmptyBatchReport = "Required_Fields_Not_Empty";
    	String recordNotMatchedToMVIBatchReport = "Record_Not_Matched_to_MVI";
    	int recordNotMatchedCounter = 0;
    	int notEmptyCounter = 0;
    	long totalCounter = 0l;
        if(recordsNotMatchedToMVI != null && !recordsNotMatchedToMVI.isEmpty()) {
        	if (LOG.isInfoEnabled()) {
                LOG.info("*********** PatientServiceImpl.processAccountSynchronization() number of records not matched to MVI:" + recordNotMatchedToMVIListBO.size());
                System.out.println("*********** PatientServiceImpl.processAccountSynchronization() number of records not matched to MVI:" + recordNotMatchedToMVIListBO.size());
            }
        	for(PatientBO patientBO : recordsNotMatchedToMVI) {
            	//PatientBO patientBO = (PatientBO)object[1];
            	//recordNotMatchedToMVIListBO.add(patientBO);
            	//for (int i = 0; i < recordNotMatchedToMVIListBO.size(); i++) {
        		++totalCounter;
        		//PatientBO patientBO = recordNotMatchedToMVIListBO.get(i);
        		Patient patient = patientBO.getPatientValues();
        		if(patient.getIcn() != null) {  //if no ICN then basic account
        			if(patient.getIcn().contains("V")) {  //must have a "V" for active account
   						recordNotMatchedToMVIList.add(patient);
   						recordNotMatchedCounter++;
   						LOG.info("*********** PatientServiceImpl.processAccountSynchronization() processing patient record not matched to MVI icn:" + patient.getIcn());
   						System.out.println("*********** PatientServiceImpl.processAccountSynchronization() processing patient record not matched to MVI icn:" + patient.getIcn());
   						LOG.info("*********** PatientServiceImpl.processAccountSynchronization() record not matched to MVI counter:" + recordNotMatchedCounter);
   						System.out.println("*********** PatientServiceImpl.processAccountSynchronization() processing patient record not matched to MVI icn:" + recordNotMatchedCounter);
  					}
   				}
            }
        }
        if(requiredFieldsNotEmpty != null && !requiredFieldsNotEmpty.isEmpty()) {
        	for(PatientBO patientBO : requiredFieldsNotEmpty) {
            	//PatientBO patientBO = (PatientBO)object[1];
            	//requiredFieldsNotEmptyListBO.add(patientBO);
            	//for (int i = 0; i < requiredFieldsNotEmptyListBO.size(); i++) {
        		++totalCounter;
        		//PatientBO patientBO = requiredFieldsNotEmptyListBO.get(i);
        		Patient patient = patientBO.getPatientValues();
   				requiredFieldsNotEmptyList.add(patient);
       			notEmptyCounter++;
       		    LOG.info("*********** PatientServiceImpl.processAccountSynchronization() processing patient for required fields not empty icn:" + patient.getIcn());
                System.out.println("*********** PatientServiceImpl.processAccountSynchronization() processing patient required fields not empty icn:" + patient.getIcn());
      		    LOG.info("*********** PatientServiceImpl.processAccountSynchronization() required fields not empty counter:" + notEmptyCounter);
                System.out.println("*********** PatientServiceImpl.processAccountSynchronization() processing patient required fields not empty counter:" + notEmptyCounter);
   			}
        }
//        			if(patient.getIcn() != null) {  //if no ICN then basic account
//	        			if(patient.getIcn().contains("V")) {  //must have a "V" for active account
////		        			PatientServiceResponse response = 
////		        			ServiceFactory.createMviIntegrationService().searchPersonInMVIByICN(patient.getIcn());
////		        			Patient mviPatient = response.getPatient();
//	        				if(patient.getUserProfile().getIsVeteran() == true || patient.getUserProfile().getIsPatient() == true) {
//	        					if(patient.getMatchedDateTime() == null) {
//	        						recordNotMatchedToMVIList.add(patient);
//	        						recordNotMatchedCounter++;
//	        					    LOG.info("*********** PatientServiceImpl.processAccountSynchronization() processing patient record not matched to MVI icn:" + patient.getIcn());
//	        	                    System.out.println("*********** PatientServiceImpl.processAccountSynchronization() processing patient record not matched to MVI icn:" + patient.getIcn());
//	        	                    LOG.info("*********** PatientServiceImpl.processAccountSynchronization() record not matched to MVI counter:" + recordNotMatchedCounter);
//	        	                    System.out.println("*********** PatientServiceImpl.processAccountSynchronization() processing patient record not matched to MVI icn:" + recordNotMatchedCounter);
//	        					}
//	        				}
//	        				List<PatientBO> mviPatients = getMVIPatients();
//	        				PatientBO mviPatientBO = mviPatients.get(i);
//	                		Patient mviPatient = mviPatientBO.getPatientValues();
//		        			if(mviPatient != null) {
//		        				boolean icnMatch = false;
//			        			if(mviPatient.getIcn() == null) {
//			        				icnMissingList.add(patient);
//			            			icnMissingBatchReport.setTotalNumberReviewed((icnMissingBatchReport.getTotalNumberReviewed() + 1));
//			        			} else {
//			        				icnMatch = mviPatient.getIcn().equals(patient.getIcn()) ? true : false;
//			        			}
//			        			boolean firstNameMatch = mviPatient.getUserProfile().getFirstName().equals(patient.getUserProfile().getFirstName()) ? true : false;
//			        			boolean lastNameMatch = mviPatient.getUserProfile().getLastName().equals(patient.getUserProfile().getLastName()) ? true : false;
//			        			boolean genderMatch = mviPatient.getUserProfile().getGender().equals(patient.getUserProfile().getGender()) ? true : false;
//			        			boolean birthdateMatch = mviPatient.getUserProfile().getBirthDate().equals(patient.getUserProfile().getBirthDate()) ? true : false;
//			        			boolean ssnMatch = mviPatient.getUserProfile().getSsn().equals(patient.getUserProfile().getSsn()) ? true : false;
//			        			int matchCounter = 0;
//			        			matchCounter = firstNameMatch ? ++matchCounter : matchCounter;
//			        			matchCounter = lastNameMatch ? ++matchCounter : matchCounter;
//			        			matchCounter = genderMatch ? ++matchCounter : matchCounter;
//			        			matchCounter = birthdateMatch ? ++matchCounter : matchCounter;
//			        			matchCounter = ssnMatch ? ++matchCounter : matchCounter;
//			        			matchCounter = icnMatch ? ++matchCounter : matchCounter;
//			        			if(matchCounter > 5) {
//			        				break;  //If the ICN and four or more traits Match (first name, last name, middle name, SSN, DOB, Gender), MHV System shall skip that record and take no action.
//			        			}
//			        			if(matchCounter < 2) {
//			        				mhvmviMisMatchList.add(patient);
//			            			mhvmviMisMatchBatchReport.setTotalNumberReviewed((mhvmviMisMatchBatchReport.getTotalNumberReviewed() + 1));
//			        			}
//			        			if(!patient.getIcn().equals(mviPatient.getIcn())) {
//			        				mhvmviMisMatchList.add(patient);
//			            			mhvmviMisMatchBatchReport.setTotalNumberReviewed((mhvmviMisMatchBatchReport.getTotalNumberReviewed() + 1));
//			        			}
//		        			}
//	        			}
//        			}
//        		} else {
//        			requiredFieldsNotEmptyList.add(patient);
//        			notEmptyCounter++;
//        		    LOG.info("*********** PatientServiceImpl.processAccountSynchronization() processing patient for required fields not empty icn:" + patient.getIcn());
//                    System.out.println("*********** PatientServiceImpl.processAccountSynchronization() processing patient required fields not empty icn:" + patient.getIcn());
//        		    LOG.info("*********** PatientServiceImpl.processAccountSynchronization() required fields not empty counter:" + notEmptyCounter);
//                    System.out.println("*********** PatientServiceImpl.processAccountSynchronization() processing patient required fields not empty counter:" + notEmptyCounter);
//        		}
//			}
        try {
        	if(requiredFieldsNotEmptyList.size() > 0) {
        		createPdfReport(requiredFieldsNotEmptyList, requiredFieldsNotEmptyBatchReport, totalCounter, requiredFieldsNotEmptyList.size());
        		System.out.println("*********** Finished Creating "  + requiredFieldsNotEmptyBatchReport + " report.");
        		LOG.info("*********** Finished Creating "  + requiredFieldsNotEmptyBatchReport + " report.");
        	}
//      	if(icnMissingList.size() > 0) {
//      		createPdfReport(icnMissingList, icnMissingDataBatchReport, totalRecordsProcessed, icnMissingBatchReport.getTotalNumberReviewed());
//      	}
        	if(recordNotMatchedToMVIList.size() > 0) {
        		createPdfReport(recordNotMatchedToMVIList, recordNotMatchedToMVIBatchReport, totalCounter, recordNotMatchedToMVIList.size());
        		System.out.println("*********** Finished Creating "  + recordNotMatchedToMVIBatchReport + " report.");
        		LOG.info("*********** Finished Creating "  + recordNotMatchedToMVIBatchReport + " report.");
        	}
//        	if(mhvmviMisMatchList.size() > 0) {
//        		createPdfReport(mhvmviMisMatchList, mhvmviMatchBatchReport, totalRecordsProcessed, mhvmviMisMatchBatchReport.getTotalNumberReviewed());
//        	}
        } catch (DocumentException e) {
        	e.printStackTrace();
        }
		return voidResponse;
    }
    
    public void createPdfReport(List<Patient> patients, String fileName, long totalNumberProcessed, int totalReviewed) throws DocumentException {
    	SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
    	String veteranRel = "Veteran";
    	String patientRel = "Patient";
    	String healthCareProviderRel = "Health Care Provider";
    	String otherRel = "Other";
    	String recordsProcessed= "Total MHV Records Processed";
    	String exceptionReported = "Total Exceptions Reported Here";
    	fileName = fileName + ".pdf";
    	Document document = new Document(PageSize.LETTER.rotate());
        try {
        	File file = new File("../../../../../../" + fileName);
        	FileOutputStream fop = new FileOutputStream(file);
			PdfWriter.getInstance(document, fop);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		document.open();
	    document.addTitle(fileName);
		document.addSubject(fileName);
		document.addAuthor("MyHealtheVet");
		document.addCreator("MyHealtheVet");
		document.addCreationDate();
		
		PdfPTable table1 = new PdfPTable(1);
		table1.setHorizontalAlignment(Element.ALIGN_LEFT);
		table1.getDefaultCell().setBackgroundColor(BaseColor.WHITE);
		table1.getDefaultCell().setBorder(Rectangle.NO_BORDER);
		table1.getDefaultCell().setNoWrap(true);
		PdfPCell table1cell1 = new PdfPCell(new Phrase(new Chunk("Title:        " + fileName, titleFont)));
		table1cell1.setHorizontalAlignment(Element.ALIGN_LEFT);
		table1cell1.setBorder(Rectangle.NO_BORDER);
		table1cell1.setBorderColor(BaseColor.WHITE);
		table1cell1.setColspan(1);
		table1.addCell(table1cell1);
		document.add(table1);
		
		PdfPTable table2 = new PdfPTable(1);
		table2.setHorizontalAlignment(Element.ALIGN_LEFT);
		table2.getDefaultCell().setBackgroundColor(BaseColor.WHITE);
		table2.getDefaultCell().setBorder(Rectangle.NO_BORDER);
		table2.getDefaultCell().setNoWrap(true);
		Calendar calendar = Calendar.getInstance();
		Timestamp today = new Timestamp(calendar.getTime().getTime());
		String runDate = sdf.format(today);
		String[] runDateValues = runDate.split("-");
		PdfPCell table2cell1 = new PdfPCell(new Phrase(new Chunk("Run Date: " + runDateValues[2]+ "/" + runDateValues[1] + "/" + runDateValues[0], titleFont)));
		table2cell1.setHorizontalAlignment(Element.ALIGN_LEFT);
		table2cell1.setBorder(Rectangle.NO_BORDER);
		table2cell1.setBorderColor(BaseColor.WHITE);
		table2cell1.setColspan(1);
		table2.addCell(table2cell1);
		document.add(table2);
		
		PdfPTable table3 = new PdfPTable(33);
		table3.setHorizontalAlignment(Element.ALIGN_LEFT);
		table3.getDefaultCell().setHorizontalAlignment(Element.ALIGN_LEFT);
		table3.getDefaultCell().setBorder(Rectangle.NO_BORDER);
		table3.getDefaultCell().setBackgroundColor(BaseColor.WHITE);
		table3.getDefaultCell().setNoWrap(false);
		
		PdfPCell table3cell1 = new PdfPCell(new Phrase(new Chunk("Unique Profile Id", tableFont)));
		table3cell1.setBorderColor(BaseColor.GRAY);
		table3cell1.setBorderWidth(1);
		table3cell1.setBorderWidthLeft(1);
		table3cell1.setBorderWidthTop(1);
		table3cell1.setBorderWidthBottom(1);
		table3cell1.setBorderWidthRight(1);
		table3cell1.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell1.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell1.setNoWrap(false);
		table3cell1.setColspan(2);
		table3.addCell(table3cell1);
		
		PdfPCell table3cell2 = new PdfPCell(new Phrase(new Chunk("Relationship to the VA", tableFont)));
		table3cell2.setBorderColor(BaseColor.GRAY);
		table3cell2.setBorderWidth(1);
		table3cell2.setBorderWidthLeft(1);
		table3cell2.setBorderWidthTop(1);
		table3cell2.setBorderWidthBottom(1);
		table3cell2.setBorderWidthRight(1);
		table3cell2.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell2.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell2.setNoWrap(false);
		table3cell2.setColspan(3);
		table3.addCell(table3cell2);
		
		PdfPCell table3cell3 = new PdfPCell(new Phrase(new Chunk("ICN", tableFont)));
		table3cell3.setBorderColor(BaseColor.GRAY);
		table3cell3.setBorderWidth(1);
		table3cell3.setBorderWidthLeft(1);
		table3cell3.setBorderWidthTop(1);
		table3cell3.setBorderWidthBottom(1);
		table3cell3.setBorderWidthRight(1);
		table3cell3.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell3.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell3.setNoWrap(false);
		table3cell3.setColspan(5);
		table3.addCell(table3cell3);
		
		PdfPCell table3cell4 = new PdfPCell(new Phrase(new Chunk("First Name", tableFont)));
		table3cell4.setBorderColor(BaseColor.GRAY);
		table3cell4.setBorderWidth(1);
		table3cell4.setBorderWidthLeft(1);
		table3cell4.setBorderWidthTop(1);
		table3cell4.setBorderWidthBottom(1);
		table3cell4.setBorderWidthRight(1);
		table3cell4.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell4.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell4.setNoWrap(false);
		table3cell4.setColspan(2);
		table3.addCell(table3cell4);
		
		PdfPCell table3cell5 = new PdfPCell(new Phrase(new Chunk("Middle Name", tableFont)));
		table3cell5.setBorderColor(BaseColor.GRAY);
		table3cell5.setBorderWidth(1);
		table3cell5.setBorderWidthLeft(1);
		table3cell5.setBorderWidthTop(1);
		table3cell5.setBorderWidthBottom(1);
		table3cell5.setBorderWidthRight(1);
		table3cell5.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell5.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell5.setNoWrap(false);
		table3cell5.setColspan(2);
		table3.addCell(table3cell5);
		
		PdfPCell table3cell6 = new PdfPCell(new Phrase(new Chunk("Last Name", tableFont)));
		table3cell6.setBorderColor(BaseColor.GRAY);
		table3cell6.setBorderWidth(1);
		table3cell6.setBorderWidthLeft(1);
		table3cell6.setBorderWidthTop(1);
		table3cell6.setBorderWidthBottom(1);
		table3cell6.setBorderWidthRight(1);
		table3cell6.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell6.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell6.setNoWrap(false);
		table3cell6.setColspan(2);
		table3.addCell(table3cell6);
		
		PdfPCell table3cell7 = new PdfPCell(new Phrase(new Chunk("Gender", tableFont)));
		table3cell7.setBorderColor(BaseColor.GRAY);
		table3cell7.setBorderWidth(1);
		table3cell7.setBorderWidthLeft(1);
		table3cell7.setBorderWidthTop(1);
		table3cell7.setBorderWidthBottom(1);
		table3cell7.setBorderWidthRight(1);
		table3cell7.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell7.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell7.setNoWrap(false);
		table3cell7.setColspan(2);
		table3.addCell(table3cell7);
		
		PdfPCell table3cell8 = new PdfPCell(new Phrase(new Chunk("Date of Birth", tableFont)));
		table3cell8.setBorderColor(BaseColor.GRAY);
		table3cell8.setBorderWidth(1);
		table3cell8.setBorderWidthLeft(1);
		table3cell8.setBorderWidthTop(1);
		table3cell8.setBorderWidthBottom(1);
		table3cell8.setBorderWidthRight(1);
		table3cell8.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell8.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell8.setNoWrap(false);
		table3cell8.setColspan(3);
		table3.addCell(table3cell8);
		
		PdfPCell table3cell9 = new PdfPCell(new Phrase(new Chunk("SSN", tableFont)));
		table3cell9.setBorderColor(BaseColor.GRAY);
		table3cell9.setBorderWidth(1);
		table3cell9.setBorderWidthLeft(1);
		table3cell9.setBorderWidthTop(1);
		table3cell9.setBorderWidthBottom(1);
		table3cell9.setBorderWidthRight(1);
		table3cell9.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell9.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell9.setNoWrap(false);
		table3cell9.setColspan(3);
		table3.addCell(table3cell9);
		
		PdfPCell table3cell10 = new PdfPCell(new Phrase(new Chunk("Correlation Date", tableFont)));
		table3cell10.setBorderColor(BaseColor.GRAY);
		table3cell10.setBorderWidth(1);
		table3cell10.setBorderWidthLeft(1);
		table3cell10.setBorderWidthTop(1);
		table3cell10.setBorderWidthBottom(1);
		table3cell10.setBorderWidthRight(1);
		table3cell10.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell10.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell10.setNoWrap(false);
		table3cell10.setColspan(3);
		table3.addCell(table3cell10);
		
		PdfPCell table3cell11 = new PdfPCell(new Phrase(new Chunk("Match Date", tableFont)));
		table3cell11.setBorderColor(BaseColor.GRAY);
		table3cell11.setBorderWidth(1);
		table3cell11.setBorderWidthLeft(1);
		table3cell11.setBorderWidthTop(1);
		table3cell11.setBorderWidthBottom(1);
		table3cell11.setBorderWidthRight(1);
		table3cell11.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell11.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell11.setNoWrap(false);
		table3cell11.setColspan(3);
		table3.addCell(table3cell11);
		
		PdfPCell table3cell12 = new PdfPCell(new Phrase(new Chunk("Account Created Date", tableFont)));
		table3cell12.setBorderColor(BaseColor.GRAY);
		table3cell12.setBorderWidth(1);
		table3cell12.setBorderWidthLeft(1);
		table3cell12.setBorderWidthTop(1);
		table3cell12.setBorderWidthBottom(1);
		table3cell12.setBorderWidthRight(1);
		table3cell12.setHorizontalAlignment(Element.ALIGN_CENTER);
		table3cell12.setVerticalAlignment(Element.ALIGN_CENTER);
		table3cell12.setNoWrap(false);
		table3cell12.setColspan(3);
		table3.addCell(table3cell12);
		document.add(table3);
		
		for(Patient patient : patients) {
			PdfPTable table4 = new PdfPTable(33);
			table4.setHorizontalAlignment(Element.ALIGN_LEFT);
			table4.getDefaultCell().setHorizontalAlignment(Element.ALIGN_LEFT);
			table4.getDefaultCell().setBorder(Rectangle.NO_BORDER);
			table4.getDefaultCell().setBackgroundColor(BaseColor.WHITE);
			table4.getDefaultCell().setNoWrap(false);
			String uniqueId = patient.getUserProfile().getId() != null ? String.valueOf(patient.getUserProfile().getId()) : "";
			PdfPCell table4cell1 = new PdfPCell(new Phrase(new Chunk(uniqueId, subFont)));
			table4cell1.setBorderColor(BaseColor.GRAY);
			table4cell1.setBorderWidth(1);
			table4cell1.setBorderWidthLeft(1);
			table4cell1.setBorderWidthTop(1);
			table4cell1.setBorderWidthBottom(1);
			table4cell1.setBorderWidthRight(1);
			table4cell1.setHorizontalAlignment(Element.ALIGN_CENTER);
			table4cell1.setVerticalAlignment(Element.ALIGN_CENTER);
			table4cell1.setNoWrap(true);
			table4cell1.setColspan(2);
			table4.addCell(table4cell1);
			
	    	String relationship = "";
	    	String relationshipSeparator = ", ";
	    	if(patient.getUserProfile().getIsVeteran() != null) {
	    		if(patient.getUserProfile().getIsVeteran()) {
	    			relationship += veteranRel;
	    		}
	    	}
	    	if(patient.getUserProfile().getIsPatient() != null) {
	    		if(patient.getUserProfile().getIsPatient()) {
	    			if(relationship.length() > 1) {
	    				relationship += relationshipSeparator + patientRel;
	    			} else {
	    				relationship += patientRel;
	    			}
	    		}
	    	}
	    	if(patient.getUserProfile().getIsHealthCareProvider() != null) {
	    		if(patient.getUserProfile().getIsHealthCareProvider()) {
	    			if(relationship.length() > 1) {
	    				relationship += relationshipSeparator + healthCareProviderRel;
	    			} else {
	    				relationship += patientRel;
	    			}
	    		}
	    	}
	    	if(patient.getUserProfile().getIsOther() != null) {
	    		if(patient.getUserProfile().getIsOther()) {
	    			if(relationship.length() > 1) {
	    				relationship += relationshipSeparator + otherRel;
	    			} else {
	    				relationship += patientRel;
	    			}
	    		}
	    	}
	    	PdfPCell table4cell2 = new PdfPCell(new Phrase(new Chunk(relationship, subFont)));
			table4cell2.setBorderColor(BaseColor.GRAY);
			table4cell2.setBorderWidth(1);
			table4cell2.setBorderWidthLeft(1);
			table4cell2.setBorderWidthTop(1);
			table4cell2.setBorderWidthBottom(1);
			table4cell2.setBorderWidthRight(1);
			table4cell2.setHorizontalAlignment(Element.ALIGN_CENTER);
			table4cell2.setVerticalAlignment(Element.ALIGN_CENTER);
			table4cell2.setNoWrap(false);
			table4cell2.setColspan(3);
			table4.addCell(table4cell2);
			
			String icn = patient.getIcn() != null ? patient.getIcn() : "";
			PdfPCell table4cell3 = new PdfPCell(new Phrase(new Chunk(icn, subFont)));
			table4cell3.setBorderColor(BaseColor.GRAY);
			table4cell3.setBorderWidth(1);
			table4cell3.setBorderWidthLeft(1);
			table4cell3.setBorderWidthTop(1);
			table4cell3.setBorderWidthBottom(1);
			table4cell3.setBorderWidthRight(1);
			table4cell3.setHorizontalAlignment(Element.ALIGN_CENTER);
			table4cell3.setVerticalAlignment(Element.ALIGN_CENTER);
			table4cell3.setNoWrap(true);
			table4cell3.setColspan(5);
			table4.addCell(table4cell3);
			
			String firstName = patient.getUserProfile().getFirstName() != null ? patient.getUserProfile().getFirstName() : "";
			PdfPCell table4cell4 = new PdfPCell(new Phrase(new Chunk(firstName, subFont)));
			table4cell4.setBorderColor(BaseColor.GRAY);
			table4cell4.setBorderWidth(1);
			table4cell4.setBorderWidthLeft(1);
			table4cell4.setBorderWidthTop(1);
			table4cell4.setBorderWidthBottom(1);
			table4cell4.setBorderWidthRight(1);
			table4cell4.setHorizontalAlignment(Element.ALIGN_CENTER);
			table4cell4.setVerticalAlignment(Element.ALIGN_CENTER);
			table4cell4.setNoWrap(false);
			table4cell4.setColspan(2);
			table4.addCell(table4cell4);
			
			String middleName = patient.getUserProfile().getMiddleName() != null ? patient.getUserProfile().getMiddleName() : "";
			PdfPCell table4cell5 = new PdfPCell(new Phrase(new Chunk(middleName, subFont)));
			table4cell5.setBorderColor(BaseColor.GRAY);			
			table4cell5.setBorderWidth(1);
			table4cell5.setBorderWidthLeft(1);
			table4cell5.setBorderWidthTop(1);
			table4cell5.setBorderWidthBottom(1);
			table4cell5.setBorderWidthRight(1);
			table4cell5.setHorizontalAlignment(Element.ALIGN_CENTER);
			table4cell5.setVerticalAlignment(Element.ALIGN_CENTER);
			table4cell5.setNoWrap(false);
			table4cell5.setColspan(2);
			table4.addCell(table4cell5);
		
			String lastName = patient.getUserProfile().getLastName() != null ? patient.getUserProfile().getLastName() : "";
			PdfPCell table4cell6 = new PdfPCell(new Phrase(new Chunk(lastName, subFont)));
			table4cell6.setBorderColor(BaseColor.GRAY);
			table4cell6.setBorderWidth(1);
			table4cell6.setBorderWidthLeft(1);
			table4cell6.setBorderWidthTop(1);
			table4cell6.setBorderWidthBottom(1);
			table4cell6.setBorderWidthRight(1);
			table4cell6.setHorizontalAlignment(Element.ALIGN_CENTER);
			table4cell6.setVerticalAlignment(Element.ALIGN_CENTER);
			table4cell6.setNoWrap(false);
			table4cell6.setColspan(2);
			table4.addCell(table4cell6);
			
			String gender = patient.getUserProfile().getGender() != null ? patient.getUserProfile().getGender().getName() : "";
			PdfPCell table4cell7 = new PdfPCell(new Phrase(new Chunk(gender, subFont)));
			table4cell7.setBorderColor(BaseColor.GRAY);
			table4cell7.setBorderWidth(1);
			table4cell7.setBorderWidthLeft(1);
			table4cell7.setBorderWidthTop(1);
			table4cell7.setBorderWidthBottom(1);
			table4cell7.setBorderWidthRight(1);
			table4cell7.setHorizontalAlignment(Element.ALIGN_CENTER);
			table4cell7.setVerticalAlignment(Element.ALIGN_CENTER);
			table4cell7.setNoWrap(true);
			table4cell7.setColspan(2);
			table4.addCell(table4cell7);
			
			String birthDate = "";
			String[] birthDateValues = new String[3];
			if(patient.getUserProfile().getBirthDate() != null) {
				birthDate = sdf.format(patient.getUserProfile().getBirthDate());
				birthDateValues = birthDate.split("-"); 
				birthDate = birthDateValues[2] + "/" + birthDateValues[1] + "/" + birthDateValues[0];
			}
			PdfPCell table4cell8 = new PdfPCell(new Phrase(new Chunk(birthDate, subFont)));
			table4cell8.setBorderColor(BaseColor.GRAY);
			table4cell8.setBorderWidth(1);
			table4cell8.setBorderWidthLeft(1);
			table4cell8.setBorderWidthTop(1);
			table4cell8.setBorderWidthBottom(1);
			table4cell8.setBorderWidthRight(1);
			table4cell8.setHorizontalAlignment(Element.ALIGN_CENTER);
			table4cell8.setVerticalAlignment(Element.ALIGN_CENTER);
			table4cell8.setNoWrap(true);
			table4cell8.setColspan(3);
			table4.addCell(table4cell8);
			
			String ssn = patient.getUserProfile().getSsn() != null ? patient.getUserProfile().getSsn() : "";
			PdfPCell table4cell9 = new PdfPCell(new Phrase(new Chunk(ssn, subFont)));
			table4cell9.setBorderColor(BaseColor.GRAY);
			table4cell9.setBorderWidth(1);
			table4cell9.setBorderWidthLeft(1);
			table4cell9.setBorderWidthTop(1);
			table4cell9.setBorderWidthBottom(1);
			table4cell9.setBorderWidthRight(1);
			table4cell9.setHorizontalAlignment(Element.ALIGN_CENTER);
			table4cell9.setVerticalAlignment(Element.ALIGN_CENTER);
			table4cell9.setNoWrap(true);
			table4cell9.setColspan(3);
			table4.addCell(table4cell9);
			
			String correlatedDate = "";
			String[] correlatedDateValues = new String[3];
			if(patient.getCorrelatedDateTime() != null) {
				Timestamp stamp = patient.getCorrelatedDateTime();
				correlatedDate = sdf.format(new Date(stamp.getTime()));
				correlatedDateValues = correlatedDate.split("-");
				correlatedDate = correlatedDateValues[2] + "/" + correlatedDateValues[1] + "/" + correlatedDateValues[0];
			}
			PdfPCell table4cell10 = new PdfPCell(new Phrase(new Chunk(correlatedDate, subFont)));
			table4cell10.setBorderColor(BaseColor.GRAY);
			table4cell10.setBorderWidth(1);
			table4cell10.setBorderWidthLeft(1);
			table4cell10.setBorderWidthTop(1);
			table4cell10.setBorderWidthBottom(1);
			table4cell10.setBorderWidthRight(1);
			table4cell10.setHorizontalAlignment(Element.ALIGN_CENTER);
			table4cell10.setVerticalAlignment(Element.ALIGN_CENTER);
			table4cell10.setNoWrap(true);
			table4cell10.setColspan(3);
			table4.addCell(table4cell10);
			
			String matchedDate = "";
			String[] matchedDateValues = new String[3];
			if(patient.getMatchedDateTime() != null) {
				Timestamp stamp = patient.getMatchedDateTime();
				matchedDate = sdf.format(new Date(stamp.getTime()));
				matchedDateValues = matchedDate.split("-");
				matchedDate = matchedDateValues[2] + "/" + matchedDateValues[1] + "/" + matchedDateValues[0];
			}
			PdfPCell table4cell11 = new PdfPCell(new Phrase(new Chunk(matchedDate, subFont)));
			table4cell11.setBorderColor(BaseColor.GRAY);
			table4cell11.setBorderWidth(1);
			table4cell11.setBorderWidthLeft(1);
			table4cell11.setBorderWidthTop(1);
			table4cell11.setBorderWidthBottom(1);
			table4cell11.setBorderWidthRight(1);
			table4cell11.setHorizontalAlignment(Element.ALIGN_CENTER);
			table4cell11.setVerticalAlignment(Element.ALIGN_CENTER);
			table4cell11.setNoWrap(true);
			table4cell11.setColspan(3);
			table4.addCell(table4cell11);
			
			String createdDate = "";
			String[] createdDateValues = new String[3];
			if(patient.getUserProfile().getCreatedDate() != null) {
				Timestamp stamp = patient.getUserProfile().getCreatedDate();
				createdDate = sdf.format(new Date(stamp.getTime()));
				createdDateValues = createdDate.split("-");
				createdDate = createdDateValues[2] + "/" + createdDateValues[1] + "/" + createdDateValues[0];
			}
			PdfPCell table4cell12 = new PdfPCell(new Phrase(new Chunk(createdDate, subFont)));
			table4cell12.setBorderColor(BaseColor.GRAY);
			table4cell12.setBorderWidth(1);
			table4cell12.setBorderWidthLeft(1);
			table4cell12.setBorderWidthTop(1);
			table4cell12.setBorderWidthBottom(1);
			table4cell12.setBorderWidthRight(1);
			table4cell12.setHorizontalAlignment(Element.ALIGN_CENTER);
			table4cell12.setVerticalAlignment(Element.ALIGN_CENTER);
			table4cell12.setNoWrap(true);
			table4cell12.setColspan(3);
			table4.addCell(table4cell12);
			document.add(table4);
		}
		PdfPTable table5 = new PdfPTable(1);
		table5.setHorizontalAlignment(Element.ALIGN_LEFT);
		table5.getDefaultCell().setHorizontalAlignment(Element.ALIGN_LEFT);
		table5.getDefaultCell().setBorder(Rectangle.NO_BORDER);
		table5.getDefaultCell().setBackgroundColor(BaseColor.WHITE);
		table5.getDefaultCell().setNoWrap(false);
		
		PdfPCell table5cell1 = new PdfPCell(new Phrase(new Chunk(recordsProcessed + "   " + String.valueOf(totalNumberProcessed), subFont)));
		table5cell1.setBorderColor(BaseColor.WHITE);
		table5cell1.setBorder(Rectangle.NO_BORDER);
		table5cell1.setHorizontalAlignment(Element.ALIGN_LEFT);
		table5cell1.setVerticalAlignment(Element.ALIGN_LEFT);
		table5cell1.setNoWrap(true);
		table5cell1.setColspan(1);
		table5.addCell(table5cell1);
		document.add(table5);
		
		PdfPTable table6 = new PdfPTable(1);
		table6.setHorizontalAlignment(Element.ALIGN_LEFT);
		table6.getDefaultCell().setHorizontalAlignment(Element.ALIGN_LEFT);
		table6.getDefaultCell().setBorder(Rectangle.NO_BORDER);
		table6.getDefaultCell().setBackgroundColor(BaseColor.WHITE);
		table6.getDefaultCell().setNoWrap(false);
		
		PdfPCell table6cell1 = new PdfPCell(new Phrase(new Chunk(exceptionReported + "  " + String.valueOf(totalReviewed), subFont)));
		table6cell1.setBorderColor(BaseColor.WHITE);
		table6cell1.setBorder(Rectangle.NO_BORDER);
		table6cell1.setHorizontalAlignment(Element.ALIGN_LEFT);
		table6cell1.setVerticalAlignment(Element.ALIGN_LEFT);
		table6cell1.setNoWrap(true);
		table6cell1.setColspan(1);
		table6.addCell(table6cell1);
		document.add(table6);
		document.close();
    }
    
    private List<PatientBO> getTestPatients() {
    	Calendar calendar = Calendar.getInstance(); 
    	Date now = calendar.getTime(); 
    	Timestamp currentTimestamp = new Timestamp(now.getTime());
    	List<PatientBO> patients = new ArrayList<PatientBO>();
    	PatientBO patientBO = new PatientBO();
    	patientBO.setCorrelatedBy("Hall, Lisa (vhaiswhalll)");
    	patientBO.setCorrelatedDateTime(currentTimestamp);
    	patientBO.setCorrelateRequestDateTime(currentTimestamp);
    	patientBO.setCorrelationErrorCode("");
    	PatientCorrelationStatus pcs = PatientCorrelationStatus.getEnum(PatientCorrelationStatus.CORRELATED);
    	patientBO.setCorrelationStatus(pcs);
    	patientBO.setIcn("1012638925V204624");
    	patientBO.setId(495714l);
    	//patientBO.setMatchedDateTime(currentTimestamp);
    	UserProfileBO userProfileBO = new UserProfileBO();
    	userProfileBO.setAddressStreet1("1818 Lorranie ave st.");
    	userProfileBO.setAddressCity("Landover");
    	CountryEnumeration country = CountryEnumeration.getEnum(CountryEnumeration.UNITED_STATES);
    	userProfileBO.setAddressCountry(country);
    	userProfileBO.setAddressPostalCode("81087");
    	StateEnumeration state = StateEnumeration.getEnum(StateEnumeration.CO);
    	userProfileBO.setAddressState(state);
    	userProfileBO.setFirstName("teste");
    	userProfileBO.setLastName("mhvjteste");
    	GenderEnumeration gender = GenderEnumeration.getEnum("Male");
    	userProfileBO.setGender(gender);
    	userProfileBO.setContactInfoEmail("ken@hkconsulting.biz");
    	MaritalStatusEnumeration maritalStatus = MaritalStatusEnumeration.getEnum(MaritalStatusEnumeration.MARRIED);
    	userProfileBO.setMaritalStatus(maritalStatus);
    	userProfileBO.setSsn("xxx-xx-xxxx");
    	userProfileBO.setMhvId("MTMM-32-8776");
    	Date birthDate = new Date("05/01/2032");
    	userProfileBO.setBirthDate(birthDate);
    	ContactMethodEnumeration contactInfoContactMethod = ContactMethodEnumeration.getEnum(ContactMethodEnumeration.EMAIL);
    	userProfileBO.setContactInfoContactMethod(contactInfoContactMethod);
    	userProfileBO.setIsVeteran(true);
    	userProfileBO.setIsPatient(true);
    	userProfileBO.setCreatedDate(currentTimestamp);
    	patientBO.setUserProfile(userProfileBO);
    	patients.add(patientBO);
    	
    	patientBO = new PatientBO();
    	patientBO.setCorrelatedBy("Hall, Lisa (vhaiswhalll)");
    	patientBO.setCorrelatedDateTime(currentTimestamp);
    	patientBO.setCorrelateRequestDateTime(currentTimestamp);
    	patientBO.setCorrelationErrorCode("");
    	pcs = PatientCorrelationStatus.getEnum(PatientCorrelationStatus.CORRELATED);
    	patientBO.setCorrelationStatus(pcs);
    	patientBO.setIcn(null);  //"1012638925V204624");
    	patientBO.setId(489451l);
    	//patientBO.setMatchedDateTime(currentTimestamp);
    	userProfileBO = new UserProfileBO();
    	userProfileBO.setAddressStreet1("550 Footfill Blvd.");
    	userProfileBO.setAddressCity("Salt Lake City");
    	country = CountryEnumeration.getEnum(CountryEnumeration.UNITED_STATES);
    	userProfileBO.setAddressCountry(country);
    	userProfileBO.setAddressPostalCode("84118");
    	state = StateEnumeration.getEnum(StateEnumeration.UT);
    	userProfileBO.setAddressState(state);
    	userProfileBO.setFirstName("mhvtest");
    	userProfileBO.setLastName("");
    	gender = GenderEnumeration.getEnum("Female");
    	userProfileBO.setGender(gender);
    	userProfileBO.setContactInfoEmail("abc@redrum.tv");
    	maritalStatus = MaritalStatusEnumeration.getEnum(MaritalStatusEnumeration.MARRIED);
    	userProfileBO.setMaritalStatus(maritalStatus);
    	userProfileBO.setSsn("xxx-xx-xxxx");
    	userProfileBO.setMhvId("MTMM-87-1234");
    	birthDate = new Date("06/11/1974");
    	userProfileBO.setBirthDate(birthDate);
    	contactInfoContactMethod = ContactMethodEnumeration.getEnum(ContactMethodEnumeration.EMAIL);
    	userProfileBO.setContactInfoContactMethod(contactInfoContactMethod);
    	patientBO.setUserProfile(userProfileBO);
    	patients.add(patientBO);
    	
    	patientBO = new PatientBO();
    	patientBO.setCorrelatedBy("Hall, Lisa (vhaiswhalll)");
    	patientBO.setCorrelatedDateTime(currentTimestamp);
    	patientBO.setCorrelateRequestDateTime(currentTimestamp);
    	patientBO.setCorrelationErrorCode("");
    	pcs = PatientCorrelationStatus.getEnum(PatientCorrelationStatus.CORRELATED);
    	patientBO.setCorrelationStatus(pcs);
    	patientBO.setIcn("1012638925V201234");
    	patientBO.setId(489451l);
    	patientBO.setMatchedDateTime(currentTimestamp);
    	userProfileBO = new UserProfileBO();
    	userProfileBO.setAddressStreet1("550 Footfill Blvd.");
    	userProfileBO.setAddressCity("Salt Lake City");
    	country = CountryEnumeration.getEnum(CountryEnumeration.UNITED_STATES);
    	userProfileBO.setAddressCountry(country);
    	userProfileBO.setAddressPostalCode("84118");
    	state = StateEnumeration.getEnum(StateEnumeration.UT);
    	userProfileBO.setAddressState(state);
    	userProfileBO.setFirstName("");
    	userProfileBO.setLastName("Master");
    	gender = GenderEnumeration.getEnum("Female");
    	userProfileBO.setGender(gender);
    	userProfileBO.setContactInfoEmail("abc123@redrum.tv");
    	maritalStatus = MaritalStatusEnumeration.getEnum(MaritalStatusEnumeration.MARRIED);
    	userProfileBO.setMaritalStatus(maritalStatus);
    	userProfileBO.setSsn("xxx-xx-xxxx");
    	userProfileBO.setMhvId("MTMM-87-1235");
    	birthDate = new Date("07/11/1975");
    	userProfileBO.setBirthDate(birthDate);
    	contactInfoContactMethod = ContactMethodEnumeration.getEnum(ContactMethodEnumeration.EMAIL);
    	userProfileBO.setContactInfoContactMethod(contactInfoContactMethod);
    	patientBO.setUserProfile(userProfileBO);
    	patients.add(patientBO);
    	
    	patientBO = new PatientBO();
    	patientBO.setCorrelatedBy("Hall, Lisa (vhaiswhalll)");
    	patientBO.setCorrelatedDateTime(currentTimestamp);
    	patientBO.setCorrelateRequestDateTime(currentTimestamp);
    	patientBO.setCorrelationErrorCode("");
    	pcs = PatientCorrelationStatus.getEnum(PatientCorrelationStatus.CORRELATED);
    	patientBO.setCorrelationStatus(pcs);
    	patientBO.setIcn("1012638925V204624");
    	patientBO.setId(489451l);
    	patientBO.setMatchedDateTime(currentTimestamp);
    	userProfileBO = new UserProfileBO();
    	userProfileBO.setAddressStreet1("550 Footfill Blvd.");
    	userProfileBO.setAddressCity("Salt Lake City");
    	country = CountryEnumeration.getEnum(CountryEnumeration.UNITED_STATES);
    	userProfileBO.setAddressCountry(country);
    	userProfileBO.setAddressPostalCode("84118");
    	state = StateEnumeration.getEnum(StateEnumeration.UT);
    	userProfileBO.setAddressState(state);
    	userProfileBO.setFirstName("firstName");
    	userProfileBO.setLastName("lastName");
    	//gender = GenderEnumeration.getEnum("Female");
    	userProfileBO.setGender(null);
    	userProfileBO.setContactInfoEmail("abc456@redrum.tv");
    	maritalStatus = MaritalStatusEnumeration.getEnum(MaritalStatusEnumeration.MARRIED);
    	userProfileBO.setMaritalStatus(maritalStatus);
    	userProfileBO.setSsn("xxx-xx-xxxx");
    	userProfileBO.setMhvId("MTMM-87-1236");
    	birthDate = new Date("08/11/1964");
    	userProfileBO.setBirthDate(birthDate);
    	contactInfoContactMethod = ContactMethodEnumeration.getEnum(ContactMethodEnumeration.EMAIL);
    	userProfileBO.setContactInfoContactMethod(contactInfoContactMethod);
    	patientBO.setUserProfile(userProfileBO);
    	patients.add(patientBO);
    	
    	patientBO = new PatientBO();
    	patientBO.setCorrelatedBy("Hall, Lisa (vhaiswhalll)");
    	patientBO.setCorrelatedDateTime(currentTimestamp);
    	patientBO.setCorrelateRequestDateTime(currentTimestamp);
    	patientBO.setCorrelationErrorCode("");
    	pcs = PatientCorrelationStatus.getEnum(PatientCorrelationStatus.CORRELATED);
    	patientBO.setCorrelationStatus(pcs);
    	patientBO.setIcn("1012638925V209412");
    	patientBO.setId(489451l);
    	patientBO.setMatchedDateTime(currentTimestamp);
    	userProfileBO = new UserProfileBO();
    	userProfileBO.setAddressStreet1("550 Footfill Blvd.");
    	userProfileBO.setAddressCity("Salt Lake City");
    	country = CountryEnumeration.getEnum(CountryEnumeration.UNITED_STATES);
    	userProfileBO.setAddressCountry(country);
    	userProfileBO.setAddressPostalCode("84118");
    	state = StateEnumeration.getEnum(StateEnumeration. UT);
    	userProfileBO.setAddressState(state);
    	userProfileBO.setFirstName(null);
    	userProfileBO.setLastName("terrible");
    	gender = GenderEnumeration.getEnum("Female");
    	userProfileBO.setGender(gender);
    	userProfileBO.setContactInfoEmail("abc@redrum.tv");
    	maritalStatus = MaritalStatusEnumeration.getEnum(MaritalStatusEnumeration.MARRIED);
    	userProfileBO.setMaritalStatus(maritalStatus);
    	userProfileBO.setSsn("xxx-xx-xxxx");
    	userProfileBO.setMhvId("MTMM-87-1237");
    	birthDate = new Date("05/19/1971");
    	userProfileBO.setBirthDate(birthDate);
    	contactInfoContactMethod = ContactMethodEnumeration.getEnum(ContactMethodEnumeration.EMAIL);
    	userProfileBO.setContactInfoContactMethod(contactInfoContactMethod);
    	patientBO.setUserProfile(userProfileBO);
    	patients.add(patientBO);
    	
    	patientBO = new PatientBO();
    	patientBO.setCorrelatedBy("Hall, Lisa (vhaiswhalll)");
    	patientBO.setCorrelatedDateTime(currentTimestamp);
    	patientBO.setCorrelateRequestDateTime(currentTimestamp);
    	patientBO.setCorrelationErrorCode("");
    	pcs = PatientCorrelationStatus.getEnum(PatientCorrelationStatus.CORRELATED);
    	patientBO.setCorrelationStatus(pcs);
    	patientBO.setIcn("1012638925V201111");
    	patientBO.setId(489451l);
    	patientBO.setMatchedDateTime(currentTimestamp);
    	userProfileBO = new UserProfileBO();
    	userProfileBO.setAddressStreet1("550 Footfill Blvd.");
    	userProfileBO.setAddressCity("Salt Lake City");
    	country = CountryEnumeration.getEnum(CountryEnumeration.UNITED_STATES);
    	userProfileBO.setAddressCountry(country);
    	userProfileBO.setAddressPostalCode("84118");
    	state = StateEnumeration.getEnum(StateEnumeration. UT);
    	userProfileBO.setAddressState(state);
    	userProfileBO.setFirstName("loser");
    	userProfileBO.setLastName(null);
    	gender = GenderEnumeration.getEnum("Female");
    	userProfileBO.setGender(gender);
    	userProfileBO.setContactInfoEmail("abc@redrum.tv");
    	maritalStatus = MaritalStatusEnumeration.getEnum(MaritalStatusEnumeration.MARRIED);
    	userProfileBO.setMaritalStatus(maritalStatus);
    	userProfileBO.setSsn("xxx-xx-xxxx");
    	userProfileBO.setMhvId("MTMM-87-1238");
    	birthDate = new Date("02/02/1922");
    	userProfileBO.setBirthDate(birthDate);
    	contactInfoContactMethod = ContactMethodEnumeration.getEnum(ContactMethodEnumeration.EMAIL);
    	userProfileBO.setContactInfoContactMethod(contactInfoContactMethod);
    	patientBO.setUserProfile(userProfileBO);
    	patients.add(patientBO);
    	
    	patientBO = new PatientBO();
    	patientBO.setCorrelatedBy("Hall, Lisa (vhaiswhalll)");
    	patientBO.setCorrelatedDateTime(currentTimestamp);
    	patientBO.setCorrelateRequestDateTime(currentTimestamp);
    	patientBO.setCorrelationErrorCode("");
    	pcs = PatientCorrelationStatus.getEnum(PatientCorrelationStatus.CORRELATED);
    	patientBO.setCorrelationStatus(pcs);
    	patientBO.setIcn("1012638925V207894");
    	patientBO.setId(489451l);
    	patientBO.setMatchedDateTime(currentTimestamp);
    	userProfileBO = new UserProfileBO();
    	userProfileBO.setAddressStreet1("550 Footfill Blvd.");
    	userProfileBO.setAddressCity("Salt Lake City");
    	country = CountryEnumeration.getEnum(CountryEnumeration.UNITED_STATES);
    	userProfileBO.setAddressCountry(country);
    	userProfileBO.setAddressPostalCode("84118");
    	state = StateEnumeration.getEnum(StateEnumeration. UT);
    	userProfileBO.setAddressState(state);
    	userProfileBO.setFirstName("best");
    	userProfileBO.setLastName("better");
    	//gender = GenderEnumeration.getEnum("Female");
    	userProfileBO.setGender(null);
    	userProfileBO.setContactInfoEmail("abc@redrum.tv");
    	maritalStatus = MaritalStatusEnumeration.getEnum(MaritalStatusEnumeration.MARRIED);
    	userProfileBO.setMaritalStatus(maritalStatus);
    	userProfileBO.setSsn("xxx-xx-xxxx");
    	userProfileBO.setMhvId("MTMM-87-1239");
    	birthDate = new Date("09/10/1995");
    	userProfileBO.setBirthDate(birthDate);
    	contactInfoContactMethod = ContactMethodEnumeration.getEnum(ContactMethodEnumeration.EMAIL);
    	userProfileBO.setContactInfoContactMethod(contactInfoContactMethod);
    	patientBO.setUserProfile(userProfileBO);
    	patients.add(patientBO);
    	
    	patientBO = new PatientBO();
    	patientBO.setCorrelatedBy("Hall, Lisa (vhaiswhalll)");
    	patientBO.setCorrelatedDateTime(currentTimestamp);
    	patientBO.setCorrelateRequestDateTime(currentTimestamp);
    	patientBO.setCorrelationErrorCode("");
    	pcs = PatientCorrelationStatus.getEnum(PatientCorrelationStatus.CORRELATED);
    	patientBO.setCorrelationStatus(pcs);
    	patientBO.setIcn("1012638925V201122");
    	patientBO.setId(489451l);
    	patientBO.setMatchedDateTime(currentTimestamp);
    	userProfileBO = new UserProfileBO();
    	userProfileBO.setAddressStreet1("550 Footfill Blvd.");
    	userProfileBO.setAddressCity("Salt Lake City");
    	country = CountryEnumeration.getEnum(CountryEnumeration.UNITED_STATES);
    	userProfileBO.setAddressCountry(country);
    	userProfileBO.setAddressPostalCode("84118");
    	state = StateEnumeration.getEnum(StateEnumeration. UT);
    	userProfileBO.setAddressState(state);
    	userProfileBO.setFirstName("faster");
    	userProfileBO.setLastName("player");
    	gender = GenderEnumeration.getEnum("Female");
    	userProfileBO.setGender(gender);
    	userProfileBO.setContactInfoEmail("abc@redrum.tv");
    	maritalStatus = MaritalStatusEnumeration.getEnum(MaritalStatusEnumeration.MARRIED);
    	userProfileBO.setMaritalStatus(maritalStatus);
    	userProfileBO.setSsn(null);
    	userProfileBO.setMhvId("MTMM-87-1210");
    	birthDate = new Date("08/16/1991");
    	userProfileBO.setBirthDate(birthDate);
    	contactInfoContactMethod = ContactMethodEnumeration.getEnum(ContactMethodEnumeration.EMAIL);
    	userProfileBO.setContactInfoContactMethod(contactInfoContactMethod);
    	patientBO.setUserProfile(userProfileBO);
    	patients.add(patientBO);
    	
    	patientBO = new PatientBO();
    	patientBO.setCorrelatedBy("Hall, Lisa (vhaiswhalll)");
    	patientBO.setCorrelatedDateTime(currentTimestamp);
    	patientBO.setCorrelateRequestDateTime(currentTimestamp);
    	patientBO.setCorrelationErrorCode("");
    	pcs = PatientCorrelationStatus.getEnum(PatientCorrelationStatus.CORRELATED);
    	patientBO.setCorrelationStatus(pcs);
    	patientBO.setIcn("1012638925V207894");
    	patientBO.setId(489451l);
    	patientBO.setMatchedDateTime(currentTimestamp);
    	userProfileBO = new UserProfileBO();
    	userProfileBO.setAddressStreet1("550 Footfill Blvd.");
    	userProfileBO.setAddressCity("Salt Lake City");
    	country = CountryEnumeration.getEnum(CountryEnumeration.UNITED_STATES);
    	userProfileBO.setAddressCountry(country);
    	userProfileBO.setAddressPostalCode("84118");
    	state = StateEnumeration.getEnum(StateEnumeration. UT);
    	userProfileBO.setAddressState(state);
    	userProfileBO.setFirstName("slowest");
    	userProfileBO.setLastName("player");
    	gender = GenderEnumeration.getEnum("Female");
    	userProfileBO.setGender(gender);
    	userProfileBO.setContactInfoEmail("abc@redrum.tv");
    	maritalStatus = MaritalStatusEnumeration.getEnum(MaritalStatusEnumeration.MARRIED);
    	userProfileBO.setMaritalStatus(maritalStatus);
    	userProfileBO.setSsn("xxx-xx-xxxx");
    	userProfileBO.setMhvId("MTMM-87-1234");
    	birthDate = new Date(null);
    	userProfileBO.setBirthDate(birthDate);
    	contactInfoContactMethod = ContactMethodEnumeration.getEnum(ContactMethodEnumeration.EMAIL);
    	userProfileBO.setContactInfoContactMethod(contactInfoContactMethod);
    	patientBO.setUserProfile(userProfileBO);
    	patients.add(patientBO);
    	
    	patientBO = new PatientBO();
    	patientBO.setCorrelatedBy("Hall, Lisa (vhaiswhalll)");
    	patientBO.setCorrelatedDateTime(currentTimestamp);
    	patientBO.setCorrelateRequestDateTime(currentTimestamp);
    	patientBO.setCorrelationErrorCode("");
    	pcs = PatientCorrelationStatus.getEnum(PatientCorrelationStatus.CORRELATED);
    	patientBO.setCorrelationStatus(pcs);
    	patientBO.setIcn("1012638925V203322");
    	patientBO.setId(489451l);
    	patientBO.setMatchedDateTime(currentTimestamp);
    	userProfileBO = new UserProfileBO();
    	userProfileBO.setAddressStreet1("550 Footfill Blvd.");
    	userProfileBO.setAddressCity("Salt Lake City");
    	country = CountryEnumeration.getEnum(CountryEnumeration.UNITED_STATES);
    	userProfileBO.setAddressCountry(country);
    	userProfileBO.setAddressPostalCode("84118");
    	state = StateEnumeration.getEnum(StateEnumeration. UT);
    	userProfileBO.setAddressState(state);
    	userProfileBO.setFirstName("biggest");
    	userProfileBO.setLastName("player");
    	gender = GenderEnumeration.getEnum("Female");
    	userProfileBO.setGender(gender);
    	userProfileBO.setContactInfoEmail("abc@redrum.tv");
    	maritalStatus = MaritalStatusEnumeration.getEnum(MaritalStatusEnumeration.MARRIED);
    	userProfileBO.setMaritalStatus(maritalStatus);
    	userProfileBO.setSsn("xxx-xx-xxxx");
    	userProfileBO.setMhvId("MTMM-87-1563");
    	birthDate = new Date("01/01/2001");
    	userProfileBO.setBirthDate(birthDate);
    	contactInfoContactMethod = ContactMethodEnumeration.getEnum(ContactMethodEnumeration.EMAIL);
    	userProfileBO.setContactInfoContactMethod(contactInfoContactMethod);
    	patientBO.setUserProfile(userProfileBO);
    	patients.add(patientBO);
    	return patients;
    }
	public PatientCollectionServiceResponse getCorrelatedPatients() {
		PatientCollectionServiceResponse response = new PatientCollectionServiceResponse();
		MviProperties properties = MviProperties.getInstance();
		//int numberToRun = properties.getMviAuthBatchNumberToRun();
		//TODO - Replace with the one above.
		int numberToRun = 0;
        List<InPersonAuthentication> bos = InPersonAuthenticationBO.getMVIPendingAuthPatients();
        if(bos != null && !bos.isEmpty()) {
	        Collection<InPersonAuthentication> ipas = InPersonAuthenticationAssembler.getInPersonAuthenticationCollection(bos);
	        int count = 0;
	        for (InPersonAuthentication ipa : ipas) {
	        	if(count < numberToRun) {
		        	Patient patient = ipa.getPatient();
		        	response.addItem(patient);
		        	count++;
	        	}
	        }
        }
		return response;
	}
	
	public PatientCollectionServiceResponse getMVIPendingPatients(Boolean isAuth) {
		PatientCollectionServiceResponse response = new PatientCollectionServiceResponse();
		MviProperties properties = MviProperties.getInstance();
		//int numberToRun = properties.getMviAuthBatchNumberToRun();
		//TODO - Replace with the one above.
		int numberToRun = 0;
		if(isAuth) {
	        List<InPersonAuthentication> bos = InPersonAuthenticationBO.getMVIPendingAuthPatients();
	        if(bos != null && !bos.isEmpty()) {
		        Collection<InPersonAuthentication> ipas = InPersonAuthenticationAssembler.getInPersonAuthenticationCollection(bos);
		        int count = 0;
		        for (InPersonAuthentication ipa : ipas) {
		        	if(count < numberToRun) {
			        	Patient patient = ipa.getPatient();
			        	response.addItem(patient);
			        	count++;
		        	}
		        }
	        }
		} else {
			//numberToRun = properties.getMviUnauthBatchNumberToRun();
			//TODO - Replace with the one above.
			numberToRun = 0;
			List<InPersonAuthentication> bos = InPersonAuthenticationBO.getMVIPendingUnauthPatients();
	        if(bos != null && !bos.isEmpty()) {
		        Collection<InPersonAuthentication> ipas = InPersonAuthenticationAssembler.getInPersonAuthenticationCollection(bos);
		        int count = 0;
		        for (InPersonAuthentication ipa : ipas) {
		        	if(count < numberToRun) {
			        	Patient patient = ipa.getPatient();
			        	response.addItem(patient);
		        	}
		        }
	        }
		}
		return response;
	}

	public StringServiceResponse getMVIAutBatchAdmin() {
		MviProperties properties = MviProperties.getInstance();
		StringServiceResponse response = new StringServiceResponse();
		//response.setString(properties.getMviAuthBatchAdmin());
		response.setString("");
		return response;
	}

	public StringServiceResponse getMviAuthBatchIntervalHours() {
		MviProperties properties = MviProperties.getInstance();
		StringServiceResponse response = new StringServiceResponse();
		//response.setString(properties.getMviAuthBatchIntervalHours().toString());
		response.setString("");
		return response;
	}

	public PatientServiceResponse setPendingAuthAndDeletePatientSynchronization(
			Patient patient) {
        InPersonAuthenticationServiceResponse response = 
        	ServiceFactory.createInPersonAuthenticationService().getIPAPatientById(
        		patient.getId());
        InPersonAuthentication ipa = response.getInPersonAuthentication();
        ipa.setMviAuthenticationStatus("PENDING_AUTH");
        AuthenticationStatus status = InPersonAuthenticationStatusUtils.AUTHENTICATED;
        ipa = saveIPA(ipa, status);
        patient.setPatientSynchronizations(null);
        ServiceFactory.createEntityMaintenanceService().save(patient);
		PatientServiceResponse pResponse = new PatientServiceResponse();
		return pResponse;
	}

}