/**
 *
 */
package gov.va.med.mhv.usermgmt.util;

import java.io.IOException;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.config.ConfigurationManager;

import gov.va.med.mhv.core.util.DescriptionBuilder;
import gov.va.med.mhv.core.util.Precondition;
import gov.va.med.mhv.core.util.PropertiesUtil;

public class MHVAdminPortalProperties {

    private static final Log LOG = LogFactory.getLog(MHVAdminPortalProperties.
        class);

    private static final String DEFAULT_RESOURCE_NAME =
        "mhvadminportal.properties";

    private static final String RESOURCE_NAME = "/" + DEFAULT_RESOURCE_NAME;

    private static final String USE_PIV_AUTHENTICATION = "mhvadminportal.usePIVAuthentication";

    private static final MHVAdminPortalProperties instance =
        new MHVAdminPortalProperties();

    /**
     * Returns a MHVAdminPortalProperties instance.
     * @return a MHVAdminPortalProperties instance.
     */
    public static final MHVAdminPortalProperties getInstance() {
        return instance;
    }

    private final boolean usePIVAuthentication;

    private MHVAdminPortalProperties() {
        java.util.Properties properties = null;
        final boolean defaultUsePIVAuthentication = false;
        try {
            properties = ConfigurationManager.getConfiguration(
                RESOURCE_NAME);
        } catch (IllegalStateException e) {
            LOG.warn("Using defaults, because failed to load " +
                "properties from '" + RESOURCE_NAME + "'.");
        }
        LOG.debug("properties: " + DescriptionBuilder.describe(properties));

        usePIVAuthentication = getValue(USE_PIV_AUTHENTICATION, properties, defaultUsePIVAuthentication);

    }
    public boolean getUsePIVAuthentication() {
        return usePIVAuthentication;
    }

    private boolean getValue(String name, java.util.Properties properties,
        boolean defaultValue)
    {
        assert !StringUtils.isBlank(name);
        return PropertiesUtil.getBooleanValue(RESOURCE_NAME, properties, name, defaultValue);
    }

}