package gov.va.med.mhv.usermgmt.util.bbdownload;

import java.sql.Clob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.tigris.atlas.config.ConfigurationManager;

public class DownloadHelper {

	private static boolean closeConnection = true;

	private static final String XHTML_SQL = "select file_contents from mhv_download_data where mhv_download_report_id = ? and file_type = 'XHTML'";
	private static final String TXT_SQL = "select file_contents from mhv_download_data where mhv_download_report_id = ? and file_type = 'TXT'";
	private static final String CLOB_FIELD_NAME = "file_contents";

	private static final Properties PROPS;
	private static final String JNDI_NAME = "bb.mhvdatasource_jndiname";
	
	private static String JNDI;
	private static String DEFAULT_JNDI = "mhvDataSource";
		
	static {
		PROPS = ConfigurationManager.getConfiguration("/downloadData.properties");
		JNDI = PROPS.getProperty(JNDI_NAME, DEFAULT_JNDI);
	}
	
	public static DataSource getMHVDatasource() throws NamingException {
		Context ctx = new InitialContext();
		return (DataSource)ctx.lookup(JNDI);
	}
	
	public static String retrieveReport(DataSource ds, String downloadFormat, long uid) throws SQLException {
		Connection con = null;
		ResultSet rs = null;
		PreparedStatement ps = null;
		String result = null;

		String sql = null;
		if( downloadFormat.equalsIgnoreCase("pdfFormat")) {
			sql = XHTML_SQL;
		} else {
			sql = TXT_SQL;
		}
		
		// perform query
		try {
			con = ds.getConnection();
			con.setAutoCommit(false);
			ps = con.prepareStatement(sql);
			ps.setLong(1, uid);
			rs = ps.executeQuery();
			//Fetch first record in result set.
			while (rs.next()) {
				Clob valueClob = rs.getClob(CLOB_FIELD_NAME);
				if (valueClob != null && valueClob.length() > 0) {
					result = valueClob
							.getSubString(1, (int) valueClob.length());
				}
				break;
			}
		} finally {
			cleanup(rs, ps);
			closeConnection(con);
		}
		
		//System.out.println("PAYLOAD SIZE="+result.length());

		return result;
	}

	protected static void cleanup(ResultSet rs, PreparedStatement ps) {
		if (rs != null) {
			try {
				rs.close();
			} catch (SQLException e) {
				//Ignore
			}
		}
		cleanup(ps);
	}

	protected static void cleanup(Statement ps) {
		if (ps != null) {
			try {
				ps.close();
			} catch (SQLException e) {
				//Ignore
			}
		}
	}

	protected static void closeConnection(Connection con) {
		try {
			if (closeConnection && con != null) {
				con.close();
				con = null;
			}
		} catch (SQLException e) {
			//Ignore
		}
	}
}
