package gov.va.med.mhv.usermgmt.decorator;

import gov.va.med.mhv.usermgmt.enumeration.AuthenticationStatus;
import gov.va.med.mhv.usermgmt.form.IPACriteriaForm;
import gov.va.med.mhv.usermgmt.transfer.PatientInformation;

import java.text.SimpleDateFormat;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.displaytag.decorator.TableDecorator;
import org.tigris.atlas.util.CalendarUtils;


public class IPASearchResultsDecorator extends TableDecorator {
	private static final Log LOG = LogFactory.getLog(IPASearchResultsDecorator.class);
	//MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - Correlation Statuses 
	private static final int UNCORRELATED_VALUE = 0;
	private static final int PENDINGCORRELATION_VALUE = 1;
	private static final int CORRELATED_VALUE = 2;
	private static final int PENDINGUNCORRELATION_VALUE = 3;
	private static final int FAILEDCORRELATION_VALUE = 4;
	private static final int FAILEDUNCORRELATION_VALUE = 5;
	private static final int CORRELATIONINVALID_VALUE = 6;
	private static final int MVIDATAMATCH_VALUE = 7;

	//MHV_CodeCR1386 - Added Gender to display
	public String getGender() {
		PatientInformation info = getPatientInfo();
		
		if(info.getGender() == null) {
			return "";
		} else if(info.getGender().equals("Male")) {
			return "M";
		} else if (info.getGender().equals("Female")) {
			return "F";
		}
		return "";
	}
	
	public String getName() {
		PatientInformation info = getPatientInfo();
		StringBuffer buff = new StringBuffer(100);
		buff.append(info.getLastName());
		buff.append(", ");
		buff.append(info.getFirstName());

		if (LOG.isDebugEnabled()) {
			LOG.debug("************** In IPASearchResultsDecorator.getName() info.getMiddleName(): " + info.getMiddleName());
		}
		
		//MHV_CodeCR1386 - adding middle name if exists in the search result
		if(info.getMiddleName() != null && info.getMiddleName().length() > 0) {
			buff.append(" ");
			buff.append(info.getMiddleName());
		}
		return buff.toString();
	}
	
	public String getLastFourSsn() {
		PatientInformation info = getPatientInfo();
		if(!StringUtils.isBlank(info.getSsn())){
			String ssn = info.getSsn();
			if(ssn.length() > 4){
				return ssn.substring(ssn.length()-4, ssn.length());
			}
		}
		return "N/A";
	}
	
	public String getDateOfBirth() {
		PatientInformation info = getPatientInfo();
		return CalendarUtils.getYear(info.getBirthDate());
	}
	
	public String getAuthenticatingFacilityInfoName() {
		PatientInformation info = getPatientInfo();
		String name = info.getAuthenticatingFacilityInfoName();
		return StringUtils.isBlank(name) ? "N/A" : name;
	}
	
	public String getAuthenticationStatus() {
		PatientInformation info = getPatientInfo();
		StringBuffer buff = new StringBuffer(50);
		AuthenticationStatus status = info.getAuthenticationStatus() == null ? AuthenticationStatus.getEnum(AuthenticationStatus.UNAUTHENTICATED) : info.getAuthenticationStatus();
		buff.append(IPACriteriaForm.getPatientInfoAuthenticationStatusName(status));
		if (status.equals(AuthenticationStatus.getEnum(AuthenticationStatus.AUTHENTICATED))) {
			buff.append(" on ");
			buff.append(CalendarUtils.getDateAsMDY(info.getAuthenticationDate()));
		}
		return buff.toString();
	}
	
	//MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - Correlation Status to display in Admin Portal IPA Search Results page. 
	public String getCorrelationStatus() {
		PatientInformation info = getPatientInfo();
		Long correlationStatusCode = info.getCorrelationStatus();
		if(correlationStatusCode == UNCORRELATED_VALUE) {
			return "Uncorrelated";
		} else if(correlationStatusCode == PENDINGCORRELATION_VALUE) {
			return "Pending Correlation";
		} else if(correlationStatusCode == CORRELATED_VALUE) {
			return "Correlated";
		} else if(correlationStatusCode == PENDINGUNCORRELATION_VALUE) {
			return "Pending Uncorrelation";
		} else if(correlationStatusCode == FAILEDCORRELATION_VALUE) {
			return "Failed Correlation";
		} else if(correlationStatusCode == FAILEDUNCORRELATION_VALUE) {
			return "Failed Uncorrelation";
		} else if(correlationStatusCode == CORRELATIONINVALID_VALUE) {
			return "Correlation Invalid";
		} else if(correlationStatusCode == MVIDATAMATCH_VALUE) {
			// CR 1838
			return "Uncorrelated";
		}
		return "";
	}
	
	private PatientInformation getPatientInfo() {
		return (PatientInformation) getCurrentRowObject();
	}
}
