package gov.va.med.mhv.usermgmt.ice;

import java.util.List;

import org.apache.beehive.netui.pageflow.Forward;
import org.apache.beehive.netui.pageflow.annotations.Jpf;
import org.apache.commons.lang.StringUtils;
import org.tigris.atlas.beehive.EntityMaintenancePageFlowController;
import org.tigris.atlas.messages.MessageFactory;
import org.tigris.atlas.messages.MessageUtils;
import org.tigris.atlas.messages.Messages;
import org.tigris.atlas.service.EntityServiceResponse;
import org.tigris.atlas.web.struts.InputForm;

import gov.va.med.mhv.usermgmt.form.EmergencyContactForm;
import gov.va.med.mhv.usermgmt.portlet.UserProfileLabelManager;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.transfer.EmergencyContact;
import gov.va.med.mhv.usermgmt.transfer.TransferObjectFactory;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;
import gov.va.med.mhv.usermgmt.util.EmergencyContactSorter;
import gov.va.med.mhv.usermgmt.util.MessageKeys;

/**
 * @jpf:controller struts-merge="/WEB-INF/struts-override.xml"
 */
@Jpf.Controller(strutsMerge = "/WEB-INF/struts-override.xml")
public class EmergencyContactController extends
    EntityMaintenancePageFlowController
{

    private EmergencyContact currentEmergencyContact = null;

    // Uncomment this declaration to access Global.app.
    // 
    // protected global.Global globalApp;
    // 

    // For an example of page flow exception handling see the example "catch"
    // and "exception-handler"
    // annotations in {project}/WEB-INF/src/global/Global.app

    public List getStateLabels() {
        return UserProfileLabelManager.getStateLabels();
    }

    public List getCountryLabels() {
        return UserProfileLabelManager.getCountryLabels();
    }

    public List getContactMethodLabels() {
        return UserProfileLabelManager.getContactMethodLabels();
    }

    public List getTitleLabels() {
        return UserProfileLabelManager.getTitleLabels();
    }

    public List getSuffixLabels() {
        return UserProfileLabelManager.getSuffixLabels();
    }

    /**
     * This method represents the point of entry into the pageflow
     * @jpf:action
     * @jpf:forward name="success" path="emergencyContacts.jsp"
     */
    @Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "emergencyContacts.jsp") })
    public Forward begin()
    {
        Forward f = super.begin();

        getSession().setAttribute("EmergencyContact",
            getCurrentUserProfile().getEmergencyContacts());

        return f;
    }

    protected UserProfile getCurrentUserProfile() {
        return (UserProfile) getCurrentEntity();
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="editEmergencyContact.jsp"
     */
    @Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "editEmergencyContact.jsp") })
    protected Forward editEmergencyContact(EmergencyContactForm form)
    {
        Long contactId = new Long(Long.parseLong(getRequest()
            .getParameter("id")));
        for (Object ecs : getCurrentUserProfile().getEmergencyContacts()) {
            EmergencyContact ec = (EmergencyContact) ecs;
            if (contactId.equals(ec.getId())) {
                currentEmergencyContact = ec;
                form.copyFrom(currentEmergencyContact);
                return new Forward("success");
            }
        }

        return new Forward("success");

    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="editEmergencyContact.jsp"
     */
    @Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "editEmergencyContact.jsp") })
    protected Forward addEmergencyContact(EmergencyContactForm form)
    {
        currentEmergencyContact = TransferObjectFactory
            .createEmergencyContact();
        form.copyFrom(currentEmergencyContact);

        return new Forward("success");
    }

    /**
     * @jpf:action
     * @jpf:validation-error-forward name="failure" return-to="currentPage"
     * @jpf:forward name="success" path="begin.do"
     */
    @Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "begin.do") }, validationErrorForward = @Jpf.Forward(name = "failure", navigateTo = Jpf.NavigateTo.currentPage))
    protected Forward saveEmergencyContact(EmergencyContactForm form)
    {
        EmergencyContact updatedPrimary = null;
        boolean isNew = (currentEmergencyContact.getUserProfile() == null);
        if (isNew) {
            // First attempt to save a new contact,
            form.copyTo(currentEmergencyContact);
            getCurrentUserProfile().addEmergencyContact(
                currentEmergencyContact);
        } else {
            for (Object o : getCurrentUserProfile().getEmergencyContacts()) {
                EmergencyContact ec = (EmergencyContact) o;
                if (isNew(ec) || isModified(ec)) {
                    currentEmergencyContact = ec;
                    form.copyTo(currentEmergencyContact);
                    break;
                }
            }
        }
        updatedPrimary = updatePrimaryIfChanged(currentEmergencyContact);

        try {
            Forward f = super.saveAction(form);
            if (f.getName().equals("success")) {
                handleMessages(addSaveContactSuccessfulMessages(null, isNew,
                    updatedPrimary));
                updatedPrimary = null; // Prevent reset in 'finally'
                currentEmergencyContact = null;
            }
            return f;
        } finally {
            if (updatedPrimary != null) {
                // Undo change in instance in session
                // Otherwise, the primary may be changed adversely
                updatedPrimary.setIsPrimary(true);
            }
        }
    }

    private boolean isNew(EmergencyContact contact) {
        // Second or later attempt to save a NEW contact,
        // because validation failed, the ID is yet unknown
        return (currentEmergencyContact != null) && (contact != null)
            && (currentEmergencyContact.getId() == null)
            && (contact.getId() == null);
    }

    private boolean isModified(EmergencyContact contact) {
        return (currentEmergencyContact != null) && (contact != null)
            && (currentEmergencyContact.getId() != null)
            && contact.getId().equals(currentEmergencyContact.getId());
    }

    /**
     * Change the primary contact if the given contact is primary and not the
     * current primary contact
     * @param contact
     */
    protected EmergencyContact updatePrimaryIfChanged(EmergencyContact contact)
    {
        if ((contact == null) || !contact.getIsPrimary()) {
            // No contact given or not a primary: not a change
            return null;
        }
        EmergencyContact primary = findPrimaryContactDifferentFrom(contact);
        if (primary == null) {
            // No other primary could be found: not a change
            return null;
        }
        // Mark the other primary as non-primary
        primary.setIsPrimary(false);
        return primary;
    }

    protected EmergencyContact findPrimaryContact() {
        return findPrimaryContactDifferentFrom(null);
    }

    /**
     * Finds the primary contact in the current user's emergency contacts.
     * @param contact If not null, the primary contact returned must be
     *        another contact than the one given
     * @return The primary contact, that differs form the the given contact
     *         (if one is given)
     */
    private EmergencyContact findPrimaryContactDifferentFrom(
        EmergencyContact contact)
    {
        for (Object o : getCurrentUserProfile().getEmergencyContacts()) {
            EmergencyContact ec = (EmergencyContact) o;
            if (!isSameInstance(contact, ec) && ec.getIsPrimary()) {
                return ec;
            }
        }
        return null;
    }

    private boolean isSameInstance(EmergencyContact contact,
        EmergencyContact otherContact)
    {
        return (contact != null) && (otherContact != null)
            && (contact.hashCode() == otherContact.hashCode());
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="begin.do"
     */
    @Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "begin.do") })
    protected Forward deleteEmergencyContact(EmergencyContactForm form)
    {
        String sId = getRequest().getParameter("id");
        Long contactId = new Long(Long.parseLong(sId));
        EmergencyContact ecToDelete = null;
        for (Object ecs : getCurrentUserProfile().getEmergencyContacts()) {
            EmergencyContact ec = (EmergencyContact) ecs;
            if (contactId.equals(ec.getId())) {
                ecToDelete = ec;
                break;
            }
        }

        if (ecToDelete != null) {
            getCurrentUserProfile().removeEmergencyContact(ecToDelete);
            currentEmergencyContact = null;
        }

        Forward f = super.saveAction(form);
        if (f.getName().equals("success")) {
            if (ecToDelete != null) {
                handleMessages(addRemoveContactSuccessfulMessages(null, 
                    ecToDelete));
            }
        }
        return f;
    }

    protected EntityServiceResponse lookupEntity() {
        return ServiceDelegateFactory.createUserProfileServiceDelegate()
            .getProfileForUser(getCurrentUserName());
    }

    protected EntityServiceResponse saveEntity() {
        return ServiceDelegateFactory.createEntityMaintenanceServiceDelegate()
            .save(getCurrentUserProfile());
    }

    protected void setupNewEntity() {
    }

    protected EntityServiceResponse deleteEntity() {
        return null;
    }

    protected Long getCurrentUserId() {
        return null;
    }

    protected String getEntityName() {
        return UserProfile.ENTITY;
    }

    protected void populateEntity(InputForm arg0) {
    }

    protected void populateForm(InputForm arg0) {
    }

    protected Class getFormClass() {
        return EmergencyContactForm.class;
    }

    protected Class getTransferObjectClass() {
        return UserProfile.class;
    }

    private Messages addSaveContactSuccessfulMessages(Messages messages, 
        boolean isNew, EmergencyContact priorPrimaryContact) 
   {
        if (priorPrimaryContact != null) {
           messages = addInfoMessage(messages, 
               MessageKeys.PRIMARY_CONTACT_CHANGED, null, 
               new Object[] {
                   getContactFullName(priorPrimaryContact),
                   getContactFullName(currentEmergencyContact) 
               });
        }
        messages = addInfoMessage(messages, ((isNew) 
            ? MessageKeys.CONTACT_ADDED_SUCCESSFULLY
            : MessageKeys.CONTACT_CHANGED_SUCCESSFULLY), 
        null, new Object[] { getContactFullName(currentEmergencyContact) });
        return messages;
    }

    private Messages addRemoveContactSuccessfulMessages(Messages messages, 
        EmergencyContact removedContact) 
    {
        if ((removedContact != null) && removedContact.getIsPrimary()) {
            messages = addInfoMessage(messages, 
                MessageKeys.PRIMARY_CONTACT_REMOVED, null, 
                new Object[] { getContactFullName(removedContact) });
        }
        messages = addInfoMessage(messages, 
            MessageKeys.CONTACT_REMOVED_SUCCESSFULLY, null, 
            new Object[] { getContactFullName(removedContact) });
        return messages;
    }

    private String getContactFullName(EmergencyContact contact) {
        assert (contact != null) : "Must provide a contact";
        return StringUtils.join(new String[] { 
                StringUtils.trim(contact.getFirstName()),
                StringUtils.trim(contact.getLastName())
            }, " ");
    }

    private Messages addInfoMessage(Messages messages, String key,
        String[] properties, Object[] inserts) 
    {
        if (messages == null) {
            messages = MessageFactory.createMessages();
        }
        messages.addMessage(MessageUtils.createInformationalMessage(key,
            properties, inserts));
        return messages;
    }

    private void handleMessages(Messages messages) {
        getMessagesHandler().handleMessages(messages,
            EmergencyContactForm.class, getRequest());

    }

    public String getTableSorter() { 
        return EmergencyContactSorter.class.getName();
    }
}
