/**
 *
 */
package gov.va.med.mhv.usermgmt.portal.backing;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Enumeration;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.bea.netuix.servlets.controls.content.backing.AbstractJspBacking;
import com.bea.netuix.servlets.manager.AppContext;
import com.bea.p13n.security.Authentication;
import com.bea.portlet.GenericURL;
import com.bea.portlet.PostbackURL;

import gov.va.med.mhv.usermgmt.util.MHVAdminPortalProperties;
import gov.va.med.mhv.usermgmt.service.UserProfileServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.util.DirectorUtil;


/**
 * Supports backing for the admin portal.
 * This support includes:
 * <ul>
 * <li>Verifying user authentication</li>
 * </ul>
 * @author Rob Proper
 */
public class AdminDesktopNavigationBacking extends AbstractJspBacking {

	private static final Log LOG = LogFactory.getLog(
		AdminDesktopNavigationBacking.class);

	/**
     * Handles an pre rendering of any of the admin desktop's pages.
     */
    public boolean preRender(HttpServletRequest request,
    	HttpServletResponse response)
    {
		System.out.println(">>>>>>>> 2000 entering preRender in AdminDesktopNavigationBacking.");

		boolean usePIVAuthentication = MHVAdminPortalProperties.getInstance().getUsePIVAuthentication();

    	boolean hasValidUser = true;

    	if (!usePIVAuthentication) {
			if (request.getUserPrincipal() != null) {
				String userName = request.getUserPrincipal().getName();
				System.out.println(">>>>>>>> 2002 User Name:"+userName);
				if (hasUserProfile(userName)) {
					System.out.println(">>>>>>>> 2003 User is NOT an employee.");
					LOG.warn("Non-employee user '" + userName + "' is trying "
						+ "to access the admin portal. The user is logged out.");
					System.out.println(">>>>>>>> 2003A Non-employee user '" + userName + "' is trying "
						+ "to access the admin portal. The user is logged out.");
					logout(request);
					hasValidUser = false;
				}
			}
		}

    	//if (!hasValidUser) {
     	//   	redirect(response, DirectorUtil.getDefaultAdminPortalPage(request, response));
    	//}

		//New for 10.3.2 fix
		if(LOG.isDebugEnabled())
			LOG.debug("In pre-Render.  EmployeeLoginOrLogout attribute value="+request.getSession().getAttribute("EmployeeLoginOrLogout"));

			System.out.println(">>>>>>>> 2004 In pre-Render.  EmployeeLoginOrLogout attribute value="+request.getSession().getAttribute("EmployeeLoginOrLogout"));

		if( request.getSession().getAttribute("EmployeeLoginOrLogout") != null) {
			System.out.println(">>>>>>>> 2005 In pre-Render. EmployeeLoginOrLogout is not null.");
			request.getSession().setAttribute("EmployeeLoginOrLogout",null);
			System.out.println(">>>>>>>> 2006 before redirect is called.");

			if (usePIVAuthentication) {

				Enumeration headerNames = request.getHeaderNames();
				while(headerNames.hasMoreElements()) {
				  String headerName = (String)headerNames.nextElement();
				  System.out.println("" + headerName +" :: " + request.getHeader(headerName));
				}

				String lastname = request.getHeader("SM_LASTNAME");
				String networkid = request.getHeader("SM_NETWORKID");
				String firstname = request.getHeader("SM_FIRSTNAME");

				request.getSession().setAttribute("SM_LASTNAME",lastname);
				request.getSession().setAttribute("SM_NETWORKID",networkid);
				request.getSession().setAttribute("SM_FIRSTNAME",firstname);

				System.out.println(">>>>>>>> 2010 lastname::"+lastname);
				System.out.println(">>>>>>>> 2010 networkid::"+networkid);
				System.out.println(">>>>>>>> 2010 firstname::"+firstname);
			}

			redirect(response, getDefaultAdminHomePage(request, response));
		}

    	return hasValidUser;
    }

    private String getDefaultAdminHomePage(
			HttpServletRequest request, HttpServletResponse response)
	{
			PostbackURL url = PostbackURL.createPostbackURL(request, response);
			url.removeParameter(GenericURL.PAGE_LABEL_PARAM);
			url.removeParameter("_nfpb");
			url.setPath("/" + AppContext.getAppContext(request).getWebAppName()
				+ "/admin/mhvAdmin/mhvAdminDesktop/home");

			System.out.println(">>>>>>>> 2007 In pre-Render. URL used for redirect:" + url.toString());

			return url.toString();
	}

    private String getSSOiAdminLoginPage(
			HttpServletRequest request, HttpServletResponse response)
	{
			PostbackURL url = PostbackURL.createPostbackURL(request, response);
			url.removeParameter(GenericURL.PAGE_LABEL_PARAM);
			url.removeParameter("_nfpb");
			url.setPath("/" + AppContext.getAppContext(request).getWebAppName()
				+ "/admin/mhvAdmin/mhvAdminDesktop/loginhome");

			System.out.println(">>>>>>>> 2001 In pre-Render. URL used for redirect:" + url.toString());

			return url.toString();
	}

	private boolean hasUserProfile(String userName) {
		UserProfileServiceResponse response = ServiceDelegateFactory.
			createUserProfileServiceDelegate().getProfileForUser(userName);
		return (response != null) && (response.getUserProfile() != null);
	}

	private void logout(HttpServletRequest request) {
		assert request != null;
		Authentication.logout(request);
		request.getSession().invalidate();
	}

	private void redirect(HttpServletResponse response, String url) {
		assert response != null;
		try {
			response.sendRedirect(url);
		} catch (Exception ex) {
			throw new RuntimeException("Unable to forward to URL " + url, ex);
		}
	}

}
