package gov.va.med.mhv.usermgmt.portlet;

import gov.va.med.mhv.core.util.Precondition;
import gov.va.med.mhv.usermgmt.form.UserProfileForm;
import gov.va.med.mhv.usermgmt.service.PatientServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.transfer.Patient;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;
import gov.va.med.mhv.usermgmt.util.PHRAccessControl;
import gov.va.med.mhv.usermgmt.util.PatientCorrelationStatusUtils;

import java.util.Iterator;

import org.apache.beehive.netui.pageflow.Forward;
import org.apache.beehive.netui.pageflow.annotations.Jpf;
import org.apache.commons.lang.BooleanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.messages.CoreMessages;
import org.tigris.atlas.messages.Message;
import org.tigris.atlas.messages.MessageUtils;
import org.tigris.atlas.messages.Messages;
import org.tigris.atlas.service.BooleanServiceResponse;
import org.tigris.atlas.service.EntityServiceResponse;
import org.tigris.atlas.service.VoidServiceResponse;
import org.tigris.atlas.transfer.TransferObject;

import com.bea.portlet.PageURL;

/**
 * @jpf:controller struts-merge="/WEB-INF/struts-override.xml"
 */
@Jpf.Controller(strutsMerge = "/WEB-INF/struts-override.xml")
public class ManageUserProfileController extends UserProfilePageFlowController
{

    private static final Log LOG = LogFactory
        .getLog(ManageUserProfileController.class);
    
    private boolean hasReadOnlyMPIInfo = false; 

    /**
     * This method represents the point of entry into the pageflow
     * @jpf:action
     * @jpf:forward name="success" path="editAction.do"
     */
    @Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "editAction.do") })
    public Forward begin()
    {
        return new Forward("success");
    }

    protected EntityServiceResponse saveEntity() {
        EntityServiceResponse response = super.saveEntity();
        TransferObject entity = response.getEntity();
        //US7.2 The user shall be able click on the terms "Basic", "Advanced" and "Premium" on the 
        //"Relationship to VA" section and the system will display a separate window/tab with the definitions for the term
        //Adding the url for advaced, premium and basic
		String contextPath= getRequest().getContextPath();
		Message message = getMessage(response.getMessages(), "ssn.required.for.patient");
		if (message != null) {
			message.addInsert(contextPath + "/ShowBinary/BEA%20Repository/Registration/mhv_account_status.html");
			message.addInsert(contextPath + "/ShowBinary/BEA%20Repository/Registration/mhv_account_status.html");
			message.addInsert(contextPath + "/ShowBinary/BEA%20Repository/Registration/mhv_account_status.html");
		}
		
		message = getMessage(response.getMessages(), "ssn.required.for.veteran");
		if (message != null) {
			message.addInsert(contextPath + "/ShowBinary/BEA%20Repository/Registration/mhv_account_status.html");
			message.addInsert(contextPath + "/ShowBinary/BEA%20Repository/Registration/mhv_account_status.html");
			message.addInsert(contextPath + "/ShowBinary/BEA%20Repository/Registration/mhv_account_status.html");
		}
		
        if (!response.getMessages().hasErrorMessages() && entity != null
            && !entity.getAllMessages().hasErrorMessages())
        {
            response.getMessages()
                .addMessage(
                    MessageUtils.createInformationalMessage(
                        CoreMessages.ENTITY_MAINTENANCE_SAVE_SUCCEEDED, null,
                        null));
        }
        determineReadOnlyMPIInfo();
        return response;
    }

	private static Message getMessage(Messages messages, String key) {
		Message message = null;
		if (messages != null) {
			for (Iterator i = messages.getErrorMessages().iterator(); 
			     (message == null) && i.hasNext();)
			{
				message = (Message) i.next();
				if ((message != null) && !message.getKey().equals(key)) {
					message = null; // not found, keep looking
				}
			}
			for (Iterator i = messages.getInformationalMessages().iterator(); 
		     (message == null) && i.hasNext();)
			{
				message = (Message) i.next();
				if ((message != null) && !message.getKey().equals(key)) {
					message = null; // not found, keep looking
				}
			}
		}

		return message;
	}
	
    /**
     * @jpf:action
     * @jpf:forward name="success" path="editUserProfile.jsp"
     */
    @Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "editUserProfile.jsp") })
    protected Forward editAction(UserProfileForm form)
    {
        String newUserName = (String) getRequest().getAttribute("newUserName");
        Forward forward = null;

        if (newUserName == null) {
            forward = super.editAction(form);
        } else {
            forward = super.newAction(form);
            getCurrentUserProfile().setUserName(newUserName);
            form.copyFrom(getCurrentUserProfile());
        }
        
        determineReadOnlyMPIInfo();

        return forward;
    }

    /**
     * @jpf:action
     * @jpf:validation-error-forward name="failure" return-to="currentPage"
     * @jpf:forward name="success" path="success.jsp"
     * 
     * US7.12 - While middle Name is optional, if blank, validity check should confirm that user 
     * has a middle name or initial. 
     * This method is called when the user is confirmed that there is no middle name and proceed with updating the profile.
     */
    @Jpf.Action(forwards = { 
            @Jpf.Forward(name = "success", path = "editUserProfile.jsp") }, 
        validationErrorForward = @Jpf.Forward(name = "failure", 
            navigateTo = Jpf.NavigateTo.currentPage))
    protected Forward saveNoMiddleNameAction(UserProfileForm form) {
    	getRequest().setAttribute("CANCEL_INFO_MESSAGE",false);
    	getRequest().setAttribute("MIDDLE_NAME_INFO_MESSAGE",false);
    	
    	Forward f = super.saveAction(form);
        return f;
    }
    
    /**
     * @jpf:action
     * @jpf:validation-error-forward name="failure" return-to="currentPage"
     * @jpf:forward name="success" path="editUserProfile.jsp"
     */
    @Jpf.Action(forwards = { 
            @Jpf.Forward(name = "success", path = "editUserProfile.jsp") }, 
        validationErrorForward = @Jpf.Forward(name = "failure", 
            navigateTo = Jpf.NavigateTo.currentPage))
    public Forward saveAction(UserProfileForm form)
    {
        UserProfile userProfile = getCurrentUserProfile();
     
        //US7.12 - While middle Name is optional, if blank, validity check should confirm that user 
        //has a middle name or initial. 
        //Make sure that if it is not read only, and middle name check is required and if the user is patient
        //or veteran, if the middle name is blank, show the error message.
        if(!isReadOnlyMPIInfo()) {
	        Boolean noMiddleNameCheck = (Boolean)getSession().getAttribute("NO_MIDDLE_NAME_CHECK");
	        if(form != null && (noMiddleNameCheck==null ||!noMiddleNameCheck)){
	        	if(form.getUserProfileMiddleName()==null ||form.getUserProfileMiddleName().length()==0) {
	            	if(new Boolean(form.getUserProfileIsVeteran()) || new Boolean(form.getUserProfileIsPatient())) {
	            		getRequest().setAttribute("MIDDLE_NAME_INFO_MESSAGE",true);
	            		
	            		//Make sure that the it won't ask again about the middle name.
	            		getSession().setAttribute("NO_MIDDLE_NAME_CHECK",true);
	            		return new Forward("failure");
	            	}
	        	}
	        }
        }
        
    	Forward f = super.saveAction(form);
        return f;
    }

	//USR7.8 - Implement Cancel Button - Creating cancel action when the user clicks on the Cancel Button
    //Needs to come back to the current page and display the message on the top.
	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", navigateTo = Jpf.NavigateTo.currentPage) })
    protected Forward cancelAction(UserProfileForm form)
        throws Exception
    {
		getRequest().setAttribute("CANCEL_INFO_MESSAGE",true);
		VoidServiceResponse vsr = new VoidServiceResponse();
		//handleCancelMessages(vsr);
		return new Forward("success");
    }
	
	//E2USR7.9 - Implement Cancel Button - Creating canceling cancel action when the user clicks on the No Button
	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", navigateTo = Jpf.NavigateTo.currentPage) })
    protected Forward cancelCancelAction(UserProfileForm form)
        throws Exception
    {
		getRequest().removeAttribute("CANCEL_INFO_MESSAGE");
		getRequest().removeAttribute("MIDDLE_NAME_INFO_MESSAGE");
		//VoidServiceResponse vsr = new VoidServiceResponse();
		//handleCancelMessages(vsr);
		return new Forward("success");
    }
	
	//E2USR7.9 - Implement Cancel Button - Creating processing cancel action when the user clicks on the Yes Button
	@Jpf.Action(
	         forwards = {@Jpf.Forward(name = "success", path = "editAction.do")}
	     )
    protected Forward anonymCancelAction()
        throws Exception
    {
		getRequest().removeAttribute("CANCEL_INFO_MESSAGE");
		return new Forward("success");
    }
	
    public String getShowWalletCardURL() {
        final String url = PageURL.createPageURL(getRequest(), getResponse(),
            "walletcard").toString();
        if (LOG.isDebugEnabled()) {
            LOG.debug("Show Wallet Card URL = [" + url + "]");
        }
        return url;
    }

    protected Class getFormClass() {
        return UserProfileForm.class;
    }

    protected Class getTransferObjectClass() {
        return UserProfile.class;
    }

    @Override
    public boolean isReadOnlyMPIInfo() {
    	return hasReadOnlyMPIInfo;
    }
        
    protected void determineReadOnlyMPIInfo() {
        hasReadOnlyMPIInfo = super.isReadOnlyMPIInfo();
        UserProfile userProfile = getCurrentUserProfile();
        Precondition.assertNotNull("this.currentUserProfile", userProfile);
        if (BooleanUtils.isTrue(userProfile.getIsPatient()) || BooleanUtils.isTrue(userProfile.getIsVeteran())) {
            hasReadOnlyMPIInfo = true;
            PatientServiceResponse response = ServiceDelegateFactory.
                createPatientServiceDelegate().getPatientForUser(userProfile);
            if (hasErrorMessages(response)) {
                LOG.error("Problem finding patient for '" + userProfile.
                    getUserName() + "'. Assuming patient is correlated.");
                logMessages(response);
            } else if (response.getPatient() == null) {
                LOG.error("No patient found for '" + userProfile.getUserName() 
                    + "'. Assuming patient is correlated.");
                hasReadOnlyMPIInfo = false;
            } else {
                hasReadOnlyMPIInfo = !PatientCorrelationStatusUtils.
                        isUncorrelated(response.getPatient())
                    || (hasPhrAccess(response.getPatient())
                        && isPendingSynchronization(response.getPatient()));
            }
        }
    }
    
    private boolean isPendingSynchronization(Patient patient) {
        return (patient != null) 
            && (patient.getPatientSynchronizations() != null) 
            && (patient.getPatientSynchronizations().size() > 0);
    }
    private boolean hasPhrAccess(Patient patient) {
        BooleanServiceResponse response = PHRAccessControl.hasPhrAccess(
            patient);
        return !hasErrorMessages(response) && BooleanUtils.isTrue(response.
            getBoolean());
    }
}
