package gov.va.med.mhv.usermgmt.portlet; 

import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeMap;

import org.apache.commons.lang.StringUtils;
import org.tigris.atlas.service.EntityServiceResponse;
import org.tigris.atlas.web.struts.InputForm;

import gov.va.med.mhv.beehive.EntityMaintenanceBasePageFlowController;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActionTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityTypeEnumeration;
import gov.va.med.mhv.usermgmt.form.UserProfileForm;
import gov.va.med.mhv.usermgmt.service.PatientServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.transfer.Patient;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;
import gov.va.med.mhv.usermgmt.util.Auditor;
import gov.va.med.mhv.usermgmt.util.PatientCorrelationStatusUtils;

public abstract class UserProfilePageFlowController 
    extends EntityMaintenanceBasePageFlowController { 

    protected String verifySsn;

    public List getStateLabels() {
        return UserProfileLabelManager.getStateLabels();
    }
    
    public List getCountryLabels() {
        return UserProfileLabelManager.getCountryLabels();
    }
    
    
    public List getContactMethodLabels() {
        return UserProfileLabelManager.getContactMethodLabels();
    }
    
    
    public List getTitleLabels() {
        return UserProfileLabelManager.getTitleLabels();
    }
    
    
    public List getSuffixLabels() {
        return UserProfileLabelManager.getSuffixLabels();
    }
    
    
    public List getBloodTypeLabels() {
        return UserProfileLabelManager.getBloodTypeLabels();
    }
    
    
    public List getGenderLabels() {
        return UserProfileLabelManager.getGenderLabels();
    }
    
    public List getMaritalStatusLabels() {
        return UserProfileLabelManager.getMaritalStatusLabels();
    }
    
    
    public List getPasswordHintLabels() {
        return UserProfileLabelManager.getPasswordHintLabels();
    }
    
    
    protected UserProfile getCurrentUserProfile() {
        return (UserProfile) getCurrentEntity();
    }
    
    
    protected EntityServiceResponse lookupEntity()
    {
        return ServiceDelegateFactory.createUserProfileServiceDelegate().getProfileForUser( getCurrentUserName() );
    }

    protected EntityServiceResponse deleteEntity()
    {
        return null;
    }

    protected String getEntityName()
    {
        return UserProfile.ENTITY;
    }

    protected EntityServiceResponse saveEntity() {
    	UserProfile userProfile = getCurrentUserProfile();
    	EntityServiceResponse response = ServiceDelegateFactory.createUserProfileServiceDelegate().saveUserProfile( userProfile, verifySsn );
    	//Adding for Correlation
    	Patient patient = null;
    	if ( !response.getMessages().hasErrorMessages()) {
	        
	    	PatientServiceResponse patientResponse = ServiceDelegateFactory.createPatientServiceDelegate().getPatientForUser(userProfile);
	    	patient = patientResponse.getPatient();
	    	if(patient != null && patient.getCorrelationStatus().equals(PatientCorrelationStatusUtils.MVI_DATA_MATCH)) {
	    		ServiceDelegateFactory.createMviAsyncIntegrationServiceDelegate().addPersonForCorrelation(patient, userProfile, false);
	    	}
    	}
    	return response;
    }

    protected void populateEntity(InputForm form)  {
        UserProfile up = getCurrentUserProfile();
        UserProfileForm upf = (UserProfileForm) form;
        assembleSsn( upf, up ); 
        upf.copyTo(up );
        
        if( StringUtils.isBlank( up.getUserName() ) ) {
            up.setUserName( getRequest().getRemoteUser() );
        } 
    }
    
    protected void populateForm(InputForm form) { 
        UserProfile up = getCurrentUserProfile();
        UserProfileForm upf = (UserProfileForm) form;
        disassembleSsn( upf, up ); 
        upf.copyFrom( up );
    }

    protected Long getCurrentUserId()  {
        return null;
    }
    
    protected void setupNewEntity() {
    }
    
    protected void assembleSsn(UserProfileForm form, UserProfile up) {
    	String ssn = StringUtils.trimToEmpty( form.getSsn1() ) + "-" + StringUtils.trimToEmpty( form.getSsn2() ) + "-" + StringUtils.trimToEmpty( form.getSsn3() );
    	ssn = ssn.equals( "--" ) ? null : ssn;

    	verifySsn = StringUtils.trimToEmpty( form.getVerifySsn1() ) + "-" + StringUtils.trimToEmpty( form.getVerifySsn2() ) + "-" + StringUtils.trimToEmpty( form.getVerifySsn3() );
    	verifySsn = verifySsn.equals( "--" ) ? null : verifySsn;

    	//System.out.println("assembleSsn >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
    	//System.out.println("     ssn = " + ssn);
    	//System.out.println("     verifySsn = " + verifySsn);

    	//MHV_CodeCR1514 - SSN fix - These values need to be saved because when we reload the page, these values will go
    	//blank. In case the user misses to correct the values, these values need to be reloaded.
    	if((ssn!=null)&&(verifySsn!=null)) {
    		getSession().setAttribute("REGISTRATION_SSN", ssn);
    		getSession().setAttribute("REGISTRATION_VERIFY_SSN", verifySsn);
    	}
    	
    	//MHV_CodeCR1514 - SSN fix - These values need to be saved because when we reload the page, these values will go
    	//blank. In case the user misses to correct the values, these values need to be reloaded.
    	if((ssn==null) && (verifySsn==null)) {
    		if(getSession().getAttribute("REGISTRATION_SSN")!=null) {
    			ssn = (String)getSession().getAttribute("REGISTRATION_SSN");
    		}
    		
    		if(getSession().getAttribute("REGISTRATION_VERIFY_SSN")!=null) {
    			verifySsn = (String)getSession().getAttribute("REGISTRATION_VERIFY_SSN");
    		}
    	}
    	
    	if (ssn != null) {
    		// only copy the SSN to the userProfile if the user filled in a value
    		form.setUserProfileSsn( ssn );
    		//System.out.println("     A");
    	} else {
    		//System.out.println("     B");
    		// If ssn and verifySsn = null then the use has not tried to change his SSN.  We need to copy the SSN from the 
    		// userProfile into verifySsn and ssn so that the 2 values are equal and the DB doesn't get changed.
    		if (verifySsn == null) {
    			//System.out.println("     C");
    			String oldSsn = null;
    			if (up != null) {
    				//System.out.println("     D");
    				// up will be null if we are creating this user for the first time
    				oldSsn = up.getSsn();
    			}
    			ssn = oldSsn;
    			verifySsn = oldSsn;
    			form.setUserProfileSsn( ssn );
    		}
    	}
    	// reset the component SSN variables to blanks so that we NEVER send actual SSN data to the screen
    	form.setSsn1( "   " );
    	form.setSsn2( "  " );
    	form.setSsn3( "    " );
    	form.setVerifySsn1( "   " );
    	form.setVerifySsn2( "  " );
    	form.setVerifySsn3( "    " );
    	//System.out.println("     ssn = " + ssn);
    	//System.out.println("     verifySsn = " + verifySsn);
    }
    
    protected void disassembleSsn(UserProfileForm form, UserProfile up) {     
    	// set the component SSN variables to blanks so that we NEVER send actual SSN data to the screen
        form.setSsn1( "   " );
        form.setSsn2( "  " );
        form.setSsn3( "    " );
        form.setVerifySsn1( "   " );
        form.setVerifySsn2( "  " );
        form.setVerifySsn3( "    " );
//       if( up.getSsn() == null ) {
//            return;
//        }
//        
//        if( verifySsn == null ) {
//            verifySsn = up.getSsn();
//        }
//           
//        StringTokenizer st = new StringTokenizer( up.getSsn(), "-" );
//        if( st.countTokens() == 3 ) {
//            form.setSsn1( st.nextToken() );
//            form.setSsn2( st.nextToken() );
//            form.setSsn3( st.nextToken() );
//        }
//        
//        st = new StringTokenizer( verifySsn, "-" );
//        if( st.countTokens() == 3 ) {
//            form.setVerifySsn1( st.nextToken() );
//            form.setVerifySsn2( st.nextToken() );
//            form.setVerifySsn3( st.nextToken() );
//        }
    }
    
    public boolean isReadOnlyMPIInfo() {
        return false;
    }
    
    public String getReadOnlyMPIInfoStyleClass() {
        return (isReadOnlyMPIInfo()) ? "mhv-input-readonly" : "";
    }

} 
