package gov.va.med.mhv.usermgmt.portlet.aal.controller;

import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.beehive.netui.pageflow.Forward;
import org.apache.beehive.netui.pageflow.annotations.Jpf;
import org.apache.commons.lang.StringUtils;
import org.displaytag.properties.SortOrderEnum;
import org.tigris.atlas.service.EntityCollectionServiceResponse;
import org.tigris.atlas.util.CalendarUtils;
import gov.va.med.mhv.core.util.MHVCalendarUtils;

import gov.va.med.mhv.beehive.EntityReadOnlyListPageFlowController;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.form.AccountActivityCriteriaForm;
import gov.va.med.mhv.usermgmt.portlet.aal.decorator.ActivityDetailDecorator;
import gov.va.med.mhv.usermgmt.portlet.aal.decorator.UserProfileDecorator;
import gov.va.med.mhv.usermgmt.service.ActivityCollectionServiceResponse;
import gov.va.med.mhv.usermgmt.service.ActivityServiceResponse;
import gov.va.med.mhv.usermgmt.service.UserProfileServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.ActivityAuditServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.EntityMaintenanceServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.service.delegate.UserProfileServiceDelegate;
import gov.va.med.mhv.usermgmt.transfer.Activity;
import gov.va.med.mhv.usermgmt.transfer.ActivityPK;
import gov.va.med.mhv.usermgmt.transfer.TransferObjectFactory;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;


@Jpf.Controller(strutsMerge = "/WEB-INF/struts-override.xml")
public class AalController extends EntityReadOnlyListPageFlowController {
	private static final long serialVersionUID = 1205561710L;
	private static final String TABLE_SESSION_KEY = AalController.class.getName() + ".activities";

	private	java.util.Collection currentActivities = null;
	private UserProfileDecorator currentUserProfile = null;
	private	ActivityDetailDecorator	currentActivity = null;
	private Date twelveMonthsFromToday;
	private Date today;

	/******************************************************************
	 * ACTIONS
	 ******************************************************************/

	@Jpf.Action(forwards = { @Jpf.Forward(name = "default", path = "/gov/va/med/mhv/usermgmt/portlet/aal/accountActivityHistory.jsp")})
	public Forward begin(AccountActivityCriteriaForm form) {
		currentUserProfile = new UserProfileDecorator(getCurrentUser());
		currentActivities = new ArrayList();

		today = new Date();
		Calendar newTime = Calendar.getInstance();
		newTime.add(Calendar.MONTH, -12);
		twelveMonthsFromToday = newTime.getTime();

		form = getDefaultCriteria();
		currentActivities = getUserAccountActivities(form);
		return new Forward("default");
	}

	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "/gov/va/med/mhv/usermgmt/portlet/aal/accountActivityHistory.jsp")})
	public Forward queryForUserAccountActivities(AccountActivityCriteriaForm form){
		form.validate(this.getRequest());
		if(form.getMessages() != null && form.getMessages().hasErrorMessages()){
			this.getMessagesHandler().handleMessages(form.getMessages(),AccountActivityCriteriaForm.class,this.getRequest());
			return new Forward("success");
		}
		if(currentUserProfile != null){
			currentActivities = getUserAccountActivities(form);
		}
		return new Forward("success");
	}
	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "/gov/va/med/mhv/usermgmt/portlet/aal/accountActivityHistory.jsp")})
	public Forward backToSummary(){
		return new Forward("success");
	}

	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "/gov/va/med/mhv/usermgmt/portlet/aal/accountActivityHistoryPF.jsp")})
	public Forward printerFriendlyListView(){
		return new Forward("success");
	}

	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "/gov/va/med/mhv/usermgmt/portlet/aal/accountActivityDetailPF.jsp")})
	public Forward printerFriendlyDetailView(){
		return new Forward("success");
	}
    
    protected String getActionEntityIdParameter() {
        return getRequest().getParameter("activityId");
    }

	@Jpf.Action(forwards = {
            @Jpf.Forward(name = "success", path="../accountActivityDetail.jsp"),
            @Jpf.Forward(name = "failure", action = "begin")} )
	public Forward viewAccountActivityDetail(){
        Forward forward = null;
		String id = getActionEntityIdParameter(); 
        if (isValidActivityId(id)) {
    		Long activityId = Long.parseLong(id);
    		ActivityPK pk = TransferObjectFactory.createActivityPK();
    		pk.setActivityId(activityId);
    		EntityMaintenanceServiceDelegate delegate = ServiceDelegateFactory.
                createEntityMaintenanceServiceDelegate();
    		ActivityServiceResponse response = delegate.findByPrimaryKey(pk);
    		handleMessages(response);
    		currentActivity = new ActivityDetailDecorator(response.getActivity());
            forward = determineForward();
        } else {
            forward = determineForwardOnUnknownEntity("view");
        }
		return forward;
	}
    
    private boolean isValidActivityId(String activityId) {
        if ((currentActivities != null) && !StringUtils.isBlank((activityId))) {
            for (Object a : currentActivities) {
                Activity activity = (Activity) a;
                if (activityId.equals(activity.getActivityId().toString())) {
                    return true;
                }
            }
        }
        return false;
    }

	@Jpf.Action(forwards = { @Jpf.Forward(name = "default", path = "/gov/va/med/mhv/usermgmt/portlet/aal/accountActivityHistory.jsp")})
	public Forward reset(AccountActivityCriteriaForm form){
		form = new AccountActivityCriteriaForm();
		return new Forward("default");
	}

	@Jpf.Action(forwards = { @Jpf.Forward(name = "default", path = "/gov/va/med/mhv/usermgmt/portlet/aal/accountActivityHistory.jsp")})
	public Forward printerFriendlyAAHView() {
		return new Forward("default");
	}

	@Jpf.Action(forwards = {
			@Jpf.Forward(name = "success", path = "/gov/va/med/mhv/usermgmt/portlet/aal/accountActivityHistory.jsp")
	})
	@Override
	public Forward handlePagingAndSorting() {
		return super.handlePagingAndSorting();
	}

    @Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "/gov/va/med/mhv/usermgmt/portlet/aal/accountActivityHistory.jsp")})
    public Forward returnToViewAction(){
		return new Forward("success");
	}
	/******************************************************************
	 * END OF ACTIONS
	 ******************************************************************/
	/******************************************************************
	 * REQUIRED OVERRIDDEN METHODS
	 ******************************************************************/
	@Override
	protected final EntityCollectionServiceResponse lookupEntities() {
		AccountActivityCriteriaForm form = getDefaultCriteria();
		return ServiceDelegateFactory.createActivityAuditServiceDelegate().getActivitiesForSubject(currentUserProfile.getId(),form.getActivityPerformerType(),new Timestamp(form.getFromDate().getTime()), new Timestamp(form.getToDate().getTime()));
	}

	@Override
	protected final Class getTransferObjectClass() {
		return Activity.class;
	}

	@Override
	protected final String getEntityName() {
		return Activity.ENTITY;
	}

	@Override
	protected final Long getCurrentUserId() {
		if(currentUserProfile == null){
			currentUserProfile = new UserProfileDecorator(getCurrentUser());
		}
		return (currentUserProfile != null) ? currentUserProfile.getId() : null;
	}

	@Override
	protected Class getFormClass() {
		return AccountActivityCriteriaForm.class;
	}
	/**
     * Specifies the UID of the table.
     * This is used to initialize paging setting.
     * Must match the value of the uid attribute of display:table
     * <pre>
     *    &lt;diplay-table uid="${pageFlow.tableUID}" .... &gt;
     *</pre>
     * or
     * <pre>
     *    &lt;diplay-table uid="${pageFlow.paging.tableUID}" .... &gt;
     *</pre>
     * @return The UID of the table.
     */
    public String getTableUID(){
    	return "accountActivity";
    }

    @Override
	protected final String getTableSessionKey() {
		return TABLE_SESSION_KEY;
	}

    @Override
	public SortOrderEnum getDefaultSortOrder() {
		return SortOrderEnum.DESCENDING;
	}

	@Override
	public Integer getDefaultSortProperty() {
		return 1;
	}
	/******************************************************************
	 * END OF REQUIRED OVERRIDDEN METHODS
	 ******************************************************************/
	/******************************************************************
	 * DROP DOWN COLLECTION METHODS
	 ******************************************************************/
	public Map getDayOptions() {
    	return CalendarUtils.getDays();
    }

    public Map getMonthOptions() {
    	return CalendarUtils.getMonths();
    }

    public List getYearOptions() {
    	List years = MHVCalendarUtils.getYears();
    	if(!years.get(0).equals(MHVCalendarUtils.getLastYear()+"")){
    		Collections.reverse(years);
    	}
    	return years;
    }

    public List getPerformedByOptions() {
    	return AccountActivityCriteriaForm.getActivityPerformerTypeLabels();
    }
	/******************************************************************
	 * END DROP DOWN COLLECTION METHODS
	 ******************************************************************/
    /******************************************************************
	 * ACCESSOR AND UTILITY METHODS
	 ******************************************************************/
	private AccountActivityCriteriaForm getDefaultCriteria(){
		AccountActivityCriteriaForm form = new AccountActivityCriteriaForm();
		Calendar twelveMonthsAgo = Calendar.getInstance();
		twelveMonthsAgo.setTime(new Date());
		twelveMonthsAgo.add(Calendar.MONTH, -12);
		form.setFromDate(new Timestamp(twelveMonthsAgo.getTime().getTime()));
		form.setToDate(new Timestamp(new Date().getTime()));
		form.setActivityPerformerType(ActivityActorTypeEnumeration.EVERYONE);
		return form;
	}

	public String getTodayMonth(){
		return CalendarUtils.getMonth(today);
	}
	public String getTodayYear(){
		return CalendarUtils.getYear(today);
	}
	public String getTodayDay(){
		return CalendarUtils.getDay(today);
	}
	public String getTwelveMonthsFromTodayMonth(){
		return CalendarUtils.getMonth(twelveMonthsFromToday);
	}
	public String getTwelveMonthsFromTodayYear(){
		return CalendarUtils.getYear(twelveMonthsFromToday);
	}
	public String getTwelveMonthsFromTodayDay(){
		return CalendarUtils.getDay(twelveMonthsFromToday);
	}
	public String getTwelveMonthsFromTodayAsString(){
		return SimpleDateFormat.getDateInstance().format(twelveMonthsFromToday.getTime());
	}

    public UserProfile getCurrentUser(){
    	UserProfileServiceDelegate sd = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createUserProfileServiceDelegate();
    	UserProfileServiceResponse response = sd.getProfileForUser(getCurrentUserName());
    	handleMessages(response);
    	return response.getUserProfile();
    }

    public UserProfileDecorator getCurrentUserProfile(){
    	return this.currentUserProfile;
    }

    public Collection getCurrentActivities(){
    	return this.currentActivities;
    }

    public ActivityDetailDecorator getCurrentActivity(){
    	return this.currentActivity;
    }
    private Collection getUserAccountActivities(AccountActivityCriteriaForm form){
		ActivityAuditServiceDelegate delegate = ServiceDelegateFactory.createActivityAuditServiceDelegate();
		ActivityCollectionServiceResponse response = delegate.getActivitiesForSubject(currentUserProfile.getId(), form.getActivityPerformerType(), new Timestamp(form.getFromDate().getTime()), new Timestamp(form.getToDate().getTime()));
		handleMessages(response);
		return response.getActivitys();
	}
}