package gov.va.med.mhv.usermgmt.portlet.aal.decorator;

import java.io.Serializable;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;

import gov.va.med.mhv.usermgmt.enumeration.ActivityActionTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityErrorTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityTypeEnumeration;
import gov.va.med.mhv.usermgmt.form.AccountActivityCriteriaForm;
import gov.va.med.mhv.usermgmt.transfer.Activity;
import gov.va.med.mhv.usermgmt.transfer.ActivityError;

public class ActivityDetailDecorator implements Serializable {
	private static final long serialVersionUID = 5388615976997002379L;
		private Activity activity;
		//static private SimpleDateFormat df = new SimpleDateFormat("MM/dd/yyyy hh:mm");
		static private SimpleDateFormat df = new SimpleDateFormat("M/d/yyyy h:mm:ss a zzz");


		public ActivityDetailDecorator(Activity activity) {
			this.activity = activity;
		}

		public String getCompletionTime(){
			Timestamp completionTime = activity.getCompletionTime();
			String sCompletionTime = df.format(completionTime);
			return sCompletionTime;
		}

		public String getPerformedByType(){
			return AccountActivityCriteriaForm.getActivityPerformerTypeName(activity.getPerformerType());
		}

		public String getActivityType(){
			String label = activity.getActionType();
			label = label.replaceAll("_","/");
			return label;
		}

		public String getActivityDetails(){
			if(StringUtils.isBlank(activity.getDetailValue())){
				return "None";
			}
			return activity.getDetailValue();
		}

		public String getAction(){
			return getNameForEnumeration(ActivityActionTypeEnumeration.getEnumMap(), activity.getAction());
		}

		public String getStatus() {
			String result = "";
			if (activity.getStatus() == true)
				result = "Successful";
			else result = "Unsuccessful";
			return result;
		}

		public String getErrors(){
			Set errors = activity.getActivityErrors();
			if(errors == null || errors.size()==0){
				return "None";
			}
			StringBuffer sb = new StringBuffer(50);
			Iterator i = errors.iterator();
			while(i.hasNext()){
				ActivityError error = (ActivityError)i.next();
				sb.append("<I>"+getNameForEnumeration(ActivityErrorTypeEnumeration.getEnumMap(),error.getType())+"</I>");
				if(!StringUtils.isBlank(error.getDetail())){
					sb.append(" - ");
					sb.append(error.getDetail());
				}
				sb.append("\n");
			}
			return sb.toString();
		}

		private String getNameForEnumeration(Map enumerations, Object e){
			Set entries = enumerations.entrySet();
			for (Iterator i = entries.iterator(); i.hasNext();) {
				Map.Entry entry = (Map.Entry) i.next();
				if (entry.getValue().equals(e)) {
					return (String) entry.getKey();
				}
			}
			throw new IllegalArgumentException("The enumerated type " + e + " cannot be mapped to a name");
		}



}
