package gov.va.med.mhv.usermgmt.portlet.changepassword;

import org.apache.beehive.netui.pageflow.Forward;
import org.apache.beehive.netui.pageflow.annotations.Jpf;
import org.apache.commons.lang.StringUtils;
import org.tigris.atlas.messages.MessageUtils;
import org.tigris.atlas.service.ServiceResponse;
import org.tigris.atlas.service.VoidServiceResponse;

import gov.va.med.mhv.usermgmt.enumeration.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.form.ChangePasswordForm;
import gov.va.med.mhv.usermgmt.portlet.login.loginbase.LoginNavigateBaseController;
import gov.va.med.mhv.usermgmt.service.UserProfileServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.util.Auditor;

/**
 * @jpf:controller struts-merge="/WEB-INF/struts-override.xml"
 *  */
@Jpf.Controller(strutsMerge = "/WEB-INF/struts-override.xml")
public class ChangePasswordController extends LoginNavigateBaseController {

	private static final long serialVersionUID = 1L;
	
    /**
     * This method represents the point of entry into the pageflow
     * @jpf:action
     * @jpf:forward name="success" path="changePassword.jsp"
     */
    @Jpf.Action(forwards = { 
        	@Jpf.Forward(name = "success", path = "changePassword.jsp")
    	})
    protected Forward begin() {
    	if (isPasswordTemporary()) {
       		handleMessages(createTemporaryPasswordResponse());
    	}
        return new Forward("success");
    }
   
    /**
     * @jpf:action
     * @jpf:forward name="success" path="succuss.jsp"
     * @jpf:validation-error-forward name="failure" return-to="currentPage"
     */
    @Jpf.Action(forwards = { 
    		@Jpf.Forward(name = "success", path = "succuss.jsp")
       	}, 
        validationErrorForward = @Jpf.Forward(name = "failure", 
        	navigateTo = Jpf.NavigateTo.currentPage)
	)
    protected Forward changePassword(ChangePasswordForm form) {
    	
    	
    	Boolean success = Boolean.TRUE;
    	
        String currentUserName = getRequest().getRemoteUser();
        VoidServiceResponse response = ServiceDelegateFactory.
        	createUserServiceDelegate().changePassword(currentUserName,
        	form.getCurrentPassword(), form.getNewPassword(),
        	form.getConfirmNewPassword());
		Long currentId = ServiceDelegateFactory.createUserProfileServiceDelegate().
		getProfileForUser(currentUserName).getUserProfile().getId();
		
		if (response.getMessages().hasErrorMessages() ){
			success = Boolean.FALSE;
			Auditor.auditChangePasswordEvent(currentId, ActivityActorTypeEnumeration.SELF, success);	
		}
		else{
			Auditor.auditChangePasswordEvent(currentId, ActivityActorTypeEnumeration.SELF, Boolean.TRUE);
		}
        
        handleMessages(response);
        Forward forward = determineForward();
        if (StringUtils.equalsIgnoreCase("success", forward.getName())) {
        	addSuccessMessage(response);
        	handleMessages(response);
        	if (isPasswordTemporary()) {
        		setPasswordTemporary(false);
	            String redirectUrl = getRedirectUrl();
	        	if (redirectUrl != null) {
	        		forward.setPath(redirectUrl);
	        	}
        	}
        }
        // Keep this here just to be safe
        form.setCurrentPassword(null);
        form.setNewPassword(null);
        form.setConfirmNewPassword(null);
        return forward;
        
        //return new Forward("failure");
    }
    
    protected Class getFormClass(){
        return ChangePasswordForm.class;
    }

    private ServiceResponse createTemporaryPasswordResponse() {
    	// Create a response instance to be able to reuse the 
    	// handleMessages method, rather than duplicating the
    	// functionality here
   		ServiceResponse response = new VoidServiceResponse();
   		response.getMessages().addMessage(MessageUtils.
   			createInformationalMessage("current.password.temporary", null, 
   			null));
   		return response;
    }
    
    private void addSuccessMessage(ServiceResponse response) {
    	// Create a response instance to be able to reuse the 
    	// handleMessages method, rather than duplicating the
    	// functionality here
    	response.getMessages().addMessage(MessageUtils.
    		createInformationalMessage("password.changed.successfully", null, 
    		null));
    }
    
}
