package gov.va.med.mhv.usermgmt.portlet.employee.login;

import javax.security.auth.login.LoginException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.beehive.netui.pageflow.Forward;
import org.apache.beehive.netui.pageflow.annotations.Jpf;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.beehive.AbstractPageFlowController;
import org.tigris.atlas.messages.MessageUtils;

import com.bea.netuix.servlets.manager.AppContext;
import com.bea.p13n.security.Authentication;
import com.bea.portlet.GenericURL;
import com.bea.portlet.PostbackURL;

import gov.va.med.mhv.usermgmt.util.MHVAdminPortalProperties;
import gov.va.med.mhv.usermgmt.form.LoginForm;
import gov.va.med.mhv.usermgmt.service.UserProfileServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;

/**
 * @jpf:controller struts-merge="/WEB-INF/struts-override.xml"
 */
@Jpf.Controller(strutsMerge = "/WEB-INF/struts-override.xml")
public class EmployeeLoginPageFlowController extends AbstractPageFlowController
{

	private static final Log LOG = LogFactory.getLog(
		EmployeeLoginPageFlowController.class);

	@Jpf.Action(
		forwards = {
			@Jpf.Forward(name = "login", path = "employeeLogin.jsp"),
			@Jpf.Forward(name = "pivlogin", path = "employeePIVLogin.jsp"),
			@Jpf.Forward(name = "loggedIn", path = "employeeLoggedIn.jsp"),
			@Jpf.Forward(name = "pivloggedIn", path = "employeePIVLoggedIn.jsp")
		}
	)
	public Forward begin() {
		// Assume that any authenticated user is an employee
		// This must be verified in the portal backing
		System.out.println(">>>>>>>> 1000 entering begin in EmployeeLoginPageFlowController.");

		boolean usePIVAuthentication = MHVAdminPortalProperties.getInstance().getUsePIVAuthentication();

		if (usePIVAuthentication)
		{
			System.out.println(">>>>>>>> 1000a in side EmployeeLoginPageFlowController. PIVUsername:"+getRequest().getSession().getAttribute("SM_NETWORKID"));
			return new Forward((getRequest().getSession().getAttribute("SM_NETWORKID") != null) ? "pivloggedIn" : "pivlogin");
		}
		else
		{
			System.out.println(">>>>>>>> 1000b in side EmployeeLoginPageFlowController. getUserPrincipal:"+getRequest().getUserPrincipal());
			return new Forward((getRequest().getUserPrincipal() != null) ? "loggedIn" : "login");
		}

		//return new Forward((getRequest().getUserPrincipal() != null) ? "loggedIn" : "login");
	}

	@Jpf.Action(
		forwards = {
			@Jpf.Forward(name = "success", path = "employeeLoggedIn.jsp")
		}, validationErrorForward = @Jpf.Forward(name = "failure",
			navigateTo = Jpf.NavigateTo.currentPage),
		preventDoubleSubmit = true
	)
	protected Forward login(LoginForm form) throws Exception {
		System.out.println(">>>>>>>> 1001 entering login in EmployeeLoginPageFlowController.");
		System.out.println(">>>>>>>> 1002 User Name:"+form.getUserName());
		if (hasUserProfile(form.getUserName())) {
			// Must be an employee, not a user
			System.out.println(">>>>>>>> 1003 User is NOT an employee.");
			form.getMessages().addMessage(MessageUtils.createErrorMessage(
				"invalid.credentials", null, null));
			LOG.warn("Attempt to login to admin portal with non-employee "
				+ " user account '" + form.getUserName() + "'");
			System.out.println(">>>>>>>> 1003A Attempt to login to admin portal with non-employee "
				+ " user account '" + form.getUserName() + "'");
		} else {
			System.out.println(">>>>>>>> 1004 User is an employee.");
			try {
				System.out.println(">>>>>>>> 1005 Authenticating user using Weblogic Authentication.");
				Authentication.login(form.getUserName(), form.getPassword(),getRequest(), getResponse());
				System.out.println(">>>>>>>> 1006 Authenticating user using Weblogic Authentication SUCCESSFUL.");
			} catch (LoginException e) {
				form.getMessages().addMessage(MessageUtils.createErrorMessage(
					"invalid.credentials", null, null));
				System.out.println(">>>>>>>> 1007 Authenticating user using Weblogic Authentication FAILED.");
			}
		}

		//New for 10.3.2 fix
		getRequest().getSession().setAttribute("EmployeeLoginOrLogout","true");
		System.out.println(">>>>>>>> 1008 Value of EmployeeLoginOrLogout attribute:" + getRequest().getSession().getAttribute("EmployeeLoginOrLogout"));

		getMessagesHandler().handleMessages(form.getMessages(), getFormClass(),
			getRequest());
		form.setPassword(null);
		return determineForward();
	}

	@Jpf.Action(
		forwards = {
			@Jpf.Forward(name = "success", path = "employeePIVLoggedIn.jsp")
		}, validationErrorForward = @Jpf.Forward(name = "failure",
			navigateTo = Jpf.NavigateTo.currentPage),
		preventDoubleSubmit = true
	)
	protected Forward pivlogin() throws Exception {
		System.out.println(">>>>>>>> 1001 entering pivlogin in EmployeeLoginPageFlowController.");
		try {
			System.out.println(">>>>>>>> 1005 Authenticating user using PIV.");
			redirect(getResponse(), getSSOiAdminLoginPage(getRequest(), getResponse()));
			//getRequest().getSession().setAttribute("PIVLoggedIn","true");
			System.out.println(">>>>>>>> 1006 Authenticating user using PIV SUCCESSFUL. NetworkID::"+ getRequest().getHeader("SM_NETWORKID"));
		} catch (Exception e) {
			System.out.println(">>>>>>>> 1007 Authenticating user using PIV FAILED.");
		}

		//New for 10.3.2 fix
		getRequest().getSession().setAttribute("EmployeeLoginOrLogout","true");
		System.out.println(">>>>>>>> 1008 Value of EmployeeLoginOrLogout attribute:" + getRequest().getSession().getAttribute("EmployeeLoginOrLogout"));

		return determineForward();
	}


	@Jpf.Action(
		forwards = {
			@Jpf.Forward(name = "pivsuccess", path = "employeePIVLogin.jsp"),
			@Jpf.Forward(name = "success", path = "employeeLogin.jsp")
		}
	)
	public Forward logout() {
		Authentication.logout(getRequest());
		getRequest().getSession().invalidate();
		//New for 10.3.2 fix
		getRequest().getSession().setAttribute("EmployeeLoginOrLogout","true");

		//return new Forward((usePIVAuthentication == true) ? "pivsuccess" : "success");
		return new Forward("success");
	}


	@Jpf.Action(
		forwards = {
			@Jpf.Forward(name = "pivsuccess", path = "employeePIVLogin.jsp"),
			@Jpf.Forward(name = "success", path = "employeeLogin.jsp")
		}
	)
	public Forward pivlogout() {
		System.out.println(">>>>>>>> 1009 Inside pivLogout. About to call logouthome url.");
		redirect(getResponse(), getSSOiAdminLogoutPage(getRequest(), getResponse()));
		getRequest().getSession().invalidate();

		System.out.println(">>>>>>>> 1010 Inside pivLogout. check if ssoi session is cleared."+ getRequest().getHeader("SM_NETWORKID"));

		return new Forward("pivsuccess");
	}

	protected Class getFormClass() {
		return LoginForm.class;
	}

	private boolean hasUserProfile(String userName) {
		UserProfileServiceResponse response = ServiceDelegateFactory.
			createUserProfileServiceDelegate().getProfileForUser(userName);
		return (response != null) && (response.getUserProfile() != null);
	}

    private String getSSOiAdminLoginPage(
			HttpServletRequest request, HttpServletResponse response)
	{
			PostbackURL url = PostbackURL.createPostbackURL(request, response);
			url.removeParameter(GenericURL.PAGE_LABEL_PARAM);
			url.removeParameter("_nfpb");
			url.setPath("/" + AppContext.getAppContext(request).getWebAppName()
				+ "/admin/mhvAdmin/mhvAdminDesktop/loginhome");

			System.out.println(">>>>>>>> 1005a In getSSOiAdminLoginPage. URL used for redirect:" + url.toString());

			return url.toString();
	}

    private String getSSOiAdminLogoutPage(
			HttpServletRequest request, HttpServletResponse response)
	{
			PostbackURL url = PostbackURL.createPostbackURL(request, response);
			url.removeParameter(GenericURL.PAGE_LABEL_PARAM);
			url.removeParameter("_nfpb");
			url.setPath("/" + AppContext.getAppContext(request).getWebAppName()
				+ "/admin/mhvAdmin/mhvAdminDesktop/logouthome");

			System.out.println(">>>>>>>> 1009a In getSSOiAdminLogoutPage. URL used for redirect:" + url.toString());

			return url.toString();
	}

	private void redirect(HttpServletResponse response, String url) {
		assert response != null;
		try {
			response.sendRedirect(url);
		} catch (Exception ex) {
			throw new RuntimeException("Unable to forward to URL " + url, ex);
		}
	}

}
