package gov.va.med.mhv.usermgmt.portlet.employee.role;

import gov.va.med.mhv.usermgmt.form.EmployeeForm;
import gov.va.med.mhv.usermgmt.form.EmployeeOrganizationRoleForm;
import gov.va.med.mhv.usermgmt.form.EmployeeSearchForm;
import gov.va.med.mhv.usermgmt.service.EmployeeOrganizationRoleCollectionServiceResponse;
import gov.va.med.mhv.usermgmt.service.EmployeeOrganizationRoleHistoryCollectionServiceResponse;
import gov.va.med.mhv.usermgmt.service.EmployeeOrganizationRoleServiceResponse;
import gov.va.med.mhv.usermgmt.service.EmployeeSearchResultCollectionServiceResponse;
import gov.va.med.mhv.usermgmt.service.EmployeeSearchResultServiceResponse;
import gov.va.med.mhv.usermgmt.service.EmployeeServiceResponse;
import gov.va.med.mhv.usermgmt.service.OrganizationCollectionServiceResponse;
import gov.va.med.mhv.usermgmt.service.RoleCollectionServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.EmployeeRoleManagementServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.EmployeeSearchServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.EntityMaintenanceServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.transfer.Employee;
import gov.va.med.mhv.usermgmt.transfer.EmployeeOrganizationRole;
import gov.va.med.mhv.usermgmt.transfer.EmployeeOrganizationRolePK;
import gov.va.med.mhv.usermgmt.transfer.EmployeeSearchResult;
import gov.va.med.mhv.usermgmt.transfer.TransferObjectFactory;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.GregorianCalendar;

import org.apache.beehive.netui.pageflow.Forward;
import org.apache.beehive.netui.pageflow.NotLoggedInException;
import org.apache.beehive.netui.pageflow.annotations.Jpf;
import org.apache.commons.lang.StringUtils;
import org.tigris.atlas.beehive.AbstractPageFlowController;
import org.tigris.atlas.messages.MessageUtils;

/**
 * @jpf:controller struts-merge="/WEB-INF/struts-override.xml"
 */
@Jpf.Controller(strutsMerge = "/WEB-INF/struts-override.xml", loginRequired = true,
		catches = { @Jpf.Catch(type = NotLoggedInException.class, path = "notLoggedIn.jsp") },
		simpleActions = { @Jpf.SimpleAction(name = "begin", path = "employeeSearch.jsp"),
						  @Jpf.SimpleAction(name = "viewCurrentRoleAssignments", path = "viewCurrentRoleAssignments.jsp"),
						  @Jpf.SimpleAction(name = "returnToSearchResults", path = "employeeSearchResults.jsp"),
						  @Jpf.SimpleAction(name = "newSearch", path = "employeeSearch.jsp"),
						  @Jpf.SimpleAction(name = "handleDisplayTagAction", navigateTo = Jpf.NavigateTo.currentPage),
						  @Jpf.SimpleAction(name = "gotoRoleHistoryTab", path = "viewRoleHistory.jsp"),
						  @Jpf.SimpleAction(name = "saveEmployeeCancel", action = "gotoDefaultStateManageRoleTab", useFormBeanType = gov.va.med.mhv.usermgmt.form.EmployeeForm.class),
						  @Jpf.SimpleAction(name = "saveEmployeeReset", action = "gotoDefaultStateManageCommentsTab", useFormBeanType = gov.va.med.mhv.usermgmt.form.EmployeeForm.class),
						  @Jpf.SimpleAction(name = "assignRoleWizardCancel", path = "employeeSearchResults.jsp", useFormBeanType = gov.va.med.mhv.usermgmt.form.EmployeeOrganizationRoleForm.class),
						  @Jpf.SimpleAction(name = "assignRoleWizardReset", action = "gotoDefaultStateManageRoleTab", useFormBeanType = gov.va.med.mhv.usermgmt.form.EmployeeOrganizationRoleForm.class)})
public class ManageEmployeeRoleController extends AbstractPageFlowController {
	private static final long serialVersionUID = -476411119L;

	private transient EntityMaintenanceServiceDelegate entityMaintenanceServiceDelegate = ServiceDelegateFactory.createEntityMaintenanceServiceDelegate();
	private transient EmployeeRoleManagementServiceDelegate employeeRoleManagementServiceDelegate = ServiceDelegateFactory.createEmployeeRoleManagementServiceDelegate();
	private transient EmployeeSearchServiceDelegate employeeSearchServiceDelegate = ServiceDelegateFactory.createEmployeeSearchServiceDelegate();

	private Collection<EmployeeSearchResult> employeeSearchResultsList = null;
	private Employee employeeToManage = null;
	private EmployeeSearchResult employeeSearchResult = null;
	private Employee authenticatedEmployee = null;

	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "employeeSearchResults.jsp"),
							 @Jpf.Forward(name="failure", path="employeeSearch.jsp") },
							 validationErrorForward = @Jpf.Forward(name = "validationFailure", navigateTo = Jpf.NavigateTo.currentPage))
	public Forward searchForEmployee(EmployeeSearchForm form) {
		employeeSearchResultsList = new ArrayList<EmployeeSearchResult>();
		if(form.isValidLastNameSearch()){
//			System.out.println("Controller, Start Search by calling service at: " + new GregorianCalendar().getTime());
			EmployeeSearchResultCollectionServiceResponse searchServiceResponse = employeeSearchServiceDelegate.searchByName(form.getFirstName(),form.getLastName());
//			System.out.println("Controller, Search is Complete at: " + new GregorianCalendar().getTime());
			handleMessages(searchServiceResponse);
			if(!searchServiceResponse.getMessages().hasErrorMessages() && !searchServiceResponse.getMessages().hasInformationalMessages()){
				employeeSearchResultsList.addAll(searchServiceResponse.getEmployeeSearchResults());
			}
		} else {
//			System.out.println("Controller, Start Search by calling service at: " + new GregorianCalendar().getTime());
			EmployeeSearchResultServiceResponse searchServiceResponse = employeeSearchServiceDelegate.searchByUserName(form.getUserName());
//			System.out.println("Controller, Search is Complete at: " + new GregorianCalendar().getTime());
			handleMessages(searchServiceResponse);
			if(!searchServiceResponse.getMessages().hasErrorMessages() && !searchServiceResponse.getMessages().hasInformationalMessages()){
				employeeSearchResultsList.add(searchServiceResponse.getEmployeeSearchResult());
			}
		}
		return determineForward();
	}
	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", action = "gotoDefaultStateManageRoleTab")})
	public Forward manageEmployeeRoleAssignments() {
		//get employee from Active Directory
		EmployeeSearchResultServiceResponse employeeSearchResponse = employeeSearchServiceDelegate.searchByUserName(getRequest().getParameter("userName"));
		handleMessages(employeeSearchResponse);
		employeeSearchResult = employeeSearchResponse.getEmployeeSearchResult();

		//get employee data from local database
		EmployeeServiceResponse employeeServiceResponse = employeeRoleManagementServiceDelegate.getEmployeeByUserName(getRequest().getParameter("userName"));
		handleMessages(employeeServiceResponse);
		employeeToManage = employeeServiceResponse.getEmployee();

		return new Forward("success");
	}
	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "manageComments.jsp")})
	public Forward saveEmployee(EmployeeForm form) {
		Employee employeeUpdated = getEmployeeToManage();
		if(employeeUpdated != null){
			employeeUpdated.setComments(form.getEmployeeComments());
			EmployeeServiceResponse response = entityMaintenanceServiceDelegate.save(employeeUpdated);
			if(!response.getMessages().hasErrorMessages()){
				employeeToManage = response.getEmployee();
				response.getMessages().addMessage(MessageUtils.createInformationalMessage("employee.edit.success",null,null));
			}
			handleMessages(response);
		}
		return new Forward("success");
	}
	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "manageComments.jsp")})
	public Forward gotoDefaultStateManageCommentsTab() {
		return new Forward("success",new EmployeeForm());
	}
	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "manageRoleAssignment.jsp")})
	public Forward gotoDefaultStateManageRoleTab() {
		return new Forward("success",populateEmpOrgRoleWizardForm(new EmployeeOrganizationRoleForm()));
	}
	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "manageRoleAssignment.jsp")})
	public Forward assignRoleWizardStep1ChooseOrgType(EmployeeOrganizationRoleForm form) {
		return new Forward("success",populateEmpOrgRoleWizardForm(form));
	}
	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", path = "manageRoleAssignment.jsp")})
	public Forward assignRoleWizardStep2ChooseOrg(EmployeeOrganizationRoleForm form) {
		return new Forward("success",populateEmpOrgRoleWizardForm(form));
	}
	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", action = "gotoDefaultStateManageRoleTab")})
	public Forward assignRoleWizardStep3ChooseRole(EmployeeOrganizationRoleForm form) {
		form = populateEmpOrgRoleWizardForm(form);
		Employee emp = getEmployeeToManage();
		if(emp != null){
			EmployeeOrganizationRoleServiceResponse response = employeeRoleManagementServiceDelegate.assignEmployeeOrganizationRole(
					getAuthenticatedEmployee(),emp.getUserName(),form.getSelectedOrganization(),form.getSelectedRole());
			handleMessages(response);
			if(response != null && !response.getMessages().hasErrorMessages() &&
			   response.getEmployeeOrganizationRole() != null && response.getEmployeeOrganizationRole().getEmployee() != null){
				employeeToManage = response.getEmployeeOrganizationRole().getEmployee();
			}
		}
		return new Forward("success");
	}
	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", action = "gotoDefaultStateManageRoleTab")})
	public Forward activateEmployeeOrgRole() {
		changeEmployeeOrgRoleStatus(true);
		return new Forward("success");
	}
	@Jpf.Action(forwards = { @Jpf.Forward(name = "success", action = "gotoDefaultStateManageRoleTab")})
	public Forward deactivateEmployeeOrgRole() {
		changeEmployeeOrgRoleStatus(false);
		return new Forward("success");
	}
	private void changeEmployeeOrgRoleStatus(boolean status){
		//lookup the employeeOrgRole to edit
		EmployeeOrganizationRolePK pk = TransferObjectFactory.createEmployeeOrganizationRolePK();
        pk.setId(Long.valueOf(getRequest().getParameter("id")));
        EmployeeOrganizationRoleServiceResponse response = entityMaintenanceServiceDelegate.findByPrimaryKey(pk);
		EmployeeOrganizationRole employeeOrgRole = response.getEmployeeOrganizationRole();
		employeeOrgRole.setActive(status);   //change its status

		//persist the entity
		handleMessages(employeeRoleManagementServiceDelegate.updateEmployeeOrganizationRole(getAuthenticatedEmployee(), employeeOrgRole));
	}
	public Employee getAuthenticatedEmployee(){
		if(authenticatedEmployee == null){
			EmployeeServiceResponse response = employeeRoleManagementServiceDelegate.getEmployeeByUserName(getRequest().getUserPrincipal().getName());
			handleMessages(response);
			authenticatedEmployee = response.getEmployee();
		}
		return authenticatedEmployee;
	}
	public Collection getEmployeeSearchResultsList() {
		return employeeSearchResultsList;
	}
	public EmployeeSearchResult getEmployeeSearchResult() {
		return employeeSearchResult;
	}
	public Employee getEmployeeToManage() {
		return employeeToManage;
	}
	public Collection getEmployeeToManageRoles() {
		EmployeeOrganizationRoleCollectionServiceResponse response = employeeRoleManagementServiceDelegate.getEmployeeOrganizationRolesAssignedToEmployee(getEmployeeToManage());
		handleMessages(response);
		return response.getEmployeeOrganizationRoles();
	}
	public Collection getEmployeeToManageRoleHistory() {
		EmployeeOrganizationRoleHistoryCollectionServiceResponse response = employeeRoleManagementServiceDelegate.getRoleHistoriesForEmployee(getEmployeeToManage());
		handleMessages(response);
		return response.getEmployeeOrganizationRoleHistorys();
	}
	public Collection getEmployeeOrganizationRolesEmployeeCanManage() {
		EmployeeOrganizationRoleCollectionServiceResponse orgRolesResponse = employeeRoleManagementServiceDelegate.getEmployeeOrganizationRolesEmployeeCanManage(getAuthenticatedEmployee());
		handleMessages(orgRolesResponse);
		return orgRolesResponse.getEmployeeOrganizationRoles();
	}
	private EmployeeOrganizationRoleForm populateEmpOrgRoleWizardForm(EmployeeOrganizationRoleForm form){
		//populate orgs employee can manage onto form
		OrganizationCollectionServiceResponse response = employeeRoleManagementServiceDelegate.getOrganizationsEmployeeCanManage(getAuthenticatedEmployee());
		handleMessages(response);
		form.setOrganizations(response.getOrganizations());

		//if employee has selected an org, populate roles at org they can manage
		if(StringUtils.isNotEmpty(form.getOrganizationId())){
			RoleCollectionServiceResponse rolesResponse = employeeRoleManagementServiceDelegate.getRolesAtOrganizationEmployeeCanManage(getAuthenticatedEmployee(),form.getSelectedOrganization());
			handleMessages(rolesResponse);
			form.setRoles(rolesResponse.getRoles());
		}
		return form;
	}
	@Override
	protected Class getFormClass() {
		return EmployeeSearchForm.class;
	}
}