package gov.va.med.mhv.usermgmt.portlet.forgotpassword;

import gov.va.med.mhv.core.service.delegate.EmailServiceDelegate;
import gov.va.med.mhv.usermgmt.enumeration.GenderEnumeration;
import gov.va.med.mhv.usermgmt.form.UserProfileForm;
import gov.va.med.mhv.usermgmt.portlet.UserProfileLabelManager;
import gov.va.med.mhv.usermgmt.service.UserProfileServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.beehive.netui.pageflow.Forward;
import org.apache.beehive.netui.pageflow.annotations.Jpf;
import org.apache.commons.lang.StringUtils;
import org.tigris.atlas.beehive.AbstractPageFlowController;
import org.tigris.atlas.config.ConfigurationManager;
import org.tigris.atlas.service.BooleanServiceResponse;
import org.tigris.atlas.service.VoidServiceResponse;
import org.tigris.atlas.util.CalendarUtils;


/**
 * @jpf:controller struts-merge="/WEB-INF/struts-override.xml"
 *  */
@Jpf.Controller(strutsMerge = "/WEB-INF/struts-override.xml")
public class ForgotPasswordController extends AbstractPageFlowController
{

    private UserProfile userProfile;
    private String question1;
    private String question2;
    private String userName;

    private static final Properties PROPS = ConfigurationManager.getConfiguration("/mailCfg.properties");
	private static final String EMAIL_SUBJECT = "My HealtheVet Account Activity Update";

    /**
	 * @jpf:action
	 * @jpf:forward name="begin" path="begin.do"
     * @jpf:forward name="refresh" return-to="currentPage"
	 */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "begin",
                     path = "begin.do"),
        @Jpf.Forward(name = "refresh",
                     navigateTo = Jpf.NavigateTo.currentPage)
    })
    protected Forward portalRefresh()
    {
        return new Forward("begin");
    }

    /**
     * This method represents the point of entry into the pageflow
     * @jpf:action
     * @jpf:forward name="success" path="userInfo.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success",
                     path = "userInfo.jsp")
    })
    protected Forward begin()
    {
        return new Forward("success");
    }

    /**
     * @jpf:action
	 * @jpf:validation-error-forward name="failure" return-to="currentPage"
	 * @jpf:forward name="success" path="passwordHints.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success",
                     path = "passwordHints.jsp")
    },
                validationErrorForward = @Jpf.Forward(name = "failure",
                                                      navigateTo = Jpf.NavigateTo.currentPage), preventDoubleSubmit=true )
    protected Forward getPasswordHints(UserProfileForm form)
    {
        Date birthDate = CalendarUtils.getDate(form.getUserProfileBirthDateMonth(), form.getUserProfileBirthDateDay(), form.getUserProfileBirthDateYear());
        GenderEnumeration gender = GenderEnumeration.getEnum(form.getUserProfileGender());

        UserProfileServiceResponse response =
                ServiceDelegateFactory.createUserProfileServiceDelegate().getPasswordHintsForForgottenPassword(
					form.getUserName(),
					form.getUserProfileFirstName(),
					form.getUserProfileLastName(),
					birthDate,
                    gender
				);

        userName = form.getUserName();
        userProfile = response.getUserProfile();
        if (userProfile != null)
        {
            question1 = UserProfileForm.getUserProfilePasswordHintQuestion1Name(userProfile.getPasswordHintQuestion1());
            question2 = UserProfileForm.getUserProfilePasswordHintQuestion2Name(userProfile.getPasswordHintQuestion2());
        }

        handleMessages(response);

        return determineForward();
    }

    /**
     * @jpf:action
	 * @jpf:validation-error-forward name="failure" return-to="currentPage"
	 * @jpf:forward name="success" path="newPassword.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success",
                     path = "newPassword.jsp")
    },
                validationErrorForward = @Jpf.Forward(name = "failure",
                                                      navigateTo = Jpf.NavigateTo.currentPage), preventDoubleSubmit=true )
    protected Forward verifyPasswordHints(UserProfileForm form)
    {
    	//CR2162 Kumar, Vyasa changes
    	//String passWordHintAnswer1= (form.getUserProfilePasswordHintAnswer1()!=null)? form.getUserProfilePasswordHintAnswer1()+"FP":"FP";

        BooleanServiceResponse response =
			ServiceDelegateFactory.createUserProfileServiceDelegate().verifyPasswordHints(
				userProfile,
				form.getUserProfilePasswordHintAnswer1(),
				form.getUserProfilePasswordHintAnswer2()
			);

        handleMessages(response);


        return determineForward();

    }

    /**
     * @jpf:action
	 * @jpf:validation-error-forward name="failure" return-to="currentPage"
	 * @jpf:forward name="success" path="confirmPasswordReset.jsp"
     */
	@Jpf.Action(
		forwards = {
			@Jpf.Forward(name = "success", path = "confirmPasswordReset.jsp")
		},
		validationErrorForward = @Jpf.Forward(name = "failure", navigateTo = Jpf.NavigateTo.currentPage), preventDoubleSubmit=true
	)
    protected Forward resetPassword(UserProfileForm form)
    {
		VoidServiceResponse response =
            ServiceDelegateFactory.createUserServiceDelegate().resetPassword(
			userName, form.getPassword(), form.getConfirmPassword(),
			Boolean.FALSE);

		handleMessages(response);


		/*
		 * CR2897: Sends out an email once the user's password gets reset successfully and also if the user has an email address in the user profile
		 */
		if ( !response.getMessages().hasErrorMessages() ){

			String userEmail = ServiceDelegateFactory.createUserProfileServiceDelegate().
								getProfileForUser(userName).getUserProfile().getContactInfoEmail();

			EmailServiceDelegate service = gov.va.med.mhv.core.service.delegate.ServiceDelegateFactory.createEmailServiceDelegate();

	    	if (!StringUtils.isBlank(userEmail)) {
				/*
				 * CR 3350; replyTo address has been changed and is now coming from email config properties file
				 */
	    		//service.sendMessage(PROPS.getProperty("userprofile.from.email"), EMAIL_SUBJECT, getMessage(), userEmail.toString());
	    		service.sendMessageWithReplyTo(PROPS.getProperty("userprofile.from.email"), EMAIL_SUBJECT, getMessage(), userEmail.toString(), PROPS.getProperty("userprofile.replyTo.email"));
	    	}
		}

		 // This code is also in ResetPasswordController.sendPassword(SendPasswordForm form)
        // This is supposed to be temporary and should be placed in UserServiceImpl as soon as user and usermgmt
        // are combined.  This cannot be added yet because it would cause a circular dependency between user and usermgmt.

        /*if ( !response.getMessages().hasErrorMessages() ){

            //send mail
            String to = ServiceDelegateFactory.createUserProfileServiceDelegate().getProfileForUser(userName).getUserProfile().getContactInfoEmail();
            String message="test";
            String subject="test";
            String from="mhv@mhv.com";
            gov.va.med.mhv.core.service.delegate.ServiceDelegateFactory.createEmailServiceDelegate().sendMessage(from, subject, message,to);


        }*/

        return determineForward();
    }


	/*
	 * Builds an email body for forgot password portlet
	 */
	private static String getMessage() {
	    StringBuilder messageText = new StringBuilder();

	    messageText.append("This is a courtesy message to notify you of recent activity involving your My HealtheVet password or User ID." +
	    		           " If you did not recently change your password or request your User ID, please notify us by using the Contact MHV" +
	    		           " link provided on the My HealtheVet website - " + "www.myhealth.domain");

	    messageText.append("\n\n ------------------------------------------------------------ ");
	    messageText.append("\n **This is an automated message.  Please do not reply** ");
	    messageText.append("\n -------------------------------------------------------------- \n\n");

	    messageText.append("This e-mail is intended only for the person or entity to which it is addressed, and may contain information that" +
	    		           " is privileged, confidential, or otherwise protected from disclosure.  Dissemination, distribution, or copying of" +
	    		           " this e-mail or the information herein by anyone other than the intended recipient or for official internal VHA " +
	    		           " and/or VA business is prohibited.");

	    messageText.append("\n\n");

	    return messageText.toString();
	  }


	// ------------------------- properties ---------------------------------
	public String getQuestion1() {
        return question1;
    }

    public String getQuestion2() {
        return question2;
    }

	public Map getMonthOptions() {
    	return CalendarUtils.getMonths();
    }

    public Map getDayOptions() {
    	return CalendarUtils.getDays();
    }

    public List getYearOptions() {
    	return CalendarUtils.getYears();
    }

    public String getUserName() {
        return userName;
    }


    public List getGenderOptions() {
        return UserProfileLabelManager.getGenderLabels();
    }

    protected Class getFormClass()
    {
        return UserProfileForm.class;
    }
}
