package gov.va.med.mhv.usermgmt.portlet.forgotusername;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.GenderEnumeration;
import gov.va.med.mhv.usermgmt.form.RequestUserNameForm;
import gov.va.med.mhv.usermgmt.form.UserProfileForm;
import gov.va.med.mhv.usermgmt.portlet.UserProfileLabelManager;
import gov.va.med.mhv.usermgmt.service.UserProfileServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;
import gov.va.med.mhv.usermgmt.util.Auditor;

import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.beehive.netui.pageflow.Forward;
import org.apache.beehive.netui.pageflow.annotations.Jpf;
import org.apache.commons.lang.StringUtils;
import org.tigris.atlas.beehive.AbstractPageFlowController;
import org.tigris.atlas.service.StringServiceResponse;
import org.tigris.atlas.util.CalendarUtils;

/**
 * @jpf:controller struts-merge="/WEB-INF/struts-override.xml"
 *  */
@Jpf.Controller(strutsMerge = "/WEB-INF/struts-override.xml")
public class ForgotUserNameController extends AbstractPageFlowController
{

    private UserProfile userProfile = null;
    private String question1 = null;
    private String question2 = null;
    private String userName;

    // Uncomment this declaration to access Global.app.
    //
    //     protected global.Global globalApp;
    //

    // For an example of page flow exception handling see the example "catch" and "exception-handler"
    // annotations in {project}/WEB-INF/src/global/Global.app

    public String getQuestion1()
    {
        return question1;
    }

    public String getQuestion2()
    {
        return question2;
    }

    public String getUserName()
    {
        return userName;
    }

    /**
     * This method represents the point of entry into the pageflow
     * @jpf:action
     * @jpf:forward name="success" path="userInfo.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success",
                     path = "userInfo.jsp")
    })
    protected Forward begin()
    {
        return new Forward("success");
    }

    /**
     * @jpf:action
     * @jpf:validation-error-forward name="failure" return-to="currentPage"
     * @jpf:forward name="success" path="passwordHints.jsp"
     * @jpf:forward name="success1" path="userInfo.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success",
                     path = "passwordHints.jsp"),
        @Jpf.Forward(name = "success1",
                     path = "userInfo.jsp")
    },
                validationErrorForward = @Jpf.Forward(name = "failure",
                                                      navigateTo = Jpf.NavigateTo.currentPage), preventDoubleSubmit=true )
    protected Forward getPasswordHints(UserProfileForm form)
    {
        Date bd = CalendarUtils.getDate(form.getUserProfileBirthDateMonth(), form.getUserProfileBirthDateDay(), form.getUserProfileBirthDateYear());
        GenderEnumeration gender = GenderEnumeration.getEnum(form.getUserProfileGender());

        UserProfileServiceResponse response =
                ServiceDelegateFactory.createUserProfileServiceDelegate().getPasswordHintsForForgottenUserName( form.getUserProfileFirstName(),
                                                                                    form.getUserProfileLastName(),
                                                                                    bd,
                                                                                    gender);

        userProfile = response.getUserProfile();
        if (userProfile != null)
        {
            question1 = UserProfileForm.getUserProfilePasswordHintQuestion1Name(userProfile.getPasswordHintQuestion1());
            question2 = UserProfileForm.getUserProfilePasswordHintQuestion2Name(userProfile.getPasswordHintQuestion2());
        }

        handleMessages(response);

        return determineForward();

    }

    /**
     * @jpf:action
     * @jpf:validation-error-forward name="failure" return-to="currentPage"
     * @jpf:forward name="success" path="userName.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success", path = "userName.jsp")
		},
		validationErrorForward = @Jpf.Forward(name = "failure", navigateTo = Jpf.NavigateTo.currentPage), preventDoubleSubmit=true
	)
    protected Forward getUserName(RequestUserNameForm form)  {
    	//CR2162 Kumar,Vyasa Changes
      // String passWordHintAnswer1= (form.getUserProfilePasswordHintAnswer1()!=null)? form.getUserProfilePasswordHintAnswer1()+"FP":"FP";


    	StringServiceResponse response =  ServiceDelegateFactory.createUserProfileServiceDelegate().getForgottenUserName( userProfile,
    																form.getUserProfilePasswordHintAnswer1(),
                                                                    form.getUserProfilePasswordHintAnswer2() );


        Auditor.auditForgotUserIdEvent(userProfile.getId(), ActivityActorTypeEnumeration.SELF,!response.getMessages().hasErrorMessages());

        userName = response.getString();

        handleMessages( response );

        return determineForward();
    }


    public Map getMonthOptions() {
    	return CalendarUtils.getMonths();
    }

    public Map getDayOptions() {
    	return CalendarUtils.getDays();
    }

    public List getYearOptions() {
    	return CalendarUtils.getYears();
    }

    public List getGenderOptions() {
        return UserProfileLabelManager.getGenderLabels();
    }


    protected Class getFormClass()
    {
        return RequestUserNameForm.class;
    }
}
