package gov.va.med.mhv.usermgmt.portlet.ipa;

import gov.va.med.mhv.usermgmt.enumeration.AuthenticationStatus;
import gov.va.med.mhv.usermgmt.enumeration.PatientCorrelationStatus;
import gov.va.med.mhv.usermgmt.form.IPACriteriaForm;
import gov.va.med.mhv.usermgmt.transfer.Facility;
import gov.va.med.mhv.usermgmt.transfer.FacilityInfo;
import gov.va.med.mhv.usermgmt.transfer.InPersonAuthentication;
import gov.va.med.mhv.usermgmt.transfer.Patient;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.util.CalendarUtils;

public class PatientSummaryDecorator implements Serializable {
	//MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - Correlation Statuses 
	private static final int UNCORRELATED_VALUE = 0;
	private static final int PENDINGCORRELATION_VALUE = 1;
	private static final int CORRELATED_VALUE = 2;
	private static final int PENDINGUNCORRELATION_VALUE = 3;
	private static final int FAILEDCORRELATION_VALUE = 4;
	private static final int FAILEDUNCORRELATION_VALUE = 5;
	private static final int CORRELATIONINVALID_VALUE = 6;
	private static final int MVIDATAMATCH_VALUE = 7;
	private static final String TIME_FORMAT = "MM/dd/yyyy hh:mm a z";

	private static final long serialVersionUID = 5388615976997002378L;
    private static Log LOG = LogFactory.getLog(
    		PatientSummaryDecorator.class);

	private Patient patient;
	private InPersonAuthentication ipa;
	
	public PatientSummaryDecorator(Patient patient, InPersonAuthentication ipa) {
		this.patient = patient;
		this.ipa = ipa;
	}

	public String getStartUpgradeManualDateTime() {
		String returnValue = "";
		if(ipa.getStartUpgradeManualDateTime() != null) {
			returnValue = new SimpleDateFormat(TIME_FORMAT).format(ipa.getStartUpgradeManualDateTime());
		}
		return returnValue;
	}
	
	public String getStartUpgradeOnlineDateTime() {
		String returnValue = "";
		if(ipa.getStartUpgradeOnlineDateTime() != null) {
			returnValue = new SimpleDateFormat(TIME_FORMAT).format(ipa.getStartUpgradeOnlineDateTime());
		}
		return returnValue;
	}
	
	public String getSignedFormOnlineDateTime() {
		String returnValue = "";
		if(ipa.getSignedFormOnlineDateTime() != null) {
			returnValue = new SimpleDateFormat(TIME_FORMAT).format(ipa.getSignedFormOnlineDateTime());
		}
		return returnValue;
	}
	
	//MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - ICN 
	public String getIcn() {
		if(patient.getIcn() != null) {
			return patient.getIcn();
		} else {
			return "";
		}
	}
	
	//MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - Correlation Statuses 
	public String getUserProfileId() {
		return patient.getUserProfile().getId().toString();
	}
	
	//MHV_CodeCR1387 - Added middle name to display
	public String getMiddleName() {
		String middleName = "";
		if(patient.getUserProfile().getMiddleName() != null) {
			middleName = patient.getUserProfile().getMiddleName();
		}
		
		if (LOG.isDebugEnabled()) {
			LOG.debug("************** In PatientSummaryDecorator.getMiddleName() middleName: " + middleName);
		}
		
		return middleName;
	}
	
	//MHV_CodeCR1387 - Added gender to display
	public String getGender() {
		String gender = "";
		if(patient.getUserProfile().getGender() != null) {
			gender = patient.getUserProfile().getGender().getName();
		}
		
		if (LOG.isDebugEnabled()) {
			LOG.debug("************** In PatientSummaryDecorator.getGender() gender: " + gender);
		}
		
		return gender;
	}
	
	public String getRelationshipToVA() {
		StringBuffer relationshipToVA = new StringBuffer("");
		if(patient.getUserProfile().getIsPatient()) {
			relationshipToVA.append("VA Patient");		
		}
		
		if(patient.getUserProfile().getIsVeteran()) {
			relationshipToVA.append(", Veteran");		
		}
		
		if(patient.getUserProfile().getIsHealthCareProvider()) {
			relationshipToVA.append(", Health Care Provider");		
		}
		
		if(patient.getUserProfile().getIsPatientAdvocate()) {
			relationshipToVA.append(", Veteran Advocate/Family Member/Friend");		
		}
		
		if(patient.getUserProfile().getIsEmployee()) {
			relationshipToVA.append(", VA Employee");		
		}
		
		if(patient.getUserProfile().getIsEmployee()) {
			relationshipToVA.append(", Other");		
		}
		
		if (LOG.isDebugEnabled()) {
			LOG.debug("************** In PatientSummaryDecorator.getRelationshipToVA() relationshipToVA: " + relationshipToVA);
		}
		
		return relationshipToVA.toString();
	}
	
	public String getFullName() {
		String firstName = patient.getUserProfile().getFirstName();
		String middleName = patient.getUserProfile().getMiddleName();
		String lastName = patient.getUserProfile().getLastName();
		
		StringBuffer buff = new StringBuffer();
		buff.append(firstName);
		if (middleName != null) {
			buff.append(' ');
			buff.append(middleName);
		}
		buff.append(' ');
		buff.append(lastName);
		
		return(buff.toString());
	}
	
	public String getUserName() {
		return(patient.getUserProfile().getUserName());
	}
	
	public String getMhvId() {
		return(patient.getUserProfile().getMhvId());
	}
	
	public String getSsn() {
		return(patient.getUserProfile().getSsn());
	}
	
	public String getBirthDate() {
		return(CalendarUtils.getDateAsMDY(patient.getUserProfile().getBirthDate()));
	}
	
	public String getEmail() {
		return(patient.getUserProfile().getContactInfoEmail());
	}

	public String getTxFacilities() {
		StringBuffer buff = new StringBuffer(); 
		if(patient.getFacilitys()==null || patient.getFacilitys().size()==0){
			return "N/A";
		}
		Iterator itr = patient.getFacilitys().iterator();
		while (itr.hasNext()) {
			Facility facility = (Facility)itr.next();
			FacilityInfo fi = facility.getFacilityInfo();
			if(fi != null){
				//JAZZ: Task#20186 - Task:Add comma at the end of each treating facility name in IPA detail pages.
				if (buff.length() > 0)
					buff.append(",<br/>");
				buff.append(fi.getName());
			}
			else{
				//JAZZ: Task#20186 - Task:Add comma at the end of each treating facility name in IPA detail pages.
				if (buff.length() > 0)
					buff.append(",<br/>");
				buff.append(facility.getName());
			}
		}
		return buff.toString();
	}
	
	public String getAuthenticationStatus() {
		if(ipa.getStatus()==null){
			return "N/A";
		}
		AuthenticationStatus status = ipa.getStatus().getName() == null ? AuthenticationStatus.getEnum(AuthenticationStatus.UNAUTHENTICATED) : AuthenticationStatus.getEnum(ipa.getStatus().getName());
		return IPACriteriaForm.getPatientInfoAuthenticationStatusName(status);
	}
	
	public String getAuthenticationDate() {
		if(ipa.getAuthenticationDate()==null){
			return "N/A";
		}
		return new SimpleDateFormat(TIME_FORMAT).format(ipa.getAuthenticationDate());
	}
	
	public String getAuthenticatingFacility() {
		if(ipa.getAuthenticatingFacility()==null){
			return "N/A";
		}
		return(ipa.getAuthenticatingFacility().getName());
	}

	//MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - Displaying new fields in Admin Portal IPA Search Detail Page 
	public String getAuthenticatedBy() {
		String returnValue = "";
		if(ipa.getAuthenticationDate() != null && ipa.getAuthenticatedBy() != null) {
			returnValue = ipa.getAuthenticatedBy();
		}
		return returnValue;
	}
	
	//MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - Displaying new fields in Admin Portal IPA Search Detail Page 
	public String getCorrelatedBy() {
		String returnValue = "";
		if(patient.getCorrelatedBy() != null) {
			returnValue = patient.getCorrelatedBy();
		}
		return returnValue;
	}
	
	//MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - Displaying new fields in Admin Portal IPA Search Detail Page 
	public String getCorrelateRequestDate() {
		String returnValue = "";
		if(patient.getCorrelateRequestDateTime() != null) {
			returnValue = new SimpleDateFormat(TIME_FORMAT).format(patient.getCorrelateRequestDateTime());
		}
		return returnValue;
	}
	
	//MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - Displaying new fields in Admin Portal IPA Search Detail Page 
	public String getCorrelationDate() {
		String returnValue = "";
		if(patient.getCorrelatedDateTime() != null) {
			returnValue = new SimpleDateFormat(TIME_FORMAT).format(patient.getCorrelatedDateTime());
		}
		return returnValue;
	}
	
	//MHV_CodeCR1514 - US12.4 MVI Compliance Implementation - Displaying new fields in Admin Portal IPA Search Detail Page 
	public String getCorrelationStatus() {
		StringBuffer buff = new StringBuffer(50);
		PatientCorrelationStatus correlationStatus = patient.getCorrelationStatus() ; 
		if(correlationStatus == PatientCorrelationStatus.getEnum(PatientCorrelationStatus.UNCORRELATED)) {
			return "Uncorrelated";
		} else if(correlationStatus == PatientCorrelationStatus.getEnum(PatientCorrelationStatus.PENDINGCORRELATION)) {
			return "Pending Correlation";
		//Admin Portal_CodeCR1843, 1844 - Formatted the statuses.
		} else if(correlationStatus == PatientCorrelationStatus.getEnum(PatientCorrelationStatus.PENDINGUNCORRELATION)) {
			return "Pending Uncorrelation";
		} else if(correlationStatus == PatientCorrelationStatus.getEnum(PatientCorrelationStatus.CORRELATED)) {
			String returnValue = "Correlated";
			return returnValue;
		} else if(correlationStatus == PatientCorrelationStatus.getEnum(PatientCorrelationStatus.CORRELATIONINVALID)) {
			return "Correlation Invalid";
		} else if(correlationStatus == PatientCorrelationStatus.getEnum(PatientCorrelationStatus.FAILEDCORRELATION)) {
			return "Failed Correlation";
		} else if(correlationStatus == PatientCorrelationStatus.getEnum(PatientCorrelationStatus.FAILEDUNCORRELATION)) {
			return "Failed Uncorrelation";
		} else if(correlationStatus == PatientCorrelationStatus.getEnum(PatientCorrelationStatus.MVIDATAMATCH)) {	
			//Admin Portal_CodeCR1846 - Made sure that Uncorrelated displays.
			return "Uncorrelated";
		}
		return "";
	}
	
	public String getAccountCreatedDate() {
		if(patient.getUserProfile().getCreatedDate() != null) {
			return new SimpleDateFormat(TIME_FORMAT).format(patient.getUserProfile().getCreatedDate());
		}
		return "";
	}
}
