package gov.va.med.mhv.usermgmt.portlet.login;

import java.sql.Timestamp;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.beehive.netui.pageflow.Forward;
import org.apache.beehive.netui.pageflow.annotations.Jpf;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.messages.Message;
import org.tigris.atlas.messages.Messages;
import org.tigris.atlas.service.BooleanServiceResponse;
import org.tigris.atlas.service.ServiceResponse;
import org.tigris.atlas.service.StringServiceResponse;

import gov.va.med.mhv.usermgmt.enumeration.AccountStatusEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActionTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityTypeEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.AuthenticationStatus;
import gov.va.med.mhv.usermgmt.form.LoginForm;
import gov.va.med.mhv.usermgmt.service.FacilityInfoServiceResponse;
import gov.va.med.mhv.usermgmt.service.InPersonAuthenticationServiceResponse;
import gov.va.med.mhv.usermgmt.service.MviSyncCommErrServiceResponse;
import gov.va.med.mhv.usermgmt.service.PatientServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.FacilityServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.InPersonAuthenticationServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.transfer.FacilityInfo;
import gov.va.med.mhv.usermgmt.transfer.InPersonAuthentication;
import gov.va.med.mhv.usermgmt.transfer.MviSyncCommErr;
import gov.va.med.mhv.usermgmt.transfer.Patient;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;
import gov.va.med.mhv.usermgmt.util.Auditor;
import gov.va.med.mhv.usermgmt.util.DirectorUtil;
import gov.va.med.mhv.usermgmt.util.PatientCorrelationStatusUtils;
import gov.va.med.mhv.usermgmt.portlet.login.loginbase.LoginNavigateBaseController;
//imports for added eauth code
import gov.va.med.mhv.core.util.ServiceResponseUtils;
import gov.va.med.mhv.eauth.EauthConstants;
import gov.va.med.mhv.eauth.service.delegate.EAuthHashServiceDelegate;
import gov.va.med.mhv.eauth.service.delegate.EAuthLogServiceDelegate;
import javax.servlet.http.HttpServletRequest;
import org.apache.beehive.netui.pageflow.scoping.ScopedServletUtils;
import org.tigris.atlas.service.VoidServiceResponse;
import gov.va.med.mhv.usermgmt.portlet.constants.SessionAttributeConstants;



/**
 * @jpf:controller struts-merge="/WEB-INF/struts-override.xml"
 *  */
@Jpf.Controller(strutsMerge = "/WEB-INF/struts-override.xml"
)
public class LoginPageFlowController extends LoginNavigateBaseController {

	private static final Log LOG = LogFactory.getLog(LoginPageFlowController.
		class);
    private String fccxUrl;

    /**
	 *
	 */
	private static final long serialVersionUID = 1L;

	public final Set<String> BOGUS_SSN_LIST = new HashSet<String>(Arrays.asList(
			"000-00-0000",
			"111-11-1111",
			"999-99-9999")
	);

	private static final String MVI_PENDING_AUTH = "PENDING_AUTH";
	private static final String MVI_PENDING_UNAUTH = "PENDING_UNAUTH";

	public String getFccxUrl() {
		return fccxUrl;
	}
	
    /**
     * This method represents the point of entry into the pageflow
     * @jpf:action
     * @jpf:forward name="success" path="login.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success", path = "login.jsp")
    })
    public Forward begin() {
    	//JAZZ # 143252 - Federated Credential - retrieving the fccx (accessVa) url for the right environment.
    	fccxUrl = ServiceDelegateFactory.createUserProfileServiceDelegate().getFccxUrl().getString();
System.out.println("******* fccxUrl="+fccxUrl);
		BooleanServiceResponse response = ServiceDelegateFactory.createUserProfileServiceDelegate().useFccxLogin();
		if(response.getBoolean() != null && response.getBoolean()) {
			getRequest().getSession().setAttribute("fccxLogin", "true");
		}
        return new Forward("success");
    }

    /**
     * This method represents the point of entry into the pageflow, when
     * refreshing the portlet.
     * @jpf:action
     * @jpf:forward name="success" path="login.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success", path = "login.jsp")
    })
    public Forward refresh() {
    	return begin();
    }

    /**
     * @jpf:action
     * @jpf:validation-error-forward name="failure" return-to="currentPage"
     * @jpf:forward name="success" path="login.jsp"
     */
    @Jpf.Action(
    	forwards = {
    		@Jpf.Forward(name = "success", path = "login.jsp")
    	},
        validationErrorForward = @Jpf.Forward(name = "failure",
        	navigateTo = Jpf.NavigateTo.currentPage),
        preventDoubleSubmit = true
	)
	protected Forward login(LoginForm form) throws Exception  {
    	Forward forward = determineForward();
    	String userName = form.getUserName();
    	String password = form.getPassword();
    	if(isTokenValid())
    		resetToken();
    	else
    		saveToken(getRequest());
    	BooleanServiceResponse response = gov.va.med.mhv.usermgmt.util.Authentication.getInstance().authenticate(form);

    	if (getError(response.getMessages(), "account.deactivated") != null)
    	{
    		getRequest().setAttribute(SessionAttributeConstants.USER_ACCOUNT_DEACTIVATED, Boolean.TRUE);
    	}
    	handleMessages(response);

    	if (response.getBoolean()) {
    		//add a flag in the session for temp_password_check_required
    		getRequest().getSession().setAttribute(SessionAttributeConstants.TEMPORARY_PASSWORD_CHECK_REQUIRED,Boolean.TRUE);
    		com.bea.p13n.security.Authentication.login(userName, password,
    			getRequest(), getResponse());
    		//ServiceDelegateFactory.createPatientServiceDelegate().updatePatientFacilities(StringUtils.lowerCase(userName));

    		Forward eAuthForward = useEAuth(form);
    		if (eAuthForward != null) {
    			forward = eAuthForward;
    		}

	    	UserProfile userProfile = ServiceDelegateFactory.createUserProfileServiceDelegate().getProfileForUser(userName).getUserProfile();

            //Switch desktops here
            if (StringUtils.equalsIgnoreCase(forward.getName(), "success")) {
            	forward.setPath(getRedirectUrl());
                //JAZZ # 111941 - Remote ID Proofing US REQ#3.11 - Remote ID Proofing - When the User lands into their MHV account already logged in,
				//MHV system shall record this login as the last time the user logged in to their MHV account.
            	processLastLogin(userProfile);
            	processDisplayRegisterFccx(userProfile);
            }

	       	//Adding for MVI Match Check in case where MVI was DOWN during registration.
            boolean isTakenCare = false;
	    	if(userProfile != null && userProfile.getId() != null) {
	    	MviSyncCommErrServiceResponse mviSyncCommErrSerRes = ServiceDelegateFactory.createMviSyncCommErrServiceDelegate().getRequiredMviSyncByProfileId(userProfile.getId());
				MviSyncCommErr mviSyncCommError = mviSyncCommErrSerRes.getMviSyncCommErr();
				if(mviSyncCommError != null) {
					PatientServiceResponse patientRegistryResponse = ServiceDelegateFactory.createPatientServiceDelegate().getPatientFromMVI(userProfile, false);
					Patient patient = patientRegistryResponse.getPatient();
					if(patient != null) {
						patient.setUserProfile(userProfile);
						patient.setCorrelationStatus(PatientCorrelationStatusUtils.MVI_DATA_MATCH);
						patient.setMatchedDateTime(new Timestamp(Calendar.getInstance().getTime().getTime()));
						ServiceResponseUtils.copyMessages(response, patientRegistryResponse);
						//copyMessages(response, patientRegistryResponse);
						PatientServiceResponse psr = ServiceDelegateFactory.createEntityMaintenanceServiceDelegate().save(patient);
						//Auditor.auditMviLoginEvent(userProfile.getId(),ActivityActorTypeEnumeration.SELF,true, ActivityActionTypeEnumeration.UPGRADE_TO_ADVANCED);
						Auditor.auditMviEvent(userProfile.getId(),ActivityActorTypeEnumeration.SYSTEM,true, ActivityActionTypeEnumeration.UPGRADE_TO_ADVANCED, ActivityTypeEnumeration.ACCOUNT_UPGRADE);
						if(patient.getFacilitys() != null)
							userProfile.setIsPatient(true);
						userProfile.setIsMPIControlled(true);
						ServiceDelegateFactory.createEntityMaintenanceServiceDelegate().save(userProfile);
						ServiceResponseUtils.copyMessages(response, psr);
						//copyMessages(response, psr);
						ServiceDelegateFactory.createEntityMaintenanceServiceDelegate().delete(mviSyncCommError.getMviSyncCommErrPK());

					} else {
						Auditor.auditMviEvent(userProfile.getId(),ActivityActorTypeEnumeration.SYSTEM, false, ActivityActionTypeEnumeration.UPGRADE_TO_ADVANCED, ActivityTypeEnumeration.ACCOUNT_UPGRADE);
					}
				}
	    	}
	    	PatientServiceResponse patientResponse = ServiceDelegateFactory.createPatientServiceDelegate().getPatientForUser(userProfile);
	    	//Adding for Correlation
	    	Patient patient = null;
	    	if ( !response.getMessages().hasErrorMessages()) {

		    	//PatientServiceResponse patientResponse = ServiceDelegateFactory.createPatientServiceDelegate().getPatientForUser(userProfile);
		    	patient = patientResponse.getPatient();

		    	if(patient != null && patient.getCorrelationStatus().equals(PatientCorrelationStatusUtils.MVI_DATA_MATCH)) {
		    		//Auditor.auditMviLoginEvent(userProfile.getId(),ActivityActorTypeEnumeration.SYSTEM,true, ActivityActionTypeEnumeration.ACCOUNT_CORRELATION_REQUESTED);
		    		isTakenCare = true;
		    		ServiceDelegateFactory.createMviAsyncIntegrationServiceDelegate().addPersonForCorrelation(patient, userProfile, false);
		    		/*BooleanServiceResponse corrResponse = ServiceDelegateFactory.createMviAsyncIntegrationServiceDelegate().addPersonForCorrelation(patient, userProfile);
		    		if (corrResponse.getBoolean().booleanValue())
		    			Auditor.auditMviLoginEvent(userProfile.getId(),ActivityActorTypeEnumeration.SYSTEM,true, ActivityActionTypeEnumeration.ACCOUNT_CORRELATED);
		    		else
		    			Auditor.auditMviLoginEvent(userProfile.getId(),ActivityActorTypeEnumeration.SYSTEM,false, ActivityActionTypeEnumeration.ACCOUNT_CORRELATED);
		    		*/
		    	}

		    	if(patient != null) {
			    	//CR2055 - Call Mvi Auth/UnAuth during login when the user is in Pending Auth/Pending Unauth.
		    		InPersonAuthenticationServiceResponse ipaResponse =
		    			ServiceDelegateFactory.createInPersonAuthenticationServiceDelegate().getIPAPatientById(patient.getId());
		    		InPersonAuthentication ipa = ipaResponse.getInPersonAuthentication();
		    		if(ipa != null && (ipa.getMviAuthenticationStatus() != null)) {
		    			if(ipa.getMviAuthenticationStatus().equals(MVI_PENDING_AUTH)) {
		    				ServiceDelegateFactory.createMviAsyncIntegrationServiceDelegate().mviAuthenticate(
		    					patient, patient.getUserProfile(), true, false);
		    			} else if(ipa.getMviAuthenticationStatus().equals(MVI_PENDING_UNAUTH)){
		    				ServiceDelegateFactory.createMviAsyncIntegrationServiceDelegate().mviUnauthenticateUncorrelateForBatch(
		    					patient, patient.getUserProfile(), "System");
		    			}
		    		}

		    		//Jazz #110471 US REQ#4.1-4.3 RemoteID Proofing - If the MHV user has a signed form; MHV system shall set the Signed 10-5345a-MHV form online field to the Date
		    		//and Time the MHV user signed the form
		    		if(ipa!=null && ipa.getStartUpgradeOnlineDateTime()!=null && ipa.getSignedFormOnlineDateTime() == null) {
		    			checkAndProcessOnlineERARUpgrade(ipa);
		    		}

		    		//JAZZ # 128616 - Process Facilities Update
		    		if(patient.getCorrelationStatus().equals(PatientCorrelationStatusUtils.CORRELATED)) {
		    			System.out.println("****************************** LoginPageFlowController.processFacilitiesUpdate(patient)");
		    			processFacilitiesUpdate(patient);
		    		}
		    	}
	    	}

	    	//Adding to deal with the existing users who are AUTHENTICATED but not in CORRELATED State. God knows for how long
	    	//Note: Need to ignore the patients who are in authenticated status. Try to correlate patients who got matched at least once with MPI.
	    	if(!isTakenCare) {
		    	InPersonAuthentication ipaPatient = null;
		    	//PatientServiceResponse patientResponse = ServiceDelegateFactory.createPatientServiceDelegate().getPatientForUser(userProfile);
		    	InPersonAuthenticationServiceResponse ipaResponse = null;
		    	Patient existingPatient = patientResponse.getPatient();
		    	if(existingPatient != null)
		    		ipaResponse = ServiceDelegateFactory.createInPersonAuthenticationServiceDelegate().getIPAPatientById(existingPatient.getId());
		    	if (ipaResponse != null)
		    		ipaPatient = ipaResponse.getInPersonAuthentication();

		    	if( ipaPatient != null && ipaPatient.getStatus().equals(AuthenticationStatus.getEnum(AuthenticationStatus.AUTHENTICATED)) &&
		    				(existingPatient != null && !PatientCorrelationStatusUtils.isCorrelated(existingPatient)) ) {
		    		ServiceDelegateFactory.createMviAsyncIntegrationServiceDelegate().existingUserWithPendingDataMismatch(userProfile);
		    	}
		    	else if ( ipaPatient == null && (existingPatient != null && !PatientCorrelationStatusUtils.isCorrelated(existingPatient))) {
		    		ServiceDelegateFactory.createMviAsyncIntegrationServiceDelegate().existingUserWithPendingDataMismatch(userProfile);
		    	}
		    	// Sometimes they want Veteran to be checked and sometimes they don't want - as of 02112013 per Bhanu - Not Needed.

		    	else if (existingPatient == null && (userProfile.getIsVeteran() && userProfile.getSsn() != null && !BOGUS_SSN_LIST.contains(userProfile.getSsn())) ) {
		    		ServiceDelegateFactory.createMviAsyncIntegrationServiceDelegate().existingUserWithPendingDataMismatch(userProfile);
		    	}

		    	//ServiceDelegateFactory.createMviIntegrationServiceDelegate().updateTreatmentFacilitiesForExistingPatients(existingPatient);
	    	}


    	}

       // Keep this here just to be safe
       form.setPassword(null);

       return forward;
    }

	//JAZZ # 128616 - Process Facilities Update
    private void processFacilitiesUpdate(Patient patient) {
		boolean sameDay = false;

		if(patient.getLastFacilitiesUpdateDateTime() != null) {
			Calendar now = Calendar.getInstance();
			//now.setTime(now.getTime() - 1000*60*60*24);
			Calendar lastFacilitiesUpdate = Calendar.getInstance();
			lastFacilitiesUpdate.setTime(patient.getLastFacilitiesUpdateDateTime());

			sameDay = now.get(Calendar.YEAR) == lastFacilitiesUpdate.get(Calendar.YEAR) &&
				now.get(Calendar.DAY_OF_YEAR) == lastFacilitiesUpdate.get(Calendar.DAY_OF_YEAR);
		}
		if(!sameDay) {
System.out.println("************************* updateTreatmentFacilitiesForExistingPatients is called for dfn:" + patient.getUserProfile().getId());
			ServiceDelegateFactory.createMviIntegrationServiceDelegate().updateTreatmentFacilitiesForExistingPatients(patient);
		} else {
System.out.println("************************* updateTreatmentFacilitiesForExistingPatients is not called for dfn:" + patient.getUserProfile().getId());
		}
    }

    //Jazz #110471 US REQ#4.1-4.3 RemoteID Proofing - MHV System checks the Date/Time Start Upgrade Online field to see
    //if user started to sign the form online.
    //The system shall also check to see if the Signed 10-5345a-MHV form online field is blank.
    //The MHV system shall make a call to VAPii to see if 10-5345a-MHV form was signed and stored in VAPii.
    //If the MHV user has a signed form; MHV system shall set the Signed 10-5345a-MHV form online field to the Date and
    //Time the MHV user signed the form.
    private void checkAndProcessOnlineERARUpgrade(InPersonAuthentication ipa) {
    	Patient patient = ipa.getPatient();
    	StringServiceResponse vapiiResponse =
    		ServiceDelegateFactory.createUserProfileServiceDelegate().getVapiiFormStatusForUser(patient.getUserProfile());
    	String selectedUserProfileVapiiFormStatus = vapiiResponse.getString();
System.out.println("selectedUserProfileVapiiFormStatus = " + selectedUserProfileVapiiFormStatus);
		if(selectedUserProfileVapiiFormStatus.endsWith("SUBMITTED")) {
			ipa.setSignedFormOnlineDateTime(ipa.getStartUpgradeOnlineDateTime());

			//JAZZ#32762 US02 REQ#6.2 The MHV system shall set Prerequisite fields:
			//User has presented the required identification and Signed Request to Receive Access to
			//Personal Health Record (PHR) check box.
			ipa.setVideoViewed(Boolean.TRUE);
			ipa.setIdentificationPresented(Boolean.TRUE);
			ipa.setParticipationFormSigned(Boolean.TRUE);
            ipa.setApprovedForRecordsAccess(Boolean.TRUE);

			//JAZZ#32762 US02 REQ#6.3 MHV System shall set the Authenticating Facility field to AUSTIN MHV
			ipa.setAuthenticatingFacility(getAustinMHVFacilityInfo());

			//JAZZ#32762 US02 REQ#6.4 MHV system shall not set the Reason for Delay field.
			ipa.setDefermentReason(null);

			//JAZZ#32762 US02 REQ#6.1 The MHV System shall set the NEW field Signed 10-5345a-MHV form online field to
			//date and time the MHV form was signed online and confirmation is received that the user has a signed form
			//on file; otherwise blank.
			ipa.setSignedFormOnlineDateTime(new Timestamp(new Date().getTime()));

			//JAZZ#32762 US02 REQ#6.7 The MHV system shall set the Name of Authenticator (VA Staff):   field to System
			ipa.setAuthenticatedBy("System");
			ipa.setStatus(AuthenticationStatus.getEnum(AuthenticationStatus.AUTHENTICATED));

			ServiceDelegateFactory.createInPersonAuthenticationServiceDelegate().saveIPA(ipa);

			//JAZZ#32762 US02 REQ#6 After confirming the Submitted form status from VAPii, MHV System shall upgrade the users account to a Premium account type.
			//JAZZ#32762 US02 REQ#6.5 The MHV system shall set the status of this account to Authenticated/Premium
			//JAZZ#32762 US02 REQ#6.6 The MHV system shall notify MVI the Release of Information (ROI) flag was set.
			InPersonAuthenticationServiceResponse ipaResp = ServiceDelegateFactory.createInPersonAuthenticationServiceDelegate().authenticate(ipa);

			if (ipaResp.getMessages().hasErrorMessages()) {
		          for (Object item:ipaResp.getMessages().getErrorMessages()) {
		        	  LOG.error("IPA authentication action Error: " + item);
		          }
				//JAZZ#32762 US02 REQ#6.8 MHV System shall capture the following information in the Account Activity Log (AAL) table in case of failure:
				Auditor.auditOnlineAuthenticationEvent(patient.getUserProfile().getId(), false, ActivityTypeEnumeration.ONLINE_FORM_AVAILABLE,
						ActivityActionTypeEnumeration.ONLINE_10_5345A_MHV_FORM_FILED, "");
		    } else {
				//JAZZ#32762 US02 REQ#6.8 MHV System shall capture the following information in the Account Activity Log (AAL) table in case of success:
				Auditor.auditOnlineAuthenticationEvent(patient.getUserProfile().getId(), true, ActivityTypeEnumeration.ONLINE_FORM_AVAILABLE,
					ActivityActionTypeEnumeration.ONLINE_10_5345A_MHV_FORM_FILED, "");
		    }
		}
    }

	private FacilityInfo getAustinMHVFacilityInfo() {
		FacilityServiceDelegate facilityInfoDelegate = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createFacilityServiceDelegate();
		FacilityInfoServiceResponse facilityInfoResponse = facilityInfoDelegate.queryFacilityInfoByStationNumber("200MH");
		return facilityInfoResponse.getFacilityInfo();
	}

    //JAZZ # 111941 - Remote ID Proofing - Last Login
	private void processLastLogin(UserProfile uProfile) {
		if(uProfile.getCurrentLogin()!=null) {
			uProfile.setLastLogin(uProfile.getCurrentLogin());
		}
		//Jazz136316 Logout box date time change
		uProfile.setCurrentLogin(new Timestamp( new Date().getTime()));
		ServiceDelegateFactory.createEntityMaintenanceServiceDelegate().save(uProfile);
	}

    //JAZZ # 143252 - Federated Credential - retrieving the fccx (accessVa) url for the right environment.
	private void processDisplayRegisterFccx(UserProfile uProfile) {
		if(uProfile.getFccxDate()==null) {
			getRequest().getSession().setAttribute("login", "mhv");    	
		}
	}
	
    private Forward useEAuth(LoginForm form) {
    	Forward forward = null;
		// save eauth hash_value
	    HttpServletRequest outerRequest = ScopedServletUtils.getOuterRequest(
	    	getRequest());
		Object loginType = outerRequest.getSession().getAttribute(EauthConstants.LOGIN_TYPE);
		if (LOG.isDebugEnabled()) {
			LOG.debug("loginType= " + loginType);
    	}

		if (EauthConstants.EAUTH.equals(loginType)){
			String hash = (String)outerRequest.getSession().getAttribute(
					EauthConstants.HASH_ATTR);
			String csid = (String)outerRequest.getSession().getAttribute(
					EauthConstants.CSID_ATTR);
			if (LOG.isDebugEnabled()) {
				LOG.debug("or userhash= " + hash);
				LOG.debug("or csid= " + csid);
			}
		    if(null != hash && hash.length() > 0){
		        EAuthHashServiceDelegate esd = gov.va.med.mhv.eauth.service.
		        	delegate.ServiceDelegateFactory.
		        	createEAuthHashServiceDelegate();
		        VoidServiceResponse vsr = esd.saveHash(form.getUserName(),hash);
		        handleMessages(vsr);
		        forward = determineForward();
		        boolean err = vsr.getMessages().hasErrorMessages();
		        if (!err) {
		          EAuthLogServiceDelegate elsd = gov.va.med.mhv.eauth.service.
		          	delegate.ServiceDelegateFactory.
		          	createEAuthLogServiceDelegate();
		          vsr = elsd.logEauthEvent(hash,form.getUserName(),csid,"MAP");
		          if (vsr.getMessages().hasErrorMessages()) {
		        	for (Object item:vsr.getMessages().getErrorMessages()) {
				      LOG.error("Error: " + item);
		        	}
		          }
		        }
		    }
		}
		return forward;
    }

    /**
     * This method represents the point of entry into the pageflow
     * @jpf:action
     * @jpf:forward name="success" path="login.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success", path = "login.jsp")
    })
    public Forward navigate() {
    	Forward forward=new Forward("success");
    	forward.setPath(getRedirectUrl());
        return forward;
    }

    protected Class getFormClass() {
        return LoginForm.class;
    }

	protected void handleMessages(ServiceResponse response) {
		Message message = getError(response.getMessages(), "account.locked");
		if (message != null) {
			// Add links to message
			message.addInsert(DirectorUtil.getForgotPasswordURL(getRequest()));
			message.addInsert(DirectorUtil.getContactUsURL(getRequest()));
		}
		super.handleMessages(response);
	}

	private static Message getError(Messages messages, String key) {
		Message message = null;
		if (messages != null) {
			for (Iterator i = messages.getErrorMessages().iterator();
			     (message == null) && i.hasNext();)
			{
				message = (Message) i.next();
				if ((message != null) && !message.getKey().equals(key)) {
					message = null; // not found, keep looking
				}
			}
		}

		return message;
	}

}
