package gov.va.med.mhv.usermgmt.portlet.login.loginbase;

import java.sql.Timestamp;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.service.BooleanServiceResponse;

import com.bea.content.ContentContext;
import com.bea.content.Node;
import com.bea.content.expression.Search;
import com.bea.content.federated.ContentManagerFactory;
import com.bea.content.federated.ISearchManager;
import com.bea.content.paging.ISortableFilterablePagedList;

import gov.va.med.mhv.usermgmt.portlet.base.UserBaseController;
import gov.va.med.mhv.usermgmt.portlet.constants.SessionAttributeConstants;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;
import gov.va.med.mhv.usermgmt.util.DirectorUtil;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

public abstract class LoginNavigateBaseController extends
		UserBaseController
{
	private static final Log LOG = LogFactory.getLog(LoginNavigateBaseController.class);
	private static final long serialVersionUID = -1778973456L;
	
	/**
	 * Callback that is invoked when this controller instance is created.
	 */

	/**
	 * determine the flow once a user i s authenticated
	 */
	protected String getRedirectUrl() {
		
		if (!hasTermsBeenAccepted(getRequest())) {
			return DirectorUtil.getTermsAndConditionsRedirectURL(getRequest());
		}

		if (hasTemporaryPassword(getRequest())) {
			return DirectorUtil.getChangeTemporaryPasswordURL(getRequest());

		}
		//clean up the session
		this.removeSessionAttributes();
		return DirectorUtil.getLoginRedirectURL(getRequest(), getResponse());

	}

	protected static Timestamp getTermsModifiedTime() {
		String query = "cm_isContent = true && active_flag=true && keywords likeignorecase 'Terms and Conditions'";
		try {
			Search search = new Search(query);
			// search.setUseCache(false);

			// Do the search using the ISearchManager.
			ISearchManager searchManager = ContentManagerFactory
				.getSearchManager();
			ISortableFilterablePagedList<Node> resultList = searchManager
				.search(new ContentContext(), search);
			Iterator iterator = resultList.iterator();
			if (iterator.hasNext()) {
				Node temp = resultList.iterator().next();
				return new Timestamp(temp.getModifiedDate().getTimeInMillis());
			}
			return null;
		} catch (Exception e) {
			return null;
		}
	}

	public static boolean hasTermsBeenAccepted(HttpServletRequest request) {
		Boolean termsAcceptedFlag = (Boolean) request.getSession().
			getAttribute(SessionAttributeConstants.TERMS_ACCEPTED_FLAG);
		if (termsAcceptedFlag == null) {
			UserProfile user = getUserProfile(request);
			Timestamp modifiedTime = getTermsModifiedTime();
			boolean accepted = false;
			if ( modifiedTime != null ) {
				BooleanServiceResponse response = ServiceDelegateFactory.
					createUserProfileServiceDelegate().verifyLatestTermsAccepted(
						user, modifiedTime);
				accepted = response.getBoolean();
			} else {
				LOG.error("ERROR: No T and C modification time found.");
				accepted = true;
			}
			request.getSession().setAttribute(SessionAttributeConstants.
				TERMS_ACCEPTED_FLAG, accepted);
			return accepted;
		}
		return termsAcceptedFlag.booleanValue();
	}

	public static boolean hasTemporaryPassword(HttpServletRequest request) {
		Boolean tempPasswordCheckRequired=(Boolean)request.getSession().getAttribute(SessionAttributeConstants.TEMPORARY_PASSWORD_CHECK_REQUIRED);
		if (tempPasswordCheckRequired==null){return false;}
		else{
			Boolean isTemporaryPassword = (Boolean) request.getSession().
				getAttribute(SessionAttributeConstants.TEMPORARY_PASSWORD);
			if (isTemporaryPassword == null) {
				UserProfile user = getUserProfile(request);
				BooleanServiceResponse response = ServiceDelegateFactory.
				createUserProfileServiceDelegate().isCurrentPasswordTemporary(
				user);
				isTemporaryPassword = response.getBoolean();
				setPasswordTemporary(request.getSession(), isTemporaryPassword.booleanValue());
			}
			return isTemporaryPassword.booleanValue();
		}
	}
	
	
	/**
	 * Denotes if the current user has a temporary password. 
	 * @return True is the user has a temporary password; false otherwise.
	 */
    protected boolean isPasswordTemporary() {
		Boolean isTemporaryPassword = (Boolean) getRequest().getSession().
			getAttribute(SessionAttributeConstants.TEMPORARY_PASSWORD);
    	return (isTemporaryPassword != null) && isTemporaryPassword.
    		booleanValue(); 
    }

	/**
	 * Sets whether the current user has a temporary password.
	 * @param isPasswordTemporary true is temporary; false otherswise. 
	 */
    protected void setPasswordTemporary(boolean isPasswordTemporary) {
    	setPasswordTemporary(getRequest().getSession(), isPasswordTemporary);
    }
    
    /**
     * Sets whether the current user has a temporary password.
     * @param session is the session to set teh attribute in
     * @param isPasswordTemporary true is temporary; false otherswise. 
     */
    protected static void setPasswordTemporary(HttpSession session, boolean isPasswordTemporary) {
    	session.setAttribute(SessionAttributeConstants.TEMPORARY_PASSWORD, 
    			Boolean.valueOf(isPasswordTemporary));
    }
    
    /**
     * clean up the session by removing some unused attributes
     *
     */
    private void removeSessionAttributes(){
    	getRequest().getSession().removeAttribute(SessionAttributeConstants.
    		TEMPORARY_PASSWORD);
    	getRequest().getSession().removeAttribute(SessionAttributeConstants.TEMPORARY_PASSWORD_CHECK_REQUIRED);
    	getRequest().getSession().removeAttribute(SessionAttributeConstants.TERMS_ACCEPTED_FLAG);
    }
    

}