package gov.va.med.mhv.usermgmt.portlet.logout;

import gov.va.med.mhv.usermgmt.enumeration.AccountStatusEnumeration;
import gov.va.med.mhv.usermgmt.enumeration.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.portlet.base.UserBaseController;
import gov.va.med.mhv.usermgmt.service.AccessControlCollectionServiceResponse;
import gov.va.med.mhv.usermgmt.service.PatientServiceResponse;
import gov.va.med.mhv.usermgmt.service.UserProfileServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.AccessControlServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.InPersonAuthenticationServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.service.delegate.UserProfileServiceDelegate;
import gov.va.med.mhv.usermgmt.transfer.Patient;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;
import gov.va.med.mhv.usermgmt.util.AccessControlUtils;
import gov.va.med.mhv.usermgmt.util.AccessDomainUtils;
import gov.va.med.mhv.usermgmt.util.AccessPermissionUtils;
import gov.va.med.mhv.usermgmt.util.Auditor;
import gov.va.med.mhv.usermgmt.util.DirectorUtil;
import gov.va.med.mhv.usermgmt.util.PatientCorrelationStatusUtils;

import java.io.StringWriter;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

import org.apache.beehive.netui.pageflow.Forward;
import org.apache.beehive.netui.pageflow.annotations.Jpf;
import org.apache.commons.lang.WordUtils;

import com.bea.p13n.security.Authentication;

/**
 * @jpf:controller
 *  */
@Jpf.Controller()
public class LogoutPageFlowController extends UserBaseController
{
    // Uncomment this declaration to access Global.app.
    //
    //     protected global.Global globalApp;
    //

    // For an example of page flow exception handling see the example "catch" and "exception-handler"
    // annotations in {project}/WEB-INF/src/global/Global.app

	public static final String HASH_PARAM = "va_eauth_hash";
	public static final String CSID_PARAM = "va_eauth_csid";
    private String fccxUrl;

	public String ACCOUNT_STATUS="account_status";
	
	
	public String getFccxUrl() {
		return fccxUrl;
	}
	
	/**
     * This method represents the point of entry into the pageflow
     * @jpf:action
     * @jpf:forward name="success" path="logout.jsp"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success",
                     path = "logout.jsp")
    })
    protected Forward begin()
    {
    	//JAZZ # 143252 - Federated Credential - retrieving the fccx (accessVa) url for the right environment.
    	fccxUrl = ServiceDelegateFactory.createUserProfileServiceDelegate().getFccxUrl().getString();
System.out.println("******* fccxUrl="+fccxUrl);    	

		// Adding for MVI Account Status Check
    	UserProfile profile=this.getCurrentUser();
		Boolean isPatientAuthenticated = null;
		PatientServiceResponse patientResponse = ServiceDelegateFactory.createPatientServiceDelegate().getPatientForUser(profile);
    	Patient patient = patientResponse.getPatient();
    	InPersonAuthenticationServiceDelegate delegate = ServiceDelegateFactory.createInPersonAuthenticationServiceDelegate();
    	if (patient != null) {
    		isPatientAuthenticated = delegate.isPatientAuthenticated(patient.getId()).getBoolean();
    	}


    	if(patient != null) {
    		if(patient.getMatchedDateTime() != null){
	    		if(isPatientAuthenticated != null) {
	    			if(isPatientAuthenticated == true && patient.getCorrelationStatus().equals(PatientCorrelationStatusUtils.CORRELATED))
	        			getRequest().setAttribute(ACCOUNT_STATUS, AccountStatusEnumeration.PREMIUM);
	    			else if(isPatientAuthenticated == true && !patient.getCorrelationStatus().equals(PatientCorrelationStatusUtils.CORRELATED))
	    				getRequest().setAttribute(ACCOUNT_STATUS, AccountStatusEnumeration.ADVANCED);
	        		else if(isPatientAuthenticated == false)
	        			getRequest().setAttribute(ACCOUNT_STATUS, AccountStatusEnumeration.ADVANCED);
	    		}
	    		else {
	    			getRequest().setAttribute(ACCOUNT_STATUS, AccountStatusEnumeration.ADVANCED);
	    		}
    		}else{
    			getRequest().setAttribute(ACCOUNT_STATUS, AccountStatusEnumeration.BASIC);
    		}
    	}else {
    		getRequest().setAttribute(ACCOUNT_STATUS, AccountStatusEnumeration.BASIC);
    	}

    	//JAZZ#11941 Remote ID Proofing Implementation - Allow only the test user group
    	if(profile != null && profile.getId() != null){
			if(hasErarAccess(profile.getUserName())) {
				getRequest().getSession().setAttribute("hasERarAccess", "true");
			} else {
				getRequest().getSession().setAttribute("hasERarAccess", "false");
			}
    	}

        return new Forward("success");
    }

    /**
     * @jpf:action
     * @jpf:forward name="success" path="begin.do"
     */
    @Jpf.Action(forwards = {
        @Jpf.Forward(name = "success", path = "begin.do")
		}
	)
    protected Forward logout()
    {
    	UserProfileServiceDelegate userService = ServiceDelegateFactory.createUserProfileServiceDelegate();
    	UserProfileServiceResponse response = userService.getProfileForUser(this.getUserPrincipal().getName());
    	UserProfile userProfile = response.getUserProfile();
    	if(userProfile != null && userProfile.getId() != null){
    		Auditor.auditLogoutEvent(userProfile.getId(), ActivityActorTypeEnumeration.SELF);
    	}

    	Authentication.logout( getRequest() );

        //switch desktops
        Forward forward = new Forward("success");

        String path = DirectorUtil.getDefaultSplashPage(getRequest(), getResponse());
        if (path != null) forward.setPath(path);

        /*
         * Checking if the VAAFI hash parameter exists in the header before forwarding the user
         * to VAAFI logout page instead of MHV Logout page
         */
        String ahash = getRequest().getHeader(HASH_PARAM);
        String csid = getRequest().getHeader(CSID_PARAM);
        if (null != ahash && null != csid) {
        	if(csid.equalsIgnoreCase("DSLogon"))
        		Auditor.auditDSLogonLogoutEvent(userProfile.getId(), ActivityActorTypeEnumeration.SELF, "eBenefits");
        	if(csid.equalsIgnoreCase("USAA"))
        		Auditor.auditDSLogonLogoutEvent(userProfile.getId(), ActivityActorTypeEnumeration.SELF, "AccessVA");
        	path = DirectorUtil.getLoggedOutVAAFIPage(getRequest(), getResponse());
        	if (path != null) forward.setPath(path);
        }
        return forward ;
    }

    public String getUserName() {
		UserProfile profile=this.getCurrentUser();
		if (profile == null) {
			return "";
		}
		StringWriter name = new StringWriter();
		if (profile.getFirstName() != null) {
			name.append(profile.getFirstName());
		}

		return name.toString();
    }

    public String getFullName() {
		UserProfile profile=this.getCurrentUser();
		if (profile == null) {
			return "";
		}
		StringWriter name = new StringWriter();
		if (profile.getFirstName() != null) {
			if(profile.getFirstName().length()>40)
			name.append("<BR/>");
			name.append( WordUtils.capitalize(profile.getFirstName().toLowerCase()));
		}

		return name.toString();
    }

public String getLastLoginLabel() {
		UserProfile profile=this.getCurrentUser();
		if (profile == null) {
			return "";
		}
	StringWriter name = new StringWriter();

	DateFormat dateFormat = new SimpleDateFormat ("dd MMM yyyy '@' HHmm zz");
    dateFormat.setTimeZone(TimeZone.getTimeZone("America/New_York"));

	if (profile.getLastLogin() != null) {
		 String dateTime=dateFormat.format(profile.getLastLogin());
           if(!dateTime.contains("0000"))
        	   name.append("Last successful login:  ");
           else
        	   name.append(" ");
		}else{
			name.append(" ");
		}
	return name.toString();

}
    public String getLastLoginDate() {
		UserProfile profile=this.getCurrentUser();
		if (profile == null) {
			return "";
		}


		StringWriter name = new StringWriter();

		DateFormat dateFormat = new SimpleDateFormat ("dd MMM yyyy '@' HHmm zz");
	//	DateFormat dateFormat = new SimpleDateFormat ("dd MMM yyyy");
	    dateFormat.setTimeZone(TimeZone.getTimeZone("America/New_York"));

		if (profile.getLastLogin() != null) {
			 String dateTime=dateFormat.format(profile.getLastLogin());
		       if(!dateTime.contains("0000"))
		       {
		    	   dateTime=dateTime.replace("EDT", "ET");
		    	   name.append(dateTime.replace("EST", "ET"));
		       }else{
		    	   name.append(" ");
		       }
		//	 name.append(dateTime);
		}else{

		//	String dateTime=dateFormat.format(new Date());
			//name.append(dateTime);
			name.append(" ");
//			 if(profile.getCurrentLogin()!=null)
//			 {
//				 String dateTime=dateFormat.format(profile.getCurrentLogin());
//				 dateTime=	dateTime.replace("EDT", "ET");
//			  		name.append(dateTime.replace("EST", "ET"));
//			 }else
//			 {
//				 if(profile.getCreatedDate()!=null){
//					 String dateTime=dateFormat.format(profile.getCreatedDate());
//					 dateTime=dateTime.replace("EDT", "ET");
//					 	name.append(dateTime.replace("EST", "ET"));
//				 }else
//				 {
//					 name.append("Now");
//				 }
//			 }
		}

		return name.toString();
    }

    /*
     * This class is just here because the abstract class requires it.
     * @see org.tigris.atlas.beehive.AbstractPageFlowController#getFormClass()
     */
	protected Class getFormClass() {
		return null;
	}

    //JAZZ#11941 Remote ID Proofing Implementation - Allow only the test user group
	public boolean hasErarAccess(String username) {
		boolean hasExtractAccess = false;
		AccessControlCollectionServiceResponse accessResponse =
			getAccessControlServiceDelegate().findAccessControls(username);
		//   Precondition.assertNotNull("response", response);
		hasExtractAccess = AccessControlUtils.hasAccess(AccessDomainUtils.PHR,
				"VAeRAR",AccessPermissionUtils.READ, accessResponse.getAccessControls()); // "unchecked"
		return hasExtractAccess;
	}

    //JAZZ#11941 Remote ID Proofing Implementation - Allow only the test user group
	public AccessControlServiceDelegate getAccessControlServiceDelegate() {
		return gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory.createAccessControlServiceDelegate();
	}
}
