package gov.va.med.mhv.usermgmt.portlet.upgrade;

import gov.va.med.mhv.usermgmt.enumeration.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.form.InPersonAuthenticationForm;
import gov.va.med.mhv.usermgmt.portlet.base.UserBaseController;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;
import gov.va.med.mhv.usermgmt.util.Auditor;

import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;

import org.apache.beehive.netui.pageflow.Forward;
import org.apache.beehive.netui.pageflow.annotations.Jpf;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.service.StringServiceResponse;

/**
 * Jazz #110486 RemoteID Proofing - User Starts on the MHV Home Page - Landing page controller for login with different accounts
 * @jpf:controller
 *  */
@Jpf.Controller()
public class UpgradeAccountPageFlowController extends UserBaseController
{
    // Uncomment this declaration to access Global.app.
    //
    //     protected global.Global globalApp;
    //

    // For an example of page flow exception handling see the example "catch" and "exception-handler"
    // annotations in {project}/WEB-INF/src/global/Global.app
	
	public static final String HASH_PARAM = "va_eauth_hash";
	public static final String CSID_PARAM = "va_eauth_csid";
	private static final String DISPLAY_ERROR = "displayError";	
	private static final String VAPII_UNAVAILABLE = "vapiiUnavail";	

	public String ACCOUNT_STATUS="account_status";
	Log clog = LogFactory.getFactory().getInstance(UpgradeAccountPageFlowController.class);
	/**
     * This method represents the point of entry into the pageflow
     * @jpf:action
     * @jpf:forward name="success" path="logout.jsp"
     */
    @Jpf.Action(forwards = {
    		@Jpf.Forward(name = "success", navigateTo = Jpf.NavigateTo.currentPage),
    		@Jpf.Forward(name = "failure", path = "loginWithDiffAccount.jsp")
        })
    protected Forward begin() {
    	Forward forward = determineForward();
    	UserProfile profile=this.getCurrentUser();
    	StringServiceResponse response = ServiceDelegateFactory.createUserProfileServiceDelegate().getVapiiFormStatusForUser(profile);

    	if(response.getMessages().hasErrorMessages()) {
    		getSession().setAttribute(DISPLAY_ERROR, VAPII_UNAVAILABLE);
    		return new Forward("failure");
    	}
		
    	//Jazz #110471 US REQ#5.5.1 RemoteID Proofing -  System shall create an Account Activity Log 
    	//entry in background when user selects Continue button.
		Auditor.auditAccessVAAccountUpgradeEvent(profile.getId(), ActivityActorTypeEnumeration.SELF);
    	String accessVAURL = retrieveAccessVAURL();
        forward.setPath(accessVAURL);
    	return forward;
    }
    
	@Jpf.Action(forwards = {@Jpf.Forward(name = "success", path = "/gov/va/med/mhv/usermgmt/portlet/upgrade/loginWithDiffAccountPF.jsp")})
	public Forward viewPrint() {
		return new Forward("success");
	}
    
    
    
    private String retrieveAccessVAURL() {
		Properties props = new Properties();
		String propertyFile = "eauth.properties";
		String path = System.getProperty("config.location");
System.out.println("Path:"+path);		
		String accessVAURL = null;
		try {
			props.load(new FileInputStream(path + File.separator + propertyFile));
System.out.println("Props:"+props);			
			String env = props.getProperty("env.type");
			if(env.equals("dev")) {
				accessVAURL = props.getProperty("dev.access.va.url", "https://int.access.domain");
			} else if(env.equals("int")) {
				accessVAURL = props.getProperty("int.access.va.url", "https://int.access.domain");
			} else if(env.equals("systest")) {
				accessVAURL = props.getProperty("systest.access.va.url", "https://pint.access.domain");
			} else if(env.equals("prod")) {
				accessVAURL = props.getProperty("prod.access.va.url", "https://access.domain");
			}
		} catch (Exception io) {
			clog.error("error getting properties", io);
		}
		return accessVAURL;
    }
    
	protected Class getFormClass() {
		return null;
	}
}
