package gov.va.med.mhv.usermgmt.vapii;

import gov.va.med.mhv.usermgmt.enumeration.ActivityActionTypeEnumeration;
import gov.va.med.mhv.usermgmt.service.UserProfileServiceResponse;
import gov.va.med.mhv.usermgmt.service.delegate.DownloadYourReportServiceDelegate;
import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.service.delegate.UserProfileServiceDelegate;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;
import gov.va.med.mhv.usermgmt.util.Auditor;

import java.io.IOException;
import java.util.Properties;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tigris.atlas.config.ConfigurationManager;
import org.tigris.atlas.service.BlobServiceResponse;

/**
 *
 */
public class DownloadVapiiServlet extends HttpServlet {

	private static final Log LOG = LogFactory
			.getLog(DownloadVapiiServlet.class);
	private static final long serialVersionUID = -3626747660141050375L;

	public static final String DOWNLOAD_RPT_ERROR = "DownloadServlet.BB_ERROR";

	// Configuration Properties
	Properties properties = ConfigurationManager
			.getConfiguration("/downloadData.properties");

	public static final String REPORT_ID = "reportId";
	public static final String DOWNLOAD_FORMAT = "downloadFormat";
	public static final String FORMAT_TYPE_PDF = "pdfFormat";
	public static final String FORMAT_TYPE_BB = "bbFormat";
	public static final String FORMAT_TYPE_TEXT = "textFormat";
	public static final String FORMAT_TYPE_PDF_INLINE = "pdfInlineFormat";
	public static final String WATERMARK_LOCATION = "waterMarkImageLocation";
	ServletContext context;

	public void init(ServletConfig config) throws ServletException {
		context = config.getServletContext();
	}

	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		doPost(request, response);
	}

	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		response.reset();
		response.setContentType("application/pdf");
		String contentDisType = "attachment; fileName=eRAR.pdf";
		if(request.getParameter("format")!=null && request.getParameter("format").equalsIgnoreCase("inline")) {
			contentDisType = "inline; fileName=eRAR.pdf";
		}
		response.setHeader( "Content-Disposition", contentDisType);

		ServletOutputStream outs;

		byte[] allPdfBytes = null;
		
		String userName = request.getParameter("username");
		if (StringUtils.isBlank(userName))
			throw new IllegalStateException("Not logged in");
		else {
			UserProfileServiceDelegate delegate = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory
					.createUserProfileServiceDelegate();
			UserProfileServiceResponse userProfileServiceResponse = delegate.getProfileForUser(userName);
			UserProfile userProfile = userProfileServiceResponse.getUserProfile();
		
			BlobServiceResponse bResponse = delegate.downloadVapiiFormForUser(userProfile);
			allPdfBytes = (byte[]) bResponse.getValue();
	
			outs = response.getOutputStream();
			if (allPdfBytes != null)
				outs.write(allPdfBytes);
			outs.flush();
			String audit = request.getParameter("audit");
			if(audit!=null && audit.equalsIgnoreCase("true")) {
				//JAZZ#32763 US01 RQ#9 For Download, MHV system shall make following entry in the AAL table: :  
				Auditor.auditDownloadViewPrintERAREvent(userProfile.getId(), true, ActivityActionTypeEnumeration.DOWNLOAD_10_5345A_MHV_FORM_ONLINE, "");
			}
			outs.close();
		}
	}

	/*
	 * Method to get DownloadServiceDelegate
	 */
	private DownloadYourReportServiceDelegate getDownloadYourServiceDelegate() {
		return ServiceDelegateFactory.createDownloadYourReportServiceDelegate();
	}

	/*
	 * Method to Get the Current User Profile
	 */
	protected final Long getCurrentUserId(HttpServletRequest request) {
		String userName = request.getRemoteUser();
		if (StringUtils.isBlank(userName))
			throw new IllegalStateException("Not logged in");
		else {
			UserProfileServiceDelegate delegate = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory
					.createUserProfileServiceDelegate();
			UserProfileServiceResponse response = delegate
					.getProfileForUser(userName);
			UserProfile userProfile = response.getUserProfile();
			return userProfile.getId();
		}
	}

	/*
	 * Method to get current User Name
	 */
	protected final String getCurrentUserName(HttpServletRequest request) {
		String userName = request.getRemoteUser();
		StringBuffer userFirstLastName = new StringBuffer();
		if (StringUtils.isBlank(userName))
			throw new IllegalStateException("Not logged in");
		else {
			UserProfileServiceDelegate delegate = gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory
					.createUserProfileServiceDelegate();
			UserProfileServiceResponse response = delegate
					.getProfileForUser(userName);
			UserProfile userProfile = response.getUserProfile();
			userFirstLastName
					.append(userProfile.getLastName() != null ? userProfile
							.getLastName() : "");

			if (userProfile.getFirstName() != null) {
				userFirstLastName.append(", ");
				userFirstLastName.append(userProfile.getFirstName());
			} else {
				userFirstLastName.append("");
			}
			if (userProfile.getMiddleName() != null) {
				userFirstLastName.append(" " + userProfile.getMiddleName());
			}

		}

		return userFirstLastName.toString();
	}

}
