package gov.va.med.mhv.usermgmt.uup.ejb;

import gov.va.med.mhv.usermgmt.service.delegate.ServiceDelegateFactory;
import gov.va.med.mhv.usermgmt.transfer.TransferObjectFactory;
import gov.va.med.mhv.usermgmt.transfer.UserProfile;

import java.rmi.RemoteException;

import javax.ejb.EJBException;
import javax.ejb.NoSuchEntityException;

import org.apache.commons.beanutils.BeanUtils;

import com.bea.p13n.cache.CacheManager;
import com.bea.p13n.property.PropertyLocator;
import com.bea.p13n.property.PropertyValidationException;
import com.bea.p13n.property.internal.EntityPropertyManagerImpl;

public class UserPropertyManagerBean extends EntityPropertyManagerImpl {
	private static final String ENTITY_PROPERTY_CACHE = "entityPropertyCache";

	//  Begin delegation to User Profile Services

	public void setProperty(String userName, String propertySet, String propertyName, Object value) throws NoSuchEntityException {
		UserProfile up = getProfileForUser(userName);
		setProfileProperty(up, propertyName, value);
		ServiceDelegateFactory.createEntityMaintenanceServiceDelegate().save(up);
	}

	public Object removeProperty(String userName, String propertySet, String propertyName) throws NoSuchEntityException {
		UserProfile up = getProfileForUser(userName);
		Object value = getProfileProperty(up, propertyName);
		setProfileProperty(up, propertyName, null);
		return value;
	}

	public Object getProperty(String userName, String propertySet, String propertyName) 
		throws NoSuchEntityException {
		
		UserProfile up = getProfileForUser(userName);
		return getProfileProperty(up, propertyName);
	}

	public String getPropertyAsString(String userName, String propertySet, String propertyName) throws NoSuchEntityException {
		Object value = getProperty(userName, propertySet, propertyName);		
		return value == null || value instanceof String ? (String)value : value.toString();
	}

	public void createUserProfile(String userName) throws RemoteException {
		UserProfile up = TransferObjectFactory.createUserProfile();
		up.setUserName(userName);
		ServiceDelegateFactory.createEntityMaintenanceServiceDelegate().save(up);
	}

	public void deleteUserProfile(String userName) throws NoSuchEntityException {
		UserProfile up = getProfileForUser(userName);
		ServiceDelegateFactory.createEntityMaintenanceServiceDelegate().delete(up.getUserProfilePK());
	}

	public boolean userProfileExists(String userName) {
		try {
			getProfileForUser(userName);
			return true;
		}
		catch(NoSuchEntityException e) {
			return false;
		}
	}

	//  End delegation to user profile services

	public void setProperty(PropertyLocator locator, String propertySet, String propertyName, Object value) 
		throws PropertyValidationException, NoSuchEntityException {
		
		setProperty(locator.getPkString(), propertySet, propertyName, value);
	}
	
	public Object getProperty(PropertyLocator locator, String propertySet, String propertyName) 
		throws NoSuchEntityException {
		
		return getProperty(locator.getPkString(), propertySet, propertyName);
	}

	public Object removeProperty(PropertyLocator locator, String propertySet, String propertyName) throws NoSuchEntityException {
		return removeProperty(locator.getPkString(), propertySet, propertyName);
	}

	public long createUniqueId(String jndiHomeName, String pkString) {
		return -1L;
	}

	public void removeEntity(PropertyLocator locator) throws NoSuchEntityException {
		removePropertyCache(locator);
		deleteUserProfile(locator.getPkString());
	}
	
	// convenience methods
	
	private void removePropertyCache(PropertyLocator locator) {
		CacheManager.flushAll(ENTITY_PROPERTY_CACHE, locator);
	}

	private UserProfile getProfileForUser(String userName) throws NoSuchEntityException {
		UserProfile up = ServiceDelegateFactory.createUserProfileServiceDelegate().getProfileForUser(userName).getUserProfile();
		
		if(up == null)
			throw new NoSuchEntityException("No UserProfile exists for user \"" + userName + "\".");
		
		return up;
	}
	
	private Object getProfileProperty(UserProfile up, String propertyName) {
		try {
			return BeanUtils.getProperty(up, propertyName);
		}
		catch(Exception e) {
			throw new EJBException("Unable to get profile property \"" + propertyName + "\" for user \"" + up.getUserName() + "\"", e);
		}
	}
	
	private void setProfileProperty(UserProfile up, String propertyName, Object value) {
		try {
			BeanUtils.setProperty(up, propertyName, value);
		}
		catch(Exception e) {
			throw new EJBException("Unable to set profile property \"" + propertyName + "\" for user \"" + up.getUserName() + "\"", e);
		}
	}
}